(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[   1074195,      25039]
NotebookOptionsPosition[    955348,      23031]
NotebookOutlinePosition[    982878,      23575]
CellTagsIndexPosition[    981620,      23532]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Graphs", "BookChapterTitle",
 CounterAssignments->{{
   "ChapterNumber", 
    10}},ExpressionUUID->"1491c7f9-4f9a-4a9e-9c29-c96188c7b0da"],

Cell[CellGroupData[{

Cell["Introduction", "Section",ExpressionUUID->"66dd4a72-0321-46bb-8367-82f921295cf2"],

Cell[TextData[{
 "In this chapter, we consider ways in which ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can help you explore and understand graphs. In particular, we describe how \
to visualize and perform computations on graphs."
}], "Text",ExpressionUUID->"1e7096ef-be0f-4fd5-a7fb-d384f5c0101e"]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.1 Graphs and Graph Models", "Section",
 CellTags->"Section1",ExpressionUUID->"7e1d154e-2090-47d3-8bc4-b8c9bc44c92c"],

Cell[TextData[{
 "In the Wolfram Language, a graph is represented by a special raw object \
with head ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ". You should think about ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " not as a function, but as a way to describe and represent an object. To \
explain this distinction, consider a fraction, such as ",
 Cell[BoxData[
  FormBox[
   FractionBox["5", "9"], TraditionalForm]],ExpressionUUID->
  "dfe1b95b-58fc-4c24-b3cf-a72c7dd78343"],
 ". We think about this as a single number, and the Wolfram Language treats \
it as a simple object, but if you delve down using ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 ", you see that it is a bit more complicated."
}], "Text",ExpressionUUID->"91ac5a55-9f9a-4232-aac0-270f33f8635c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"5", "/", "9"}], " ", "//", "FullForm"}]], "Input",
 CellLabel->"In[1]:=",ExpressionUUID->"d3b511e6-087a-4929-8d51-e510d059bcee"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"Rational", "[", 
    RowBox[{"5", ",", "9"}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellLabel->
  "Out[1]//FullForm=",ExpressionUUID->"64261b8f-8cd4-4421-bef7-9ffe99a70b64"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " reveals that the fraction ",
 Cell[BoxData[
  FormBox[
   RowBox[{"5", "/", "9"}], TraditionalForm]],ExpressionUUID->
  "c5372caf-6c52-4a38-95e1-c163828bd626"],
 " is actually represented by ",
 StyleBox[ButtonBox["Rational",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rational"], "FunctionLink"],
 " applied to two integers. ",
 StyleBox[ButtonBox["Rational",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rational"], "FunctionLink"],
 " is not a function, in the usual sense. Rather, it is a head that tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " what the contents mean. ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " is exactly the same, if a bit more complicated. It is a head that tells ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " that the contents represent a graph object. Moreover, just like a ",
 StyleBox[ButtonBox["Rational",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rational"], "FunctionLink"],
 " is typically displayed in the usual fraction notation, a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object is displayed as a drawing of the graph. "
}], "Text",ExpressionUUID->"bd8d4154-8723-4815-bb0b-e23b48170a40"],

Cell[TextData[{
 "The simplest way to specify a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object in the Wolfram Language is by specifying the edges as a list of \
rules. You typically use positive integers or strings for the vertices \
(although other expressions can be used). An edge between the vertex \
represented by 1 and the vertex represented by \[OpenCurlyDoubleQuote]a\
\[CloseCurlyDoubleQuote] is given as the rule ",
 Cell[BoxData[
  RowBox[{"1", "->", "\"\<a\>\""}]], "myCode",ExpressionUUID->
  "9fa22d8a-d507-4f38-a07a-52827d19e8ce"],
 "."
}], "Text",ExpressionUUID->"eb36443a-e5fa-4028-8ee0-86f771de608a"],

Cell["\<\
As an example, the following defines a directed version of the graph shown in \
Exercise 3 in Section 10.1.\
\>", "Text",ExpressionUUID->"45faebc0-0474-45de-a7cb-9578a1eab641"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise3directed", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], "]"}]}]], "Input",\

 InitializationCell->True,
 CellLabel->"In[2]:=",ExpressionUUID->"9892c346-ced9-4372-a8d6-e23182e27da3"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {{{1, 2}, {1, 3}, {2, 3}, {2, 4}}, Null}]]}, 
    
    TagBox[GraphicsGroupBox[GraphicsComplexBox[CompressedData["
1:eJxTTMoPSmViYGCQB2IQjQYcINQHe1QaHRAWXz5vY/d1+/9wftJEow8l2f/g
fL0bTbo3mv7C+TzFi5wZ2/7A+b8SQRp+I/hL/EEIod7m7ix5sZ9wvqG6lUjC
ie9wfhoYfIPzF4O1f4HzH1v0AdFnOF9ZCQQ+wfmJ3NsSuLch/GP/crLdy8nv
4Pw9u0HgNZxvCTLu8Qs4H6Q7gfsZnG8MBo/hfHDw5D+A88/xd09pXH4bzhfP
OSiWc/AanC9ZscvUXucSnL8i/c2jr+dPw/lT7EAOPAznP3h57c8Gj51w/j+t
M518rCvsAYH5cLQ=
       "], {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
         ArrowBox[{1, 2}, 0.02261146496815286], 
         ArrowBox[BezierCurveBox[{1, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 
          17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 3}], 
          0.02261146496815286], ArrowBox[{2, 3}, 0.02261146496815286], 
         ArrowBox[{2, 4}, 0.02261146496815286]}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02261146496815286], DiskBox[2, 0.02261146496815286], 
         DiskBox[3, 0.02261146496815286], DiskBox[4, 0.02261146496815286]}}]],
     
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[2]=",ExpressionUUID->"d3db0b7b-8f80-4def-9422-03f77a9801af"]
}, Open  ]],

Cell[TextData[{
 "To produce an undirected simple graph, you can set the option ",
 StyleBox[ButtonBox["DirectedEdges",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdges"], "FunctionLink"],
 " to ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"08e46637-b7d5-41cf-831d-ed8325c581b5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise3", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[3]:=",ExpressionUUID->"c9956d2e-2233-4057-9fd3-9706cdca4f45"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}]]}, 
    
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
       0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
       0.42430280790670055`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.022873923537282098`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.022873923537282098], DiskBox[2, 0.022873923537282098], 
         DiskBox[3, 0.022873923537282098], 
         DiskBox[4, 0.022873923537282098]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[3]=",ExpressionUUID->"0e42c780-3d52-4b85-b19c-64c4ff2e76ab"]
}, Open  ]],

Cell["\<\
Note that the difference in appearance in the directed and undirected \
versions is a result of a preference in the Wolfram Language to draw directed \
graphs with arrows pointing down. In the undirected case, there is no such \
concern and a more compact image results.\
\>", "Text",ExpressionUUID->"4617c9bf-0eec-47fd-8445-3255a62524f1"],

Cell[TextData[{
 "You can also create (undirected) simple graphs using a ",
 StyleBox[ButtonBox["TwoWayRule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TwoWayRule"], "FunctionLink"],
 " (",
 StyleBox["<->", "Code"],
 ", ",
 StyleBox["\[TwoWayRule]", "Code"],
 ") or the symbol \[UndirectedEdge] in place of ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 ", ",
 StyleBox["\[Rule]", "Code"],
 "). The undirected edge symbol is entered by typing \[EscapeKey]",
 StyleBox["ue",
  IgnoreSpellCheck->True],
 "\[EscapeKey]. "
}], "Text",ExpressionUUID->"131a2c41-3fd8-41a8-b8ab-bfd6df1bb55f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\"\<a\>\"", "\[TwoWayRule]", "\"\<b\>\""}], ",", 
    RowBox[{"\"\<a\>\"", "\[TwoWayRule]", "\"\<c\>\""}], ",", 
    RowBox[{"\"\<b\>\"", "\[TwoWayRule]", "\"\<c\>\""}], ",", 
    RowBox[{"\"\<b\>\"", "\[TwoWayRule]", "\"\<d\>\""}]}], "}"}], 
  "]"}]], "Input",
 CellLabel->"In[4]:=",ExpressionUUID->"88d3a644-34d2-4668-bcd2-dd1fff27c14f"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}]]}, 
    
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
       0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
       0.42430280790670055`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.022873923537282098`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.022873923537282098], DiskBox[2, 0.022873923537282098], 
         DiskBox[3, 0.022873923537282098], 
         DiskBox[4, 0.022873923537282098]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[4]=",ExpressionUUID->"8f67ba99-a7e0-4fc0-bef9-493a4516519d"]
}, Open  ]],

Cell[TextData[{
 "If you prefer, you can use the symbol \[DirectedEdge], entered as \
\[EscapeKey]de\[EscapeKey] in place of ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 ") for a directed graph. In this manual, we will generally use ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 ") to specify all edges together with the ",
 StyleBox[ButtonBox["DirectedEdges",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdges"], "FunctionLink"],
 " option, when needed, as this frequently involves the least typing and \
typos do not result in mixed graphs. There are, however, some functions that \
apply to edges and require that you correctly distinguish between directed \
(using ",
 StyleBox["->", "Code"],
 " or \[EscapeKey]de\[EscapeKey]) and undirected edges (using ",
 StyleBox["<->", "Code"],
 " or \[EscapeKey]",
 StyleBox["ue",
  IgnoreSpellCheck->True],
 "\[EscapeKey]). Pay special attention to those situations."
}], "Text",ExpressionUUID->"03a29ee4-2656-493a-a9fa-5b7b153390ef"],

Cell[TextData[{
 "If you wish, you may give an explicit list of vertices as the first \
argument to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ". This is only required in the situation when a graph has a vertex not \
adjacent to any other, as in the example below."
}], "Text",ExpressionUUID->"655bbf22-af78-41a3-aa9c-eacb87db9877"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "2"}], ",", 
     RowBox[{"2", "\[Rule]", "3"}], ",", 
     RowBox[{"3", "\[Rule]", "4"}], ",", 
     RowBox[{"4", "\[Rule]", "1"}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellLabel->"In[5]:=",ExpressionUUID->"04cc0727-f9a4-4dbc-81ad-5ec1dcae5a5f"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {2, 3}, {3, 4}, {4, 1}}}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{1.1570827321497008`, -1.1281685887629571`}, {
       0.15768973023584465`, -1.154236031031331}, {
       0.12841548684339066`, -0.15485132588994366`}, {
       1.1283539266140916`, -0.12841548684339066`}, {
       0.12841548684339066`, -1.4110670047181124`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.016512279377655925`]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.016512279377655925`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.016512279377655925`]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.016512279377655925`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.016512279377655925], DiskBox[2, 0.016512279377655925], 
         DiskBox[3, 0.016512279377655925], DiskBox[4, 0.016512279377655925], 
         DiskBox[5, 0.016512279377655925]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[5]=",ExpressionUUID->"ad012f9b-243a-478f-b57c-48affe5545d4"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["EdgeList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeList"], "FunctionLink"],
 " functions applied to a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object produce lists of the vertices and edges of the graph."
}], "Text",ExpressionUUID->"128bdea1-c1bb-4bcf-9a63-9df6f548f883"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexList", "[", "exercise3", "]"}]], "Input",
 CellLabel->"In[6]:=",ExpressionUUID->"a89caa0d-de96-4233-954c-7cb5cfa5ea54"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>"}],
   "}"}]], "Output",
 CellLabel->"Out[6]=",ExpressionUUID->"c44a7b70-49cf-45dd-9d06-43c556264835"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EdgeList", "[", "exercise3", "]"}]], "Input",
 CellLabel->"In[7]:=",ExpressionUUID->"f235a7d0-f96b-48ae-a368-24e5202cd162"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"b\"\>"}], ",", 
   RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"d\"\>"}]}], "}"}]], "Output",\

 CellLabel->"Out[7]=",ExpressionUUID->"aab3addd-63f0-4d3c-96c6-c4cc1a2e300a"]
}, Open  ]],

Cell[TextData[{
 "Observe that the output of ",
 StyleBox[ButtonBox["EdgeList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeList"], "FunctionLink"],
 " is a list of undirected edges, using the symbol \[UndirectedEdge], despite \
the fact that we defined the graph using rules. This is because the effect of \
setting the ",
 StyleBox[ButtonBox["DirectedEdges",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdges"], "FunctionLink"],
 " option to ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " is to transform the rules into undirected edges. Using ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 ", you can see that these are stored using the head ",
 StyleBox[ButtonBox["UndirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UndirectedEdge"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"f61db585-31e9-4f29-918e-185c0d028be8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"EdgeList", "[", "exercise3", "]"}], "//", "FullForm"}]], "Input",
 CellLabel->"In[8]:=",ExpressionUUID->"4d79ae5c-7fee-4917-a989-d67b494537ca"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"List", "[", 
    RowBox[{
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<a\>\"", ",", "\"\<b\>\""}], "]"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<a\>\"", ",", "\"\<c\>\""}], "]"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<c\>\""}], "]"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<d\>\""}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellLabel->
  "Out[8]//FullForm=",ExpressionUUID->"f67ec78c-57ea-4ddc-b89a-9a6178a2cd77"]
}, Open  ]],

Cell[TextData[{
 "Note that the same is true in the directed case, with the edges represented \
internally as ",
 StyleBox[ButtonBox["DirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdge"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"00519f47-789f-4070-b1c7-673859a63c79"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EdgeList", "[", "exercise3directed", "]"}]], "Input",
 CellLabel->"In[9]:=",ExpressionUUID->"bcdc60e9-c18a-4b56-bd6a-c6edfba23f93"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"a\"\>", "\[DirectedEdge]", "\<\"b\"\>"}], ",", 
   RowBox[{"\<\"a\"\>", "\[DirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[DirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[DirectedEdge]", "\<\"d\"\>"}]}], "}"}]], "Output",
 CellLabel->"Out[9]=",ExpressionUUID->"a016a09b-a2ab-467d-a756-0f92e767f465"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"EdgeList", "[", "exercise3directed", "]"}], "//", 
  "FullForm"}]], "Input",
 CellLabel->"In[10]:=",ExpressionUUID->"54e86a64-6b9f-4bd8-a39e-a541aa1c8b47"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"List", "[", 
    RowBox[{
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"\"\<a\>\"", ",", "\"\<b\>\""}], "]"}], ",", 
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"\"\<a\>\"", ",", "\"\<c\>\""}], "]"}], ",", 
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<c\>\""}], "]"}], ",", 
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<d\>\""}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellLabel->
  "Out[10]//FullForm=",ExpressionUUID->"adaff00c-bc64-4814-9941-839d58ddda74"]
}, Open  ]],

Cell[TextData[{
 "You can use ",
 StyleBox[ButtonBox["DirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdge"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["UndirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UndirectedEdge"], "FunctionLink"],
 " yourself in creating graphs."
}], "Text",ExpressionUUID->"cb73f302-ccf5-4530-bcea-16c63f4e5552"],

Cell[CellGroupData[{

Cell[TextData[{
 "Options Affecting the Visual Output of ",
 StyleBox["Graph", "Code"]
}], "Subsection",
 CellTags->"Section1a",ExpressionUUID->"edbfee8f-9a39-43d0-bfd2-9599e02df672"],

Cell[TextData[{
 "In this section, we will explain just a few of the options available for \
changing the visual appearance of ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " objects. Readers interested in greater control over the display of ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " objects should refer to the documentation pages."
}], "Text",ExpressionUUID->"0ed072f2-3a7c-44a0-9116-5edef16453b8"],

Cell[TextData[{
 "To display the names of vertices, use the ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " option. Probably the most common value for this option is ",
 StyleBox["\"Name\"", "Code"],
 ", including the quotation marks, which causes each vertex to be labeled \
with its name. As an example, we modify the definition given above of the \
graph from Exercise 3, with this option invoked."
}], "Text",ExpressionUUID->"e45996c5-bf35-41a0-bbca-a3ad1d8e8c77"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]], "Input",
 CellLabel->"In[11]:=",ExpressionUUID->"0a09f387-735a-45a4-a6bb-2bfe5ae96556"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {0.024962990061063034, 0.022873923537282098}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9175848200953383, 0.4241670904205272}, 
          0.022873923537282098], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {0.9404587436326204, 0.44704101395780926}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {0.022873923537282098, 0.8702454747554453}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.022873923537282098], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {2.0552874113911086, 0.4471767314439826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[11]=",ExpressionUUID->"b17f4419-2b8d-4552-b9f1-d49f7259c25b"]
}, Open  ]],

Cell[TextData[{
 "Another use of the ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " option is to specify labels for specific vertices. You do this by \
identifying the option with a list consisting of rules associating the name \
of the vertex with the desired label. The following illustrates this by \
labeling two of the vertices of the graph from Exercise 3."
}], "Text",ExpressionUUID->"d07f2506-e100-47dc-89ca-e5bd9d7051bd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<A\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "4"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[12]:=",ExpressionUUID->"5d11ba58-4fbf-4530-a6b2-b980edc260c8"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {"d" -> 4, "a" -> "A"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
         InsetBox["\<\"A\"\>", 
          Offset[{2, 2}, {0.024962990061063034, 0.022873923537282098}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, 
        DiskBox[{0.9175848200953383, 0.4241670904205272}, 
         0.022873923537282098], 
        DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.022873923537282098], 
         InsetBox["4", 
          Offset[{2, 2}, {2.0552874113911086, 0.4471767314439826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[12]=",ExpressionUUID->"c669670e-f87b-4ed9-9ce4-8e39d4249183"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " option can be usefully combined with the ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " symbol. The basic syntax of ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " is with a first argument being an expression to be displayed and a second \
argument indicating the position. Common choices for position include ",
 StyleBox["Above", "Code"],
 ", ",
 StyleBox["Below", "Code"],
 ", ",
 StyleBox["Before", "Code"],
 ", ",
 StyleBox["After", "Code"],
 ", and ",
 StyleBox["Center", "Code"],
 ". ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " can be used with first argument ",
 StyleBox["\[OpenCurlyDoubleQuote]Name\[CloseCurlyDoubleQuote]", "Code"],
 " in order to have all vertices labeled with their name or with a specific \
label for a particular vertex. "
}], "Text",ExpressionUUID->"38daf13f-184c-433b-a174-99c4fa8cdb2c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Below"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[13]:=",ExpressionUUID->"c2144ee1-9ce7-496e-85a8-0273128b1157"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {
         Placed["Name", Below]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
         InsetBox["\<\"a\"\>", 
          Offset[{0, -2}, {0.002089066523780936, -0.022873923537282098}], 
          ImageScaled[{0.5, 1}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9175848200953383, 0.4241670904205272}, 
          0.022873923537282098], 
         InsetBox["\<\"b\"\>", 
          Offset[{0, -2}, {0.9175848200953383, 0.4012931668832451}], 
          ImageScaled[{0.5, 1}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], 
         InsetBox["\<\"c\"\>", Offset[{0, -2}, {0., 0.8244976276808811}], 
          ImageScaled[{0.5, 1}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.022873923537282098], 
         InsetBox["\<\"d\"\>", 
          Offset[{0, -2}, {2.0324134878538267, 0.4014288843694185}], 
          ImageScaled[{0.5, 1}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[13]=",ExpressionUUID->"11ff6366-de35-4065-a94b-ae09ca1da85b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "->", 
       RowBox[{"Placed", "[", 
        RowBox[{"\"\<A\>\"", ",", "Below"}], "]"}]}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", 
       RowBox[{"Placed", "[", 
        RowBox[{"4", ",", "After"}], "]"}]}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[14]:=",ExpressionUUID->"d4685fdc-d947-497d-9201-1454f507da40"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {
        "d" -> Placed[4, After], "a" -> Placed["A", Below]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
         InsetBox["\<\"A\"\>", 
          Offset[{0, -2}, {0.002089066523780936, -0.022873923537282098}], 
          ImageScaled[{0.5, 1}],
          BaseStyle->"Graphics"]}, 
        DiskBox[{0.9175848200953383, 0.4241670904205272}, 
         0.022873923537282098], 
        DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.022873923537282098], 
         InsetBox["4", 
          Offset[{2, 0}, {2.0552874113911086, 0.42430280790670055}], 
          ImageScaled[{0, 0.5}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[14]=",ExpressionUUID->"cd0b7eb6-c159-4464-a092-5b9e9d76c970"]
}, Open  ]],

Cell["\<\
Observe that you can combine rules specifying labels for individual vertices \
with a default label specification.\
\>", "Text",ExpressionUUID->"0a80cc13-6060-4389-890a-c7c78ce58dc8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "->", 
       RowBox[{"Placed", "[", 
        RowBox[{"\"\<Name\>\"", ",", "Below"}], "]"}]}], ",", 
      RowBox[{"Placed", "[", 
       RowBox[{"\"\<Name\>\"", ",", "Above"}], "]"}]}], "}"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[15]:=",ExpressionUUID->"4a4d5d27-9fbb-49a8-a9da-00602306e219"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {
         Placed["Name", Above], "a" -> Placed["Name", Below]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
         InsetBox["\<\"a\"\>", 
          Offset[{0, -2}, {0.002089066523780936, -0.022873923537282098}], 
          ImageScaled[{0.5, 1}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9175848200953383, 0.4241670904205272}, 
          0.022873923537282098], 
         InsetBox["\<\"b\"\>", 
          Offset[{0, 2}, {0.9175848200953383, 0.44704101395780926}], 
          ImageScaled[{0.5, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], 
         InsetBox["\<\"c\"\>", Offset[{0, 2}, {0., 0.8702454747554453}], 
          ImageScaled[{0.5, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.022873923537282098], 
         InsetBox["\<\"d\"\>", 
          Offset[{0, 2}, {2.0324134878538267, 0.4471767314439826}], 
          ImageScaled[{0.5, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[15]=",ExpressionUUID->"ef8f17d6-31f5-4082-bc16-f96207900bfe"]
}, Open  ]],

Cell[TextData[{
 "Similarly, edges in a graph can be labeled using the ",
 StyleBox[ButtonBox["EdgeLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeLabels"], "FunctionLink"],
 " option. Once again, the value ",
 StyleBox["\[OpenCurlyDoubleQuote]Name\[CloseCurlyDoubleQuote]", "Code"],
 " will cause all edges to be labeled."
}], "Text",ExpressionUUID->"1f7d8997-696e-4eb4-a79c-7395e58b7bb8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"EdgeLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]], "Input",
 CellLabel->"In[16]:=",ExpressionUUID->"5033f8ba-04c0-48c1-b168-7225d9706985"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      EdgeLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], {
         {Arrowheads[0.], 
          ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
           0.4241670904205272}}, 0.022873923537282098`]}, InsetBox[
          RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"b\"\>"}], {0.4598369433095596, 0.2120835452102636},
           ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], 
          ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
           0.022873923537282098`]}, InsetBox[
          RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], {0.001044533261890468, 0.4236857756090816},
           ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], 
          ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
           0.8473715512181632}}, 0.022873923537282098`]}, InsetBox[
          RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], {0.45879241004766913, 0.6357693208193452},
           ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], 
          ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
           2.0324134878538267`, 0.42430280790670055`}}, 
           0.022873923537282098`]}, InsetBox[
          RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"d\"\>"}], {1.4749991539745824, 0.4242349491636139},
           ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
        DiskBox[{0.9175848200953383, 0.4241670904205272}, 
         0.022873923537282098], 
        DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], 
        DiskBox[{2.0324134878538267, 0.42430280790670055}, 
         0.022873923537282098]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[16]=",ExpressionUUID->"1b844450-ccb0-42fa-98a4-b1ff2d969695"]
}, Open  ]],

Cell[TextData[{
 "As with vertices, you can also choose to label specific edges with \
arbitrary labels by giving a list of rules as the value to ",
 StyleBox[ButtonBox["EdgeLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeLabels"], "FunctionLink"],
 ". When doing this, the edges should be specified using either \
\[UndirectedEdge] (\[EscapeKey]",
 StyleBox["ue",
  IgnoreSpellCheck->True],
 "\[EscapeKey]) or \[DirectedEdge] (\[EscapeKey]de\[EscapeKey]); \
alternatively, the edge can be specified with a ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 ") provided that the edge is surrounded by parentheses."
}], "Text",ExpressionUUID->"77cb1f23-515d-47a7-8aa4-7346588e0bf6"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"EdgeLabels", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[UndirectedEdge]", "\"\<b\>\""}], "\[Rule]", 
      SubscriptBox["e", "1"]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[17]:=",ExpressionUUID->"c30db9d3-2516-43ff-aca9-be07cc001c7b"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      EdgeLabels -> {
        UndirectedEdge["a", "b"] -> Subscript[$CellContext`e, 1]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], {
         {Arrowheads[0.], 
          ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
           0.4241670904205272}}, 0.022873923537282098`]}, InsetBox[
          SubscriptBox["e", "1"], {0.4598369433095596, 0.2120835452102636}, 
          ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
        DiskBox[{0.9175848200953383, 0.4241670904205272}, 
         0.022873923537282098], 
        DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], 
        DiskBox[{2.0324134878538267, 0.42430280790670055}, 
         0.022873923537282098]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[17]=",ExpressionUUID->"aea6637e-a4da-4458-9f64-354e11a39e30"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"EdgeLabels", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\"\<a\>\"", "->", "\"\<b\>\""}], ")"}], "\[Rule]", 
      SubscriptBox["e", "1"]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[18]:=",ExpressionUUID->"57c5bbe1-4c7b-4bf5-bf19-d799122f8d11"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      EdgeLabels -> {
        UndirectedEdge["a", "b"] -> Subscript[$CellContext`e, 1]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], {
         {Arrowheads[0.], 
          ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
           0.4241670904205272}}, 0.022873923537282098`]}, InsetBox[
          SubscriptBox["e", "1"], {0.4598369433095596, 0.2120835452102636}, 
          ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 
          0.022873923537282098`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], 
        DiskBox[{0.9175848200953383, 0.4241670904205272}, 
         0.022873923537282098], 
        DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], 
        DiskBox[{2.0324134878538267, 0.42430280790670055}, 
         0.022873923537282098]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[18]=",ExpressionUUID->"a61f3acb-0e8b-4f18-945a-b2be40a309f6"]
}, Open  ]],

Cell[TextData[{
 "As an alternative to ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["EdgeLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeLabels"], "FunctionLink"],
 ", you can use the wrapper ",
 StyleBox[ButtonBox["Labeled",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Labeled"], "FunctionLink"],
 " around an edge in the list of edges or, in conjunction with the optional \
list of vertices, around a vertex. The first element in ",
 StyleBox[ButtonBox["Labeled",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Labeled"], "FunctionLink"],
 " is the name of the vertex or the edge definition. The second element is \
the label to be used. "
}], "Text",ExpressionUUID->"e74bd55a-3dd2-40cc-8729-cfbc077c60dd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", 
     RowBox[{"Labeled", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<b\>\""}], "]"}], ",", "\"\<c\>\"", ",", 
     RowBox[{"Labeled", "[", 
      RowBox[{"\"\<d\>\"", ",", "4"}], "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"Labeled", "[", 
      RowBox[{
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", "\"\<edge\>\""}], 
      "]"}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellLabel->"In[19]:=",ExpressionUUID->"2bc7ba42-88e2-4334-b50b-2f14fc4e362a"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      EdgeLabels -> {UndirectedEdge["b", "d"] -> "edge"}, 
       VertexLabels -> {"d" -> 4, "b" -> "b"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.022873923537282098`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.022873923537282098`]}, {
         {Arrowheads[0.], 
          ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
           2.0324134878538267`, 0.42430280790670055`}}, 
           0.022873923537282098`]}, 
         InsetBox["\<\"edge\"\>", {1.4749991539745824, 0.4242349491636139}, 
          ImageScaled[{0.5, 0.5}],
          BaseStyle->"Graphics"]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        DiskBox[{0.002089066523780936, 0.}, 0.022873923537282098], {
         DiskBox[{0.9175848200953383, 0.4241670904205272}, 
          0.022873923537282098], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {0.9404587436326204, 0.44704101395780926}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, 
        DiskBox[{0., 0.8473715512181632}, 0.022873923537282098], {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.022873923537282098], 
         InsetBox["4", 
          Offset[{2, 2}, {2.0552874113911086, 0.4471767314439826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[19]=",ExpressionUUID->"9650bbe3-bdb2-476f-836e-ebc6429b9bb9"]
}, Open  ]],

Cell[TextData[{
 "You can change the size of vertices with the ",
 StyleBox[ButtonBox["VertexSize",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexSize"], "FunctionLink"],
 " option. Some possible values include ",
 StyleBox["Tiny", "Code"],
 ", ",
 StyleBox["Small", "Code"],
 ", ",
 StyleBox["Medium", "Code"],
 ", and ",
 StyleBox["Large", "Code"],
 ", or a number between 0 and 1. A numerical value indicates that the size of \
the vertex should be that proportion of the distance between the closest two \
vertices. This can be used in conjunction with ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " to have vertex labels appear inside the vertex."
}], "Text",ExpressionUUID->"332c89da-7e34-4b44-8b84-ce598c224f54"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]], "Input",
 CellLabel->"In[20]:=",ExpressionUUID->"5545b2a7-1cbe-4d8f-9aa7-3ab979fc13d4"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
          0.4241670904205272}}, 0.08473741263531812]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.002089066523780936, 0.}, {0., 0.8473715512181632}}, 
          0.08473741263531812]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {0., 
          0.8473715512181632}}, 0.08473741263531812]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9175848200953383, 0.4241670904205272}, {
          2.0324134878538267`, 0.42430280790670055`}}, 0.08473741263531812]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.002089066523780936, 0.}, 0.08473741263531812], 
         InsetBox["\<\"a\"\>", {0.002089066523780936, 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9175848200953383, 0.4241670904205272}, 
          0.08473741263531812], 
         InsetBox["\<\"b\"\>", {0.9175848200953383, 0.4241670904205272},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.8473715512181632}, 0.08473741263531812], 
         InsetBox["\<\"c\"\>", {0., 0.8473715512181632},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.0324134878538267, 0.42430280790670055}, 
          0.08473741263531812], 
         InsetBox["\<\"d\"\>", {2.0324134878538267, 0.42430280790670055},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[20]=",ExpressionUUID->"9b5e51b4-3522-4c05-9a4e-992892c93689"]
}, Open  ]],

Cell[TextData[{
 "You can further control the appearance, such as the color, of vertices and \
edges with the ",
 StyleBox[ButtonBox["VertexStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexStyle"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["EdgeStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeStyle"], "FunctionLink"],
 " options. Multiple styles can be combined with the ",
 StyleBox[ButtonBox["Directive",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Directive"], "FunctionLink"],
 " wrapper. For example, the following illustrates how to create red vertices \
and thick green edges. Readers interested in exploring the various options \
available should consult the ",
 ButtonBox["Graphics Directives",
  BaseStyle->"Link",
  ButtonData->"paclet:guide/GraphicsDirectives"],
 " guide."
}], "Text",ExpressionUUID->"b545811d-046b-40a6-9cde-be940bb47a09"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexStyle", "\[Rule]", "Red"}], ",", 
   RowBox[{"EdgeStyle", "\[Rule]", 
    RowBox[{"Directive", "[", 
     RowBox[{"Green", ",", "Thick"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[21]:=",ExpressionUUID->"ea30b30e-6cab-4fa9-9078-a2f7799a4d5c"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      EdgeStyle -> {
         Directive[
          RGBColor[0, 1, 0], 
          Thickness[Large]]}, VertexStyle -> {
         RGBColor[1, 0, 0]}}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
       0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
       0.42430280790670055`}}, {
        {RGBColor[0, 1, 0], Thickness[Large], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.022873923537282098`]}}, 
        {RGBColor[1, 0, 0], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.022873923537282098], DiskBox[2, 0.022873923537282098], 
         DiskBox[3, 0.022873923537282098], 
         DiskBox[4, 0.022873923537282098]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[21]=",ExpressionUUID->"38e20e89-a218-4fe9-98be-75da7717c9ff"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " option is used to specify the locations of vertices. The value for the ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " option is a list of pairs of the form ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      StyleBox[
       SubscriptBox["x", "1"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox[
        StyleBox["y", "CodeVar"], "1"], "CodeVar"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      StyleBox[
       SubscriptBox[
        StyleBox["x", "CodeVar"], "2"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox[
        StyleBox["y", "CodeVar"], "2"], "CodeVar"]}], "}"}], ",", "..."}], 
   "}"}]], "myCode",ExpressionUUID->"f04fb139-9587-4b14-a51c-a0ce6fa3c79f"],
 ", with the pairs specifying the coordinates of a vertex. The order of the \
locations must correspond to the order of the vertices in the graph. When an \
explicit list of vertices is given as the optional first element of ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ", that list specifies the order. Otherwise, the order is determined by when \
the vertex is first encountered in the list of edges, and is the same as the \
output of ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 ". Below, we use ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " to redraw the graph from Exercise 3 with the vertices in the same \
positions as in the image in the textbook."
}], "Text",ExpressionUUID->"a2948603-61fe-4a57-9431-5daaa8468d4b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"VertexCoordinates", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[22]:=",ExpressionUUID->"04997973-1027-4639-8fb6-9c0088a19692"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}, {
      VertexLabels -> {"Name"}, 
       VertexCoordinates -> {{0, 1}, {1, 1}, {0, 0}, {1, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {1., 1.}}, 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {0., 0.}}, 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {0., 0.}}, 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 0.}}, 0.01273]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 1.}, 0.01273], 
         InsetBox["\<\"a\"\>", Offset[{2, 2}, {0.01273, 1.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.01273], 
         InsetBox["\<\"b\"\>", Offset[{2, 2}, {1.01273, 1.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.01273], 
         InsetBox["\<\"c\"\>", Offset[{2, 2}, {0.01273, 0.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.01273], 
         InsetBox["\<\"d\"\>", Offset[{2, 2}, {1.01273, 0.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[22]=",ExpressionUUID->"c119a6df-26f0-40bd-be0b-65b417b3bc46"]
}, Open  ]],

Cell[TextData[{
 "You can also exert control over the layout of the vertices of a graph, but \
without specifying each coordinate, by using the ",
 StyleBox[ButtonBox["GraphLayout",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphLayout"], "FunctionLink"],
 " option to specify the algorithm used to choose the vertex location. Common \
choices include: ",
 StyleBox["\"SpringEmbedding\"", "Code"],
 ", which treats edges as springs and minimizes the mechanical energy of the \
system; ",
 StyleBox["\"SpringElectricalEmbedding\"", "Code"],
 ", which treats edges as springs and vertices as electrical charges and \
minimizes mechanical and electrical energy; ",
 StyleBox["\"HighDimensionalEmbedding\"", "Code"],
 ", which is like the spring-electrical method but computes in high \
dimensions and then projects down to two; ",
 StyleBox["\"CircularEmbedding\"", "Code"],
 ", which places the vertices on a circle; and ",
 StyleBox["\"LayeredDrawing\"", "Code"],
 ", which places vertices in layers and attempts to reduce the number of \
edges between non-adjacent layers."
}], "Text",ExpressionUUID->"b1e93fde-a9e9-4889-929f-7ba385b41e36"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Modifying Graphs", "Subsection",
 CellTags->"Section1b",ExpressionUUID->"ba8b3a85-5f88-4759-baa1-259a619565f1"],

Cell[TextData[{
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " objects may contain loops, as illustrated below with a replica of Exercise \
7 from Section 10.1. Loops are created simply by including an edge from a \
vertex to itself."
}], "Text",ExpressionUUID->"306ff252-4c10-412e-bb4b-4aa9c3fdf4cf"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise7", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<a\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<e\>\""}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[23]:=",ExpressionUUID->"8f8bea1f-8afe-4673-9755-a2ade768a75b"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e"}, {{{1, 2}, {2, 5}, {3, 2}, {3, 3}, {3, 4}, {4, 
       3}, {5, 1}, {5, 4}, {5, 5}}, Null}, {
      VertexLabels -> {"Name"}, 
       VertexCoordinates -> {{1, 1}, {2, 2}, {3, 2}, {3, 1}, {2, 1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
        ArrowBox[{{1., 1.}, {2., 2.}}, 0.02261146496815286], 
        ArrowBox[{{2., 2.}, {2., 1.}}, 0.02261146496815286], 
        ArrowBox[{{3., 2.}, {2., 2.}}, 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{3., 2.}, {2.9226652357765426`, 
         2.211102824883941}, {3.03037168156465, 2.404072374917325}, {
         3.1710710001067524`, 2.4556369686221933`}, {3.4556369686221933`, 
         2.1710710001067524`}, {3.404072374917325, 2.03037168156465}, {
         3.211102824883941, 1.9226652357765426`}, {3., 2.}},
         SplineDegree->7], 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{3., 2.}, {3.164440747183118, 
         1.499999999998105}, {3., 1.}}], 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{3., 1.}, {2.835559252816882, 
         1.5000000000037899`}, {3., 2.}}], 0.02261146496815286], 
        ArrowBox[{{2., 1.}, {1., 1.}}, 0.02261146496815286], 
        ArrowBox[{{2., 1.}, {3., 1.}}, 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{2., 1.}, {2.2304450364563646`, 
         0.8476276972960466}, {2.250002442847489, 0.6526270006898856}, {
         2.1815918401357854`, 0.5391807550302723}, {1.8184081598642148`, 
         0.5391807550302722}, {1.7499975571525106`, 0.6526270006898856}, {
         1.7695549635436354`, 0.8476276972960466}, {2., 1.}},
         SplineDegree->7], 0.02261146496815286]}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{1., 1.}, 0.02261146496815286], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {1.0226114649681528, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 2.}, 0.02261146496815286], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {2.022611464968153, 2.022611464968153}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 2.}, 0.02261146496815286], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {3.022611464968153, 2.022611464968153}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 1.}, 0.02261146496815286], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {3.022611464968153, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.02261146496815286], 
         InsetBox["\<\"e\"\>", 
          Offset[{2, 2}, {2.022611464968153, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[23]=",ExpressionUUID->"5c439ffb-e128-4c59-bd7b-9c951356f056"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " objects may also include multiple edges. The following is the graph from \
Exercise 4 from Section 10.1."
}], "Text",ExpressionUUID->"0923ac16-f6fc-4454-885d-98c528516e4a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise4", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[24]:=",ExpressionUUID->"4075caa7-818b-473a-b14b-b4f5cfac54b7"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {2, 4}, {2, 4}, {2, 4}, {3, 4}}}, {
      VertexLabels -> {"Name"}, 
       VertexCoordinates -> {{1, 2}, {2, 2}, {1, 1}, {2, 1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1., 2.}, {1.4999999999999987`, 
          2.1644407471839577`}, {2., 2.}}], 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1., 2.}, {1.4999999999999987`, 
          1.8355592528160423`}, {2., 2.}}], 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {1., 1.}}, 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2., 2.}, {2.1644407471831175`, 
          1.5000000000009477`}, {2., 1.}}], 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {2., 1.}}, 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2., 2.}, {1.8355592528168825`, 
          1.5000000000009477`}, {2., 1.}}], 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.01273]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{1., 2.}, 0.01273], 
         InsetBox["\<\"a\"\>", Offset[{2, 2}, {1.01273, 2.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 2.}, 0.01273], 
         InsetBox["\<\"b\"\>", Offset[{2, 2}, {2.01273, 2.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.01273], 
         InsetBox["\<\"c\"\>", Offset[{2, 2}, {1.01273, 1.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.01273], 
         InsetBox["\<\"d\"\>", Offset[{2, 2}, {2.01273, 1.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[24]=",ExpressionUUID->"1f6c9bd5-c21f-4755-b03d-c9e397157ee6"]
}, Open  ]],

Cell[TextData[{
 "You can modify existing graphs by adding and deleting edges and vertices \
with the functions ",
 StyleBox[ButtonBox["VertexAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexAdd"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["VertexDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDelete"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["EdgeAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeAdd"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["EdgeDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeDelete"], "FunctionLink"],
 ". Each of these functions requires a graph as the first argument. The \
second argument is usually either a vertex, an edge, or a list of vertices or \
edges. The deletion functions can, in place of a vertex, edge, or list, \
accept a pattern as the second argument in order to delete all of the \
vertices or edges that match the pattern."
}], "Text",ExpressionUUID->"6ed9357f-6779-4e40-b9cf-84ff26975b59"],

Cell[TextData[{
 "Beginning with the graph from Exercise 4, we use ",
 StyleBox[ButtonBox["VertexAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexAdd"], "FunctionLink"],
 " to add two new vertices to the graph. "
}], "Text",ExpressionUUID->"94f96250-df0d-41f1-8a1a-909900f11189"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"modifyExercise4", "=", 
  RowBox[{"VertexAdd", "[", 
   RowBox[{"exercise4", ",", 
    RowBox[{"{", 
     RowBox[{"\"\<y\>\"", ",", "\"\<z\>\""}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[25]:=",ExpressionUUID->"87877f43-721e-4565-acf5-19feb048ef54"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "y", "z"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {2, 4}, {2, 4}, {2, 4}, {3, 4}}}, {
      VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{
          1.1158951306234963`, -0.11578978443249777`}, {
          0.6109257028526557, -0.26527315995309775`}, {
          0.11580448506865626`, -0.08584522946197004}}], 
          0.015786999451181005`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{
          1.1158951306234963`, -0.11578978443249777`}, {0.6207739128394941, 
          0.063638146058651}, {0.11580448506865626`, -0.08584522946197004}}], 
          0.015786999451181005`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1158951306234963`, -0.11578978443249777`}, {
          1.0847677461503926`, -1.115840835387724}}, 
          0.015786999451181005`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{
          0.11580448506865626`, -0.08584522946197004}, {
          0.26511352349851064`, -0.5903093331485807}, {
          0.08584522946197004, -1.0849203920811903`}}], 
          0.015786999451181005`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.11580448506865626`, -0.08584522946197004}, {
          0.08584522946197004, -1.0849203920811903`}}, 
          0.015786999451181005`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{
          0.11580448506865626`, -0.08584522946197004}, {-0.06346380896788516, \
-0.5804562883945427}, {0.08584522946197004, -1.0849203920811903`}}], 
          0.015786999451181005`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.0847677461503926`, -1.115840835387724}, {
          0.08584522946197004, -1.0849203920811903`}}, 
          0.015786999451181005`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{1.1158951306234963, -0.11578978443249777}, 
          0.015786999451181005], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {1.1316821300746773, -0.10000278498131676}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.11580448506865626, -0.08584522946197004}, 
          0.015786999451181005], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {0.13159148451983727, -0.07005823001078904}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.0847677461503926, -1.115840835387724}, 
          0.015786999451181005], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {1.1005547456015736, -1.100053835936543}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.08584522946197004, -1.0849203920811903}, 
          0.015786999451181005], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {0.10163222891315105, -1.0691333926300093}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.08584522946197001, -1.2875312943116641}, 
          0.015786999451181005], 
         InsetBox["\<\"y\"\>", 
          Offset[{2, 2}, {0.10163222891315102, -1.2717442948604831}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6867154095047032, -1.2875312943116641}, 
          0.015786999451181005], 
         InsetBox["\<\"z\"\>", 
          Offset[{2, 2}, {0.7025024089558842, -1.2717442948604831}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[25]=",ExpressionUUID->"a25ce52c-96cc-4123-9dcf-0e76f3388233"]
}, Open  ]],

Cell["\<\
Now, we add edges to connect the new vertices with the rest of the graph. \
Note that, in this example, the Wolfram Language interprets rules as \
undirected edges since the original graph is undirected.\
\>", "Text",ExpressionUUID->"dff2e508-e3be-4eef-8ab6-f5cc831c9bf8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"modifyExercise4", "=", 
  RowBox[{"EdgeAdd", "[", 
   RowBox[{"modifyExercise4", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<z\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<y\>\""}], ",", 
      RowBox[{"\"\<y\>\"", "\[Rule]", "\"\<z\>\""}]}], "}"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[26]:=",ExpressionUUID->"d081112e-eecf-4317-81ff-3a7d70f461cf"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "y", "z"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {2, 4}, {2, 4}, {2, 4}, {3, 4}, {1, 6}, {
       1, 5}, {5, 6}}}, {VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9865249697355845, 0.5084119656762822}, {
          1.5707344479236138`, 0.4188459778346099}, {1.987736639086351, 0.}}],
           0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9865249697355845, 0.5084119656762822}, {
          1.4035271608983213`, 0.08956598784164148}, {1.987736639086351, 
          0.}}], 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9865249697355845, 0.5084119656762822}, {
          1.9870439836336748`, 1.017230340258258}}, 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9865249697355845, 0.5084119656762822}, {0., 
          0.14640950520208257`}}, 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9865249697355845, 0.5084119656762822}, {
          0.0010762706055189586`, 0.8712013662210176}}, 
          0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1.987736639086351, 0.}, {
          2.2959053377998098`, 0.3833114202700723}, {2.771434915944913, 
          0.5088789801145902}}], 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.987736639086351, 0.}, {2.771434915944913, 
          0.5088789801145902}}, 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1.987736639086351, 0.}, {
          2.4632662172314572`, 0.12556755984453322`}, {2.771434915944913, 
          0.5088789801145902}}], 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9870439836336748`, 1.017230340258258}, {
          2.771434915944913, 0.5088789801145902}}, 0.028577670360465574`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.14640950520208257`}, {0.0010762706055189586`, 
          0.8712013662210176}}, 0.028577670360465574`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.9865249697355845, 0.5084119656762822}, 
          0.028577670360465574], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {1.0151026400960501, 0.5369896360367478}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.987736639086351, 0.}, 0.028577670360465574], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {2.0163143094468166, 0.028577670360465574}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.9870439836336748, 1.017230340258258}, 
          0.028577670360465574], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {2.0156216539941405, 1.0458080106187237}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.771434915944913, 0.5088789801145902}, 
          0.028577670360465574], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {2.8000125863053786, 0.5374566504750558}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.14640950520208257}, 0.028577670360465574], 
         InsetBox["\<\"y\"\>", 
          Offset[{2, 2}, {0.028577670360465574, 0.17498717556254814}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.0010762706055189586, 0.8712013662210176}, 
          0.028577670360465574], 
         InsetBox["\<\"z\"\>", 
          Offset[{2, 2}, {0.029653940965984532, 0.8997790365814832}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[26]=",ExpressionUUID->"397b7933-5e50-41ce-92ab-2af8255c5cee"]
}, Open  ]],

Cell[TextData[{
 "Next, we delete one of the old edges. Note that to delete an edge from an \
undirected graph, you must use \[UndirectedEdge] (\[EscapeKey]",
 StyleBox["ue",
  IgnoreSpellCheck->True],
 "\[EscapeKey]) or ",
 StyleBox[ButtonBox["TwoWayRule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TwoWayRule"], "FunctionLink"],
 " (",
 StyleBox["<->", "Code"],
 "). Likewise, deleting an edge from a directed graph must use \
\[DirectedEdge] (\[EscapeKey]de\[EscapeKey]) or ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 ")."
}], "Text",ExpressionUUID->"131fe72e-4f04-48e7-9953-1a5d965282b8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"modifyExercise4", "=", 
  RowBox[{"EdgeDelete", "[", 
   RowBox[{"modifyExercise4", ",", 
    RowBox[{"\"\<d\>\"", "\[TwoWayRule]", "\"\<c\>\""}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[27]:=",ExpressionUUID->"37c54b7f-3464-4df1-aacf-82b2900216dd"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "y", "z"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {2, 4}, {2, 4}, {2, 4}, {1, 6}, {1, 5}, {
       5, 6}}}, {VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2.2219912987352175`, 
          1.0216116870138297`}, {1.5925838075438417`, 0.8774521403916186}, {
          1.0004762770135442`, 1.1350262795038841`}}], 
          0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2.2219912987352175`, 
          1.0216116870138297`}, {1.6298837682049139`, 1.2791858261274136`}, {
          1.0004762770135442`, 1.1350262795038841`}}], 
          0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.2219912987352175`, 1.0216116870138297`}, {
          2.411304025307592, 0.}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.2219912987352175`, 1.0216116870138297`}, {
          3.180789549229244, 1.26809860503561}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.2219912987352175`, 1.0216116870138297`}, {
          2.783743324494634, 1.8240358640698422`}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1.0004762770135442`, 
          1.1350262795038841`}, {0.48521728071636194`, 1.016179766909924}, {
          0., 1.2263713873784445`}}], 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.0004762770135442`, 1.1350262795038841`}, {0., 
          1.2263713873784445`}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1.0004762770135442`, 
          1.1350262795038841`}, {0.515258996297187, 1.345217899972305}, {0., 
          1.2263713873784445`}}], 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.180789549229244, 1.26809860503561}, {2.783743324494634, 
          1.8240358640698422`}}, 0.03152212939048707]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{2.2219912987352175, 1.0216116870138297}, 
          0.03152212939048707], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {2.2535134281257045, 1.0531338164043167}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.0004762770135442, 1.1350262795038841}, 
          0.03152212939048707], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {1.0319984064040313, 1.1665484088943712}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.411304025307592, 0.}, 0.03152212939048707], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {2.442826154698079, 0.03152212939048707}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 1.2263713873784445}, 0.03152212939048707], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {0.03152212939048707, 1.2578935167689316}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.180789549229244, 1.26809860503561}, 0.03152212939048707], 
         InsetBox["\<\"y\"\>", 
          Offset[{2, 2}, {3.212311678619731, 1.299620734426097}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.783743324494634, 1.8240358640698422}, 
          0.03152212939048707], 
         InsetBox["\<\"z\"\>", 
          Offset[{2, 2}, {2.815265453885121, 1.8555579934603292}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[27]=",ExpressionUUID->"75f1d671-b03d-43e4-bdfd-ac76e439b7cf"]
}, Open  ]],

Cell["\<\
Note that deleting an edge between two vertices with multiple edges will \
delete one of the edges.\
\>", "Text",ExpressionUUID->"dcecc9ec-2a92-4213-8fdf-b17a24e26d2f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"modifyExercise4", "=", 
  RowBox[{"EdgeDelete", "[", 
   RowBox[{"modifyExercise4", ",", 
    RowBox[{"\"\<d\>\"", "\[TwoWayRule]", "\"\<b\>\""}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[28]:=",ExpressionUUID->"b1b09ca4-7091-4eb2-87ac-d608918d7772"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "y", "z"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {2, 4}, {2, 4}, {1, 6}, {1, 5}, {5, 
       6}}}, {VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2.2219912987352175`, 
          1.0216116870138297`}, {1.5925838075438417`, 0.8774521403916186}, {
          1.0004762770135442`, 1.1350262795038841`}}], 
          0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2.2219912987352175`, 
          1.0216116870138297`}, {1.6298837682049139`, 1.2791858261274136`}, {
          1.0004762770135442`, 1.1350262795038841`}}], 
          0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.2219912987352175`, 1.0216116870138297`}, {
          2.411304025307592, 0.}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.2219912987352175`, 1.0216116870138297`}, {
          3.180789549229244, 1.26809860503561}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.2219912987352175`, 1.0216116870138297`}, {
          2.783743324494634, 1.8240358640698422`}}, 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1.0004762770135442`, 
          1.1350262795038841`}, {0.48521728071636194`, 1.016179766909924}, {
          0., 1.2263713873784445`}}], 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1.0004762770135442`, 
          1.1350262795038841`}, {0.515258996297187, 1.345217899972305}, {0., 
          1.2263713873784445`}}], 0.03152212939048707]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.180789549229244, 1.26809860503561}, {2.783743324494634, 
          1.8240358640698422`}}, 0.03152212939048707]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{2.2219912987352175, 1.0216116870138297}, 
          0.03152212939048707], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {2.2535134281257045, 1.0531338164043167}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.0004762770135442, 1.1350262795038841}, 
          0.03152212939048707], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {1.0319984064040313, 1.1665484088943712}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.411304025307592, 0.}, 0.03152212939048707], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {2.442826154698079, 0.03152212939048707}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 1.2263713873784445}, 0.03152212939048707], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {0.03152212939048707, 1.2578935167689316}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.180789549229244, 1.26809860503561}, 0.03152212939048707], 
         InsetBox["\<\"y\"\>", 
          Offset[{2, 2}, {3.212311678619731, 1.299620734426097}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.783743324494634, 1.8240358640698422}, 
          0.03152212939048707], 
         InsetBox["\<\"z\"\>", 
          Offset[{2, 2}, {2.815265453885121, 1.8555579934603292}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[28]=",ExpressionUUID->"54fb9206-c67d-47c9-9b94-179c1a29ddbe"]
}, Open  ]],

Cell["\<\
Finally, note that deleting a vertex also deletes all the edges incident with \
that vertex. \
\>", "Text",ExpressionUUID->"ed2e50a7-7d14-418c-9dda-930db03ffc74"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"modifyExercise4", "=", 
  RowBox[{"VertexDelete", "[", 
   RowBox[{"modifyExercise4", ",", "\"\<y\>\""}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[29]:=",ExpressionUUID->"030327b4-09aa-427a-b737-4be333ec58b0"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "z"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {2, 4}, {2, 4}, {1, 5}}}, {
      VertexLabels -> {"Name", "y" -> None}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2.0587390993139407`, 0.7350143119410218}, {
          1.5036745993414762`, 0.5524671820594976}, {0.9486120654460974, 
          0.7350202900125036}}], 0.027945497725088253`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{2.0587390993139407`, 0.7350143119410218}, {
          1.5036765654185638`, 0.9175674198940227}, {0.9486120654460974, 
          0.7350202900125036}}], 0.027945497725088253`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.0587390993139407`, 0.7350143119410218}, {
          2.6857914103076004`, 1.470339282738514}}, 0.027945497725088253`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.0587390993139407`, 0.7350143119410218}, {
          2.68612228736358, 0.}}, 0.027945497725088253`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9486120654460974, 0.7350202900125036}, {
          0.47430052003238915`, 0.5790465751198268}, {0., 
          0.7350538138851191}}], 0.027945497725088253`]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9486120654460974, 0.7350202900125036}, {
          0.47431154541370857`, 0.8910275287775387}, {0., 
          0.7350538138851191}}], 0.027945497725088253`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{2.0587390993139407, 0.7350143119410218}, 
          0.027945497725088253], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {2.086684597039029, 0.7629598096661101}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9486120654460974, 0.7350202900125036}, 
          0.027945497725088253], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {0.9765575631711857, 0.7629657877375918}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.6857914103076004, 1.470339282738514}, 
          0.027945497725088253], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {2.7137369080326885, 1.4982847804636021}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.7350538138851191}, 0.027945497725088253], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {0.027945497725088253, 0.7629993116102074}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.68612228736358, 0.}, 0.027945497725088253], 
         InsetBox["\<\"z\"\>", 
          Offset[{2, 2}, {2.7140677850886683, 0.027945497725088253}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[29]=",ExpressionUUID->"5ebe3d16-f7a4-4333-9cc1-853956272406"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Semantic Networks", "Subsection",
 CellTags->"Section1c",ExpressionUUID->"1fec1932-2ece-43ae-b25a-86b027486969"],

Cell[TextData[{
 "The textbook defines semantic networks at the end of Section 10.1. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s capabilities for working with graphs together with the \
extensive and growing Wolfram Knowledgebase are a powerful combination. We \
will create a semantic network for the word \[OpenCurlyDoubleQuote]program\
\[CloseCurlyDoubleQuote] using the Wolfram Knowledgebase."
}], "Text",ExpressionUUID->"afd2f856-2e5e-47e1-a9f0-1bb8a3b62325"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["WordData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WordData"], "FunctionLink"],
 " function will accept a string representing a word."
}], "Text",ExpressionUUID->"c03231c1-9912-4d24-9dea-40c39c4367de"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WordData", "[", "\"\<program\>\"", "]"}]], "Input",
 CellLabel->"In[30]:=",ExpressionUUID->"9666764d-665e-470c-95b1-a7e759c6a538"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
     ",", "\<\"Performance\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", ",", "\<\"Idea\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", ",", "\<\"System\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
     ",", "\<\"WrittenDocument\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
     ",", "\<\"SoftwareSystem\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", ",", "\<\"Show\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
     ",", "\<\"Information\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
     ",", "\<\"Promulgation\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Schedule\"\>"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"program\"\>", ",", "\<\"Verb\"\>", 
     ",", "\<\"CreateMentally\"\>"}], "}"}]}], "}"}]], "Output",
 CellLabel->"Out[30]=",ExpressionUUID->"937cbe1b-5e98-47bb-909d-4127d249449e"]
}, Open  ]],

Cell["\<\
The output displayed above is a list of lists, with the first member of each \
list being the word itself, the second being the part of speech, and the \
third indicating a possible \[OpenCurlyDoubleQuote]sense\
\[CloseCurlyDoubleQuote] of the word.\
\>", "Text",ExpressionUUID->"c7b02994-f421-44fb-850c-45e41db2df35"],

Cell[TextData[{
 StyleBox[ButtonBox["WordData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WordData"], "FunctionLink"],
 " accepts a second argument that allows you to get information about \
specific properties associated with the word. Some of the properties are ",
 StyleBox["\[OpenCurlyDoubleQuote]Antonyms\[CloseCurlyDoubleQuote]", 
  "Input"],
 ", ",
 StyleBox["\[OpenCurlyDoubleQuote]Synonyms\[CloseCurlyDoubleQuote]", 
  "Input"],
 ", ",
 StyleBox["\[OpenCurlyDoubleQuote]BroaderTerms\[CloseCurlyDoubleQuote]", 
  "Input"],
 ", ",
 StyleBox["\[OpenCurlyDoubleQuote]NarrowerTerms\[CloseCurlyDoubleQuote]", 
  "Input"],
 ", ",
 StyleBox["\[OpenCurlyDoubleQuote]Definitions\[CloseCurlyDoubleQuote]", 
  "Input"],
 " and ",
 StyleBox["\[OpenCurlyDoubleQuote]Examples\[CloseCurlyDoubleQuote]", 
  "Input"],
 ". "
}], "Text",ExpressionUUID->"1286ee83-71c2-4084-af46-561105a36c05"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WordData", "[", 
  RowBox[{"\"\<program\>\"", ",", "\"\<BroaderTerms\>\""}], "]"}]], "Input",
 CellLabel->"In[31]:=",ExpressionUUID->"b56f5cfb-5f7a-4f03-a341-e5b83ad7c581"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
      ",", "\<\"Performance\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", "\<\"performance\"\>", "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", ",", "\<\"Idea\"\>"}], 
     "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"idea\"\>", ",", "\<\"thought\"\>"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", ",", "\<\"System\"\>"}], 
     "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"system\"\>", ",", "\<\"system of rules\"\>"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
      ",", "\<\"WrittenDocument\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"document\"\>", ",", "\<\"papers\"\>", 
      ",", "\<\"written document\"\>"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
      ",", "\<\"SoftwareSystem\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"computer software\"\>", ",", "\<\"package\"\>", 
      ",", "\<\"software\"\>", ",", "\<\"software package\"\>", 
      ",", "\<\"software program\"\>", ",", "\<\"software system\"\>"}], 
     "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", ",", "\<\"Show\"\>"}], 
     "}"}], "\[Rule]", 
    RowBox[{"{", "\<\"show\"\>", "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
      ",", "\<\"Information\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"info\"\>", ",", "\<\"information\"\>"}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Noun\"\>", 
      ",", "\<\"Promulgation\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"announcement\"\>", ",", "\<\"promulgation\"\>"}], "}"}]}], 
   ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Verb\"\>", 
      ",", "\<\"Schedule\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", "\<\"schedule\"\>", "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"program\"\>", ",", "\<\"Verb\"\>", 
      ",", "\<\"CreateMentally\"\>"}], "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"create by mental act\"\>", ",", "\<\"create mentally\"\>"}],
      "}"}]}]}], "}"}]], "Output",
 CellLabel->"Out[31]=",ExpressionUUID->"4bf354e3-eb83-4e8b-b2cc-b7faf3afb4e5"]
}, Open  ]],

Cell[TextData[{
 "In the graph of the semantic network for the word \
\[OpenCurlyDoubleQuote]program,\[CloseCurlyDoubleQuote] the terms on the \
right-hand side of the rules in the output above will be the neighbors of \
\[OpenCurlyDoubleQuote]program.\[CloseCurlyDoubleQuote] The first step is to \
extract all of those terms. Fortunately, ",
 StyleBox[ButtonBox["WordData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WordData"], "FunctionLink"],
 " accepts a third argument to specify the form of the output. If we give ",
 StyleBox["\[OpenCurlyDoubleQuote]List\[CloseCurlyDoubleQuote]", "Input"],
 " as that third argument, the output will be a list of all the terms related \
to \[OpenCurlyDoubleQuote]program.\[CloseCurlyDoubleQuote]"
}], "Text",ExpressionUUID->"f75aabf0-d580-4347-bd37-5af958d12fbb"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WordData", "[", 
  RowBox[{
  "\"\<program\>\"", ",", "\"\<BroaderTerms\>\"", ",", "\"\<List\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[32]:=",ExpressionUUID->"171339b7-97de-41d1-a6f7-0dc0e9fe84da"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"announcement\"\>", ",", "\<\"computer software\"\>", 
   ",", "\<\"create by mental act\"\>", ",", "\<\"create mentally\"\>", 
   ",", "\<\"document\"\>", ",", "\<\"idea\"\>", ",", "\<\"info\"\>", 
   ",", "\<\"information\"\>", ",", "\<\"package\"\>", ",", "\<\"papers\"\>", 
   ",", "\<\"performance\"\>", ",", "\<\"promulgation\"\>", 
   ",", "\<\"schedule\"\>", ",", "\<\"show\"\>", ",", "\<\"software\"\>", 
   ",", "\<\"software package\"\>", ",", "\<\"software program\"\>", 
   ",", "\<\"software system\"\>", ",", "\<\"system\"\>", 
   ",", "\<\"system of rules\"\>", ",", "\<\"thought\"\>", 
   ",", "\<\"written document\"\>"}], "}"}]], "Output",
 CellLabel->"Out[32]=",ExpressionUUID->"deb21d51-1223-4a70-b968-2c564d5e7e5a"]
}, Open  ]],

Cell[TextData[{
 "We will need to create edges between \[OpenCurlyDoubleQuote]program\
\[CloseCurlyDoubleQuote] and each of these words, which we can do with ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"0ba9e9ff-96a6-4b7b-8b9c-681060da4495"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DirectedEdge", "[", 
     RowBox[{"\"\<program\>\"", ",", "#"}], "]"}], "&"}], ",", 
   RowBox[{"WordData", "[", 
    RowBox[{
    "\"\<program\>\"", ",", "\"\<BroaderTerms\>\"", ",", "\"\<List\>\""}], 
    "]"}]}], "]"}]], "Input",
 CellLabel->"In[33]:=",ExpressionUUID->"0a8fcf84-1177-4154-9bf1-e5d3a536775f"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"announcement\"\>"}], 
   ",", 
   RowBox[{"\<\"program\"\>", 
    "\[DirectedEdge]", "\<\"computer software\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", 
    "\[DirectedEdge]", "\<\"create by mental act\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"create mentally\"\>"}], 
   ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"document\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"idea\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"info\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"information\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"package\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"papers\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"performance\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"promulgation\"\>"}], 
   ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"schedule\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"show\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"software\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"software package\"\>"}],
    ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"software program\"\>"}],
    ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"software system\"\>"}], 
   ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"system\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"system of rules\"\>"}], 
   ",", 
   RowBox[{"\<\"program\"\>", "\[DirectedEdge]", "\<\"thought\"\>"}], ",", 
   RowBox[{"\<\"program\"\>", 
    "\[DirectedEdge]", "\<\"written document\"\>"}]}], "}"}]], "Output",
 CellLabel->"Out[33]=",ExpressionUUID->"9189ab00-58b9-4ec7-b998-eadeb971bacd"]
}, Open  ]],

Cell[TextData[{
 "This is not yet a very interesting network, so we take it one step further \
and find the words related to those related to \
\[OpenCurlyDoubleQuote]program.\[CloseCurlyDoubleQuote] We use a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to apply the technique above to all the words that appear in the list of \
broader terms for \[OpenCurlyDoubleQuote]program.\[CloseCurlyDoubleQuote] The \
result is rather long, so to save space, we limit the number of lines of \
output with ",
 StyleBox[ButtonBox["Short",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Short"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"723b6e50-143d-4483-9e6f-a7037f22d241"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"DirectedEdge", "[", 
         RowBox[{"word", ",", "#"}], "]"}], "&"}], ",", 
       RowBox[{"WordData", "[", 
        RowBox[{"word", ",", "\"\<BroaderTerms\>\"", ",", "\"\<List\>\""}], 
        "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"word", ",", 
       RowBox[{"WordData", "[", 
        RowBox[{
        "\"\<program\>\"", ",", "\"\<BroaderTerms\>\"", ",", "\"\<List\>\""}],
         "]"}]}], "}"}]}], "]"}], ",", "10"}], "]"}]], "Input",
 CellLabel->"In[34]:=",ExpressionUUID->"74ecd67e-a5f1-4703-a049-591cbc42ca16"],

Cell[BoxData[
 TagBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"announcement\"\>", "\[DirectedEdge]", "\<\"statement\"\>"}],
      "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\<\"computer software\"\>", 
       "\[DirectedEdge]", "\<\"code\"\>"}], ",", 
      RowBox[{"\<\"computer software\"\>", 
       "\[DirectedEdge]", "\<\"computer code\"\>"}]}], "}"}], ",", 
    RowBox[{"\[LeftSkeleton]", "18", "\[RightSkeleton]"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\<\"thought\"\>", "\[DirectedEdge]", "\<\"belief\"\>"}], ",", 
      RowBox[{"\<\"thought\"\>", 
       "\[DirectedEdge]", "\<\"cognitive content\"\>"}], ",", 
      RowBox[{"\<\"thought\"\>", "\[DirectedEdge]", "\<\"content\"\>"}], ",", 
      
      RowBox[{"\<\"thought\"\>", 
       "\[DirectedEdge]", "\<\"higher cognitive process\"\>"}], ",", 
      RowBox[{"\<\"thought\"\>", 
       "\[DirectedEdge]", "\<\"mental object\"\>"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\<\"written document\"\>", 
       "\[DirectedEdge]", "\<\"piece of writing\"\>"}], ",", 
      RowBox[{"\<\"written document\"\>", 
       "\[DirectedEdge]", "\<\"writing\"\>"}], ",", 
      RowBox[{"\<\"written document\"\>", 
       "\[DirectedEdge]", "\<\"written material\"\>"}]}], "}"}]}], "}"}],
  Short[#, 10]& ]], "Output",
 CellLabel->
  "Out[34]//Short=",ExpressionUUID->"9a55580a-c740-41bd-8adc-bcfb51da1f11"]
}, Open  ]],

Cell[TextData[{
 "Thus, the edges of the graph of this semantic network can be obtained by \
flattening that result and combining it with the edges incident on \
\[OpenCurlyDoubleQuote]program.\[CloseCurlyDoubleQuote] This graph will be \
too large to display the terms on the graph, so we label the vertices using \
",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " with second argument ",
 StyleBox[ButtonBox["Tooltip",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tooltip"], "FunctionLink"],
 ", so that if you hover the mouse pointer over a vertex, it will display the \
term associated with the vertex."
}], "Text",ExpressionUUID->"89bdfffc-4187-4da3-9016-64b4a36cad81"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"DirectedEdge", "[", 
         RowBox[{"\"\<program\>\"", ",", "#"}], "]"}], "&"}], ",", 
       RowBox[{"WordData", "[", 
        RowBox[{
        "\"\<program\>\"", ",", "\"\<BroaderTerms\>\"", ",", "\"\<List\>\""}],
         "]"}]}], "]"}], ",", 
     RowBox[{"Flatten", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"DirectedEdge", "[", 
            RowBox[{"word", ",", "#"}], "]"}], "&"}], ",", 
          RowBox[{"WordData", "[", 
           RowBox[{"word", ",", "\"\<BroaderTerms\>\"", ",", "\"\<List\>\""}],
            "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"word", ",", 
          RowBox[{"WordData", "[", 
           RowBox[{
           "\"\<program\>\"", ",", "\"\<BroaderTerms\>\"", ",", 
            "\"\<List\>\""}], "]"}]}], "}"}]}], "]"}], "]"}]}], "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Tooltip"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[35]:=",ExpressionUUID->"49a9ac19-67c1-499d-a95b-804d01ddcba8"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "program", "announcement", "computer software", "create by mental act", 
       "create mentally", "document", "idea", "info", "information", 
       "package", "papers", "performance", "promulgation", "schedule", "show",
        "software", "software package", "software program", "software system",
        "system", "system of rules", "thought", "written document", 
       "statement", "code", "computer code", "create", "make", "affirm", 
       "communication", "computer file", "confirm", "corroborate", "enter", 
       "piece of writing", "put down", "record", "representation", 
       "substantiate", "support", "sustain", "writing", "written material", 
       "aim", "air", "calculation", "cognitive content", "computation", 
       "content", "design", "figuring", "intent", "intention", "line", 
       "melodic line", "melodic phrase", "melody", "mental object", "opinion",
        "persuasion", "purpose", "reckoning", "sentiment", "strain", "tune", 
       "view", "message", "subject matter", "substance", "accumulation", 
       "accusal", "accusation", "aggregation", "assemblage", "cognition", 
       "collection", "information measure", "knowledge", "noesis", "case", 
       "container", "encase", "incase", "accomplishment", "achievement", 
       "action", "demonstration", "physical process", "presentation", 
       "presentment", "process", "act", "annunciation", "declaration", "deed",
        "human action", "human activity", "proclamation", "list", "listing", 
       "plan", "programme", "amusement", "appear", "conduct", "convey", 
       "demo", "direct", "entertainment", "feigning", "guide", "indicate", 
       "inform", "interpret", "lead", "pretence", "pretending", "pretense", 
       "race", "read", "register", "represent", "run", "simulation", 
       "social event", "take", "merchandise", "product", "ware", "body part", 
       "group", "grouping", "instrumentality", "instrumentation", "live body",
        "matter", "method", "methodicalness", "orderliness", "plan of action",
        "structure", "belief", "higher cognitive process"}, {CompressedData["
1:eJwVxWdbE0EUBtDclLk7k5l9s8mwYZcaIgKJgrFTpBdBbEizohFsgNhQKRYQ
EGz8IP6c+uE8J1demShHI5HI0T//pyjFKE4JEsTkkCRFSdJkyCVQijxKU4Ys
VZBP2WhlLIiF8ap4daIqUS1qRK2oE/UiJxpEXhwTjeK4aBLNokUURJFP8Elu
5TY+xSU+zWf4LJ/j83yBL3I7d3And/El7uYe7mWf+7jfKTkDzqAzJIfliLws
R+WYvCLHZUleldfkgLwuB+WQGlajakzdUIEaV6G6qSbUrWQ+WUgW9aSe0tN6
Rs/q2/qOvqth7pmouW8emDnz0DwyZVPpPnbn3QWX3CeowVM8w3PU4wVyWMQM
lrCMl1jBK7zGG7zFO6ziPT7gIxqxhnVsANjEJzShGS34nApSoRd4offF++pt
pYN0mAkyod223+yO3bV79rvdtwf2h/1pf9nfFQf+H7/NL/mHfkc2ny1ki38B
Mts5vA==
       "], Null}, {VertexLabels -> {
         Placed["Name", Tooltip]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[0.017232190533472188`], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {
         4.3631041244623905`, 1.8754694214310674`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {
         4.8363050880298335`, 2.6671297495001056`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {4.112960160860195, 
         2.578773432072347}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {3.987653281702678, 
         2.718336533701252}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {2.585223460441978, 
         4.092722246000099}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {7.825446110220488, 
         4.009284577296301}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {6.072539215347003, 
         2.7018806885904167`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {6.511933915422109, 
         1.8581221870658664`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {5.599553423748748, 
         1.619423130250373}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {
         3.7802063478623777`, 3.816390404643261}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {2.359428497688093, 
         2.388187267906938}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {4.301215147788383, 
         0.9856440016622097}}, 0.06171089064620776], 
        ArrowBox[BezierCurveBox[{{4.622182318024413, 3.198161318068291}, {
         5.202435574979503, 3.526761881229576}, {5.864461437644778, 
         3.446799831061418}}], 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {
         1.1458521283872245`, 3.586328117145597}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {4.820503303210917, 
         2.4935705679758575`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {5.157935045678634, 
         3.2270028460717306`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {4.972563099427484, 
         2.772128204682778}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {5.142332892460434, 
         2.8198809705013836`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {
         4.7837397487178634`, 5.912746734441349}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {4.65113645014543, 
         4.3433458761116395`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {6.562235138614136, 
         3.8329975585129916`}}, 0.06171089064620776], 
        ArrowBox[{{4.622182318024413, 3.198161318068291}, {3.713593579914663, 
         3.682008846057269}}, 0.06171089064620776], 
        ArrowBox[{{4.3631041244623905`, 1.8754694214310674`}, {
         4.121123623340167, 1.184708095072827}}, 0.06171089064620776], 
        ArrowBox[{{4.8363050880298335`, 2.6671297495001056`}, {
         5.277025892828639, 2.4635979066335727`}}, 0.06171089064620776], 
        ArrowBox[{{4.8363050880298335`, 2.6671297495001056`}, {
         5.173719556989121, 2.394677762458093}}, 0.06171089064620776], 
        ArrowBox[{{4.112960160860195, 2.578773432072347}, {
         3.6736335036669736`, 2.392981552154499}}, 0.06171089064620776], 
        ArrowBox[{{4.112960160860195, 2.578773432072347}, {
         3.8515492348022753`, 2.2107028437529386`}}, 0.06171089064620776], 
        ArrowBox[{{3.987653281702678, 2.718336533701252}, {
         3.6736335036669736`, 2.392981552154499}}, 0.06171089064620776], 
        ArrowBox[{{3.987653281702678, 2.718336533701252}, {
         3.8515492348022753`, 2.2107028437529386`}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         1.6474726513930658`, 4.061574101025762}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {2.6589244664841, 
         4.821688606284125}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {2.458161379119707, 
         4.993385328323108}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         1.6987957413186212`, 3.864263136750706}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         1.6601988266105798`, 4.263077843727271}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         2.3109349968352726`, 4.793316142390892}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         3.2320691499143477`, 3.99655849213806}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {2.302767777649875, 
         4.5400101312560235`}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         1.8435769932679644`, 3.593203187831645}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {2.133924847955065, 
         4.916620554582968}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         1.8746134829341745`, 3.932590266327317}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         1.8816864565034899`, 3.748236305174419}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {1.812239384199144, 
         4.145053181301604}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {3.302483708530289, 
         4.140144433934878}}, 0.06171089064620776], 
        ArrowBox[{{2.585223460441978, 4.092722246000099}, {
         3.1915993283817308`, 3.8398883380559736`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {6.562235138614136, 
         3.8329975585129916`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.35718491695204, 
         4.246187385899545}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.771597343109914, 
         4.2313408381134}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {7.952156369309427, 
         4.8536602089858105`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {7.292457558682633, 
         4.169644232421903}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.922710047660885, 
         3.7964784435434384`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {6.945932961060841, 
         3.045580955120631}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.720902434704197, 
         4.558968031872986}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.339455392671667, 
         3.6249094991082984`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.857138829218385, 
         4.44554800396487}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.952850556976369, 
         4.15233643169305}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.501905754349906, 
         4.865789634267212}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.639231824480916, 
         3.467752006944937}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.215599538520749, 
         4.950175746475327}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.797310605021032, 
         3.6096559745575068`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {7.335343288651176, 
         3.939403208252065}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.543429735067011, 
         4.071055195502567}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.580872624731995, 
         4.666817107678163}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.27554317579856, 
         4.74061799320876}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.450521851136562, 
         4.467309843600519}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.43501562633667, 
         3.380090957976169}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.542209220047646, 
         3.8039626455894116`}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.800436975893614, 
         3.960161635043256}}, 0.06171089064620776], 
        ArrowBox[{{7.825446110220488, 4.009284577296301}, {8.098923370080023, 
         4.563723306534194}}, 0.06171089064620776], 
        ArrowBox[{{6.072539215347003, 2.7018806885904167`}, {
         6.945932961060841, 3.045580955120631}}, 0.06171089064620776], 
        ArrowBox[{{6.072539215347003, 2.7018806885904167`}, {
         6.407839720789965, 2.185729624408543}}, 0.06171089064620776], 
        ArrowBox[{{6.072539215347003, 2.7018806885904167`}, {
         6.764933669981312, 2.3056970364129223`}}, 0.06171089064620776], 
        ArrowBox[{{6.072539215347003, 2.7018806885904167`}, {
         6.609279032057781, 2.2282354206856607`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.945932961060841, 3.045580955120631}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.407839720789965, 2.185729624408543}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.764933669981312, 2.3056970364129223`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.609279032057781, 2.2282354206856607`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.127872233913103, 1.2731960758025909`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.9193698570232325`, 1.0119305373051226`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         7.111710771878911, 1.0604556763204829`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {6.17033373229551,
          1.4861089545773196`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.307862202716697, 1.1705697624435145`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         7.407626297739556, 1.392130035525489}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         6.345867555288437, 1.3621653897219825`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         7.2836286750036106`, 1.1629143406165916`}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         7.157290466776784, 1.384215714459863}}, 0.06171089064620776], 
        ArrowBox[{{6.511933915422109, 1.8581221870658664`}, {
         7.374375668266854, 1.6290672637649148`}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {5.277025892828639, 
         2.4635979066335727`}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {5.173719556989121, 
         2.394677762458093}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {6.127872233913103, 
         1.2731960758025909`}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {6.17033373229551, 
         1.4861089545773196`}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {6.307862202716697, 
         1.1705697624435145`}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {6.345867555288437, 
         1.3621653897219825`}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {5.992458044545064, 
         0.7025036265855089}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {5.500677879263255, 
         0.7872860865967941}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {
         5.7992534602447545`, 0.8742860303930127}}, 0.06171089064620776], 
        ArrowBox[{{5.599553423748748, 1.619423130250373}, {
         5.7107691354549495`, 0.6715758066808752}}, 0.06171089064620776], 
        ArrowBox[{{3.7802063478623777`, 3.816390404643261}, {
         3.2320691499143477`, 3.99655849213806}}, 0.06171089064620776], 
        ArrowBox[{{3.7802063478623777`, 3.816390404643261}, {
         3.302483708530289, 4.140144433934878}}, 0.06171089064620776], 
        ArrowBox[{{3.7802063478623777`, 3.816390404643261}, {
         3.1915993283817308`, 3.8398883380559736`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {
         1.1458521283872245`, 3.586328117145597}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {
         1.6542544319124328`, 1.9694708409887582`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {1.661458931236567, 
         1.727545611477082}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {
         1.8282612341263391`, 1.5588038110401234`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {
         1.5761389701762476`, 2.7860224824006266`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {1.908246159604205, 
         1.9216588582012548`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {2.098384027389237, 
         1.7204629359884878`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {2.081377640895517, 
         1.4760976374650827`}}, 0.06171089064620776], 
        ArrowBox[{{2.359428497688093, 2.388187267906938}, {
         2.3473546575897437`, 1.6055566162068258`}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {
         4.3631041244623905`, 1.8754694214310674`}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {4.121123623340167,
          1.184708095072827}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {3.976179974936086,
          0.2816129325356904}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {3.76626781929311, 
         0.24976583551511355`}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {4.22953312414159, 
         0.26409149775026464`}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {4.603245530929744,
          0.21094785717960818`}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {4.265730699113267,
          0.}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {4.477575747545193,
          0.041045134869510225`}}, 0.06171089064620776], 
        ArrowBox[{{4.301215147788383, 0.9856440016622097}, {4.024410815574865,
          0.031186178462385694`}}, 0.06171089064620776], 
        ArrowBox[BezierCurveBox[{{5.864461437644778, 3.446799831061418}, {
         5.28420818068968, 3.1181992679336887`}, {4.622182318024413, 
         3.198161318068291}}], 0.06171089064620776], 
        ArrowBox[{{5.864461437644778, 3.446799831061418}, {6.437165401439212, 
         3.209833397668756}}, 0.06171089064620776], 
        ArrowBox[{{5.864461437644778, 3.446799831061418}, {6.398375930671974, 
         3.5675529000447184`}}, 0.06171089064620776], 
        ArrowBox[{{5.864461437644778, 3.446799831061418}, {6.263352704510136, 
         3.8458324812743347`}}, 0.06171089064620776], 
        ArrowBox[{{5.864461437644778, 3.446799831061418}, {6.609359674240018, 
         3.434591580319447}}, 0.06171089064620776], 
        ArrowBox[BezierCurveBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.2839107925774316`, 3.2052614611032166`}, {1.2549254920401167`, 
         3.1381425393749565`}, {1.2179422095351318`, 3.105898585630704}, {
         1.0769290386872772`, 3.1054340308253927`}, {1.0397341118204788`, 
         3.13743361134262}, {1.0103072111197509`, 3.204360099494564}, {
         1.1458521283872245`, 3.586328117145597}},
         SplineDegree->7], 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.6474726513930658`, 4.061574101025762}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.6987957413186212`, 3.864263136750706}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.6601988266105798`, 4.263077843727271}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.8435769932679644`, 3.593203187831645}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.8746134829341745`, 3.932590266327317}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.8816864565034899`, 3.748236305174419}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.812239384199144, 4.145053181301604}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         1.5761389701762476`, 2.7860224824006266`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.24039301055393736`, 4.26291485002908}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.7107408163797713, 2.977422849364468}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.20429360691830833`, 3.750859621695218}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.1381404999385092, 4.063322385611503}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.5959604974479471, 3.3231562334624924`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.45439846893572655`, 4.250965246930067}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {0., 
         3.6611742080469036`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.33262579974978035`, 3.381487559939251}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.25398337642361213`, 2.938532160476004}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.7073004422881017, 2.7576049201924615`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.37531940362037375`, 3.996012371971256}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.010244195194008654`, 3.9029430942937946`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.08465171974215746, 3.2969664586725425`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.8672103195478877, 4.14738540928295}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.6575715911730464, 4.1928750152755}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.1497274708874441, 3.134417748970323}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.015309456400993682`, 3.4676336197423727`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.8896831940701704, 4.473833079421826}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.39899372784105847`, 3.666942163261398}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.6582587614943298, 4.493505158548837}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.6166357990427835, 3.8507328478058565`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.4523125417334839, 2.8283632645386976`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.42473317003683864`, 4.4485142745874455`}}, 0.06171089064620776], 
        ArrowBox[{{1.1458521283872245`, 3.586328117145597}, {
         0.40886866045322545`, 3.111018268609982}}, 0.06171089064620776], 
        ArrowBox[{{4.820503303210917, 2.4935705679758575`}, {
         5.277025892828639, 2.4635979066335727`}}, 0.06171089064620776], 
        ArrowBox[{{4.820503303210917, 2.4935705679758575`}, {
         5.173719556989121, 2.394677762458093}}, 0.06171089064620776], 
        ArrowBox[{{5.157935045678634, 3.2270028460717306`}, {
         5.277025892828639, 2.4635979066335727`}}, 0.06171089064620776], 
        ArrowBox[{{5.157935045678634, 3.2270028460717306`}, {
         5.173719556989121, 2.394677762458093}}, 0.06171089064620776], 
        ArrowBox[{{5.157935045678634, 3.2270028460717306`}, {
         5.2078207648967725`, 3.928473504979497}}, 0.06171089064620776], 
        ArrowBox[{{5.157935045678634, 3.2270028460717306`}, {
         5.4437187157013645`, 3.776240578748179}}, 0.06171089064620776], 
        ArrowBox[{{5.157935045678634, 3.2270028460717306`}, {
         4.982791221842441, 3.7435501261259034`}}, 0.06171089064620776], 
        ArrowBox[{{4.972563099427484, 2.772128204682778}, {5.277025892828639, 
         2.4635979066335727`}}, 0.06171089064620776], 
        ArrowBox[{{4.972563099427484, 2.772128204682778}, {5.173719556989121, 
         2.394677762458093}}, 0.06171089064620776], 
        ArrowBox[{{5.142332892460434, 2.8198809705013836`}, {
         5.277025892828639, 2.4635979066335727`}}, 0.06171089064620776], 
        ArrowBox[{{5.142332892460434, 2.8198809705013836`}, {
         5.173719556989121, 2.394677762458093}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         5.045659016631458, 6.557579730402689}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         4.412564915373739, 6.791441276143009}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {5.35525549986666,
          6.679341878364559}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         5.321855068177082, 6.4307387384754175`}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         5.144062164104109, 6.792725114970653}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         4.236150871892367, 6.587345400266901}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         4.769261823041524, 6.537603995276211}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {4.73427227586752,
          5.26849893265654}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         4.641616089854216, 6.75855875330385}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         4.4440499699566045`, 6.480168650621344}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {
         4.984403847092703, 6.9327433898296915`}}, 0.06171089064620776], 
        ArrowBox[{{4.7837397487178634`, 5.912746734441349}, {4.72983527122933,
          6.945884781047834}}, 0.06171089064620776], 
        ArrowBox[{{4.65113645014543, 4.3433458761116395`}, {4.73427227586752, 
         5.26849893265654}}, 0.06171089064620776], 
        ArrowBox[{{6.562235138614136, 3.8329975585129916`}, {
         7.292457558682633, 4.169644232421903}}, 0.06171089064620776], 
        ArrowBox[{{6.562235138614136, 3.8329975585129916`}, {
         6.945932961060841, 3.045580955120631}}, 0.06171089064620776], 
        ArrowBox[{{6.562235138614136, 3.8329975585129916`}, {
         7.335343288651176, 3.939403208252065}}, 0.06171089064620776], 
        ArrowBox[{{6.562235138614136, 3.8329975585129916`}, {
         6.7687105607136955`, 4.593282760114632}}, 0.06171089064620776], 
        ArrowBox[{{6.562235138614136, 3.8329975585129916`}, {
         6.959862495276674, 4.43349701277298}}, 0.06171089064620776], 
        ArrowBox[{{3.713593579914663, 3.682008846057269}, {
         3.2320691499143477`, 3.99655849213806}}, 0.06171089064620776], 
        ArrowBox[{{3.713593579914663, 3.682008846057269}, {3.302483708530289, 
         4.140144433934878}}, 0.06171089064620776], 
        ArrowBox[{{3.713593579914663, 3.682008846057269}, {
         3.1915993283817308`, 3.8398883380559736`}}, 0.06171089064620776]}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        TagBox[
         TooltipBox[
          DiskBox[{4.622182318024413, 3.198161318068291}, 0.06171089064620776],
          "\"program\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "program", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.3631041244623905, 1.8754694214310674}, 
           0.06171089064620776],
          "\"announcement\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "announcement", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.8363050880298335, 2.6671297495001056}, 
           0.06171089064620776],
          "\"computer software\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "computer software", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.112960160860195, 2.578773432072347}, 0.06171089064620776],
          "\"create by mental act\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "create by mental act", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.987653281702678, 2.718336533701252}, 0.06171089064620776],
          "\"create mentally\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "create mentally", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.585223460441978, 4.092722246000099}, 0.06171089064620776],
          "\"document\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "document", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.825446110220488, 4.009284577296301}, 0.06171089064620776],
          "\"idea\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "idea", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.072539215347003, 2.7018806885904167}, 
           0.06171089064620776],
          "\"info\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "info", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.511933915422109, 1.8581221870658664}, 
           0.06171089064620776],
          "\"information\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "information", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.599553423748748, 1.619423130250373}, 0.06171089064620776],
          "\"package\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "package", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.7802063478623777, 3.816390404643261}, 
           0.06171089064620776],
          "\"papers\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "papers", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.359428497688093, 2.388187267906938}, 0.06171089064620776],
          "\"performance\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "performance", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.301215147788383, 0.9856440016622097}, 
           0.06171089064620776],
          "\"promulgation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "promulgation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.864461437644778, 3.446799831061418}, 0.06171089064620776],
          "\"schedule\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "schedule", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.1458521283872245, 3.586328117145597}, 
           0.06171089064620776],
          "\"show\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "show", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.820503303210917, 2.4935705679758575}, 
           0.06171089064620776],
          "\"software\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "software", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.157935045678634, 3.2270028460717306}, 
           0.06171089064620776],
          "\"software package\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "software package", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.972563099427484, 2.772128204682778}, 0.06171089064620776],
          "\"software program\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "software program", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.142332892460434, 2.8198809705013836}, 
           0.06171089064620776],
          "\"software system\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "software system", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.7837397487178634, 5.912746734441349}, 
           0.06171089064620776],
          "\"system\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "system", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.65113645014543, 4.3433458761116395}, 0.06171089064620776],
          "\"system of rules\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "system of rules", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.562235138614136, 3.8329975585129916}, 
           0.06171089064620776],
          "\"thought\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "thought", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.713593579914663, 3.682008846057269}, 0.06171089064620776],
          "\"written document\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "written document", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.121123623340167, 1.184708095072827}, 0.06171089064620776],
          "\"statement\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "statement", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.277025892828639, 2.4635979066335727}, 
           0.06171089064620776],
          "\"code\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "code", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.173719556989121, 2.394677762458093}, 0.06171089064620776],
          "\"computer code\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "computer code", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.6736335036669736, 2.392981552154499}, 
           0.06171089064620776],
          "\"create\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "create", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.8515492348022753, 2.2107028437529386}, 
           0.06171089064620776],
          "\"make\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "make", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.6474726513930658, 4.061574101025762}, 
           0.06171089064620776],
          "\"affirm\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "affirm", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.6589244664841, 4.821688606284125}, 0.06171089064620776],
          "\"communication\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "communication", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.458161379119707, 4.993385328323108}, 0.06171089064620776],
          "\"computer file\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "computer file", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.6987957413186212, 3.864263136750706}, 
           0.06171089064620776],
          "\"confirm\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "confirm", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.6601988266105798, 4.263077843727271}, 
           0.06171089064620776],
          "\"corroborate\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "corroborate", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.3109349968352726, 4.793316142390892}, 
           0.06171089064620776],
          "\"enter\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "enter", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.2320691499143477, 3.99655849213806}, 0.06171089064620776],
          "\"piece of writing\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "piece of writing", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.302767777649875, 4.5400101312560235}, 
           0.06171089064620776],
          "\"put down\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "put down", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.8435769932679644, 3.593203187831645}, 
           0.06171089064620776],
          "\"record\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "record", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.133924847955065, 4.916620554582968}, 0.06171089064620776],
          "\"representation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "representation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.8746134829341745, 3.932590266327317}, 
           0.06171089064620776],
          "\"substantiate\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "substantiate", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.8816864565034899, 3.748236305174419}, 
           0.06171089064620776],
          "\"support\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "support", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.812239384199144, 4.145053181301604}, 0.06171089064620776],
          "\"sustain\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "sustain", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.302483708530289, 4.140144433934878}, 0.06171089064620776],
          "\"writing\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "writing", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.1915993283817308, 3.8398883380559736}, 
           0.06171089064620776],
          "\"written material\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "written material", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.35718491695204, 4.246187385899545}, 0.06171089064620776],
          
          "\"aim\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "aim", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.771597343109914, 4.2313408381134}, 0.06171089064620776],
          "\"air\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "air", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.952156369309427, 4.8536602089858105}, 
           0.06171089064620776],
          "\"calculation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "calculation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.292457558682633, 4.169644232421903}, 0.06171089064620776],
          "\"cognitive content\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "cognitive content", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.922710047660885, 3.7964784435434384}, 
           0.06171089064620776],
          "\"computation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "computation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.945932961060841, 3.045580955120631}, 0.06171089064620776],
          "\"content\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "content", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.720902434704197, 4.558968031872986}, 0.06171089064620776],
          "\"design\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "design", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.339455392671667, 3.6249094991082984}, 
           0.06171089064620776],
          "\"figuring\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "figuring", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.857138829218385, 4.44554800396487}, 0.06171089064620776],
          
          "\"intent\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "intent", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.952850556976369, 4.15233643169305}, 0.06171089064620776],
          
          "\"intention\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "intention", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.501905754349906, 4.865789634267212}, 0.06171089064620776],
          "\"line\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "line", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.639231824480916, 3.467752006944937}, 0.06171089064620776],
          "\"melodic line\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "melodic line", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.215599538520749, 4.950175746475327}, 0.06171089064620776],
          "\"melodic phrase\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "melodic phrase", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.797310605021032, 3.6096559745575068}, 
           0.06171089064620776],
          "\"melody\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "melody", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.335343288651176, 3.939403208252065}, 0.06171089064620776],
          "\"mental object\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "mental object", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.543429735067011, 4.071055195502567}, 0.06171089064620776],
          "\"opinion\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "opinion", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.580872624731995, 4.666817107678163}, 0.06171089064620776],
          "\"persuasion\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "persuasion", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.27554317579856, 4.74061799320876}, 0.06171089064620776],
          "\"purpose\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "purpose", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.450521851136562, 4.467309843600519}, 0.06171089064620776],
          "\"reckoning\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "reckoning", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.43501562633667, 3.380090957976169}, 0.06171089064620776],
          
          "\"sentiment\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "sentiment", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.542209220047646, 3.8039626455894116}, 
           0.06171089064620776],
          "\"strain\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "strain", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.800436975893614, 3.960161635043256}, 0.06171089064620776],
          "\"tune\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "tune", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{8.098923370080023, 4.563723306534194}, 0.06171089064620776],
          "\"view\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "view", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.407839720789965, 2.185729624408543}, 0.06171089064620776],
          "\"message\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "message", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.764933669981312, 2.3056970364129223}, 
           0.06171089064620776],
          "\"subject matter\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "subject matter", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.609279032057781, 2.2282354206856607}, 
           0.06171089064620776],
          "\"substance\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "substance", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.127872233913103, 1.2731960758025909}, 
           0.06171089064620776],
          "\"accumulation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "accumulation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.9193698570232325, 1.0119305373051226}, 
           0.06171089064620776],
          "\"accusal\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "accusal", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.111710771878911, 1.0604556763204829}, 
           0.06171089064620776],
          "\"accusation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "accusation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.17033373229551, 1.4861089545773196}, 0.06171089064620776],
          "\"aggregation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "aggregation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.307862202716697, 1.1705697624435145}, 
           0.06171089064620776],
          "\"assemblage\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "assemblage", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.407626297739556, 1.392130035525489}, 0.06171089064620776],
          "\"cognition\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "cognition", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.345867555288437, 1.3621653897219825}, 
           0.06171089064620776],
          "\"collection\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "collection", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.2836286750036106, 1.1629143406165916}, 
           0.06171089064620776],
          "\"information measure\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "information measure", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.157290466776784, 1.384215714459863}, 0.06171089064620776],
          "\"knowledge\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "knowledge", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{7.374375668266854, 1.6290672637649148}, 
           0.06171089064620776],
          "\"noesis\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "noesis", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.992458044545064, 0.7025036265855089}, 
           0.06171089064620776],
          "\"case\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "case", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.500677879263255, 0.7872860865967941}, 
           0.06171089064620776],
          "\"container\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "container", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.7992534602447545, 0.8742860303930127}, 
           0.06171089064620776],
          "\"encase\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "encase", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.7107691354549495, 0.6715758066808752}, 
           0.06171089064620776],
          "\"incase\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "incase", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.6542544319124328, 1.9694708409887582}, 
           0.06171089064620776],
          "\"accomplishment\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "accomplishment", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.661458931236567, 1.727545611477082}, 0.06171089064620776],
          "\"achievement\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "achievement", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.8282612341263391, 1.5588038110401234}, 
           0.06171089064620776],
          "\"action\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "action", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.5761389701762476, 2.7860224824006266}, 
           0.06171089064620776],
          "\"demonstration\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "demonstration", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.908246159604205, 1.9216588582012548}, 
           0.06171089064620776],
          "\"physical process\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "physical process", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.098384027389237, 1.7204629359884878}, 
           0.06171089064620776],
          "\"presentation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "presentation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.081377640895517, 1.4760976374650827}, 
           0.06171089064620776],
          "\"presentment\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "presentment", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{2.3473546575897437, 1.6055566162068258}, 
           0.06171089064620776],
          "\"process\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "process", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.976179974936086, 0.2816129325356904}, 
           0.06171089064620776],
          "\"act\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "act", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.76626781929311, 0.24976583551511355}, 
           0.06171089064620776],
          "\"annunciation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "annunciation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.22953312414159, 0.26409149775026464}, 
           0.06171089064620776],
          "\"declaration\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "declaration", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.603245530929744, 0.21094785717960818}, 
           0.06171089064620776],
          "\"deed\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "deed", "Tooltip"]& ], 
        TagBox[
         TooltipBox[DiskBox[{4.265730699113267, 0.}, 0.06171089064620776],
          "\"human action\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "human action", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.477575747545193, 0.041045134869510225}, 
           0.06171089064620776],
          "\"human activity\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "human activity", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.024410815574865, 0.031186178462385694}, 
           0.06171089064620776],
          "\"proclamation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "proclamation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.437165401439212, 3.209833397668756}, 0.06171089064620776],
          "\"list\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "list", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.398375930671974, 3.5675529000447184}, 
           0.06171089064620776],
          "\"listing\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "listing", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.263352704510136, 3.8458324812743347}, 
           0.06171089064620776],
          "\"plan\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "plan", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.609359674240018, 3.434591580319447}, 0.06171089064620776],
          "\"programme\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "programme", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.24039301055393736, 4.26291485002908}, 
           0.06171089064620776],
          "\"amusement\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "amusement", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.7107408163797713, 2.977422849364468}, 
           0.06171089064620776],
          "\"appear\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "appear", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.20429360691830833, 3.750859621695218}, 
           0.06171089064620776],
          "\"conduct\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "conduct", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.1381404999385092, 4.063322385611503}, 
           0.06171089064620776],
          "\"convey\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "convey", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.5959604974479471, 3.3231562334624924}, 
           0.06171089064620776],
          "\"demo\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "demo", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.45439846893572655, 4.250965246930067}, 
           0.06171089064620776],
          "\"direct\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "direct", "Tooltip"]& ], 
        TagBox[
         TooltipBox[DiskBox[{0., 3.6611742080469036}, 0.06171089064620776],
          "\"entertainment\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "entertainment", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.33262579974978035, 3.381487559939251}, 
           0.06171089064620776],
          "\"feigning\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "feigning", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.25398337642361213, 2.938532160476004}, 
           0.06171089064620776],
          "\"guide\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "guide", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.7073004422881017, 2.7576049201924615}, 
           0.06171089064620776],
          "\"indicate\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "indicate", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.37531940362037375, 3.996012371971256}, 
           0.06171089064620776],
          "\"inform\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "inform", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.010244195194008654, 3.9029430942937946}, 
           0.06171089064620776],
          "\"interpret\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "interpret", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.08465171974215746, 3.2969664586725425}, 
           0.06171089064620776],
          "\"lead\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "lead", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.8672103195478877, 4.14738540928295}, 0.06171089064620776],
          "\"pretence\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "pretence", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.6575715911730464, 4.1928750152755}, 0.06171089064620776],
          
          "\"pretending\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "pretending", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.1497274708874441, 3.134417748970323}, 
           0.06171089064620776],
          "\"pretense\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "pretense", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.015309456400993682, 3.4676336197423727}, 
           0.06171089064620776],
          "\"race\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "race", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.8896831940701704, 4.473833079421826}, 
           0.06171089064620776],
          "\"read\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "read", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.39899372784105847, 3.666942163261398}, 
           0.06171089064620776],
          "\"register\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "register", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.6582587614943298, 4.493505158548837}, 
           0.06171089064620776],
          "\"represent\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "represent", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.6166357990427835, 3.8507328478058565}, 
           0.06171089064620776],
          "\"run\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "run", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.4523125417334839, 2.8283632645386976}, 
           0.06171089064620776],
          "\"simulation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "simulation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.42473317003683864, 4.4485142745874455}, 
           0.06171089064620776],
          "\"social event\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "social event", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.40886866045322545, 3.111018268609982}, 
           0.06171089064620776],
          "\"take\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "take", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.2078207648967725, 3.928473504979497}, 
           0.06171089064620776],
          "\"merchandise\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "merchandise", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.4437187157013645, 3.776240578748179}, 
           0.06171089064620776],
          "\"product\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "product", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.982791221842441, 3.7435501261259034}, 
           0.06171089064620776],
          "\"ware\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "ware", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.045659016631458, 6.557579730402689}, 0.06171089064620776],
          "\"body part\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "body part", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.412564915373739, 6.791441276143009}, 0.06171089064620776],
          "\"group\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "group", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.35525549986666, 6.679341878364559}, 0.06171089064620776],
          
          "\"grouping\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "grouping", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.321855068177082, 6.4307387384754175}, 
           0.06171089064620776],
          "\"instrumentality\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "instrumentality", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.144062164104109, 6.792725114970653}, 0.06171089064620776],
          "\"instrumentation\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "instrumentation", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.236150871892367, 6.587345400266901}, 0.06171089064620776],
          "\"live body\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "live body", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.769261823041524, 6.537603995276211}, 0.06171089064620776],
          "\"matter\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "matter", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.73427227586752, 5.26849893265654}, 0.06171089064620776],
          "\"method\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "method", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.641616089854216, 6.75855875330385}, 0.06171089064620776],
          
          "\"methodicalness\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "methodicalness", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.4440499699566045, 6.480168650621344}, 
           0.06171089064620776],
          "\"orderliness\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "orderliness", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.984403847092703, 6.9327433898296915}, 
           0.06171089064620776],
          "\"plan of action\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "plan of action", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.72983527122933, 6.945884781047834}, 0.06171089064620776],
          
          "\"structure\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "structure", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.7687105607136955, 4.593282760114632}, 
           0.06171089064620776],
          "\"belief\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "belief", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{6.959862495276674, 4.43349701277298}, 0.06171089064620776],
          
          "\"higher cognitive process\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "higher cognitive process", "Tooltip"]& ]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[35]=",ExpressionUUID->"1e16a6b7-09c7-4e87-b8d6-a969ee232e8a"]
}, Open  ]],

Cell[TextData[{
 "You might be surprised by the presence of a self-loop in the graph, \
indicating the presence of a word related to itself. Hovering the mouse over \
that vertex, you can see that the word in question is \
\[OpenCurlyDoubleQuote]show.\[CloseCurlyDoubleQuote] Applying ",
 StyleBox[ButtonBox["WordData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WordData"], "FunctionLink"],
 " to \[OpenCurlyDoubleQuote]show\[CloseCurlyDoubleQuote] with third argument \
",
 StyleBox["\[OpenCurlyDoubleQuote]Rules\[CloseCurlyDoubleQuote]", "Code"],
 " will give detailed information, including the part of speech and sense of \
the word on the right-hand side of the rules. Again, this output will be \
long, so we use ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " to find those with the word \[OpenCurlyDoubleQuote]show\
\[CloseCurlyDoubleQuote] on the right-hand side of the rule. "
}], "Text",ExpressionUUID->"b0e28b42-a635-4ef5-85cd-a5468de8bd28"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Select", "[", 
  RowBox[{
   RowBox[{"WordData", "[", 
    RowBox[{
    "\"\<show\>\"", ",", "\"\<BroaderTerms\>\"", ",", "\"\<Rules\>\""}], 
    "]"}], ",", 
   RowBox[{
    RowBox[{"MemberQ", "[", 
     RowBox[{
      RowBox[{"#", "[", 
       RowBox[{"[", "2", "]"}], "]"}], ",", "\"\<show\>\"", ",", "2"}], "]"}],
     "&"}]}], "]"}]], "Input",
 CellLabel->"In[36]:=",ExpressionUUID->"7947dba6-209f-4286-8211-344cc1a19067"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Register\"\>"}], 
     "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"\<\"read\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Indicate\"\>"}],
        "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\<\"record\"\>", ",", "\<\"Verb\"\>", 
        ",", "\<\"Indicate\"\>"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\<\"register\"\>", ",", "\<\"Verb\"\>", 
        ",", "\<\"Indicate\"\>"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Read\"\>"}], 
       "}"}]}], "}"}]}], ",", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Exhibit\"\>"}], 
     "}"}], "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"{", 
      RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", 
       ",", "\<\"Demonstrate\"\>"}], "}"}], "}"}]}]}], "}"}]], "Output",
 CellLabel->"Out[36]=",ExpressionUUID->"fea158fc-8744-46fb-b239-a35dc01a7a18"]
}, Open  ]],

Cell["\<\
We see that the word \[OpenCurlyDoubleQuote]show\[CloseCurlyDoubleQuote] is \
related to itself in two different ways: the sense \
\[OpenCurlyDoubleQuote]Read\[CloseCurlyDoubleQuote] is broader than the sense \
\[OpenCurlyDoubleQuote]Register,\[CloseCurlyDoubleQuote] and the sense \
\[OpenCurlyDoubleQuote]Exhibit\[CloseCurlyDoubleQuote] is broader than the \
sense \[OpenCurlyDoubleQuote]Demonstrate.\[CloseCurlyDoubleQuote]\
\>", "Text",ExpressionUUID->"522d75aa-baad-46df-880b-558b5e3e0124"],

Cell[TextData[{
 "It is not always clear from the \[OpenCurlyDoubleQuote]sense\
\[CloseCurlyDoubleQuote] what specific definition is meant. ",
 StyleBox[ButtonBox["WordData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WordData"], "FunctionLink"],
 " can be applied to a full word specification, that is, the list containing \
the word, part of speech, and sense. The second argument ",
 StyleBox["\[OpenCurlyDoubleQuote]Definition\[CloseCurlyDoubleQuote]", 
  "Input"],
 " or ",
 StyleBox["\[OpenCurlyDoubleQuote]Examples\[CloseCurlyDoubleQuote]", 
  "Input"],
 " is useful in this context."
}], "Text",ExpressionUUID->"5dfb8445-9f02-42ac-8b7d-1a5bb546bef7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WordData", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<show\>\"", ",", "\"\<Verb\>\"", ",", "\"\<Read\>\""}], "}"}],
    ",", "\"\<Definitions\>\""}], "]"}]], "Input",
 CellLabel->"In[37]:=",ExpressionUUID->"c4b353e0-6231-4630-acdc-2d29d15f7c2b"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Read\"\>"}], "}"}],
    "\[Rule]", "\<\"indicate a certain reading; of gauges and \
instruments\"\>"}], "}"}]], "Output",
 CellLabel->"Out[37]=",ExpressionUUID->"4c80c592-262e-4aec-bbd2-35471eced936"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WordData", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<show\>\"", ",", "\"\<Verb\>\"", ",", "\"\<Register\>\""}], 
    "}"}], ",", "\"\<Definitions\>\""}], "]"}]], "Input",
 CellLabel->"In[38]:=",ExpressionUUID->"d34f46ba-1b23-46bd-9b2e-7f9c522742f8"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Register\"\>"}], 
    "}"}], "\[Rule]", "\<\"give evidence of, as of records\"\>"}], 
  "}"}]], "Output",
 CellLabel->"Out[38]=",ExpressionUUID->"11e1d858-0294-41d3-9161-f80ad93b792b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WordData", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<show\>\"", ",", "\"\<Verb\>\"", ",", "\"\<Register\>\""}], 
    "}"}], ",", "\"\<Examples\>\""}], "]"}]], "Input",
 CellLabel->"In[39]:=",ExpressionUUID->"68a4b272-fbba-42d2-9995-b485fd214415"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"show\"\>", ",", "\<\"Verb\"\>", ",", "\<\"Register\"\>"}], 
    "}"}], "\[Rule]", 
   RowBox[{"{", "\<\"The diary shows his distress that evening\"\>", "}"}]}], 
  "}"}]], "Output",
 CellLabel->"Out[39]=",ExpressionUUID->"bb6944a9-ab1e-4f93-8aa4-39737cdc97e9"]
}, Open  ]],

Cell[TextData[{
 "Finally, compare the graph produced from broader terms to that consisting \
of synonyms. Note that, in this case, the edges are undirected, representing \
the symmetric nature of the synonym relation. Also note that we create the \
graph with ",
 StyleBox[ButtonBox["SimpleGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SimpleGraph"], "FunctionLink"],
 " so as to eliminate the multiple edges."
}], "Text",ExpressionUUID->"3bf252ad-07ec-46e0-8112-95b8da8144cc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimpleGraph", "[", 
  RowBox[{
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Map", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"UndirectedEdge", "[", 
         RowBox[{"\"\<program\>\"", ",", "#"}], "]"}], "&"}], ",", 
       RowBox[{"WordData", "[", 
        RowBox[{
        "\"\<program\>\"", ",", "\"\<Synonyms\>\"", ",", "\"\<List\>\""}], 
        "]"}]}], "]"}], ",", 
     RowBox[{"Flatten", "[", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"UndirectedEdge", "[", 
            RowBox[{"word", ",", "#"}], "]"}], "&"}], ",", 
          RowBox[{"WordData", "[", 
           RowBox[{"word", ",", "\"\<Synonyms\>\"", ",", "\"\<List\>\""}], 
           "]"}]}], "]"}], ",", 
        RowBox[{"{", 
         RowBox[{"word", ",", 
          RowBox[{"WordData", "[", 
           RowBox[{
           "\"\<program\>\"", ",", "\"\<Synonyms\>\"", ",", "\"\<List\>\""}], 
           "]"}]}], "}"}]}], "]"}], "]"}]}], "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Tooltip"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[40]:=",ExpressionUUID->"25859ede-b01e-44ad-adfe-6862a9131198"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "program", "broadcast", "computer program", "computer programme", 
       "course of study", "curriculum", "plan", "platform", 
       "political platform", "political program", "programme", "syllabus", 
       "air", "beam", "circularise", "circularize", "circulate", "diffuse", 
       "disperse", "disseminate", "distribute", "pass around", "propagate", 
       "send", "spread", "transmit", "class", "course", 
       "course of instruction", "architectural plan", "be after", "contrive", 
       "design", "project", "chopine", "weapons platform"}, {
      Null, {{1, 7}, {7, 32}, {1, 2}, {2, 14}, {2, 15}, {2, 17}, {2, 25}, {2, 
       18}, {1, 8}, {8, 35}, {1, 5}, {5, 27}, {5, 28}, {5, 29}, {2, 16}, {1, 
       6}, {5, 6}, {8, 36}, {1, 11}, {2, 11}, {5, 11}, {6, 11}, {7, 11}, {1, 
       3}, {3, 11}, {2, 22}, {1, 4}, {3, 4}, {4, 11}, {7, 33}, {2, 24}, {7, 
       31}, {1, 12}, {5, 12}, {6, 12}, {11, 12}, {1, 10}, {8, 10}, {2, 23}, {
       2, 20}, {2, 13}, {2, 26}, {7, 34}, {7, 30}, {2, 21}, {2, 19}, {1, 9}, {
       8, 9}, {9, 10}}}, {VertexLabels -> {
         Placed["Name", Tooltip]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          1.1828614848597978`, 2.0381540759340373`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.5314359131345396`, 1.7495263174885392`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.1510319598632233`, 1.6331758440013944`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          4.321886082721295, 2.5688542301851163`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.8312144865311746`, 2.274960943332554}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          4.090727383973352, 0.9743232531202906}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.5136398566495353`, 3.87596319692095}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.4932105008312524`, 3.3302838375325297`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.168755297116855, 3.3907062619630097`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.13753068506863, 1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.2005548670558865`, 2.4141089041934087`}, {
          3.697389972281107, 2.5483788161553353`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          3.13753068506863, 1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          1.3650326581874392`, 2.7899399717035047`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.638106815880386, 2.906838123860532}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.4470941520740608, 1.1522297508935844`}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.2697820357991976, 2.7480377129789404`}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.19245318060902017`, 1.4782422488431117`}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          1.232729171504524, 1.088307523373295}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {0., 
          1.8080068686212736`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.2469063838402823, 2.070162018460066}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.8339702541274132, 1.0195758739207907`}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.9808081079801021, 3.028609508888132}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.6467086206920842, 2.4037160125281667`}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.6943677965118562, 1.5622918000956225`}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          0.038160271782984534`, 2.385801642927307}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1828614848597978`, 2.0381540759340373`}, {
          1.547296057346342, 1.4297136644882644`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.5314359131345396`, 1.7495263174885392`}, {
          3.1510319598632233`, 1.6331758440013944`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.5314359131345396`, 1.7495263174885392`}, {
          3.13753068506863, 1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.1510319598632233`, 1.6331758440013944`}, {
          3.13753068506863, 1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.321886082721295, 2.5688542301851163`}, {
          3.8312144865311746`, 2.274960943332554}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.321886082721295, 2.5688542301851163`}, {
          3.13753068506863, 1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.321886082721295, 2.5688542301851163`}, {
          3.697389972281107, 2.5483788161553353`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.321886082721295, 2.5688542301851163`}, {
          5.04086556482721, 3.251702949362633}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.321886082721295, 2.5688542301851163`}, {
          5.303336209041085, 2.439646106029551}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.321886082721295, 2.5688542301851163`}, {
          5.304941528790886, 2.8891906675560244`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8312144865311746`, 2.274960943332554}, {
          3.13753068506863, 1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8312144865311746`, 2.274960943332554}, {
          3.697389972281107, 2.5483788161553353`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.090727383973352, 0.9743232531202906}, {3.13753068506863,
           1.9848797584629236`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.090727383973352, 0.9743232531202906}, {
          4.712253254047935, 0.1285155600948329}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.090727383973352, 0.9743232531202906}, {
          4.347798105415934, 0.}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.090727383973352, 0.9743232531202906}, {
          4.9373391673142315`, 0.4368936944403503}}, 
          0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.090727383973352, 0.9743232531202906}, {
          3.9341991477184406`, 0.06291619655269587}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4.090727383973352, 0.9743232531202906}, {
          5.041880690973688, 0.8248158483196562}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.5136398566495353`, 3.87596319692095}, {
          3.4932105008312524`, 3.3302838375325297`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.5136398566495353`, 3.87596319692095}, {
          3.168755297116855, 3.3907062619630097`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.5136398566495353`, 3.87596319692095}, {
          4.011794445798466, 4.735618318742}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.5136398566495353`, 3.87596319692095}, {
          3.4948137995465016`, 4.859174463582365}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.4932105008312524`, 3.3302838375325297`}, {
          3.168755297116855, 3.3907062619630097`}}, 0.044764211877728305`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.13753068506863, 1.9848797584629236`}, {
          3.697389972281107, 2.5483788161553353`}}, 
          0.044764211877728305`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        TagBox[
         TooltipBox[
          DiskBox[{3.2005548670558865, 2.4141089041934087}, 
           0.044764211877728305],
          "\"program\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "program", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.1828614848597978, 2.0381540759340373}, 
           0.044764211877728305],
          "\"broadcast\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "broadcast", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.5314359131345396, 1.7495263174885392}, 
           0.044764211877728305],
          "\"computer program\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "computer program", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.1510319598632233, 1.6331758440013944}, 
           0.044764211877728305],
          "\"computer programme\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "computer programme", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.321886082721295, 2.5688542301851163}, 
           0.044764211877728305],
          "\"course of study\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "course of study", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.8312144865311746, 2.274960943332554}, 
           0.044764211877728305],
          "\"curriculum\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "curriculum", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.090727383973352, 0.9743232531202906}, 
           0.044764211877728305],
          "\"plan\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "plan", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.5136398566495353, 3.87596319692095}, 
           0.044764211877728305],
          "\"platform\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "platform", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.4932105008312524, 3.3302838375325297}, 
           0.044764211877728305],
          "\"political platform\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "political platform", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.168755297116855, 3.3907062619630097}, 
           0.044764211877728305],
          "\"political program\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "political program", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.13753068506863, 1.9848797584629236}, 
           0.044764211877728305],
          "\"programme\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "programme", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.697389972281107, 2.5483788161553353}, 
           0.044764211877728305],
          "\"syllabus\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "syllabus", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.3650326581874392, 2.7899399717035047}, 
           0.044764211877728305],
          "\"air\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "air", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.638106815880386, 2.906838123860532}, 
           0.044764211877728305],
          "\"beam\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "beam", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.4470941520740608, 1.1522297508935844}, 
           0.044764211877728305],
          "\"circularise\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "circularise", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.2697820357991976, 2.7480377129789404}, 
           0.044764211877728305],
          "\"circularize\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "circularize", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.19245318060902017, 1.4782422488431117}, 
           0.044764211877728305],
          "\"circulate\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "circulate", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.232729171504524, 1.088307523373295}, 
           0.044764211877728305],
          "\"diffuse\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "diffuse", "Tooltip"]& ], 
        TagBox[
         TooltipBox[DiskBox[{0., 1.8080068686212736}, 0.044764211877728305],
          "\"disperse\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "disperse", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.2469063838402823, 2.070162018460066}, 
           0.044764211877728305],
          "\"disseminate\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "disseminate", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.8339702541274132, 1.0195758739207907}, 
           0.044764211877728305],
          "\"distribute\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "distribute", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.9808081079801021, 3.028609508888132}, 
           0.044764211877728305],
          "\"pass around\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "pass around", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.6467086206920842, 2.4037160125281667}, 
           0.044764211877728305],
          "\"propagate\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "propagate", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.6943677965118562, 1.5622918000956225}, 
           0.044764211877728305],
          "\"send\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "send", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{0.038160271782984534, 2.385801642927307}, 
           0.044764211877728305],
          "\"spread\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "spread", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{1.547296057346342, 1.4297136644882644}, 
           0.044764211877728305],
          "\"transmit\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "transmit", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.04086556482721, 3.251702949362633}, 0.044764211877728305],
          "\"class\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "class", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.303336209041085, 2.439646106029551}, 
           0.044764211877728305],
          "\"course\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "course", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.304941528790886, 2.8891906675560244}, 
           0.044764211877728305],
          "\"course of instruction\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "course of instruction", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.712253254047935, 0.1285155600948329}, 
           0.044764211877728305],
          "\"architectural plan\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "architectural plan", "Tooltip"]& ], 
        TagBox[
         TooltipBox[DiskBox[{4.347798105415934, 0.}, 0.044764211877728305],
          "\"be after\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "be after", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.9373391673142315, 0.4368936944403503}, 
           0.044764211877728305],
          "\"contrive\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "contrive", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.9341991477184406, 0.06291619655269587}, 
           0.044764211877728305],
          "\"design\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "design", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{5.041880690973688, 0.8248158483196562}, 
           0.044764211877728305],
          "\"project\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "project", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{4.011794445798466, 4.735618318742}, 0.044764211877728305],
          "\"chopine\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "chopine", "Tooltip"]& ], 
        TagBox[
         TooltipBox[
          DiskBox[{3.4948137995465016, 4.859174463582365}, 
           0.044764211877728305],
          "\"weapons platform\"",
          TooltipStyle->"TextStyling"],
         Annotation[#, "weapons platform", "Tooltip"]& ]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[40]=",ExpressionUUID->"ad7ca542-edc1-45fe-9a84-0a2ab036a10a"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GraphPlot", "Subsection",
 CellTags->"Section1d",ExpressionUUID->"f6a00912-707c-4f69-aada-8b4fdc0c602f"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " function was introduced in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " version 6. This function is older than and mostly superseded by the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. However, ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " provides some different functionality and is worth a brief discussion. \
Keep in mind that ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " was designed specifically as a way to create images of graphs, before the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object was introduced, so if your focus is on creating an image of a graph \
and are having difficulty getting ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " to display the graph how you wish, ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " is worth considering."
}], "Text",ExpressionUUID->"d2fde6e1-9fb7-4117-8656-0ca56df5b40b"],

Cell[TextData[{
 "The most important difference between ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " is the fact that ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " produces a graph object, which can be computed with and manipulated, \
whereas ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " produces a graphics object, which is the same kind of output as ",
 StyleBox[ButtonBox["Plot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["Plot3D",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plot3D"], "FunctionLink"],
 " produce."
}], "Text",ExpressionUUID->"84105a22-81a2-410b-ae71-7e7e8c6d4c37"],

Cell[TextData[{
 "The basic input for ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " is either a list of rules representing the edges of the graph or a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " does not allow the use of the symbols \[UndirectedEdge] or \
\[DirectedEdge], nor does it allow the use of ",
 StyleBox[ButtonBox["TwoWayRule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TwoWayRule"], "FunctionLink"],
 " (",
 StyleBox["<->", "Code"],
 "). "
}], "Text",ExpressionUUID->"530d5180-098c-4d82-bc39-4865b3ec76e5"],

Cell["We will use the graph from Exercise 3 as the example.", "Text",ExpressionUUID->"6f08354c-478e-45d7-bf56-e0a3b69739a4"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphPlot", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
    RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
    RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
    RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], "]"}]], "Input",
 CellLabel->"In[41]:=",ExpressionUUID->"d97fa745-225a-482c-891c-7ddd91e0c829"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
    0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
    0.42430280790670055`}}, {
     {RGBColor[0.5, 0., 0.], LineBox[{{1, 2}, {1, 3}, {2, 3}, {2, 4}}]}, 
     {RGBColor[0, 0, 0.7], 
      TagBox[
       TooltipBox[PointBox[1],
        "\"a\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "a", "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[2],
        "\"b\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "b", "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[3],
        "\"c\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "c", "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[4],
        "\"d\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "d", "Tooltip"]& ]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{0.002089066523780936, 0.}, {0.9175848200953383,
      0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
     0.42430280790670055`}}]& ],
  AspectRatio->Automatic,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output",
 CellLabel->"Out[41]=",ExpressionUUID->"1161b772-5bb0-44d5-9c05-f5ff7fcd8042"]
}, Open  ]],

Cell[TextData[{
 "Right away, you can see an important difference between ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ". Specifically, where ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "\[CloseCurlyQuote]s default behavior is to interpret rules as directed \
edges, ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " assumes that rules are not directed. To draw a directed graph, you must \
use the ",
 StyleBox[ButtonBox["DirectedEdges",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdges"], "FunctionLink"],
 " option."
}], "Text",ExpressionUUID->"6f63391b-0cea-4be0-becd-c194e19192a2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"DirectedEdges", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[42]:=",ExpressionUUID->"124b6c89-ed3f-407b-90d5-e13c39d11c64"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
    0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
    0.42430280790670055`}}, {
     {RGBColor[0.5, 0., 0.], Arrowheads[{{0.03365237218299126, 0.8}}], 
      ArrowBox[{1, 2}], ArrowBox[{1, 3}], ArrowBox[{2, 3}], ArrowBox[{2, 4}]}, 
     {RGBColor[0, 0, 0.7], 
      TagBox[
       TooltipBox[PointBox[1],
        "\"a\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "a", "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[2],
        "\"b\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "b", "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[3],
        "\"c\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "c", "Tooltip"]& ], 
      TagBox[
       TooltipBox[PointBox[4],
        "\"d\"",
        TooltipStyle->"TextStyling"],
       Annotation[#, "d", "Tooltip"]& ]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{0.002089066523780936, 0.}, {0.9175848200953383,
      0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
     0.42430280790670055`}}]& ],
  AspectRatio->Automatic,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output",
 CellLabel->"Out[42]=",ExpressionUUID->"c78482f3-6b11-4546-ab9d-c6ed9547ed38"]
}, Open  ]],

Cell[TextData[{
 "To display the names of vertices, you use the ",
 StyleBox[ButtonBox["VertexLabeling",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabeling"], "FunctionLink"],
 " option. The default behavior, provided the number of vertices is not too \
large, is for the names of vertices to appear in a tooltip when you move the \
mouse pointer over the vertex. This option can be explicitly given with value \
",
 StyleBox[ButtonBox["Tooltip",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Tooltip"], "FunctionLink"],
 ". By setting the ",
 StyleBox[ButtonBox["VertexLabeling",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabeling"], "FunctionLink"],
 " option to ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ", the names will be displayed on the graph itself."
}], "Text",ExpressionUUID->"5adf40d0-d023-4300-92b9-a6adc6b1823b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[43]:=",ExpressionUUID->"bf76a05e-750e-4f16-9881-25e945c19df1"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
    0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
    0.42430280790670055`}}, {
     {RGBColor[0.5, 0., 0.], LineBox[{{1, 2}, {1, 3}, {2, 3}, {2, 4}}]}, {
      InsetBox[
       FrameBox["\<\"a\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["\<\"b\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["\<\"c\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["\<\"d\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{0.002089066523780936, 0.}, {0.9175848200953383,
      0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
     0.42430280790670055`}}]& ],
  AspectRatio->Automatic,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output",
 CellLabel->"Out[43]=",ExpressionUUID->"ec0e17fa-f307-4305-86ec-9d2a4102446c"]
}, Open  ]],

Cell[TextData[{
 "To explicitly control the placement of vertices, you use the ",
 StyleBox[ButtonBox["VertexCoordinateRules",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinateRules"], "FunctionLink"],
 " option. The value of this option is a list of rules identifying the name \
of a vertex with a pair representing ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]],ExpressionUUID->
  "1cbce95e-a3b2-46cc-8001-5604f3e47095"],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]],ExpressionUUID->
  "0e3a3d93-5331-4775-90e9-933f4c5961c4"],
 " coordinates. You can use the symbol ",
 StyleBox[ButtonBox["Automatic",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Automatic"], "FunctionLink"],
 " in place of the coordinate pair to indicate that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " should determine the location for that vertex automatically. This is the \
default for vertices that are omitted from the rules. You can also use ",
 StyleBox[ButtonBox["Automatic",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Automatic"], "FunctionLink"],
 " in place of either the ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]],ExpressionUUID->
  "040021ee-ddbd-41b4-8e90-d0c417f028b2"],
 " or ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]],ExpressionUUID->
  "a55198f9-0e8c-46d2-b712-b188fa0de9b0"],
 " value within a pair if you wish to specify one value but leave the other \
to ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine. We use ",
 StyleBox[ButtonBox["VertexCoordinateRules",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinateRules"], "FunctionLink"],
 " to rearrange the vertices in the graph from Exercise 3 to match the \
locations in the textbook."
}], "Text",ExpressionUUID->"611c7dbb-6361-404b-ab12-418469681e92"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"VertexLabeling", "\[Rule]", "True"}], ",", 
   RowBox[{"VertexCoordinateRules", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}]}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}]}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}]}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}]}], "}"}]}]}], "]"}]], "Input",
 CellLabel->"In[44]:=",ExpressionUUID->"f54ef256-0798-49b1-8172-521cace7964e"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0., 1.}, {1., 1.}, {0., 0.}, {1., 0.}}, {
     {RGBColor[0.5, 0., 0.], LineBox[{{1, 2}, {1, 3}, {2, 3}, {2, 4}}]}, {
      InsetBox[
       FrameBox["\<\"a\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["\<\"b\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["\<\"c\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["\<\"d\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{0., 1.}, {1., 1.}, {0., 0.}, {1., 0.}}]& ],
  AspectRatio->Automatic,
  FrameTicks->None,
  PlotRange->All,
  PlotRangePadding->Scaled[0.1]]], "Output",
 CellLabel->"Out[44]=",ExpressionUUID->"f7065ca8-5c3b-4b82-b11e-553bd33ecc4c"]
}, Open  ]],

Cell[TextData[{
 "One of the benefits of ",
 StyleBox[ButtonBox["GraphPlot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphPlot"], "FunctionLink"],
 " is that the displayed graphic can be dynamically manipulated. The code \
below produces default display of the graph from Exercise 3. If you \
double-click on the image and then double-click on a vertex (so that the \
vertex is highlighted), you can adjust the position of the vertex by dragging \
it. Below, we have moved vertex \[OpenCurlyDoubleQuote]d\
\[CloseCurlyDoubleQuote] into the center of the graph. Observe that the edge \
automatically adjusts."
}], "Text",ExpressionUUID->"a711a089-ce17-4172-aa45-9c58b3e99b74"],

Cell[BoxData[
 RowBox[{"GraphPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
     RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
     RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}]}], "}"}], ",", 
   RowBox[{"VertexLabeling", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellLabel->"In[45]:=",ExpressionUUID->"51b88f06-76d5-499e-ab13-49f042f6a0e7"],

Cell[BoxData[
 GraphicsBox[
  TagBox[GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
    0.4241670904205272}, {0., 0.8473715512181632}, {0.3277581484688401, 
    0.42136321347501565`}}, {
     {RGBColor[0.5, 0., 0.], LineBox[{{1, 2}, {1, 3}, {2, 3}, {2, 4}}]}, {
      InsetBox[
       FrameBox["\<\"a\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 1], InsetBox[
       FrameBox["\<\"b\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 2], InsetBox[
       FrameBox["\<\"c\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 3], InsetBox[
       FrameBox["\<\"d\"\>",
        Background->RGBColor[1, 1, 0.8],
        FrameStyle->RGBColor[0.94, 0.85, 0.36],
        StripOnInput->False], 4]}}],
   Annotation[#, 
    VertexCoordinateRules -> {{0.002089066523780936, 0.}, {0.9175848200953383,
      0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
     0.42430280790670055`}}]& ],
  AspectRatio->Automatic,
  FrameTicks->None,
  ImagePadding->{{0., 0.8}, {0.8, 0.}},
  PlotRange->{{-0.25405168598172834`, 
   2.286465173835555}, {-0.10592144390227039`, 0.9532929951204336}},
  PlotRangePadding->Automatic]], "Input",ExpressionUUID->"abc874c8-bf09-4ad4-\
ad71-d0a6ed0cbfaf"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.2 Graph Terminology and Special Types of Graphs", "Section",
 CellTags->"Section2",ExpressionUUID->"1c5a4943-84e4-4bdc-9f5d-f582dc295695"],

Cell[TextData[{
 "In this section, we will see how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to perform computations related to some of the basic terminology of \
graphs, such as calculating degree. We will also look at some of the special \
families of graphs included in the Wolfram Language. Finally, we will discuss \
subgraphs and unions of graphs."
}], "Text",ExpressionUUID->"b827afbd-b785-4ffc-bdfd-daaa6bb09e22"],

Cell[CellGroupData[{

Cell["Degree", "Subsection",
 CellTags->"Section2a",ExpressionUUID->"80010f42-936c-49f0-9b1b-b46fe8d617f5"],

Cell[TextData[{
 "For a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object, the Wolfram Language includes the function ",
 StyleBox[ButtonBox["VertexDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDegree"], "FunctionLink"],
 " for determining the degree of a vertex. Given a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object and one of the graph\[CloseCurlyQuote]s vertices, the function \
returns the number of edges incident to that vertex. For example, we can \
check the degrees of vertices ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]],ExpressionUUID->
  "a69cfd5d-3af4-428b-9e3b-4fd7b5407014"],
 " and ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]],ExpressionUUID->
  "fce4bd42-869b-4fcc-966c-74380494a42c"],
 " of ",
 StyleBox["exercise7", "Code"],
 " from the previous section."
}], "Text",ExpressionUUID->"a4e5b323-0924-4c19-97be-6ec03d438ffd"],

Cell[CellGroupData[{

Cell[BoxData["exercise7"], "Input",
 CellLabel->"In[46]:=",ExpressionUUID->"9888b51f-1f9c-4776-90b1-53134b96e0be"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e"}, {{{1, 2}, {2, 5}, {3, 2}, {3, 3}, {3, 4}, {4, 
       3}, {5, 1}, {5, 4}, {5, 5}}, Null}, {
      VertexLabels -> {"Name"}, 
       VertexCoordinates -> {{1, 1}, {2, 2}, {3, 2}, {3, 1}, {2, 1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
        ArrowBox[{{1., 1.}, {2., 2.}}, 0.02261146496815286], 
        ArrowBox[{{2., 2.}, {2., 1.}}, 0.02261146496815286], 
        ArrowBox[{{3., 2.}, {2., 2.}}, 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{3., 2.}, {2.9226652357765426`, 
         2.211102824883941}, {3.03037168156465, 2.404072374917325}, {
         3.1710710001067524`, 2.4556369686221933`}, {3.4556369686221933`, 
         2.1710710001067524`}, {3.404072374917325, 2.03037168156465}, {
         3.211102824883941, 1.9226652357765426`}, {3., 2.}},
         SplineDegree->7], 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{3., 2.}, {3.164440747183118, 
         1.499999999998105}, {3., 1.}}], 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{3., 1.}, {2.835559252816882, 
         1.5000000000037899`}, {3., 2.}}], 0.02261146496815286], 
        ArrowBox[{{2., 1.}, {1., 1.}}, 0.02261146496815286], 
        ArrowBox[{{2., 1.}, {3., 1.}}, 0.02261146496815286], 
        ArrowBox[BezierCurveBox[{{2., 1.}, {2.2304450364563646`, 
         0.8476276972960466}, {2.250002442847489, 0.6526270006898856}, {
         2.1815918401357854`, 0.5391807550302723}, {1.8184081598642148`, 
         0.5391807550302722}, {1.7499975571525106`, 0.6526270006898856}, {
         1.7695549635436354`, 0.8476276972960466}, {2., 1.}},
         SplineDegree->7], 0.02261146496815286]}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{1., 1.}, 0.02261146496815286], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {1.0226114649681528, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 2.}, 0.02261146496815286], 
         InsetBox["\<\"b\"\>", 
          Offset[{2, 2}, {2.022611464968153, 2.022611464968153}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 2.}, 0.02261146496815286], 
         InsetBox["\<\"c\"\>", 
          Offset[{2, 2}, {3.022611464968153, 2.022611464968153}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 1.}, 0.02261146496815286], 
         InsetBox["\<\"d\"\>", 
          Offset[{2, 2}, {3.022611464968153, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.02261146496815286], 
         InsetBox["\<\"e\"\>", 
          Offset[{2, 2}, {2.022611464968153, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[46]=",ExpressionUUID->"a8d074fa-cacb-43b4-99c1-18bfa7f54fb0"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexDegree", "[", 
  RowBox[{"exercise7", ",", "\"\<a\>\""}], "]"}]], "Input",
 CellLabel->"In[47]:=",ExpressionUUID->"ee7088a3-0122-41ad-a5a2-f8ddd090d5a0"],

Cell[BoxData["2"], "Output",
 CellLabel->"Out[47]=",ExpressionUUID->"de20d504-7af5-4888-ac8e-e7440fef6791"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexDegree", "[", 
  RowBox[{"exercise7", ",", "\"\<e\>\""}], "]"}]], "Input",
 CellLabel->"In[48]:=",ExpressionUUID->"7d265ecc-326b-4779-8f7d-bff20709ac6f"],

Cell[BoxData["5"], "Output",
 CellLabel->"Out[48]=",ExpressionUUID->"74094687-0731-4228-b056-feb1b36202cb"]
}, Open  ]],

Cell[TextData[{
 "Observe that the loop at the vertex ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]],ExpressionUUID->
  "b4ca32d7-85b9-4fcf-91b2-034fa7ddb7b2"],
 " counts as 2 towards the degree of that vertex. Moreover, note that with \
this directed graph, ",
 StyleBox[ButtonBox["VertexDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDegree"], "FunctionLink"],
 " calculates the number of edges incident to the given vertex without regard \
for their direction. The Wolfram Language includes ",
 StyleBox[ButtonBox["VertexInDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexInDegree"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["VertexOutDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexOutDegree"], "FunctionLink"],
 " for calculating the directed values. As an example, consider vertex ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]],ExpressionUUID->
  "3dc619ea-a8c1-4a62-a9c6-0a1180b98c27"],
 " from above."
}], "Text",ExpressionUUID->"d974460a-3af0-41c1-84e3-e7cd13e50543"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexInDegree", "[", 
  RowBox[{"exercise7", ",", "\"\<d\>\""}], "]"}]], "Input",
 CellLabel->"In[49]:=",ExpressionUUID->"6556ff99-b52a-4332-97d5-3506de95f981"],

Cell[BoxData["2"], "Output",
 CellLabel->"Out[49]=",ExpressionUUID->"039149dd-6465-4f5e-ae6c-6bb239515f4b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexOutDegree", "[", 
  RowBox[{"exercise7", ",", "\"\<d\>\""}], "]"}]], "Input",
 CellLabel->"In[50]:=",ExpressionUUID->"260b5094-3385-42ae-a985-c44959574935"],

Cell[BoxData["1"], "Output",
 CellLabel->"Out[50]=",ExpressionUUID->"10572701-1ebf-4c48-8093-526ec03f0eb2"]
}, Open  ]],

Cell[TextData[{
 "All three of these functions can be used without a second argument. If they \
are passed only the graph as the sole argument, they will return a list of \
the degrees of the vertices. Note that the output is in the same order as the \
output from ",
 StyleBox[ButtonBox["VertexList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexList"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"cf0161a8-d5a9-415e-9f3b-d1ff15eb8fd2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexList", "[", "exercise7", "]"}]], "Input",
 CellLabel->"In[51]:=",ExpressionUUID->"127b7a10-8f57-480d-9e9e-a28a744b6d44"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"e\"\>"}], "}"}]], "Output",
 CellLabel->"Out[51]=",ExpressionUUID->"b1e22162-cfa7-4246-994d-1bbec43c3411"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexDegree", "[", "exercise7", "]"}]], "Input",
 CellLabel->"In[52]:=",ExpressionUUID->"dddc3e77-e054-4e71-aa4d-cfd90849dbb6"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "5", ",", "3", ",", "5"}], "}"}]], "Output",
 CellLabel->"Out[52]=",ExpressionUUID->"b502cbf2-5394-4a18-bf00-88a499e65332"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexInDegree", "[", "exercise7", "]"}]], "Input",
 CellLabel->"In[53]:=",ExpressionUUID->"8c32150a-1ea1-47f2-88ba-18372100cd28"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "2", ",", "2", ",", "2"}], "}"}]], "Output",
 CellLabel->"Out[53]=",ExpressionUUID->"8ed727af-b786-4ee7-a2cc-3f24d21e0512"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexOutDegree", "[", "exercise7", "]"}]], "Input",
 CellLabel->"In[54]:=",ExpressionUUID->"c01d315c-3b74-4915-bf4e-2b45c898bfb7"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "1", ",", "3", ",", "1", ",", "3"}], "}"}]], "Output",
 CellLabel->"Out[54]=",ExpressionUUID->"606d37e1-34cd-4e47-b7a6-4e0cb6fa6db5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some Special Simple Graphs", "Subsection",
 CellTags->"Section2b",ExpressionUUID->"6beef6ed-30dc-4a7d-9036-5c578f4a00cc"],

Cell["\<\
The textbook discusses several families of graphs, including complete graphs, \
cycles, and wheels. The Wolfram Language provides functions for easily \
creating these and other special graphs. \
\>", "Text",ExpressionUUID->"d276b7ff-cd86-4e74-8a45-45d4482d35bc"],

Cell[TextData[{
 "We begin with complete graphs. Recall that a complete graph is a simple, \
undirected graph on a given number of vertices that has all possible edges \
between those vertices. The complete graph on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "19ba85a5-e990-480e-a53c-d5991badb4d9"],
 " vertices is denoted ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "n"], TraditionalForm]],ExpressionUUID->
  "08f6e52d-bd2a-4a38-99f9-1bdd7ac4d731"],
 ". The complete graph on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "4a2f4d8a-a6bd-4e6a-828c-56cb64194472"],
 " vertices can be obtained with the function ",
 StyleBox[ButtonBox["CompleteGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CompleteGraph"], "FunctionLink"],
 " applied to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "fbee5c9a-6b93-404d-9b9a-30cbf27983f0"],
 ". For example, we can generate and display ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "5"], TraditionalForm]],ExpressionUUID->
  "fbd2353a-5285-4510-a572-cf944f265a73"],
 ", the complete graph on 5 vertices."
}], "Text",ExpressionUUID->"8f0fb936-1de9-4acb-9ba8-d1782e590900"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteGraph", "[", "5", "]"}]], "Input",
 CellLabel->"In[55]:=",ExpressionUUID->"f774a154-01c0-4493-af9c-5e8b7ad392f8"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 4, 8, 12, 16, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {5}, {
          1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {GraphLayout -> "CircularEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.9510565162951538, 
       0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
       0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
       0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{4, 5}, 0.021812234931106983`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.021812234931106983], DiskBox[2, 0.021812234931106983], 
         DiskBox[3, 0.021812234931106983], DiskBox[4, 0.021812234931106983], 
         DiskBox[5, 0.021812234931106983]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[55]=",ExpressionUUID->"bc718e80-a019-46fd-b981-7769209dcf47"]
}, Open  ]],

Cell[TextData[{
 "Similarly, the cycle ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "n"], TraditionalForm]],ExpressionUUID->
  "adc7fb23-072c-4453-bb69-0e861b78979f"],
 " is obtained with the function ",
 StyleBox[ButtonBox["CycleGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CycleGraph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"3067d43c-93d6-4b1b-a521-444a25e45a93"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CycleGraph", "[", "9", "]"}]], "Input",
 CellLabel->"In[56]:=",ExpressionUUID->"f66d73b8-80bb-4b3e-9e24-8dd7d902a9ef"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9}, {Null, 
       SparseArray[
       Automatic, {9, 9}, 0, {
        1, {{0, 2, 4, 6, 8, 10, 12, 14, 16, 18}, {{2}, {9}, {1}, {3}, {2}, {
          4}, {3}, {5}, {4}, {6}, {5}, {7}, {6}, {8}, {7}, {9}, {1}, {8}}}, 
         Pattern}]}, {GraphLayout -> "CircularEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.6427876096865399, 
       0.7660444431189786}, {-0.9848077530122081, 
       0.17364817766693114`}, {-0.8660254037844384, -0.4999999999999994}, \
{-0.3420201433256684, -0.9396926207859079}, {
       0.34202014332566916`, -0.9396926207859085}, {
       0.8660254037844389, -0.5000000000000012}, {0.984807753012208, 
       0.17364817766692903`}, {0.6427876096865389, 
       0.7660444431189772}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{1, 9}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{4, 5}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{6, 7}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{7, 8}, 0.022364449578799767`]}, 
         {Arrowheads[0.], ArrowBox[{8, 9}, 0.022364449578799767`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.022364449578799767], DiskBox[2, 0.022364449578799767], 
         DiskBox[3, 0.022364449578799767], DiskBox[4, 0.022364449578799767], 
         DiskBox[5, 0.022364449578799767], DiskBox[6, 0.022364449578799767], 
         DiskBox[7, 0.022364449578799767], DiskBox[8, 0.022364449578799767], 
         DiskBox[9, 0.022364449578799767]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[56]=",ExpressionUUID->"ed3c2d2f-124b-4970-9197-c0fddfe0d5dc"]
}, Open  ]],

Cell[TextData[{
 "A wheel ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["W", "n"], TraditionalForm]],ExpressionUUID->
  "cd89901c-2cb6-4dd3-b6b9-362495e5897c"],
 " is obtained from the cycle graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "n"], TraditionalForm]],ExpressionUUID->
  "17262b14-d791-4849-9abf-ad150c4fdb29"],
 " by adding one additional vertex adjacent to all ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "aedb9ff9-a5b5-4b35-a9fd-ee0203b68cb7"],
 " of the original vertices. In the Wolfram Language, wheel graphs are \
obtained by ",
 StyleBox[ButtonBox["WheelGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WheelGraph"], "FunctionLink"],
 " applied to the value ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]],ExpressionUUID->
  "98e38e2f-0728-494d-9844-9cf4f166ea1b"],
 ", the total number of vertices in the wheel, not just the outside ring."
}], "Text",ExpressionUUID->"e2b5e3d8-89d3-4536-8d6b-1abc793afdbd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WheelGraph", "[", "6", "]"}]], "Input",
 CellLabel->"In[57]:=",ExpressionUUID->"72ba3ef9-5a37-4d56-8ec7-516d4d7a8344"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 5, 8, 11, 14, 17, 20}, {{2}, {3}, {4}, {5}, {6}, {1}, {3}, {
          6}, {1}, {2}, {4}, {1}, {3}, {5}, {1}, {4}, {6}, {1}, {2}, {5}}}, 
         Pattern}]}, {GraphLayout -> "StarEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0., 
       0.}, {-0.5877852522924727, -0.8090169943749468}, {
       0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
       0.30901699437494634`}, {-7.044813998280222*^-16, 
       1.}, {-0.9510565162951538, 0.30901699437494945`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 6}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 6}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{4, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.021812234931106983`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.021812234931106983], DiskBox[2, 0.021812234931106983], 
         DiskBox[3, 0.021812234931106983], DiskBox[4, 0.021812234931106983], 
         DiskBox[5, 0.021812234931106983], 
         DiskBox[6, 0.021812234931106983]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[57]=",ExpressionUUID->"70b51613-0389-4c09-916f-5d56dfd3c000"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Hypercubes", "Subsection",
 CellTags->"Section2c",ExpressionUUID->"be3bc5df-3392-45e8-b163-ef14416610cd"],

Cell[TextData[{
 "To construct the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "7e892f8b-9e85-4319-b3a5-30c5b1f71554"],
 "-cube ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Q", "n"], TraditionalForm]],ExpressionUUID->
  "c41a29c6-d9b8-466d-b911-d4d84fbe82dd"],
 ", we use the ",
 StyleBox[ButtonBox["HypercubeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HypercubeGraph"], "FunctionLink"],
 " function applied to the dimension ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "5604aa80-a00e-499c-93ea-c47ade97a89a"],
 ". Recall the definition of the hypercube graph given in Example 8 of \
Section 10.2. There are ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "n"], TraditionalForm]],ExpressionUUID->
  "dc148574-fb38-4382-9c9d-9be5a11db44e"],
 " vertices labeled with the binary representations of the numbers 0 through ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", "n"], "-", "1"}], TraditionalForm]],ExpressionUUID->
  "30324737-2ba3-4103-af9b-55fa42c7cef7"],
 ". Two vertices are adjacent if their binary representations differ in only \
one digit. Here is the presentation of the three-dimensional cube."
}], "Text",ExpressionUUID->"5acc6a22-cc88-4ea9-a86f-458dc95a45af"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypercubeGraph", "[", "3", "]"}]], "Input",
 CellLabel->"In[58]:=",ExpressionUUID->"cdecb6ac-3f12-487c-85d1-d8534e1ee3d4"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 3, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {5}, {1}, {4}, {6}, {
          1}, {4}, {7}, {2}, {3}, {8}, {1}, {6}, {7}, {2}, {5}, {8}, {3}, {
          5}, {8}, {4}, {6}, {7}}}, Pattern}]}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.6411318907234969, 1.6256350616332038`}, {
       1.5431761475992014`, 1.6258514889129208`}, {1.3641521541128723`, 
       0.8124188351574709}, {2.186420636755495, 0.814439144539747}, {0., 
       0.8116524116085829}, {0.8207122623609914, 0.8136876580070856}, {
       0.6431241846256299, 0.}, {1.5449243126722068`, 
       0.0005338415834358345}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{2, 6}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{3, 7}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{4, 8}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{5, 7}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{6, 8}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{7, 8}, 0.02410631722965237]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02410631722965237], DiskBox[2, 0.02410631722965237], 
         DiskBox[3, 0.02410631722965237], DiskBox[4, 0.02410631722965237], 
         DiskBox[5, 0.02410631722965237], DiskBox[6, 0.02410631722965237], 
         DiskBox[7, 0.02410631722965237], DiskBox[8, 0.02410631722965237]}}]],
     
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[58]=",ExpressionUUID->"eb2db52c-5e46-4fd9-bbe5-29b4379a5667"]
}, Open  ]],

Cell[TextData[{
 "By default, the vertices are not labeled. To have label the vertices, we \
can use the ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 " option. While the named parametric graphs generally accept all the same \
options that ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " does, ",
 StyleBox[ButtonBox["HypercubeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HypercubeGraph"], "FunctionLink"],
 " in particular is more robust passed to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"4212af1c-6e17-4ed9-b5f3-6fe08cc4ab77"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"HypercubeGraph", "[", "3", "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]], "Input",
 CellLabel->"In[60]:=",ExpressionUUID->"7e5831be-2ae7-4d73-b915-4268d07931f2"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 3, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {5}, {1}, {4}, {6}, {
          1}, {4}, {7}, {2}, {3}, {8}, {1}, {6}, {7}, {2}, {5}, {8}, {3}, {
          5}, {8}, {4}, {6}, {7}}}, Pattern}]}, {VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {
          1.5431761475992014`, 1.6258514889129208`}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {
          1.3641521541128723`, 0.8124188351574709}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {0., 
          0.8116524116085829}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5431761475992014`, 1.6258514889129208`}, {
          2.186420636755495, 0.814439144539747}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5431761475992014`, 1.6258514889129208`}, {
          0.8207122623609914, 0.8136876580070856}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.3641521541128723`, 0.8124188351574709}, {
          2.186420636755495, 0.814439144539747}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.3641521541128723`, 0.8124188351574709}, {
          0.6431241846256299, 0.}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.186420636755495, 0.814439144539747}, {
          1.5449243126722068`, 0.0005338415834358345}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8116524116085829}, {0.8207122623609914, 
          0.8136876580070856}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8116524116085829}, {0.6431241846256299, 0.}}, 
          0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8207122623609914, 0.8136876580070856}, {
          1.5449243126722068`, 0.0005338415834358345}}, 0.02410631722965237]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.6431241846256299, 0.}, {1.5449243126722068`, 
          0.0005338415834358345}}, 0.02410631722965237]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.6411318907234969, 1.6256350616332038}, 
          0.02410631722965237], 
         InsetBox["1", 
          Offset[{2, 2}, {0.6652382079531493, 1.649741378862856}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5431761475992014, 1.6258514889129208}, 
          0.02410631722965237], 
         InsetBox["2", 
          Offset[{2, 2}, {1.5672824648288537, 1.649957806142573}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.3641521541128723, 0.8124188351574709}, 
          0.02410631722965237], 
         InsetBox["3", 
          Offset[{2, 2}, {1.3882584713425246, 0.8365251523871233}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.186420636755495, 0.814439144539747}, 0.02410631722965237],
          InsetBox["4", 
          Offset[{2, 2}, {2.2105269539851475, 0.8385454617693994}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.8116524116085829}, 0.02410631722965237], 
         InsetBox["5", 
          Offset[{2, 2}, {0.02410631722965237, 0.8357587288382353}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8207122623609914, 0.8136876580070856}, 
          0.02410631722965237], 
         InsetBox["6", 
          Offset[{2, 2}, {0.8448185795906438, 0.837793975236738}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6431241846256299, 0.}, 0.02410631722965237], 
         InsetBox["7", 
          Offset[{2, 2}, {0.6672305018552823, 0.02410631722965237}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5449243126722068, 0.0005338415834358345}, 
          0.02410631722965237], 
         InsetBox["8", 
          Offset[{2, 2}, {1.569030629901859, 0.024640158813088203}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[59]=",ExpressionUUID->"d3a1a97d-c2fe-42a7-9163-33d4b66dda83"]
}, Open  ]],

Cell[TextData[{
 "One more modification will allow us to see the connection between this \
image and the definition. Instead of using ",
 StyleBox["\[OpenCurlyDoubleQuote]Name\[CloseCurlyDoubleQuote]", "Code"],
 " as the argument to ",
 StyleBox[ButtonBox["VertexLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexLabels"], "FunctionLink"],
 ", we can specify the labels explicitly by setting the option to a list of \
rules identifying the integers with the binary expression."
}], "Text",ExpressionUUID->"f78e5175-78a5-4c8d-9e81-d5ca513af52a"],

Cell[TextData[{
 "The definition of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["Q", "n"], TraditionalForm]],ExpressionUUID->
  "4a2d85e1-2dd1-4c6f-886d-85aa79a75123"],
 " tells us that the vertices should be considered to be the binary \
representations of the integers from 0 to 7. To display this, we will apply \
labels by subtracting 1 from the integer vertex names and using the ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " function to obtain the binary representation. The ",
 StyleBox[ButtonBox["IntegerString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerString"], "FunctionLink"],
 " function requires two arguments, an integer and a base, and produces a \
string representing the integer in that base. A third optional argument \
allows you to specify a minimal length for the string."
}], "Text",ExpressionUUID->"e179eb9b-5885-4257-958c-ee9c9d947d1c"],

Cell[TextData[{
 "The following ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " produces a list of rules identifying the vertex names with the appropriate \
label."
}], "Text",ExpressionUUID->"eba22082-6658-4847-b0d6-5af65d530cc5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"v", "\[Rule]", 
    RowBox[{"IntegerString", "[", 
     RowBox[{
      RowBox[{"v", "-", "1"}], ",", "2", ",", "3"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "8"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[60]:=",ExpressionUUID->"e8f98f8a-bc40-48be-97b7-7d1f6bba87b1"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "\[Rule]", "\<\"000\"\>"}], ",", 
   RowBox[{"2", "\[Rule]", "\<\"001\"\>"}], ",", 
   RowBox[{"3", "\[Rule]", "\<\"010\"\>"}], ",", 
   RowBox[{"4", "\[Rule]", "\<\"011\"\>"}], ",", 
   RowBox[{"5", "\[Rule]", "\<\"100\"\>"}], ",", 
   RowBox[{"6", "\[Rule]", "\<\"101\"\>"}], ",", 
   RowBox[{"7", "\[Rule]", "\<\"110\"\>"}], ",", 
   RowBox[{"8", "\[Rule]", "\<\"111\"\>"}]}], "}"}]], "Output",
 CellLabel->"Out[60]=",ExpressionUUID->"34ca43b9-bf11-475f-a975-8b86e0ed46d9"]
}, Open  ]],

Cell[TextData[{
 "We can combined this with the ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " symbol to move the labels inside the vertices. Be warned that, while this \
is an effective method in general, it can cause the kernel to crash when \
passed as an option directly to ",
 StyleBox[ButtonBox["HypercubeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HypercubeGraph"], "FunctionLink"],
 ". If this happens, passing the ",
 StyleBox[ButtonBox["HypercubeGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HypercubeGraph"], "FunctionLink"],
 " as an option to the more general ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " can fix the problem."
}], "Text",ExpressionUUID->"2b49b231-4008-4c17-9024-abe47220b789"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"HypercubeGraph", "[", "3", "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"v", "\[Rule]", 
       RowBox[{"Placed", "[", 
        RowBox[{
         RowBox[{"IntegerString", "[", 
          RowBox[{
           RowBox[{"v", "-", "1"}], ",", "2", ",", "3"}], "]"}], ",", 
         "Center"}], "]"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"v", ",", "8"}], "}"}]}], "]"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Large"}]}], "]"}]], "Input",
 CellLabel->"In[61]:=",ExpressionUUID->"33643236-a592-4494-8364-833dca074f7f"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 3, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {5}, {1}, {4}, {6}, {
          1}, {4}, {7}, {2}, {3}, {8}, {1}, {6}, {7}, {2}, {5}, {8}, {3}, {
          5}, {8}, {4}, {6}, {7}}}, Pattern}]}, {
      GraphLayout -> {"Dimension" -> 2}, 
       VertexLabels -> {
        6 -> Placed["101", Center], 3 -> Placed["010", Center], 5 -> 
         Placed["100", Center], 8 -> Placed["111", Center], 7 -> 
         Placed["110", Center], 4 -> Placed["011", Center], 2 -> 
         Placed["001", Center], 1 -> Placed["000", Center]}, 
       VertexSize -> {Large}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {
          1.5431761475992014`, 1.6258514889129208`}}, 
          0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {
          1.3641521541128723`, 0.8124188351574709}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {0., 
          0.8116524116085829}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5431761475992014`, 1.6258514889129208`}, {
          2.186420636755495, 0.814439144539747}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5431761475992014`, 1.6258514889129208`}, {
          0.8207122623609914, 0.8136876580070856}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.3641521541128723`, 0.8124188351574709}, {
          2.186420636755495, 0.814439144539747}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.3641521541128723`, 0.8124188351574709}, {
          0.6431241846256299, 0.}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.186420636755495, 0.814439144539747}, {
          1.5449243126722068`, 0.0005338415834358345}}, 
          0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8116524116085829}, {0.8207122623609914, 
          0.8136876580070856}}, 0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8116524116085829}, {0.6431241846256299, 0.}}, 
          0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8207122623609914, 0.8136876580070856}, {
          1.5449243126722068`, 0.0005338415834358345}}, 
          0.10868827459458903`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.6431241846256299, 0.}, {1.5449243126722068`, 
          0.0005338415834358345}}, 0.10868827459458903`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.6411318907234969, 1.6256350616332038}, 
          0.10868827459458903], 
         InsetBox["\<\"000\"\>", {0.6411318907234969, 1.6256350616332038},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5431761475992014, 1.6258514889129208}, 
          0.10868827459458903], 
         InsetBox["\<\"001\"\>", {1.5431761475992014, 1.6258514889129208},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.3641521541128723, 0.8124188351574709}, 
          0.10868827459458903], 
         InsetBox["\<\"010\"\>", {1.3641521541128723, 0.8124188351574709},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.186420636755495, 0.814439144539747}, 0.10868827459458903],
          InsetBox["\<\"011\"\>", {2.186420636755495, 0.814439144539747},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.8116524116085829}, 0.10868827459458903], 
         InsetBox["\<\"100\"\>", {0., 0.8116524116085829},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8207122623609914, 0.8136876580070856}, 
          0.10868827459458903], 
         InsetBox["\<\"101\"\>", {0.8207122623609914, 0.8136876580070856},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6431241846256299, 0.}, 0.10868827459458903], 
         InsetBox["\<\"110\"\>", {0.6431241846256299, 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5449243126722068, 0.0005338415834358345}, 
          0.10868827459458903], 
         InsetBox["\<\"111\"\>", {1.5449243126722068, 0.0005338415834358345},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[61]=",ExpressionUUID->"f29c1b54-62be-43ff-91f7-503af7be7fbb"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bipartite Graphs", "Subsection",
 CellTags->"Section2d",ExpressionUUID->"fa76909e-faac-40c7-9008-9f04388b75f0"],

Cell[TextData[{
 "Another important class of graphs is the bipartite graphs. A bipartite \
graph is one whose vertex set can be partitioned into two disjoint sets such \
that every edge has one vertex in each of the partitioning sets. In other \
words, no two vertices in the same partitioning set are adjacent. We write ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V", "=", 
    RowBox[{"(", 
     RowBox[{"A", ",", "B"}], ")"}]}], TraditionalForm]],ExpressionUUID->
  "43850f31-6b8f-4c9d-9e7c-ff53125471c8"],
 " to indicate that the vertex set ",
 Cell[BoxData[
  FormBox["V", TraditionalForm]],ExpressionUUID->
  "c0a3dab1-2a71-43d4-aad6-9e2a37fc0932"],
 " is partitioned into the sets ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "9aeecd8a-3d60-4cdc-81cb-20526a647238"],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "7bc94681-3bd9-4323-938b-1d7bf7dc71e4"],
 "."
}], "Text",ExpressionUUID->"7e6066c1-6d1d-4fa4-b5a6-aaf9d60a39af"],

Cell[TextData[{
 "The complete bipartite graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"m", ",", "n"}]], TraditionalForm]],ExpressionUUID->
  "047d5283-ee90-4aa2-8046-4b2fa06be3a2"],
 " is a bipartite graph with ",
 StyleBox["bipartition",
  IgnoreSpellCheck->True],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"V", "=", 
    RowBox[{"(", 
     RowBox[{"A", ",", "B"}], ")"}]}], TraditionalForm]],ExpressionUUID->
  "b6969f7b-c8bb-470b-a2c1-7ff93c041444"],
 " such that there are ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]],ExpressionUUID->
  "ab595af3-03aa-4332-a107-d316e9240dfa"],
 " vertices in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "defacba4-80bd-4fe8-b58b-20d2f63504b1"],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "9832c95b-718e-4b2e-a51e-db5ed0360eaf"],
 " in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "95c434f9-e23b-4e08-b62f-f9d101b07c8b"],
 " and such that there is an edge for every pair of vertices ",
 Cell[BoxData[
  FormBox[
   RowBox[{"a", "\[Element]", "A"}], TraditionalForm]],ExpressionUUID->
  "bbf5bea8-374f-48f1-a623-3bfb47002cdc"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", "\[Element]", "B"}], TraditionalForm]],ExpressionUUID->
  "c485048e-59b3-4216-839c-311041539d43"],
 ". The ",
 StyleBox[ButtonBox["CompleteGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CompleteGraph"], "FunctionLink"],
 " function can be used to create complete bipartite graphs. The argument is \
the list consisting of the pair of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]],ExpressionUUID->
  "48f9c111-0b05-453d-bd96-232ecfef2e6b"],
 " and ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "d9d13865-11e4-4252-82e2-9fe85ba7b758"],
 ". "
}], "Text",ExpressionUUID->"22293f23-7cb2-4f3b-abde-4b3ecbded1c9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteGraph", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "4"}], "}"}], "]"}]], "Input",
 CellLabel->"In[62]:=",ExpressionUUID->"534bd48a-b27a-4e04-a880-abd2fdbbc718"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7}, {Null, 
       SparseArray[
       Automatic, {7, 7}, 0, {
        1, {{0, 4, 8, 12, 15, 18, 21, 24}, {{4}, {5}, {6}, {7}, {4}, {5}, {
          6}, {7}, {4}, {5}, {6}, {7}, {1}, {2}, {3}, {1}, {2}, {3}, {1}, {
          2}, {3}, {1}, {2}, {3}}}, Pattern}]}, {
      GraphLayout -> {"MultipartiteLayout", "VertexPartition" -> {3, 4}}}]]}, 
    
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0., -0.2560113295832984}, {0., 0.}, {0., 
       0.2560113295832984}, {1., -0.38401699437494763`}, {
       1., -0.1280056647916492}, {1., 0.1280056647916492}, {1., 
       0.38401699437494763`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{1, 6}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{1, 7}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 5}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 6}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 7}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{3, 5}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{3, 6}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{3, 7}, 0.01273]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.01273], DiskBox[2, 0.01273], DiskBox[3, 0.01273], 
         DiskBox[4, 0.01273], DiskBox[5, 0.01273], DiskBox[6, 0.01273], 
         DiskBox[7, 0.01273]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[62]=",ExpressionUUID->"4fb25386-10b3-4a6a-a9a7-acc4a82c8a1d"]
}, Open  ]],

Cell[TextData[{
 "Notice that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " draws the complete bipartite graph with the two partitioning sets along \
the left and right to make the partition visually clear. As with the other \
functions in this section, the usual options for ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " apply. We illustrate how to label the vertices in a meaningful way. To \
enter a subscript in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", press \[ControlKey] and -."
}], "Text",ExpressionUUID->"4250a82b-5775-40eb-be0d-6b5ff8b61d13"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteGraph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "4"}], "}"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Union", "[", 
     RowBox[{
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"i", "\[Rule]", 
         SubscriptBox["a", "i"]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "3"}], "}"}]}], "]"}], ",", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"i", "\[Rule]", 
         SubscriptBox["b", 
          RowBox[{"i", "-", "3"}]]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", "4", ",", "7"}], "}"}]}], "]"}]}], "]"}]}]}], 
  "]"}]], "Input",
 CellLabel->"In[63]:=",ExpressionUUID->"4095066a-70f6-47d3-9b35-edb20ce87511"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7}, {Null, 
       SparseArray[
       Automatic, {7, 7}, 0, {
        1, {{0, 4, 8, 12, 15, 18, 21, 24}, {{4}, {5}, {6}, {7}, {4}, {5}, {
          6}, {7}, {4}, {5}, {6}, {7}, {1}, {2}, {3}, {1}, {2}, {3}, {1}, {
          2}, {3}, {1}, {2}, {3}}}, Pattern}]}, {
      GraphLayout -> {"MultipartiteLayout", "VertexPartition" -> {3, 4}}, 
       VertexLabels -> {
        6 -> Subscript[$CellContext`b, 3], 3 -> Subscript[$CellContext`a, 3], 
         5 -> Subscript[$CellContext`b, 2], 7 -> Subscript[$CellContext`b, 4],
          4 -> Subscript[$CellContext`b, 1], 2 -> 
         Subscript[$CellContext`a, 2], 1 -> Subscript[$CellContext`a, 1]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., -0.2560113295832984}, {1., -0.38401699437494763`}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., -0.2560113295832984}, {1., -0.1280056647916492}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., -0.2560113295832984}, {1., 0.1280056647916492}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., -0.2560113295832984}, {1., 0.38401699437494763`}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {1., -0.38401699437494763`}}, 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {1., -0.1280056647916492}}, 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {1., 0.1280056647916492}}, 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {1., 0.38401699437494763`}}, 0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.2560113295832984}, {1., -0.38401699437494763`}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.2560113295832984}, {1., -0.1280056647916492}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.2560113295832984}, {1., 0.1280056647916492}}, 
          0.01273]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.2560113295832984}, {1., 0.38401699437494763`}}, 
          0.01273]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., -0.2560113295832984}, 0.01273], InsetBox[
          SubscriptBox["a", "1"], 
          Offset[{2, 2}, {0.01273, -0.24328132958329843}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.01273], InsetBox[
          SubscriptBox["a", "2"], Offset[{2, 2}, {0.01273, 0.01273}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.2560113295832984}, 0.01273], InsetBox[
          SubscriptBox["a", "3"], 
          Offset[{2, 2}, {0.01273, 0.26874132958329844}], ImageScaled[{0, 0}],
          
          BaseStyle->"Graphics"]}, {
         DiskBox[{1., -0.38401699437494763}, 0.01273], InsetBox[
          SubscriptBox["b", "1"], 
          Offset[{2, 2}, {1.01273, -0.3712869943749476}], ImageScaled[{0, 0}],
          
          BaseStyle->"Graphics"]}, {
         DiskBox[{1., -0.1280056647916492}, 0.01273], InsetBox[
          SubscriptBox["b", "2"], 
          Offset[{2, 2}, {1.01273, -0.1152756647916492}], ImageScaled[{0, 0}],
          
          BaseStyle->"Graphics"]}, {
         DiskBox[{1., 0.1280056647916492}, 0.01273], InsetBox[
          SubscriptBox["b", "3"], 
          Offset[{2, 2}, {1.01273, 0.1407356647916492}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1., 0.38401699437494763}, 0.01273], InsetBox[
          SubscriptBox["b", "4"], 
          Offset[{2, 2}, {1.01273, 0.39674699437494765}], ImageScaled[{0, 0}],
          
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[63]=",ExpressionUUID->"04667f52-069f-4820-b8cc-3469bda605e9"]
}, Open  ]],

Cell[TextData[{
 "The Wolfram Language also supports complete multipartite graphs. A ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]],ExpressionUUID->
  "bef60900-11ba-446d-b907-526983d7db75"],
 "-partite graph is a graph in which the vertices can be partitioned into ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]],ExpressionUUID->
  "9a0d6e92-43e6-42e9-91f7-86dc3b9f968a"],
 " disjoint sets so that no two vertices in any one of the partitioning sets \
are adjacent."
}], "Text",ExpressionUUID->"a076328c-884d-49ca-a705-29dacf5de6dd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteGraph", "[", 
  RowBox[{"{", 
   RowBox[{"2", ",", "3", ",", "4"}], "}"}], "]"}]], "Input",
 CellLabel->"In[64]:=",ExpressionUUID->"41b17fac-cc62-42b0-b896-f62ad061401e"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9}, {Null, 
       SparseArray[
       Automatic, {9, 9}, 0, {
        1, {{0, 7, 14, 20, 26, 32, 37, 42, 47, 52}, {{3}, {4}, {5}, {6}, {
          7}, {8}, {9}, {3}, {4}, {5}, {6}, {7}, {8}, {9}, {1}, {2}, {6}, {
          7}, {8}, {9}, {1}, {2}, {6}, {7}, {8}, {9}, {1}, {2}, {6}, {7}, {
          8}, {9}, {1}, {2}, {3}, {4}, {5}, {1}, {2}, {3}, {4}, {5}, {1}, {
          2}, {3}, {4}, {5}, {1}, {2}, {3}, {4}, {5}}}, Pattern}]}, {
      GraphLayout -> {
        "MultipartiteLayout", "VertexPartition" -> {2, 3, 4}}}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0., -0.2310113295832984}, {0., 
       0.2310113295832984}, {1., -0.4620226591665968}, {1., 0.}, {1., 
       0.4620226591665968}, {2., -0.6930339887498952}, {
       2., -0.2310113295832984}, {2., 0.2310113295832984}, {2., 
       0.6930339887498952}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 7}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 8}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 9}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 5}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 7}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 8}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 9}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 7}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 8}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 9}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{4, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{4, 7}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{4, 8}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{4, 9}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{5, 7}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{5, 8}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{5, 9}, 0.02261146496815286]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02261146496815286], DiskBox[2, 0.02261146496815286], 
         DiskBox[3, 0.02261146496815286], DiskBox[4, 0.02261146496815286], 
         DiskBox[5, 0.02261146496815286], DiskBox[6, 0.02261146496815286], 
         DiskBox[7, 0.02261146496815286], DiskBox[8, 0.02261146496815286], 
         DiskBox[9, 0.02261146496815286]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[64]=",ExpressionUUID->"5c70b557-9947-48e2-bcc7-afa995923266"]
}, Open  ]],

Cell[TextData[{
 "The function ",
 StyleBox[ButtonBox["BipartiteGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BipartiteGraphQ"], "FunctionLink"],
 " determines whether a given graph is bipartite. This function accepts a \
graph as its sole argument and returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the graph is bipartite."
}], "Text",ExpressionUUID->"6e77f061-9231-4efd-82a6-87a291f3a9c3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BipartiteGraphQ", "[", 
  RowBox[{"HypercubeGraph", "[", "3", "]"}], "]"}]], "Input",
 CellLabel->"In[65]:=",ExpressionUUID->"e75d0c96-346a-4fa9-b0e7-f9c7d4357f4d"],

Cell[BoxData["True"], "Output",
 CellLabel->"Out[65]=",ExpressionUUID->"2fc92568-ab90-4be5-9151-6a2d5d137f9b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Bipartition",
 IgnoreSpellCheck->True]], "Subsubsection",ExpressionUUID->"3030be26-376a-\
490f-8066-bc368840d52f"],

Cell[TextData[{
 "It is worthwhile, however, to create a version of ",
 StyleBox["BipartiteGraphQ", "Code"],
 " from scratch in order to better understand both the representation of \
graphs in the Wolfram Language and an algorithm that determines whether the \
graph is bipartite and finds a ",
 StyleBox["bipartition",
  IgnoreSpellCheck->True],
 ". Our function will apply to a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. Moreover, instead of just returning true, our function will, if \
the graph is bipartite, display the graph with the vertices colored red and \
green to represent the partitioning. Of course, if the graph is not \
bipartite, the function will return false."
}], "Text",ExpressionUUID->"2305fe89-c5f8-4a78-9242-d51bd3317681"],

Cell[TextData[{
 "For ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " objects, the colors of vertices can be changed by setting the ",
 StyleBox[ButtonBox["VertexStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexStyle"], "FunctionLink"],
 " option to a single style for global changes or to a list of rules to set \
options for individual vertices as is shown below."
}], "Text",ExpressionUUID->"9ed6fe55-b130-48c1-9f44-da974ef2b21f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"HypercubeGraph", "[", "2", "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"VertexStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "Red"}], ",", 
      RowBox[{"2", "\[Rule]", "Green"}], ",", 
      RowBox[{"3", "\[Rule]", "Blue"}], ",", 
      RowBox[{"4", "\[Rule]", "Black"}]}], "}"}]}]}], "]"}]], "Input",Expressi\
onUUID->"b91cbd09-faf5-46ca-bb94-734d6be02815"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4}, {Null, 
       SparseArray[
       Automatic, {4, 4}, 0, {
        1, {{0, 2, 4, 6, 8}, {{2}, {3}, {1}, {4}, {1}, {4}, {2}, {3}}}, 
         Pattern}]}, {
      GraphLayout -> "GridEmbedding", VertexLabels -> {"Name"}, 
       VertexStyle -> {
        1 -> RGBColor[1, 0, 0], 2 -> RGBColor[0, 1, 0], 3 -> 
         RGBColor[0, 0, 1], 4 -> GrayLevel[0]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 2.}}, 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {2., 2.}}, 0.01273]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 2.}}, 0.01273]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[1, 0, 0], DiskBox[{1., 1.}, 0.01273]}, 
         InsetBox["1", Offset[{2, 2}, {1.01273, 1.01273}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 1, 0], DiskBox[{1., 2.}, 0.01273]}, 
         InsetBox["2", Offset[{2, 2}, {1.01273, 2.01273}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 0, 1], DiskBox[{2., 1.}, 0.01273]}, 
         InsetBox["3", Offset[{2, 2}, {2.01273, 1.01273}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {GrayLevel[0], DiskBox[{2., 2.}, 0.01273]}, 
         InsetBox["4", Offset[{2, 2}, {2.01273, 2.01273}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[66]=",ExpressionUUID->"59d658f4-e23b-42cc-9d51-f05d9b7c5c1e"]
}, Open  ]],

Cell[TextData[{
 "In addition to the color, the shape of the vertices can be changed using \
the ",
 StyleBox[ButtonBox["VertexShapeFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexShapeFunction"], "FunctionLink"],
 " option. This option can be used to take full control over the drawing of \
vertices by defining a function that builds the vertex out of graphics \
primitives. Fortunately, there are also a variety of predefined shape \
functions, as illustrated below."
}], "Text",ExpressionUUID->"9a38d6c8-7283-4d6f-b6e5-5a0bd2278841"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"HypercubeGraph", "[", "2", "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
   RowBox[{"VertexStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "Red"}], ",", 
      RowBox[{"2", "\[Rule]", "Green"}], ",", 
      RowBox[{"3", "\[Rule]", "Blue"}], ",", 
      RowBox[{"4", "\[Rule]", "Black"}]}], "}"}]}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"VertexShapeFunction", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "\"\<Square\>\""}], ",", 
      RowBox[{"2", "->", "\"\<Star\>\""}], ",", 
      RowBox[{"3", "->", "\"\<UpTrapezoid\>\""}], ",", 
      RowBox[{"4", "\[Rule]", "\"\<Capsule\>\""}]}], "}"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]], "Input",ExpressionUU\
ID->"ff5d60e4-fd93-44f7-8e19-0f528f782fa3"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4}, {Null, 
       SparseArray[
       Automatic, {4, 4}, 0, {
        1, {{0, 2, 4, 6, 8}, {{2}, {3}, {1}, {4}, {1}, {4}, {2}, {3}}}, 
         Pattern}]}, {
      GraphLayout -> "GridEmbedding", VertexLabels -> {"Name"}, 
       VertexShapeFunction -> {
        1 -> "Square", 4 -> "Capsule", 2 -> "Star", 3 -> "UpTrapezoid"}, 
       VertexSize -> {Medium}, 
       VertexStyle -> {
        1 -> RGBColor[1, 0, 0], 2 -> RGBColor[0, 1, 0], 3 -> 
         RGBColor[0, 0, 1], 4 -> GrayLevel[0]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 2.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {2., 2.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 2.}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[1, 0, 0], 
          RectangleBox[{0.9117647, 0.9117647}, {1.0882353, 1.0882353}]}, 
         InsetBox["1", Offset[{2, 2}, {1.1, 1.1}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 1, 0], 
          PolygonBox[{{1., 2.117647}, {0.970916, 2.0296704}, {
           0.8823529999999999, 2.0277727}, {0.9529412, 1.9715025}, {0.9272901,
            1.882353}, {1., 1.9355527}, {1.0727099, 1.882353}, {1.0470588, 
           1.9715025}, {1.117647, 2.0277727}, {1.029084, 2.0296704}, {1., 
           2.117647}}]}, 
         InsetBox["2", Offset[{2, 2}, {1.1, 2.1}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 0, 1], 
          PolygonBox[{{1.894118, 0.9152941}, {2.105882, 0.9152941}, {
           2.0529412, 1.0847059}, {1.9470588, 1.0847059}, {1.894118, 
           0.9152941}}]}, 
         InsetBox["3", Offset[{2, 2}, {2.1, 1.1}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {GrayLevel[0], 
          FilledCurveBox[{{{0, 2, 1}, {1, 6, 3}, {0, 1, 1}, {1, 6, 3}}}, {{{
           1.9411765, 1.9411765}, {2.0588235, 1.9411765}, {2.0913109, 
           1.9411765}, {2.117647, 1.9675127}, {2.117647, 2.}, {2.117647, 
           2.0324873}, {2.0913109, 2.0588235}, {2.0588235, 2.0588235}, {
           1.9411765, 2.0588235}, {1.9086891, 2.0588235}, {1.882353, 
           2.0324873}, {1.882353, 2.}, {1.882353, 1.9675127}, {1.9086891, 
           1.9411765}, {1.9411765, 1.9411765}}}]}, 
         InsetBox["4", Offset[{2, 2}, {2.1, 2.1}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[67]=",ExpressionUUID->"fa69acdb-745b-4176-8bcc-351ca0d99a6f"]
}, Open  ]],

Cell[TextData[{
 "Our version of ",
 StyleBox[ButtonBox["BipartiteGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BipartiteGraphQ"], "FunctionLink"],
 " will need to determine the list of neighbors of a given vertex. For a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object, the built-in function ",
 StyleBox[ButtonBox["AdjacencyList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyList"], "FunctionLink"],
 " applied to a graph and a vertex will return the list of vertices adjacent \
to it. However, we will create our own version of this to better illustrate \
how to manipulate graphs on a basic level."
}], "Text",ExpressionUUID->"b0882cd8-0c3c-4ebc-8751-ce53b29f065c"],

Cell[TextData[{
 "To find the neighbors of a vertex in a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object, we need to find all of the edges containing the given vertex and \
output the other vertex in the edge. The ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " function can be used for this. The first argument to ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " is a list of expressions. The second argument is a pattern expressing \
which elements of the first argument should be output. In our case, we want \
to pick out those edges, one of whose elements is the desired vertex. "
}], "Text",ExpressionUUID->"7ecd9b2a-cacd-4751-a818-29e88cfbfbce"],

Cell[TextData[{
 "Before applying ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " we need to extract the edges from the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " and determine what patterns to look for. The ",
 StyleBox[ButtonBox["EdgeList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeList"], "FunctionLink"],
 " function returns a list of the edges of a graph. If we apply ",
 StyleBox[ButtonBox["FullForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullForm"], "FunctionLink"],
 " as well, we can see the internal structure. We use the graph from Exercise \
3 in Section 10.1, which we defined above, as an example."
}], "Text",ExpressionUUID->"7f34ac59-f386-405e-89fd-3fd655d4a7b8"],

Cell[CellGroupData[{

Cell[BoxData["exercise3"], "Input",
 CellLabel->"In[68]:=",ExpressionUUID->"45073772-66f8-4519-a0f5-8cd9cd72f484"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, {{1, 2}, {1, 3}, {2, 3}, {2, 4}}}]]}, 
    
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.002089066523780936, 0.}, {0.9175848200953383, 
       0.4241670904205272}, {0., 0.8473715512181632}, {2.0324134878538267`, 
       0.42430280790670055`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.022873923537282098`]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.022873923537282098`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.022873923537282098], DiskBox[2, 0.022873923537282098], 
         DiskBox[3, 0.022873923537282098], 
         DiskBox[4, 0.022873923537282098]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[68]=",ExpressionUUID->"937da144-75ea-41b9-8e37-0e8308cb88b3"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"EdgeList", "[", "exercise3", "]"}], "//", "FullForm"}]], "Input",
 CellLabel->"In[69]:=",ExpressionUUID->"e2b00adb-ef54-4b45-815a-8a87e905ea78"],

Cell[BoxData[
 TagBox[
  StyleBox[
   RowBox[{"List", "[", 
    RowBox[{
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<a\>\"", ",", "\"\<b\>\""}], "]"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<a\>\"", ",", "\"\<c\>\""}], "]"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<c\>\""}], "]"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"\"\<b\>\"", ",", "\"\<d\>\""}], "]"}]}], "]"}],
   ShowSpecialCharacters->False,
   ShowStringCharacters->True,
   NumberMarks->True],
  FullForm]], "Output",
 CellLabel->
  "Out[69]//FullForm=",ExpressionUUID->"1c75f7bf-61e8-4800-b4a5-cb3486e086a5"]
}, Open  ]],

Cell[TextData[{
 "Observe that this is a list of pairs of vertices with the head ",
 StyleBox[ButtonBox["UndirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UndirectedEdge"], "FunctionLink"],
 ". If the edges were directed, the head would be ",
 StyleBox[ButtonBox["DirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdge"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"d8bc94ef-f2df-4257-80df-8cb448e06b3c"],

Cell["\<\
Consider the following graph, with both directed and undirected edges.\
\>", "Text",ExpressionUUID->"52bddbfe-d991-4483-9e81-c40bccdbac72"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"partitionExample", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "1"}], ",", 
      RowBox[{"3", "\[TwoWayRule]", "4"}], ",", 
      RowBox[{"1", "\[TwoWayRule]", "4"}], ",", 
      RowBox[{"1", "\[TwoWayRule]", "4"}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[70]:=",ExpressionUUID->"e08ca07a-1de1-4957-b2be-9c5aa20e0632"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4}, {{{1, 2}, {2, 3}, {3, 1}}, {{3, 4}, {1, 4}, {1, 
      4}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Small}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9353870673697849, 0.8689179180057149}, {
          0.5389903663105874, 0.49831638566156367`}, {0., 
          0.43534634984504866`}}], 0.04344593429585063]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9353870673697849, 0.8689179180057149}, {
          0.3963967010591986, 0.8059478821893371}, {0., 
          0.43534634984504866`}}], 0.04344593429585063]}, 
        ArrowBox[{{0.9353870673697849, 0.8689179180057149}, {
         1.868142562589604, 0.43480952429647823`}}, 0.04344593429585063], 
        ArrowBox[{{1.868142562589604, 0.43480952429647823`}, {
         0.9342318598726096, 0.}}, 0.04344593429585063], 
        {Arrowheads[0.], 
         ArrowBox[{{0.9342318598726096, 0.}, {0., 0.43534634984504866`}}, 
          0.04344593429585063]}, 
        ArrowBox[{{0.9342318598726096, 0.}, {0.9353870673697849, 
         0.8689179180057149}}, 0.04344593429585063]}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.9353870673697849, 0.8689179180057149}, 
          0.04344593429585063], 
         InsetBox["1", {0.9353870673697849, 0.8689179180057149},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.868142562589604, 0.43480952429647823}, 
          0.04344593429585063], 
         InsetBox["2", {1.868142562589604, 0.43480952429647823},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9342318598726096, 0.}, 0.04344593429585063], 
         InsetBox["3", {0.9342318598726096, 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.43534634984504866}, 0.04344593429585063], 
         InsetBox["4", {0., 0.43534634984504866},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[70]=",ExpressionUUID->"ad6a9842-de2f-468c-afac-1607cb135e5c"]
}, Open  ]],

Cell[TextData[{
 "Suppose we are looking for edges involving the vertex 3. Those edges will \
either be of the form ",
 StyleBox["Head", "CodeVar"],
 StyleBox["[3,_]", "Code"],
 " or ",
 StyleBox["Head", "CodeVar"],
 StyleBox["[_,3]", "Code"],
 ", where ",
 StyleBox["Head", "CodeVar"],
 " is either ",
 StyleBox[ButtonBox["DirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdge"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["UndirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UndirectedEdge"], "FunctionLink"],
 ". In a pattern, we use the ",
 StyleBox[ButtonBox["Alternatives",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Alternatives"], "FunctionLink"],
 " (",
 StyleBox["|", "Code"],
 ") operator to combined two or more possibilities in a single pattern. The \
following picks out all of the edges incident to vertex 3 in ",
 StyleBox["partitionExample", "Code"],
 "."
}], "Text",ExpressionUUID->"e434a226-1b04-4ec1-89cc-e209c6326fc4"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EdgeList", "[", "partitionExample", "]"}]], "Input",
 CellLabel->"In[71]:=",ExpressionUUID->"113c7974-4c2a-44a0-8818-d9a4b43f5bb7"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "\[DirectedEdge]", "2"}], ",", 
   RowBox[{"2", "\[DirectedEdge]", "3"}], ",", 
   RowBox[{"3", "\[DirectedEdge]", "1"}], ",", 
   RowBox[{"3", "\[UndirectedEdge]", "4"}], ",", 
   RowBox[{"1", "\[UndirectedEdge]", "4"}], ",", 
   RowBox[{"1", "\[UndirectedEdge]", "4"}]}], "}"}]], "Output",
 CellLabel->"Out[71]=",ExpressionUUID->"8075834c-038c-425e-b6d8-1ff8dd0f9681"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{
   RowBox[{"EdgeList", "[", "partitionExample", "]"}], ",", 
   RowBox[{
    RowBox[{"DirectedEdge", "[", 
     RowBox[{"3", ",", "_"}], "]"}], "|", 
    RowBox[{"DirectedEdge", "[", 
     RowBox[{"_", ",", "3"}], "]"}], "|", 
    RowBox[{"UndirectedEdge", "[", 
     RowBox[{"3", ",", "_"}], "]"}], "|", 
    RowBox[{"UndirectedEdge", "[", 
     RowBox[{"_", ",", "3"}], "]"}]}]}], "]"}]], "Input",
 CellLabel->"In[72]:=",ExpressionUUID->"7a6dc0ec-08a9-4ab3-8dd1-f2ee8d552d9f"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"2", "\[DirectedEdge]", "3"}], ",", 
   RowBox[{"3", "\[DirectedEdge]", "1"}], ",", 
   RowBox[{"3", "\[UndirectedEdge]", "4"}]}], "}"}]], "Output",
 CellLabel->"Out[72]=",ExpressionUUID->"6bc02421-a0f4-409a-af05-311545abbc89"]
}, Open  ]],

Cell[TextData[{
 "Note that these edges form the neighborhood graph of vertex 3, that is, the \
subgraph consisting of vertex 3 and all of its neighbors. However, our \
interest is the vertices, not the edges. To obtain the vertices, we will take \
advantage of another feature of the ",
 StyleBox[ButtonBox["Cases",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Cases"], "FunctionLink"],
 " function: the second argument can be given as a rule. The left-hand side \
of the rule is the pattern expressing which elements of the original list \
should match and the right-hand side describes what to include in the output \
for that matching element. Since we want the other vertex in the output, not \
the entire rule, we will name the blanks in the rule and output the vertex."
}], "Text",ExpressionUUID->"12f75f7e-fecd-4a93-abf4-d317e5b0f6d8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases", "[", 
  RowBox[{
   RowBox[{"EdgeList", "[", "partitionExample", "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"3", ",", "x_"}], "]"}], "|", 
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"x_", ",", "3"}], "]"}], "|", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"3", ",", "x_"}], "]"}], "|", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"x_", ",", "3"}], "]"}]}], "\[RuleDelayed]", "x"}]}], 
  "]"}]], "Input",
 CellLabel->"In[73]:=",ExpressionUUID->"bdefbebd-2802-45d4-ab3d-7b2aa66c5a79"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "1", ",", "4"}], "}"}]], "Output",
 CellLabel->"Out[73]=",ExpressionUUID->"822d18fb-12b8-488e-a528-d900805875c5"]
}, Open  ]],

Cell[TextData[{
 "Note that we use ",
 StyleBox[ButtonBox["RuleDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RuleDelayed"], "FunctionLink"],
 " (",
 StyleBox[":>", "Code"],
 ", \[RuleDelayed]) to protect against the symbol ",
 StyleBox["x", "Code"],
 " having been assigned a value elsewhere. It is a good idea to use ",
 StyleBox[ButtonBox["SetDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SetDelayed"], "FunctionLink"],
 " (",
 StyleBox[":=", "Code"],
 ") and ",
 StyleBox[ButtonBox["RuleDelayed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RuleDelayed"], "FunctionLink"],
 " (",
 StyleBox[":>", "Code"],
 ", \[RuleDelayed]) when named blanks are involved."
}], "Text",ExpressionUUID->"2cd8830b-0ceb-458c-a9a9-967ec4797ec2"],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["Union",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Union"], "FunctionLink"],
 " will remove duplicates and sort the results. Replacing the example data \
with arguments allows us to create a function."
}], "Text",ExpressionUUID->"82c0bfbe-1fa1-4058-9c5b-ab88b583f41c"],

Cell[BoxData[
 RowBox[{
  RowBox[{"neighbors", "[", 
   RowBox[{"g_Graph", ",", "v_"}], "]"}], ":=", 
  RowBox[{"Union", "[", 
   RowBox[{"Cases", "[", 
    RowBox[{
     RowBox[{"EdgeList", "[", "g", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"DirectedEdge", "[", 
        RowBox[{"v", ",", "x_"}], "]"}], "|", 
       RowBox[{"DirectedEdge", "[", 
        RowBox[{"x_", ",", "v"}], "]"}], "|", 
       RowBox[{"UndirectedEdge", "[", 
        RowBox[{"v", ",", "x_"}], "]"}], "|", 
       RowBox[{"UndirectedEdge", "[", 
        RowBox[{"x_", ",", "v"}], "]"}]}], "\[RuleDelayed]", "x"}]}], "]"}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[74]:=",ExpressionUUID->"832b91b6-0cde-4cbe-989f-118ee76b0f8e"],

Cell[TextData[{
 "We now turn to our version of ",
 StyleBox[ButtonBox["BipartiteGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BipartiteGraphQ"], "FunctionLink"],
 ". The idea of the function is as follows. (Note that this method is based \
on forming a spanning tree of the graph, a concept discussed in Section 11.4 \
of the textbook)."
}], "Text",ExpressionUUID->"4e8de1c9-1373-4156-a5be-bd45b5065240"],

Cell[CellGroupData[{

Cell[TextData[{
 "Pick a vertex ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "04839af0-61d4-4d55-a905-0ee08dd3a1bd"],
 " from the vertex set and place it in the set ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "4dc70135-0c74-4627-84bf-db7d820c1c80"],
 "."
}], "Item1Numbered",ExpressionUUID->"e1c4ce36-664c-4dde-ac0d-1209987593f1"],

Cell[TextData[{
 "Place all of ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "e096f051-1d06-4a84-9045-65486cb14403"],
 StyleBox["\[CloseCurlyQuote]s",
  IgnoreSpellCheck->True],
 " neighbors in set ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "3bcde5a4-6773-49cd-9aab-ebc507b40b9c"],
 "."
}], "Item1Numbered",ExpressionUUID->"2a179c87-92f8-4b4f-b509-56b3af8cf293"],

Cell[TextData[{
 "For each vertex ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "d13131ef-e5ed-4f6a-8225-d2c911a6faaa"],
 " in the set ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "d71f2b0c-10fb-4958-b44f-efabf301bb8e"],
 " that has not already been processed, place all of ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "4e3110d8-c0cf-496e-89c4-3109fc362d6d"],
 StyleBox["\[CloseCurlyQuote]s",
  IgnoreSpellCheck->True],
 " neighbors that are not already in either set into the set ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "cd3f7b61-c14e-457f-bc65-1c22dcba2d61"],
 "."
}], "Item1Numbered",ExpressionUUID->"0c4eda7d-9c84-44c7-a4f3-084aef3f16c6"],

Cell[TextData[{
 "Repeat step 3, reversing ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "07bdeccd-ea06-4cf6-8a78-48f02eb17ecb"],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "1b471804-638f-4ae1-93b2-6438b60e01ed"],
 " until no more vertices remain to be processed."
}], "Item1Numbered",ExpressionUUID->"f51ac8a3-8fd7-4bf4-93a4-b3f3f3c5e33f"],

Cell[TextData[{
 "Once step 4 is complete, we have formed a disjoint partition of the \
vertices. We then examine each edge of the graph and ensure that no edge has \
both ends in ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "648780d6-795f-4e5e-a5ab-fd244e9b498c"],
 " or both ends in ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "3513e9af-10d0-4580-a201-3e23480627e7"],
 ". If some edge fails that test, then the graph is not bipartite. If all of \
the edges do pass the test, then the graph is bipartite and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"A", ",", "B"}], ")"}], TraditionalForm]],ExpressionUUID->
  "7fb75183-26e0-421c-b759-1f6822a68f61"],
 " is a ",
 StyleBox["bipartition",
  IgnoreSpellCheck->True],
 "."
}], "Item1Numbered",ExpressionUUID->"4b9c40ab-ea7b-44a7-91d6-5c95534b08bf"]
}, Open  ]],

Cell[TextData[{
 "In the implementation, rather than using two sets ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "4b8f9421-2668-43fe-9545-84b07bf2c239"],
 " and ",
 Cell[BoxData[
  FormBox["B", TraditionalForm]],ExpressionUUID->
  "626911de-7630-4f14-bb35-d54e0db043cd"],
 ", we use the symbol ",
 StyleBox["AB", "Code"],
 " as an indexed variable, with ",
 StyleBox["AB[0]", "Code"],
 " representing one set and ",
 StyleBox["AB[1]", "Code"],
 " the other. This allows us to refer to them both as ",
 StyleBox["AB[i]", "Code"],
 " and vary ",
 StyleBox["i", "Code"],
 "."
}], "Text",ExpressionUUID->"01239376-5c19-4d9d-8ccf-64cb09b0e04b"],

Cell[BoxData[
 RowBox[{
  RowBox[{"drawBipartite", "[", "g_Graph", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "AB", ",", "i", ",", "T", ",", "w", ",", "e"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V", "=", 
      RowBox[{"VertexList", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"w", "=", 
      RowBox[{"V", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"AB", "[", "0", "]"}], "=", 
      RowBox[{"{", "w", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"AB", "[", "1", "]"}], "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"V", "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"T", "=", 
         RowBox[{"Intersection", "[", 
          RowBox[{"V", ",", 
           RowBox[{"AB", "[", "i", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "=", 
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"i", "+", "1"}], ",", "2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"AB", "[", "i", "]"}], "=", 
           RowBox[{"Union", "[", 
            RowBox[{
             RowBox[{"AB", "[", "i", "]"}], ",", 
             RowBox[{"Complement", "[", 
              RowBox[{
               RowBox[{"neighbors", "[", 
                RowBox[{"g", ",", "w"}], "]"}], ",", 
               RowBox[{"Union", "[", 
                RowBox[{
                 RowBox[{"AB", "[", "0", "]"}], ",", 
                 RowBox[{"AB", "[", "1", "]"}]}], "]"}]}], "]"}]}], "]"}]}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"w", ",", "T"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"V", "=", 
         RowBox[{"Complement", "[", 
          RowBox[{"V", ",", "T"}], "]"}]}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "0", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "&&", 
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "0", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ")"}], "||", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "1", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "&&", 
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "1", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ")"}]}], ",", 
           RowBox[{"Throw", "[", "False", "]"}]}], "]"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"e", ",", 
           RowBox[{"EdgeList", "[", "g", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Graph", "[", 
        RowBox[{"g", ",", 
         RowBox[{"VertexStyle", "\[Rule]", 
          RowBox[{"Union", "[", 
           RowBox[{
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"i", "\[Rule]", "Green"}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", 
                RowBox[{"AB", "[", "0", "]"}]}], "}"}]}], "]"}], ",", 
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"i", "\[Rule]", "Yellow"}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", 
                RowBox[{"AB", "[", "1", "]"}]}], "}"}]}], "]"}]}], "]"}]}], 
         ",", 
         RowBox[{"VertexShapeFunction", "\[Rule]", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"i", "\[Rule]", "\"\<Square\>\""}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", 
              RowBox[{"AB", "[", "0", "]"}]}], "}"}]}], "]"}]}]}], "]"}]}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 CellLabel->"In[75]:=",ExpressionUUID->"7e99a805-99d3-457c-b35f-2de49fd6c396"],

Cell[TextData[{
 "Note that if the graph is not bipartite, such as the graph ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]],ExpressionUUID->
  "dc5713e2-06f3-4399-9f01-b8a01477fe16"],
 " from Figure 8 in Section 10.3 of the textbook, the function will output \
false."
}], "Text",ExpressionUUID->"ccfb0a35-6e92-4d56-8f5c-f46a66436046"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure8H", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\"", ",", "\"\<f\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<f\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"4", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[76]:=",ExpressionUUID->"1c193ff8-b12b-4e18-8e9c-12d83dc544c8"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f"}, {
      Null, {{1, 2}, {1, 5}, {1, 6}, {2, 3}, {2, 4}, {2, 5}, {2, 6}, {3, 4}, {
       3, 6}, {4, 5}, {4, 6}, {5, 6}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       VertexCoordinates -> {{1, 2}, {3, 2}, {4, 1}, {3, 0}, {1, 0}, {0, 
        1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{1., 2.}, {3., 2.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 2.}, {1., 0.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 2.}, {0., 1.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {4., 1.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {3., 0.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {1., 0.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {0., 1.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4., 1.}, {3., 0.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4., 1.}, {0., 1.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 0.}, {1., 0.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 0.}, {0., 1.}}, 0.14142135623730953`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 0.}, {0., 1.}}, 0.14142135623730953`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{1., 2.}, 0.14142135623730953], 
         InsetBox["\<\"a\"\>", {1., 2.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 2.}, 0.14142135623730953], 
         InsetBox["\<\"b\"\>", {3., 2.},
          BaseStyle->"Graphics"]}, {DiskBox[{4., 1.}, 0.14142135623730953], 
         InsetBox["\<\"c\"\>", {4., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.14142135623730953], 
         InsetBox["\<\"d\"\>", {3., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.14142135623730953], 
         InsetBox["\<\"e\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 1.}, 0.14142135623730953], 
         InsetBox["\<\"f\"\>", {0., 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[76]=",ExpressionUUID->"cd9f1585-6843-46e4-a0e3-0f90d7f26eb6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drawBipartite", "[", "figure8H", "]"}]], "Input",
 CellLabel->"In[77]:=",ExpressionUUID->"633ca587-c081-49f2-8c38-2a08d0d3d76f"],

Cell[BoxData["False"], "Output",
 CellLabel->"Out[77]=",ExpressionUUID->"133647e3-3e1a-4d16-9ccb-001add8b101b"]
}, Open  ]],

Cell[TextData[{
 "On the other hand, if the graph is bipartite, such as the graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "14b72d0b-e620-4937-8325-1ec3ebf60496"],
 " shown in Figure 8 from Section 10.3, the function will draw the graph with \
the vertices colored to illustrate the ",
 StyleBox["bipartition",
  IgnoreSpellCheck->True],
 ". Note the use of ",
 StyleBox[ButtonBox["GraphLayout",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphLayout"], "FunctionLink"],
 " in the initial creation of graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "9ad58006-c9df-4f8d-9bac-4a3bd9d66922"],
 " below. The help page lists possible layout options. Of course, you can \
always specify vertex locations manually with ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 ", as above, or omit both options and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will choose a placement for you."
}], "Text",ExpressionUUID->"c3321a87-a745-4ecf-9de5-b1bb2fa3ba5b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure8G", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<g\>\"", ",", "\"\<f\>\"", ",", "\"\<e\>\"", ",", 
      "\"\<d\>\"", ",", "\"\<c\>\"", ",", "\"\<b\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<g\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"GraphLayout", "\[Rule]", "\"\<CircularEmbedding\>\""}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[78]:=",ExpressionUUID->"8a56b41a-5452-4f66-b611-9b45a20c8105"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "g", "f", "e", "d", "c", "b"}, {
      Null, {{1, 6}, {1, 4}, {1, 3}, {1, 2}, {7, 6}, {7, 4}, {7, 3}, {6, 5}, {
       5, 4}, {5, 3}, {5, 2}}}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-0.9749279121818235, -0.22252093395631342`}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-0.43388373911755757`, -0.9009688679024187}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.43388373911755884`, -0.9009688679024196}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.43388373911755884`, -0.9009688679024196}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.43388373911755884`, -0.9009688679024196}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9749279121818237, -0.2225209339563155}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7818314824680294, 
          0.6234898018587327}, {-7.044813998280222*^-16, 1.}}, 
          0.08677674782351154]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.7818314824680302, 0.6234898018587343}, 
          0.08677674782351154], 
         InsetBox["\<\"a\"\>", {-0.7818314824680302, 0.6234898018587343},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9749279121818235, -0.22252093395631342}, 
          0.08677674782351154], 
         InsetBox["\<\"g\"\>", {-0.9749279121818235, -0.22252093395631342},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.43388373911755757, -0.9009688679024187}, 
          0.08677674782351154], 
         InsetBox["\<\"f\"\>", {-0.43388373911755757, -0.9009688679024187},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.43388373911755884, -0.9009688679024196}, 
          0.08677674782351154], 
         InsetBox["\<\"e\"\>", {0.43388373911755884, -0.9009688679024196},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9749279121818237, -0.2225209339563155}, 
          0.08677674782351154], 
         InsetBox["\<\"d\"\>", {0.9749279121818237, -0.2225209339563155},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.7818314824680294, 0.6234898018587327}, 
          0.08677674782351154], 
         InsetBox["\<\"c\"\>", {0.7818314824680294, 0.6234898018587327},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.08677674782351154], 
         InsetBox["\<\"b\"\>", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[78]=",ExpressionUUID->"abb7b6d0-81e6-4739-9666-463c109ecd35"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drawBipartite", "[", "figure8G", "]"}]], "Input",
 CellLabel->"In[84]:=",ExpressionUUID->"8b79b2d3-f626-43c8-92fa-eaa207918d29"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "g", "f", "e", "d", "c", "b"}, {
      Null, {{1, 6}, {1, 4}, {1, 3}, {1, 2}, {7, 6}, {7, 4}, {7, 3}, {6, 5}, {
       5, 4}, {5, 3}, {5, 2}}}, {
      GraphLayout -> {
        "Dimension" -> 2, "VertexLayout" -> "CircularEmbedding"}, 
       VertexLabels -> {
         Placed["Name", Center]}, 
       VertexShapeFunction -> {
        "b" -> "Square", "d" -> "Square", "a" -> "Square"}, 
       VertexSize -> {Medium}, 
       VertexStyle -> {
        "g" -> RGBColor[1, 1, 0], "c" -> RGBColor[1, 1, 0], "f" -> 
         RGBColor[1, 1, 0], "b" -> RGBColor[0, 1, 0], "e" -> 
         RGBColor[1, 1, 0], "d" -> RGBColor[0, 1, 0], "a" -> 
         RGBColor[0, 1, 0]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-0.9749279121818235, -0.22252093395631342`}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-0.43388373911755757`, -0.9009688679024187}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.43388373911755884`, -0.9009688679024196}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.43388373911755884`, -0.9009688679024196}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.43388373911755884`, -0.9009688679024196}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9749279121818237, -0.2225209339563155}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7818314824680294, 
          0.6234898018587327}, {-7.044813998280222*^-16, 1.}}, 
          0.08677674782351154]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[0, 1, 0], 
          RectangleBox[{-0.8583992062403492, 0.5469220780864154}, \
{-0.7052637586957113, 0.7000575256310532}]}, 
         InsetBox["\<\"a\"\>", {-0.7818314824680302, 0.6234898018587343},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{-0.9749279121818235, -0.22252093395631342}, 
           0.08677674782351154]}, 
         InsetBox["\<\"g\"\>", {-0.9749279121818235, -0.22252093395631342},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{-0.43388373911755757, -0.9009688679024187}, 
           0.08677674782351154]}, 
         InsetBox["\<\"f\"\>", {-0.43388373911755757, -0.9009688679024187},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{0.43388373911755884, -0.9009688679024196}, 
           0.08677674782351154]}, 
         InsetBox["\<\"e\"\>", {0.43388373911755884, -0.9009688679024196},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 1, 0], 
          RectangleBox[{0.8983601884095048, -0.29908865772863436}, \
{1.0514956359541425, -0.14595321018399662}]}, 
         InsetBox["\<\"d\"\>", {0.9749279121818237, -0.2225209339563155},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{0.7818314824680294, 0.6234898018587327}, 
           0.08677674782351154]}, 
         InsetBox["\<\"c\"\>", {0.7818314824680294, 0.6234898018587327},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 1, 0], 
          RectangleBox[{-0.07656772377231959, 0.9234322762276811}, \
{0.07656772377231817, 1.0765677237723188}]}, 
         InsetBox["\<\"b\"\>", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[84]=",ExpressionUUID->"3782a067-4a79-4ea0-b0f3-10dff45520e0"]
}, Open  ]],

Cell[TextData[{
 "Observe that applying our function has preserved the layout of the \
original. This is typical for Wolfram Language functions that apply to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " objects. Because the options are part of the data that make up the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ", the options are passed along to any functions that act on the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"3fb073bf-817d-44e1-9acd-39dceba67b1c"],

Cell[TextData[{
 "For convenience, we define a version of ",
 StyleBox["drawBipartite", "Code"],
 " that can accept additional ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " options and pass them through our function to where we create the new ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ". The easiest way to do this is to add a ",
 StyleBox[ButtonBox["BlankNullSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankNullSequence"], "FunctionLink"],
 " (",
 StyleBox["___", "Code"],
 ") to the arguments of our function. The result is that the function \
requires a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " as the first argument, but can accept any number of additional arguments \
or no additional arguments. We then just pass these to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"a10c8912-7be5-46cc-ab27-64d825c1bf57"],

Cell[BoxData[
 RowBox[{
  RowBox[{"drawBipartite", "[", 
   RowBox[{"g_Graph", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "AB", ",", "i", ",", "T", ",", "w", ",", "e"}], "}"}], 
    ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V", "=", 
      RowBox[{"VertexList", "[", "g", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"w", "=", 
      RowBox[{"V", "[", 
       RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"AB", "[", "0", "]"}], "=", 
      RowBox[{"{", "w", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"AB", "[", "1", "]"}], "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"i", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"V", "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"T", "=", 
         RowBox[{"Intersection", "[", 
          RowBox[{"V", ",", 
           RowBox[{"AB", "[", "i", "]"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"i", "=", 
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"i", "+", "1"}], ",", "2"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"AB", "[", "i", "]"}], "=", 
           RowBox[{"Union", "[", 
            RowBox[{
             RowBox[{"AB", "[", "i", "]"}], ",", 
             RowBox[{"Complement", "[", 
              RowBox[{
               RowBox[{"neighbors", "[", 
                RowBox[{"g", ",", "w"}], "]"}], ",", 
               RowBox[{"Union", "[", 
                RowBox[{
                 RowBox[{"AB", "[", "0", "]"}], ",", 
                 RowBox[{"AB", "[", "1", "]"}]}], "]"}]}], "]"}]}], "]"}]}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"w", ",", "T"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"V", "=", 
         RowBox[{"Complement", "[", 
          RowBox[{"V", ",", "T"}], "]"}]}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "0", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "&&", 
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "0", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ")"}], "||", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "1", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "&&", 
              RowBox[{"MemberQ", "[", 
               RowBox[{
                RowBox[{"AB", "[", "1", "]"}], ",", 
                RowBox[{"e", "[", 
                 RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ")"}]}], ",", 
           RowBox[{"Throw", "[", "False", "]"}]}], "]"}], 
         "\[IndentingNewLine]", ",", 
         RowBox[{"{", 
          RowBox[{"e", ",", 
           RowBox[{"EdgeList", "[", "g", "]"}]}], "}"}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Graph", "[", 
        RowBox[{"g", ",", 
         RowBox[{"VertexStyle", "\[Rule]", 
          RowBox[{"Union", "[", 
           RowBox[{
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"i", "\[Rule]", "Green"}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", 
                RowBox[{"AB", "[", "0", "]"}]}], "}"}]}], "]"}], ",", 
            RowBox[{"Table", "[", 
             RowBox[{
              RowBox[{"i", "\[Rule]", "Yellow"}], ",", 
              RowBox[{"{", 
               RowBox[{"i", ",", 
                RowBox[{"AB", "[", "1", "]"}]}], "}"}]}], "]"}]}], "]"}]}], 
         ",", 
         RowBox[{"VertexShapeFunction", "\[Rule]", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"i", "\[Rule]", "\"\<Square\>\""}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", 
              RowBox[{"AB", "[", "0", "]"}]}], "}"}]}], "]"}]}], ",", 
         "opts"}], "]"}]}], "\[IndentingNewLine]", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[82]:=",ExpressionUUID->"962bfc11-e098-4549-a16b-b6dc27286eea"],

Cell["For example, we might want to draw the edges dashed.", "Text",ExpressionUUID->"3d37175c-6a36-4d99-be62-63140158a5e2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"drawBipartite", "[", 
  RowBox[{"figure8G", ",", 
   RowBox[{"EdgeStyle", "\[Rule]", "Dashed"}]}], "]"}]], "Input",
 CellLabel->"In[83]:=",ExpressionUUID->"ed6df31c-9bde-473e-a808-c15c905d24c3"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "g", "f", "e", "d", "c", "b"}, {
      Null, {{1, 6}, {1, 4}, {1, 3}, {1, 2}, {7, 6}, {7, 4}, {7, 3}, {6, 5}, {
       5, 4}, {5, 3}, {5, 2}}}, {EdgeStyle -> {
         Dashing[{Small, Small}]}, 
       GraphLayout -> {
        "Dimension" -> 2, "VertexLayout" -> "CircularEmbedding"}, 
       VertexLabels -> {
         Placed["Name", Center]}, 
       VertexShapeFunction -> {
        "b" -> "Square", "d" -> "Square", "a" -> "Square"}, 
       VertexSize -> {Medium}, 
       VertexStyle -> {
        "g" -> RGBColor[1, 1, 0], "c" -> RGBColor[1, 1, 0], "f" -> 
         RGBColor[1, 1, 0], "b" -> RGBColor[0, 1, 0], "e" -> 
         RGBColor[1, 1, 0], "d" -> RGBColor[0, 1, 0], "a" -> 
         RGBColor[0, 1, 0]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Dashing[{Small, Small}], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-0.9749279121818235, -0.22252093395631342`}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-0.43388373911755757`, -0.9009688679024187}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.43388373911755884`, -0.9009688679024196}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.43388373911755884`, -0.9009688679024196}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.43388373911755884`, -0.9009688679024196}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9749279121818237, -0.2225209339563155}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7818314824680294, 
          0.6234898018587327}, {-7.044813998280222*^-16, 1.}}, 
          0.08677674782351154]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[0, 1, 0], 
          RectangleBox[{-0.8583992062403492, 0.5469220780864154}, \
{-0.7052637586957113, 0.7000575256310532}]}, 
         InsetBox["\<\"a\"\>", {-0.7818314824680302, 0.6234898018587343},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{-0.9749279121818235, -0.22252093395631342}, 
           0.08677674782351154]}, 
         InsetBox["\<\"g\"\>", {-0.9749279121818235, -0.22252093395631342},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{-0.43388373911755757, -0.9009688679024187}, 
           0.08677674782351154]}, 
         InsetBox["\<\"f\"\>", {-0.43388373911755757, -0.9009688679024187},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{0.43388373911755884, -0.9009688679024196}, 
           0.08677674782351154]}, 
         InsetBox["\<\"e\"\>", {0.43388373911755884, -0.9009688679024196},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 1, 0], 
          RectangleBox[{0.8983601884095048, -0.29908865772863436}, \
{1.0514956359541425, -0.14595321018399662}]}, 
         InsetBox["\<\"d\"\>", {0.9749279121818237, -0.2225209339563155},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 1, 0], 
          DiskBox[{0.7818314824680294, 0.6234898018587327}, 
           0.08677674782351154]}, 
         InsetBox["\<\"c\"\>", {0.7818314824680294, 0.6234898018587327},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0, 1, 0], 
          RectangleBox[{-0.07656772377231959, 0.9234322762276811}, \
{0.07656772377231817, 1.0765677237723188}]}, 
         InsetBox["\<\"b\"\>", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[83]=",ExpressionUUID->"221f5285-1afa-4b01-862d-b6f0b5bac761"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bipartite Graphs and Matchings", "Subsection",
 CellTags->"Section2e",ExpressionUUID->"ea23e0b8-564d-4c56-9d82-6481f39dc5da"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can help us find maximal matchings in a bipartite graph. We will use \
Figure 10a from Section 10.2 of the text as an example. To improve \
readability, we have abbreviated the names to their first letter and \
shortened the descriptions of the jobs."
}], "Text",ExpressionUUID->"cb77e345-0a68-4984-83a7-6b7cb2553f1f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure10aEdges", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<req\>\""}], ",", 
    RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<test\>\""}], ",", 
    RowBox[{"\"\<B\>\"", "\[Rule]", "\"\<arch\>\""}], ",", 
    RowBox[{"\"\<B\>\"", "\[Rule]", "\"\<imp\>\""}], ",", 
    RowBox[{"\"\<B\>\"", "\[Rule]", "\"\<test\>\""}], ",", 
    RowBox[{"\"\<C\>\"", "\[Rule]", "\"\<req\>\""}], ",", 
    RowBox[{"\"\<C\>\"", "\[Rule]", "\"\<arch\>\""}], ",", 
    RowBox[{"\"\<C\>\"", "\[Rule]", "\"\<imp\>\""}], ",", 
    RowBox[{"\"\<D\>\"", "\[Rule]", "\"\<req\>\""}]}], "}"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[85]:=",ExpressionUUID->"26371fa4-e45a-4a41-8870-d38ead04b79c"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"A\"\>", "\[Rule]", "\<\"req\"\>"}], ",", 
   RowBox[{"\<\"A\"\>", "\[Rule]", "\<\"test\"\>"}], ",", 
   RowBox[{"\<\"B\"\>", "\[Rule]", "\<\"arch\"\>"}], ",", 
   RowBox[{"\<\"B\"\>", "\[Rule]", "\<\"imp\"\>"}], ",", 
   RowBox[{"\<\"B\"\>", "\[Rule]", "\<\"test\"\>"}], ",", 
   RowBox[{"\<\"C\"\>", "\[Rule]", "\<\"req\"\>"}], ",", 
   RowBox[{"\<\"C\"\>", "\[Rule]", "\<\"arch\"\>"}], ",", 
   RowBox[{"\<\"C\"\>", "\[Rule]", "\<\"imp\"\>"}], ",", 
   RowBox[{"\<\"D\"\>", "\[Rule]", "\<\"req\"\>"}]}], "}"}]], "Output",
 CellLabel->"Out[85]=",ExpressionUUID->"75a0d48c-95e6-4ed8-989e-2626e99c297f"]
}, Open  ]],

Cell[TextData[{
 "In order to draw the graph meaningfully, in the same fashion as in the \
text, we will specify the coordinates of each vertex. This is done by setting \
the ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " option to a list of coordinates, with the order of the list matching the \
order of the vertices. To ensure that our order is correct, we will specify a \
vertex list when we create the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"4192d0d7-7f36-48f6-94e9-6eafd9b13ddf"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure10aVertices", "=", 
  RowBox[{"{", 
   RowBox[{
   "\"\<A\>\"", ",", "\"\<B\>\"", ",", "\"\<C\>\"", ",", "\"\<D\>\"", ",", 
    "\"\<req\>\"", ",", "\"\<test\>\"", ",", "\"\<arch\>\"", ",", 
    "\"\<imp\>\""}], "}"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[86]:=",ExpressionUUID->"589ba845-5ffb-4805-9d8d-29eb66b0eaf0"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"A\"\>", ",", "\<\"B\"\>", ",", "\<\"C\"\>", ",", "\<\"D\"\>", 
   ",", "\<\"req\"\>", ",", "\<\"test\"\>", ",", "\<\"arch\"\>", 
   ",", "\<\"imp\"\>"}], "}"}]], "Output",
 CellLabel->"Out[86]=",ExpressionUUID->"64854585-9c54-404f-ad7e-30dea681408d"]
}, Open  ]],

Cell[TextData[{
 "To create the list of coordinates, we use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " twice and specify that the names should have ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]],ExpressionUUID->
  "90fe6e69-596a-4595-b9b6-a70e440e1582"],
 "-coordinate 1 and the jobs should have ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]],ExpressionUUID->
  "bd6439c6-9c3b-4d7c-b25c-34242ff2796d"],
 "-coordinate 0. ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " is used to combine the two lists."
}], "Text",ExpressionUUID->"316bf266-3f33-43d2-8bd9-06145dec687a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure10aCoordinates", "=", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "4"}], "}"}]}], "]"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"i", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "4"}], "}"}]}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[87]:=",ExpressionUUID->"25d21c98-65da-41da-b557-1023cd0234dc"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "0"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "0"}], "}"}]}], "}"}]], "Output",
 CellLabel->"Out[87]=",ExpressionUUID->"76150600-37db-412c-a006-ae5806c7f9ae"]
}, Open  ]],

Cell["We now create the graph.", "Text",ExpressionUUID->"7d9c328b-7478-4635-92df-d7cbebefab9f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure10a", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{"figure10aVertices", ",", "figure10aEdges", ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Before"}], "]"}]}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", "figure10aCoordinates"}]}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[88]:=",ExpressionUUID->"8e306cb2-588d-48c5-afc5-84eaf2a83b42"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"A", "B", "C", "D", "req", "test", "arch", "imp"}, {
      Null, {{1, 5}, {1, 6}, {2, 7}, {2, 8}, {2, 6}, {3, 5}, {3, 7}, {3, 8}, {
       4, 5}}}, {VertexLabels -> {
         Placed["Name", Before]}, 
       VertexCoordinates -> {{1, 1}, {2, 1}, {3, 1}, {4, 1}, {1, 0}, {2, 0}, {
        3, 0}, {4, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{1., 1.}, {1., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 1.}, {2., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 1.}, {2., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 1.}, {3., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 1.}, {4., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 1.}, {1., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 1.}, {3., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 1.}, {4., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{4., 1.}, {1., 0.}}, 0.030239520958083826`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{1., 1.}, 0.030239520958083826], 
         InsetBox["\<\"A\"\>", Offset[{-2, 0}, {0.9697604790419162, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.030239520958083826], 
         InsetBox["\<\"B\"\>", Offset[{-2, 0}, {1.969760479041916, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 1.}, 0.030239520958083826], 
         InsetBox["\<\"C\"\>", Offset[{-2, 0}, {2.9697604790419163, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{4., 1.}, 0.030239520958083826], 
         InsetBox["\<\"D\"\>", Offset[{-2, 0}, {3.9697604790419163, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.030239520958083826], 
         InsetBox["\<\"req\"\>", Offset[{-2, 0}, {0.9697604790419162, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.030239520958083826], 
         InsetBox["\<\"test\"\>", Offset[{-2, 0}, {1.969760479041916, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.030239520958083826], 
         InsetBox["\<\"arch\"\>", Offset[{-2, 0}, {2.9697604790419163, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{4., 0.}, 0.030239520958083826], 
         InsetBox["\<\"imp\"\>", Offset[{-2, 0}, {3.9697604790419163, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[88]=",ExpressionUUID->"465b4121-9b47-4307-bea9-7633e3b6dec1"]
}, Open  ]],

Cell[TextData[{
 "To find a maximal matching, we use the function ",
 StyleBox[ButtonBox["FindIndependentEdgeSet",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindIndependentEdgeSet"], "FunctionLink"],
 ". The term ",
 StyleBox["independent edge set",
  FontSlant->"Italic"],
 " is synonymous with matching. The only allowed argument to this function is \
the graph. It returns a list of edges in a matching."
}], "Text",ExpressionUUID->"e09b8db1-6188-4de2-87be-54d364ce1b26"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure10aMatching", "=", 
  RowBox[{"FindIndependentEdgeSet", "[", "figure10a", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[89]:=",ExpressionUUID->"f8f0a348-55c1-4f5e-9c12-b9b385ef422d"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"A\"\>", "\[UndirectedEdge]", "\<\"test\"\>"}], ",", 
   RowBox[{"\<\"B\"\>", "\[UndirectedEdge]", "\<\"imp\"\>"}], ",", 
   RowBox[{"\<\"C\"\>", "\[UndirectedEdge]", "\<\"arch\"\>"}], ",", 
   RowBox[{"\<\"D\"\>", "\[UndirectedEdge]", "\<\"req\"\>"}]}], 
  "}"}]], "Output",
 CellLabel->"Out[89]=",ExpressionUUID->"8c442b01-2468-46fc-8018-f10ea7b02db3"]
}, Open  ]],

Cell[TextData[{
 "The output indicates that one maximal matching has Alvarez assigned to \
testing, ",
 StyleBox["Berkowitz",
  IgnoreSpellCheck->True],
 " to implementation, Chen to architecture, and Davis to requirements."
}], "Text",ExpressionUUID->"e9e55cb3-1e45-4011-bb11-b8917da180cd"],

Cell[TextData[{
 "We can visualize this matching by highlighting the edges that form the \
matching using the function ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 ". This function requires two arguments. The first is a graph. The second is \
a list of the elements to highlight. In this case, the second argument will \
be the output from ",
 StyleBox[ButtonBox["FindIndependentEdgeSet",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindIndependentEdgeSet"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"d2cd514e-95c2-4a92-8e09-0940ca301932"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"figure10a", ",", "figure10aMatching"}], "]"}]], "Input",
 CellLabel->"In[90]:=",ExpressionUUID->"e6630359-f588-4e47-bf83-ed6a50503a58"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"A", "B", "C", "D", "req", "test", "arch", "imp"}, {
      Null, {{1, 5}, {1, 6}, {2, 7}, {2, 8}, {2, 6}, {3, 5}, {3, 7}, {3, 8}, {
       4, 5}}}, {VertexLabels -> {
         Placed["Name", Before]}, GraphHighlight -> {
         UndirectedEdge["D", "req"], 
         UndirectedEdge["C", "arch"], 
         UndirectedEdge["B", "imp"], 
         UndirectedEdge["A", "test"]}, 
       VertexCoordinates -> {{1, 1}, {2, 1}, {3, 1}, {4, 1}, {1, 0}, {2, 0}, {
        3, 0}, {4, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{1., 1.}, {1., 0.}}, 0.030239520958083826`]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{1., 1.}, {2., 0.}}, 0.030239520958083826`]}}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 1.}, {2., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 1.}, {3., 0.}}, 0.030239520958083826`]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{2., 1.}, {4., 0.}}, 0.030239520958083826`]}}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 1.}, {1., 0.}}, 0.030239520958083826`]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{3., 1.}, {3., 0.}}, 0.030239520958083826`]}}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 1.}, {4., 0.}}, 0.030239520958083826`]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{4., 1.}, {1., 0.}}, 0.030239520958083826`]}}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{1., 1.}, 0.030239520958083826], 
         InsetBox["\<\"A\"\>", Offset[{-2, 0}, {0.9697604790419162, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.030239520958083826], 
         InsetBox["\<\"B\"\>", Offset[{-2, 0}, {1.969760479041916, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 1.}, 0.030239520958083826], 
         InsetBox["\<\"C\"\>", Offset[{-2, 0}, {2.9697604790419163, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{4., 1.}, 0.030239520958083826], 
         InsetBox["\<\"D\"\>", Offset[{-2, 0}, {3.9697604790419163, 1.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.030239520958083826], 
         InsetBox["\<\"req\"\>", Offset[{-2, 0}, {0.9697604790419162, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.030239520958083826], 
         InsetBox["\<\"test\"\>", Offset[{-2, 0}, {1.969760479041916, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.030239520958083826], 
         InsetBox["\<\"arch\"\>", Offset[{-2, 0}, {2.9697604790419163, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}, {DiskBox[{4., 0.}, 0.030239520958083826], 
         InsetBox["\<\"imp\"\>", Offset[{-2, 0}, {3.9697604790419163, 0.}], 
          ImageScaled[{1, 0.5}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[90]=",ExpressionUUID->"a54e86f4-3191-4f4d-b561-24126e251277"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Subgraphs and Induced Subgraphs", "Subsection",
 CellTags->"Section2f",ExpressionUUID->"e237cf2e-31dc-49d5-b7ea-299bee614096"],

Cell[TextData[{
 "The Wolfram Language provides the ",
 StyleBox[ButtonBox["Subgraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subgraph"], "FunctionLink"],
 " function for producing a subgraph from an existing graph. Given a graph \
and a list of edges, ",
 StyleBox[ButtonBox["Subgraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subgraph"], "FunctionLink"],
 " produces the ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " consisting of the edges and all the vertices that are an endpoint of one \
of the given edges."
}], "Text",ExpressionUUID->"f8b3b7e7-12e2-42e4-bd91-a667d63d9e6b"],

Cell[TextData[{
 "For example, we create the circulant graph with 6 vertices and jump 2, and \
then create the subgraph consisting of one of triangles. A circulant graph on \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "f54bdaf9-d0b8-417b-a604-f5d4317945bb"],
 " vertices with jump ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]],ExpressionUUID->
  "3b2297a3-b8c1-4bcd-80c4-61da5ce87c1f"],
 " is a graph with vertices ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]],ExpressionUUID->"c719b344-3a3d-4d91-9f95-57be45c6b453"],
 ", typically arranged in a circle, and such that vertex ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "fbf00d00-1671-4088-a4ec-7b941d97bb36"],
 " is connected to vertex ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "3c63b1dc-1087-462b-9df8-98f862a28a36"],
 " whenever ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[NotEqual]", "v"}], TraditionalForm]],ExpressionUUID->
  "c67fc2df-0d0f-4efa-8a60-12e19e78e305"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"u", "\[Congruent]", 
    RowBox[{"v", " ", 
     RowBox[{"(", 
      RowBox[{"mod", " ", "j"}], ")"}]}]}], TraditionalForm]],ExpressionUUID->
  "9e088792-eb41-43fc-bb49-11c41a5a94ff"],
 ". Visually, each vertex is connected to the vertex ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]],ExpressionUUID->
  "842a4a3d-ae0a-49e6-b290-5c2eca43fa68"],
 " further around the circle."
}], "Text",ExpressionUUID->"b6bb0be4-7769-48fe-b87a-57e29bd22d26"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"star", "=", 
  RowBox[{"CirculantGraph", "[", 
   RowBox[{"6", ",", "2", ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[91]:=",ExpressionUUID->"feef7aca-f227-4973-b5a7-e1e6822ed40d"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{1, 3}, {1, 5}, {2, 4}, {2, 6}, {3, 5}, {4, 6}}}, {
      GraphLayout -> {
        "VertexLayout" -> "CircularEmbedding", "PackingLayout" -> None}, 
       VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {0.8660254037844386, 
          0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
          0.02261146496815286], 
         InsetBox["1", 
          Offset[{2, 2}, {-0.843413938816286, 0.5226114649681535}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.02261146496815286], 
         InsetBox["2", 
          Offset[{2, 2}, {-0.8434139388162856, -0.47738853503184653}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.8285686989269494*^-16, -1.}, 0.02261146496815286], 
         InsetBox["3", 
          Offset[{2, 2}, {0.022611464968153243, -0.9773885350318472}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.02261146496815286], 
         InsetBox["4", 
          Offset[{2, 2}, {0.8886368687525918, -0.47738853503184836}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844386, 0.4999999999999993}, 
          0.02261146496815286], 
         InsetBox["5", 
          Offset[{2, 2}, {0.8886368687525914, 0.5226114649681521}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.02261146496815286], 
         InsetBox["6", 
          Offset[{2, 2}, {0.022611464968153045, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[91]=",ExpressionUUID->"a5c9876c-1ad6-46aa-b319-b4e8a50028bc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"substar", "=", 
  RowBox[{"Subgraph", "[", 
   RowBox[{"star", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "1"}]}], "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[92]:=",ExpressionUUID->"da90dae7-d192-4aa0-b40c-44ddbd02666c"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 3, 5}, {Null, {{1, 2}, {1, 3}, {2, 3}}}, {
      VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 
          0.020399597244776385`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          1.8369701987210297`*^-16, 1.}}, 0.020399597244776385`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          1.8369701987210297`*^-16, 1.}}, 0.020399597244776385`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.020399597244776385], 
         InsetBox["1", 
          Offset[{2, 2}, {-0.845625806539662, -0.479600402755223}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.020399597244776385], 
         InsetBox["3", 
          Offset[{2, 2}, {0.8864250010292153, -0.47960040275522486}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.020399597244776385], 
         InsetBox["5", 
          Offset[{2, 2}, {0.02039959724477657, 1.0203995972447764}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[92]=",ExpressionUUID->"e21d4cb8-a8c7-4bea-b449-05a73206ee1e"]
}, Open  ]],

Cell[TextData[{
 "Alternatively, you can give a list of vertices as the second argument to ",
 StyleBox[ButtonBox["Subgraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subgraph"], "FunctionLink"],
 ". The result is the graph induced by the given vertices, that is, the graph \
consisting of the vertices and all the edges from the original graph with \
both endpoints in the set of vertices. Below we consider a prism graph and \
one of its layers."
}], "Text",ExpressionUUID->"afdcd814-f87a-4d2b-aab1-72d21385974c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"prism", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "4"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}], ",", 
      RowBox[{"6", "\[Rule]", "1"}], ",", 
      RowBox[{"7", "\[Rule]", "8"}], ",", 
      RowBox[{"8", "\[Rule]", "9"}], ",", 
      RowBox[{"9", "\[Rule]", "10"}], ",", 
      RowBox[{"10", "\[Rule]", "11"}], ",", 
      RowBox[{"11", "\[Rule]", "12"}], ",", 
      RowBox[{"12", "\[Rule]", "7"}], ",", 
      RowBox[{"1", "\[Rule]", "7"}], ",", 
      RowBox[{"2", "\[Rule]", "8"}], ",", 
      RowBox[{"3", "\[Rule]", "9"}], ",", 
      RowBox[{"4", "\[Rule]", "10"}], ",", 
      RowBox[{"5", "\[Rule]", "11"}], ",", 
      RowBox[{"6", "\[Rule]", "12"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[93]:=",ExpressionUUID->"90a7357d-fcbe-4616-8a13-3976f7702ac7"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}, {
      Null, {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 1}, {7, 8}, {8, 9}, {
       9, 10}, {10, 11}, {11, 12}, {12, 7}, {1, 7}, {2, 8}, {3, 9}, {4, 10}, {
       5, 11}, {6, 12}}}, {VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          0.9839435806937917, 2.4015105280929165`}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          1.199176045955087, 0.4394265853678305}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {0., 
          0.9620430219770666}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          2.099708266496932, 2.3764856914994263`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          0.39587622224175434`, 2.005570642165643}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          2.7301971795384574`, 1.4401369864781073`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.333865160413755, 0.39609325735504175`}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.199176045955087, 0.4394265853678305}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.746198062974075, 0.}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.199176045955087, 0.4394265853678305}, {
          0.6298912750595992, 0.025825868311962763`}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {0.39587622224175434`, 
          2.005570642165643}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {0.6298912750595992, 
          0.025825868311962763`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5313638191412857`, 1.9620109994805106`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1323633389347414`, 1.0677650125596039`}, {
          1.746198062974075, 0.}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.746198062974075, 0.}, {0.6298912750595992, 
          0.025825868311962763`}}, 0.028272915525887662`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.5983109393792555, 1.333510780126811}, 
          0.028272915525887662], 
         InsetBox["1", 
          Offset[{2, 2}, {0.6265838549051431, 1.3617836956526985}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9839435806937917, 2.4015105280929165}, 
          0.028272915525887662], 
         InsetBox["2", 
          Offset[{2, 2}, {1.0122164962196794, 2.4297834436188044}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.099708266496932, 2.3764856914994263}, 
          0.028272915525887662], 
         InsetBox["3", Offset[{2, 2}, {2.12798118202282, 2.404758607025314}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.7301971795384574, 1.4401369864781073}, 
          0.028272915525887662], 
         InsetBox["4", 
          Offset[{2, 2}, {2.7584700950643453, 1.468409902003995}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.333865160413755, 0.39609325735504175}, 
          0.028272915525887662], 
         InsetBox["5", 
          Offset[{2, 2}, {2.362138075939643, 0.4243661728809294}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.199176045955087, 0.4394265853678305}, 
          0.028272915525887662], 
         InsetBox["6", 
          Offset[{2, 2}, {1.2274489614809747, 0.4676995008937182}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.9620430219770666}, 0.028272915525887662], 
         InsetBox["7", 
          Offset[{2, 2}, {0.028272915525887662, 0.9903159375029542}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.39587622224175434, 2.005570642165643}, 
          0.028272915525887662], 
         InsetBox["8", Offset[{2, 2}, {0.424149137767642, 2.033843557691531}],
           ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5313638191412857, 1.9620109994805106}, 
          0.028272915525887662], 
         InsetBox["9", 
          Offset[{2, 2}, {1.5596367346671733, 1.9902839150063982}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.1323633389347414, 1.0677650125596039}, 
          0.028272915525887662], 
         InsetBox["10", 
          Offset[{2, 2}, {2.1606362544606292, 1.0960379280854915}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.746198062974075, 0.}, 0.028272915525887662], 
         InsetBox["11", 
          Offset[{2, 2}, {1.7744709784999626, 0.028272915525887662}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6298912750595992, 0.025825868311962763}, 
          0.028272915525887662], 
         InsetBox["12", 
          Offset[{2, 2}, {0.6581641905854868, 0.054098783837850425}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[93]=",ExpressionUUID->"fb9207c2-49c0-4a56-8c22-aec5c8ddac9f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subprism", "=", 
  RowBox[{"Subgraph", "[", 
   RowBox[{"prism", ",", 
    RowBox[{"{", 
     RowBox[{"7", ",", "8", ",", "9", ",", "10", ",", "11", ",", "12"}], 
     "}"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[94]:=",ExpressionUUID->"689c71ee-94dd-48f3-abfd-70d6a1f67225"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{7, 8, 9, 10, 11, 12}, {
      Null, {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 1}}}, {
      VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 
          0.5000000000000007}, {-0.8660254037844384, -0.4999999999999994}}, 
          0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844386, 0.4999999999999993}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
          0.02261146496815286], 
         InsetBox["7", 
          Offset[{2, 2}, {-0.843413938816286, 0.5226114649681535}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.02261146496815286], 
         InsetBox["8", 
          Offset[{2, 2}, {-0.8434139388162856, -0.47738853503184653}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.8285686989269494*^-16, -1.}, 0.02261146496815286], 
         InsetBox["9", 
          Offset[{2, 2}, {0.022611464968153243, -0.9773885350318472}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.02261146496815286], 
         InsetBox["10", 
          Offset[{2, 2}, {0.8886368687525918, -0.47738853503184836}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844386, 0.4999999999999993}, 
          0.02261146496815286], 
         InsetBox["11", 
          Offset[{2, 2}, {0.8886368687525914, 0.5226114649681521}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.02261146496815286], 
         InsetBox["12", 
          Offset[{2, 2}, {0.022611464968153045, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[94]=",ExpressionUUID->"8ec80807-7177-49d6-afac-0a0caa257a8d"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 " function can be used to display a subgraph relative to the original by \
passing the original graph as the first argument and the subgraph as the \
second. "
}], "Text",ExpressionUUID->"03666e72-f925-402b-99bb-9ce013825b5e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"star", ",", "substar"}], "]"}]], "Input",
 CellLabel->"In[95]:=",ExpressionUUID->"a8613240-9611-467c-accf-af81b6f0e9fe"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{1, 3}, {1, 5}, {2, 4}, {2, 6}, {3, 5}, {4, 6}}}, {
      GraphHighlight -> {
         UndirectedEdge[1, 5], 1, 
         UndirectedEdge[3, 5], 5, 
         UndirectedEdge[1, 3], 3}, 
       GraphLayout -> {
        "VertexLayout" -> "CircularEmbedding", "PackingLayout" -> None}, 
       VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
           3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
           0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{3.8285686989269494`*^-16, -1.}, {0.8660254037844386, 
           0.4999999999999993}}, 0.02261146496815286]}}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
           0.02261146496815286]}, 
         InsetBox["1", 
          Offset[{2, 2}, {-0.843413938816286, 0.5226114649681535}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.02261146496815286], 
         InsetBox["2", 
          Offset[{2, 2}, {-0.8434139388162856, -0.47738853503184653}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{3.8285686989269494*^-16, -1.}, 0.02261146496815286]}, 
         InsetBox["3", 
          Offset[{2, 2}, {0.022611464968153243, -0.9773885350318472}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.02261146496815286], 
         InsetBox["4", 
          Offset[{2, 2}, {0.8886368687525918, -0.47738853503184836}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0.8660254037844386, 0.4999999999999993}, 
           0.02261146496815286]}, 
         InsetBox["5", 
          Offset[{2, 2}, {0.8886368687525914, 0.5226114649681521}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.02261146496815286], 
         InsetBox["6", 
          Offset[{2, 2}, {0.022611464968153045, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[95]=",ExpressionUUID->"89c90d2f-02c8-4629-95f5-2182e6ff5b00"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"prism", ",", "subprism"}], "]"}]], "Input",
 CellLabel->"In[96]:=",ExpressionUUID->"84a4c2b5-9f75-4d8e-a017-902c64a99610"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}, {
      Null, {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 1}, {7, 8}, {8, 9}, {
       9, 10}, {10, 11}, {11, 12}, {12, 7}, {1, 7}, {2, 8}, {3, 9}, {4, 10}, {
       5, 11}, {6, 12}}}, {VertexLabels -> {"Name"}, GraphHighlight -> {10, 
         UndirectedEdge[12, 7], 
         UndirectedEdge[8, 9], 
         UndirectedEdge[11, 12], 9, 
         UndirectedEdge[9, 10], 11, 12, 
         UndirectedEdge[10, 11], 
         UndirectedEdge[7, 8], 7, 8}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          0.9839435806937917, 2.4015105280929165`}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          1.199176045955087, 0.4394265853678305}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {0., 
          0.9620430219770666}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          2.099708266496932, 2.3764856914994263`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          0.39587622224175434`, 2.005570642165643}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          2.7301971795384574`, 1.4401369864781073`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.333865160413755, 0.39609325735504175`}}, 
          0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.199176045955087, 0.4394265853678305}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.746198062974075, 0.}}, 0.028272915525887662`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.199176045955087, 0.4394265853678305}, {
          0.6298912750595992, 0.025825868311962763`}}, 
          0.028272915525887662`]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{0., 0.9620430219770666}, {0.39587622224175434`, 
           2.005570642165643}}, 0.028272915525887662`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{0., 0.9620430219770666}, {0.6298912750595992, 
           0.025825868311962763`}}, 0.028272915525887662`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
           1.5313638191412857`, 1.9620109994805106`}}, 
           0.028272915525887662`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{1.5313638191412857`, 1.9620109994805106`}, {
           2.1323633389347414`, 1.0677650125596039`}}, 
           0.028272915525887662`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{2.1323633389347414`, 1.0677650125596039`}, {
           1.746198062974075, 0.}}, 0.028272915525887662`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{1.746198062974075, 0.}, {0.6298912750595992, 
           0.025825868311962763`}}, 0.028272915525887662`]}}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.5983109393792555, 1.333510780126811}, 
          0.028272915525887662], 
         InsetBox["1", 
          Offset[{2, 2}, {0.6265838549051431, 1.3617836956526985}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9839435806937917, 2.4015105280929165}, 
          0.028272915525887662], 
         InsetBox["2", 
          Offset[{2, 2}, {1.0122164962196794, 2.4297834436188044}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.099708266496932, 2.3764856914994263}, 
          0.028272915525887662], 
         InsetBox["3", Offset[{2, 2}, {2.12798118202282, 2.404758607025314}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.7301971795384574, 1.4401369864781073}, 
          0.028272915525887662], 
         InsetBox["4", 
          Offset[{2, 2}, {2.7584700950643453, 1.468409902003995}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.333865160413755, 0.39609325735504175}, 
          0.028272915525887662], 
         InsetBox["5", 
          Offset[{2, 2}, {2.362138075939643, 0.4243661728809294}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.199176045955087, 0.4394265853678305}, 
          0.028272915525887662], 
         InsetBox["6", 
          Offset[{2, 2}, {1.2274489614809747, 0.4676995008937182}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0., 0.9620430219770666}, 0.028272915525887662]}, 
         InsetBox["7", 
          Offset[{2, 2}, {0.028272915525887662, 0.9903159375029542}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0.39587622224175434, 2.005570642165643}, 
           0.028272915525887662]}, 
         InsetBox["8", Offset[{2, 2}, {0.424149137767642, 2.033843557691531}],
           ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{1.5313638191412857, 1.9620109994805106}, 
           0.028272915525887662]}, 
         InsetBox["9", 
          Offset[{2, 2}, {1.5596367346671733, 1.9902839150063982}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{2.1323633389347414, 1.0677650125596039}, 
           0.028272915525887662]}, 
         InsetBox["10", 
          Offset[{2, 2}, {2.1606362544606292, 1.0960379280854915}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{1.746198062974075, 0.}, 0.028272915525887662]}, 
         InsetBox["11", 
          Offset[{2, 2}, {1.7744709784999626, 0.028272915525887662}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0.6298912750595992, 0.025825868311962763}, 
           0.028272915525887662]}, 
         InsetBox["12", 
          Offset[{2, 2}, {0.6581641905854868, 0.054098783837850425}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[96]=",ExpressionUUID->"bb208353-dd5e-4597-b30b-67c5591e35a4"]
}, Open  ]],

Cell[TextData[{
 "Note that the second argument of ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 " can be a list of subgraphs. Below, we highlight the three triangles in the \
circulant graph on 9 vertices with jump 3."
}], "Text",ExpressionUUID->"d1d07186-8a21-4c70-91f6-2bc5785a085f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"circulant9", "=", 
  RowBox[{"CirculantGraph", "[", 
   RowBox[{"9", ",", "3", ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]}]], "Input",
 CellLabel->"In[97]:=",ExpressionUUID->"37934446-a773-42a1-ab72-4996500ee45e"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9}, {
      Null, {{1, 4}, {1, 7}, {2, 5}, {2, 8}, {3, 6}, {3, 9}, {4, 7}, {5, 8}, {
       6, 9}}}, {
      GraphLayout -> {
        "VertexLayout" -> "CircularEmbedding", "PackingLayout" -> None}, 
       VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.6427876096865399, 
          0.7660444431189786}, {-0.3420201433256684, -0.9396926207859079}}, 
          0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.6427876096865399, 0.7660444431189786}, {
          0.984807753012208, 0.17364817766692903`}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9848077530122081, 0.17364817766693114`}, {
          0.34202014332566916`, -0.9396926207859085}}, 
          0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9848077530122081, 0.17364817766693114`}, {
          0.6427876096865389, 0.7660444431189772}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, \
{-7.044813998280222*^-16, 1.}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.3420201433256684, -0.9396926207859079}, {
          0.984807753012208, 0.17364817766692903`}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.34202014332566916`, -0.9396926207859085}, {
          0.6427876096865389, 0.7660444431189772}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.8660254037844389, -0.5000000000000012}, {-7.044813998280222*^-16, 
          1.}}, 0.06840402866513366]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.6427876096865399, 0.7660444431189786}, 
          0.06840402866513366], 
         InsetBox["1", {-0.6427876096865399, 0.7660444431189786},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9848077530122081, 0.17364817766693114}, 
          0.06840402866513366], 
         InsetBox["2", {-0.9848077530122081, 0.17364817766693114},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.06840402866513366], 
         InsetBox["3", {-0.8660254037844384, -0.4999999999999994},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.3420201433256684, -0.9396926207859079}, 
          0.06840402866513366], 
         InsetBox["4", {-0.3420201433256684, -0.9396926207859079},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.34202014332566916, -0.9396926207859085}, 
          0.06840402866513366], 
         InsetBox["5", {0.34202014332566916, -0.9396926207859085},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.06840402866513366], 
         InsetBox["6", {0.8660254037844389, -0.5000000000000012},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.984807753012208, 0.17364817766692903}, 
          0.06840402866513366], 
         InsetBox["7", {0.984807753012208, 0.17364817766692903},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6427876096865389, 0.7660444431189772}, 
          0.06840402866513366], 
         InsetBox["8", {0.6427876096865389, 0.7660444431189772},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.06840402866513366], 
         InsetBox["9", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[97]=",ExpressionUUID->"49579771-c43d-4042-bfd5-e767c655553c"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"circulant9", ",", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Subgraph", "[", 
      RowBox[{"circulant9", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"i", "+", "3"}], ",", 
         RowBox[{"i", "+", "6"}]}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "3"}], "}"}]}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[98]:=",ExpressionUUID->"a67bcf12-2a8e-4a85-897c-0f00cc6ca563"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9}, {
      Null, {{1, 4}, {1, 7}, {2, 5}, {2, 8}, {3, 6}, {3, 9}, {4, 7}, {5, 8}, {
       6, 9}}}, {GraphHighlight -> {
         UndirectedEdge[5, 8], 
         UndirectedEdge[3, 9], 
         UndirectedEdge[4, 7], 1, 6, 5, 9, 
         UndirectedEdge[3, 6], 
         UndirectedEdge[1, 7], 
         UndirectedEdge[1, 4], 
         UndirectedEdge[6, 9], 2, 7, 8, 
         UndirectedEdge[2, 8], 3, 
         UndirectedEdge[2, 5], 4}, 
       GraphHighlightStyle -> {
        UndirectedEdge[2, 8] -> {EdgeStyle -> Hue[0.14, 1, 0.9]}, 
         UndirectedEdge[3, 6] -> {EdgeStyle -> Hue[0.8, 0.6, 0.8]}, 
         4 -> {VertexStyle -> Hue[0, 1, 0.8]}, 
         UndirectedEdge[1, 7] -> {EdgeStyle -> Hue[0, 1, 0.8]}, 
         UndirectedEdge[6, 9] -> {EdgeStyle -> Hue[0.8, 0.6, 0.8]}, 
         1 -> {VertexStyle -> Hue[0, 1, 0.8]}, 
         UndirectedEdge[4, 7] -> {EdgeStyle -> Hue[0, 1, 0.8]}, 
         2 -> {VertexStyle -> Hue[0.14, 1, 0.9]}, 
         UndirectedEdge[5, 8] -> {EdgeStyle -> Hue[0.14, 1, 0.9]}, 
         UndirectedEdge[1, 4] -> {EdgeStyle -> Hue[0, 1, 0.8]}, 
         3 -> {VertexStyle -> Hue[0.8, 0.6, 0.8]}, 
         5 -> {VertexStyle -> Hue[0.14, 1, 0.9]}, 
         9 -> {VertexStyle -> Hue[0.8, 0.6, 0.8]}, 
         6 -> {VertexStyle -> Hue[0.8, 0.6, 0.8]}, 
         UndirectedEdge[3, 9] -> {EdgeStyle -> Hue[0.8, 0.6, 0.8]}, 
         UndirectedEdge[2, 5] -> {EdgeStyle -> Hue[0.14, 1, 0.9]}, 
         8 -> {VertexStyle -> Hue[0.14, 1, 0.9]}, 
         7 -> {VertexStyle -> Hue[0, 1, 0.8]}}, 
       GraphLayout -> {
        "VertexLayout" -> "CircularEmbedding", "PackingLayout" -> None}, 
       VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Hue[0, 1, 0.8], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.6427876096865399, 
           0.7660444431189786}, {-0.3420201433256684, -0.9396926207859079}}, 
           0.06840402866513366]}}, 
        {Hue[0, 1, 0.8], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.6427876096865399, 0.7660444431189786}, {
           0.984807753012208, 0.17364817766692903`}}, 
           0.06840402866513366]}}, 
        {Hue[0.14, 1, 0.9], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9848077530122081, 0.17364817766693114`}, {
           0.34202014332566916`, -0.9396926207859085}}, 
           0.06840402866513366]}}, 
        {Hue[0.14, 1, 0.9], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9848077530122081, 0.17364817766693114`}, {
           0.6427876096865389, 0.7660444431189772}}, 0.06840402866513366]}}, 
        {Hue[0.8, 0.6, 0.8], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}}, 
           0.06840402866513366]}}, 
        {Hue[0.8, 0.6, 0.8], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, \
{-7.044813998280222*^-16, 1.}}, 0.06840402866513366]}}, 
        {Hue[0, 1, 0.8], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.3420201433256684, -0.9396926207859079}, {
           0.984807753012208, 0.17364817766692903`}}, 
           0.06840402866513366]}}, 
        {Hue[0.14, 1, 0.9], 
         {Arrowheads[0.], 
          ArrowBox[{{0.34202014332566916`, -0.9396926207859085}, {
           0.6427876096865389, 0.7660444431189772}}, 0.06840402866513366]}}, 
        {Hue[0.8, 0.6, 0.8], 
         {Arrowheads[0.], 
          ArrowBox[{{
           0.8660254037844389, -0.5000000000000012}, {-7.044813998280222*^-16,
            1.}}, 0.06840402866513366]}}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {Hue[0, 1, 0.8], 
          DiskBox[{-0.6427876096865399, 0.7660444431189786}, 
           0.06840402866513366]}, 
         InsetBox["1", {-0.6427876096865399, 0.7660444431189786},
          BaseStyle->"Graphics"]}, {
         {Hue[0.14, 1, 0.9], 
          DiskBox[{-0.9848077530122081, 0.17364817766693114}, 
           0.06840402866513366]}, 
         InsetBox["2", {-0.9848077530122081, 0.17364817766693114},
          BaseStyle->"Graphics"]}, {
         {Hue[0.8, 0.6, 0.8], 
          DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
           0.06840402866513366]}, 
         InsetBox["3", {-0.8660254037844384, -0.4999999999999994},
          BaseStyle->"Graphics"]}, {
         {Hue[0, 1, 0.8], 
          DiskBox[{-0.3420201433256684, -0.9396926207859079}, 
           0.06840402866513366]}, 
         InsetBox["4", {-0.3420201433256684, -0.9396926207859079},
          BaseStyle->"Graphics"]}, {
         {Hue[0.14, 1, 0.9], 
          DiskBox[{0.34202014332566916, -0.9396926207859085}, 
           0.06840402866513366]}, 
         InsetBox["5", {0.34202014332566916, -0.9396926207859085},
          BaseStyle->"Graphics"]}, {
         {Hue[0.8, 0.6, 0.8], 
          DiskBox[{0.8660254037844389, -0.5000000000000012}, 
           0.06840402866513366]}, 
         InsetBox["6", {0.8660254037844389, -0.5000000000000012},
          BaseStyle->"Graphics"]}, {
         {Hue[0, 1, 0.8], 
          DiskBox[{0.984807753012208, 0.17364817766692903}, 
           0.06840402866513366]}, 
         InsetBox["7", {0.984807753012208, 0.17364817766692903},
          BaseStyle->"Graphics"]}, {
         {Hue[0.14, 1, 0.9], 
          DiskBox[{0.6427876096865389, 0.7660444431189772}, 
           0.06840402866513366]}, 
         InsetBox["8", {0.6427876096865389, 0.7660444431189772},
          BaseStyle->"Graphics"]}, {
         {Hue[0.8, 0.6, 0.8], 
          DiskBox[{-7.044813998280222*^-16, 1.}, 0.06840402866513366]}, 
         InsetBox["9", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->"Out[98]=",ExpressionUUID->"170c4c52-c73d-40d5-90f9-4fda26d9ef48"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Deleting Vertices and Edges", "Subsection",
 PageBreakWithin->False,
 CellTags->"Section2g",ExpressionUUID->"f15c3815-6b55-43f0-a8c0-a3e9d8e3122a"],

Cell[TextData[{
 "Subgraphs can also be produced by deleting vertices or edges. The ",
 StyleBox[ButtonBox["VertexDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDelete"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["EdgeDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeDelete"], "FunctionLink"],
 " functions were described in Section 10.1 of this manual, but are worth \
revisiting. ",
 StyleBox[ButtonBox["VertexDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDelete"], "FunctionLink"],
 " takes two arguments: a graph and a vertex or list of vertices. The \
function returns a new graph with the vertex or vertices and all incident \
edges removed. Here we highlight the subgraph of the complete graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "4"], TraditionalForm]],ExpressionUUID->
  "f2366e51-c8de-43a5-b765-fffe42eaa19b"],
 " that is obtained by deleting a vertex."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->
  False,ExpressionUUID->"09946c38-c68c-4b05-84e9-265f5bcfcc03"],

Cell[BoxData[
 RowBox[{
  RowBox[{"deleteVExStart", "=", 
   RowBox[{"CompleteGraph", "[", "5", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellLabel->"In[99]:=",ExpressionUUID->"4d6e7b6c-1294-4c91-996b-1f1df85bf7dc"],

Cell[BoxData[
 RowBox[{
  RowBox[{"deleteVExEnd", "=", 
   RowBox[{"VertexDelete", "[", 
    RowBox[{"deleteVExStart", ",", "1"}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[100]:=",ExpressionUUID->"a03fa5f2-f401-46fa-93ec-5872b040ae23"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"deleteVExStart", ",", "deleteVExEnd", ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]], "Input",
 CellLabel->
  "In[101]:=",ExpressionUUID->"07865a80-6808-4372-9c5d-17f9cd238285"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 4, 8, 12, 16, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {5}, {
          1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {GraphHighlight -> {
         UndirectedEdge[2, 3], 
         UndirectedEdge[4, 5], 
         UndirectedEdge[3, 5], 
         UndirectedEdge[3, 4], 2, 
         UndirectedEdge[2, 5], 
         UndirectedEdge[2, 4], 4, 3, 5}, GraphLayout -> "CircularEmbedding", 
       VertexLabels -> {
        4 -> "Name", 2 -> "Name", 3 -> "Name", 5 -> "Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}}, 
          0.021812234931106983`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 0.30901699437494834`}, {
          0.5877852522924737, -0.8090169943749481}}, 
          0.021812234931106983`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 0.30901699437494834`}, {
          0.9510565162951533, 0.30901699437494634`}}, 0.021812234931106983`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
          0.021812234931106983`]}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
           0.5877852522924737, -0.8090169943749481}}, 
           0.021812234931106983`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
           0.9510565162951533, 0.30901699437494634`}}, 
           0.021812234931106983`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, \
{-7.044813998280222*^-16, 1.}}, 0.021812234931106983`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
           0.9510565162951533, 0.30901699437494634`}}, 
           0.021812234931106983`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{
           0.5877852522924737, -0.8090169943749481}, {-7.044813998280222*^-16,
            1.}}, 0.021812234931106983`]}}, 
        {Hue[1, 1, 0.7], Opacity[1], 
         {Arrowheads[0.], 
          ArrowBox[{{0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
           0.021812234931106983`]}}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
        DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
         0.021812234931106983], {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
           0.021812234931106983]}, 
         InsetBox["2", 
          Offset[{2, 2}, {-0.5659730173613657, -0.7872047594438398}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0.5877852522924737, -0.8090169943749481}, 
           0.021812234931106983]}, 
         InsetBox["3", 
          Offset[{2, 2}, {0.6095974872235806, -0.7872047594438412}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0.9510565162951533, 0.30901699437494634}, 
           0.021812234931106983]}, 
         InsetBox["4", 
          Offset[{2, 2}, {0.9728687512262603, 0.3308292293060533}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{-7.044813998280222*^-16, 1.}, 0.021812234931106983]}, 
         InsetBox["5", 
          Offset[{2, 2}, {0.02181223493110628, 1.021812234931107}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[101]=",ExpressionUUID->"00bd4676-0b81-40c3-8c61-7c20001853e7"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["EdgeDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeDelete"], "FunctionLink"],
 " also takes two arguments, a graph and an edge or a list of edges. For \
example, we can remove the outer ring of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "5"], TraditionalForm]],ExpressionUUID->
  "92052edc-50d6-46b5-a894-8777e891451a"],
 " as follows:"
}], "Text",ExpressionUUID->"f8111675-635b-46c7-a899-a3c92e904a59"],

Cell[BoxData[
 RowBox[{
  RowBox[{"deleteEexStart", "=", 
   RowBox[{"CompleteGraph", "[", "5", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[102]:=",ExpressionUUID->"6212519a-5903-40e7-92f4-976dd58d4fc4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"deleteEexEdges", "=", 
   RowBox[{"Join", "[", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"i", "\[Rule]", 
        RowBox[{"i", "+", "1"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", "4"}], "}"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", "\[Rule]", "5"}], "}"}]}], "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[103]:=",ExpressionUUID->"0db71905-0d0d-4abd-94c9-803bca599a20"],

Cell[BoxData[
 RowBox[{
  RowBox[{"deleteEexEnd", "=", 
   RowBox[{"EdgeDelete", "[", 
    RowBox[{"deleteEexStart", ",", "deleteEexEdges"}], "]"}]}], 
  ";"}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[104]:=",ExpressionUUID->"18087067-bb74-42b1-8977-cf7c33dd0dfb"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"deleteEexStart", ",", "deleteEexEnd"}], "]"}]], "Input",
 CellLabel->
  "In[105]:=",ExpressionUUID->"0b3ecda1-e552-453d-8c1f-c272b18cc14b"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 4, 8, 12, 16, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {5}, {
          1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {GraphHighlight -> {1, 
         UndirectedEdge[3, 5], 5, 
         UndirectedEdge[2, 4], 
         UndirectedEdge[1, 3], 
         UndirectedEdge[1, 4], 2, 
         UndirectedEdge[2, 5], 4, 3}, GraphLayout -> "CircularEmbedding"}]]}, 
    
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.9510565162951538, 
       0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
       0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
       0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}},
            0.021812234931106983`]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{-0.9510565162951538, 0.30901699437494834`}, {
            0.5877852522924737, -0.8090169943749481}}, 
            0.021812234931106983`]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{-0.9510565162951538, 0.30901699437494834`}, {
            0.9510565162951533, 0.30901699437494634`}}, 
            0.021812234931106983`]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
           0.021812234931106983`]}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
           0.5877852522924737, -0.8090169943749481}}, 0.021812234931106983`]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
            0.9510565162951533, 0.30901699437494634`}}, 
            0.021812234931106983`]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, \
{-7.044813998280222*^-16, 1.}}, 0.021812234931106983`]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
           0.9510565162951533, 0.30901699437494634`}}, 
           0.021812234931106983`]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{
            0.5877852522924737, -0.8090169943749481}, \
{-7.044813998280222*^-16, 1.}}, 0.021812234931106983`]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
           0.021812234931106983`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[1, 0.021812234931106983]}, 
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[2, 0.021812234931106983]}, 
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[3, 0.021812234931106983]}, 
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[4, 0.021812234931106983]}, 
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[5, 0.021812234931106983]}}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[105]=",ExpressionUUID->"80b72944-0d21-49fd-a170-95e55ec0e7aa"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Adding Vertices and Edges", "Subsection",
 PageBreakWithin->False,
 CellTags->"Section2h",ExpressionUUID->"3e726959-9a34-4b84-b7d9-03ef53fda0a7"],

Cell[TextData[{
 "The functions for adding vertices and edges are very similar. ",
 StyleBox[ButtonBox["VertexAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexAdd"], "FunctionLink"],
 " accepts a graph and either a vertex or list of vertices to add to the \
graph."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->
  False,ExpressionUUID->"ce1e26a0-1972-4db0-9e4b-8efb09cc7d1d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexAdd", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", 
    RowBox[{"5", ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}], ",", 
   "\"\<a\>\""}], "]"}]], "Input",
 CellLabel->
  "In[106]:=",ExpressionUUID->"c8aec17e-c84b-4d55-b1c5-f1fb428e5393"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, "a"}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 4, 8, 12, 16, 20, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {
          5}, {1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 
          0.5000000000000007}, {-0.8660254037844384, -0.4999999999999994}}, 
          0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {0.8660254037844386, 
          0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
          0.02261146496815286], 
         InsetBox["1", 
          Offset[{2, 2}, {-0.843413938816286, 0.5226114649681535}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.02261146496815286], 
         InsetBox["2", 
          Offset[{2, 2}, {-0.8434139388162856, -0.47738853503184653}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.8285686989269494*^-16, -1.}, 0.02261146496815286], 
         InsetBox["3", 
          Offset[{2, 2}, {0.022611464968153243, -0.9773885350318472}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.02261146496815286], 
         InsetBox["4", 
          Offset[{2, 2}, {0.8886368687525918, -0.47738853503184836}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844386, 0.4999999999999993}, 
          0.02261146496815286], 
         InsetBox["5", 
          Offset[{2, 2}, {0.8886368687525914, 0.5226114649681521}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.02261146496815286], 
         InsetBox["\<\"a\"\>", 
          Offset[{2, 2}, {0.022611464968153045, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[106]=",ExpressionUUID->"55f935e4-de63-4d43-b39c-730c34b49b57"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["EdgeAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeAdd"], "FunctionLink"],
 " acts on a graph and adds an edge or a list of edges. Note that you can use \
rules to describe the edges and ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will interpret them as directed or not depending on whether the original \
graph is directed."
}], "Text",ExpressionUUID->"bd9f9fac-77ac-42c2-ae72-c9d134c094ad"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EdgeAdd", "[", 
  RowBox[{
   RowBox[{"CycleGraph", "[", "6", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "3"}], ",", 
     RowBox[{"2", "\[Rule]", "4"}], ",", 
     RowBox[{"3", "\[Rule]", "5"}], ",", 
     RowBox[{"4", "\[Rule]", "6"}], ",", 
     RowBox[{"5", "\[Rule]", "1"}], ",", 
     RowBox[{"6", "\[Rule]", "2"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[107]:=",ExpressionUUID->"f9f0060a-85da-4a7d-a2ed-93575131d4cd"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 4, 8, 12, 16, 20, 24}, {{2}, {6}, {3}, {5}, {1}, {3}, {4}, {
          6}, {2}, {4}, {1}, {5}, {3}, {5}, {2}, {6}, {4}, {6}, {3}, {1}, {
          1}, {5}, {4}, {2}}}, Pattern}]}, {
      GraphLayout -> "CircularEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.8660254037844389, 
       0.5000000000000007}, {-0.8660254037844384, -0.4999999999999994}, {
       3.8285686989269494`*^-16, -1.}, {
       0.8660254037844389, -0.5000000000000012}, {0.8660254037844386, 
       0.4999999999999993}, {1.8369701987210297`*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 5}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{4, 5}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{4, 6}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.02261146496815286]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02261146496815286], DiskBox[2, 0.02261146496815286], 
         DiskBox[3, 0.02261146496815286], DiskBox[4, 0.02261146496815286], 
         DiskBox[5, 0.02261146496815286], DiskBox[6, 0.02261146496815286]}}]],
     
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[107]=",ExpressionUUID->"2d21e996-8b03-4d9c-a296-53de49926e4d"]
}, Open  ]],

Cell[TextData[{
 "You can specify directed edges using ",
 StyleBox[ButtonBox["DirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdge"], "FunctionLink"],
 " or \[UndirectedEdge] (\[EscapeKey]",
 StyleBox["ue",
  IgnoreSpellCheck->True],
 "\[EscapeKey]); you can specify undirected edges by ",
 StyleBox[ButtonBox["UndirectedEdge",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UndirectedEdge"], "FunctionLink"],
 ", \[DirectedEdge] (\[EscapeKey]de\[EscapeKey]), or ",
 StyleBox[ButtonBox["TwoWayRule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TwoWayRule"], "FunctionLink"],
 " (",
 StyleBox["<->", "Code"],
 "). When a graph includes any directed edges, ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 ") will be interpreted as a directed edge."
}], "Text",ExpressionUUID->"ff021e29-70a8-446e-b911-f4993031a141"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EdgeAdd", "[", 
  RowBox[{
   RowBox[{"CycleGraph", "[", 
    RowBox[{"6", ",", 
     RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"DirectedEdge", "[", 
      RowBox[{"1", ",", "3"}], "]"}], ",", 
     RowBox[{"3", "\[DirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[Rule]", "1"}], ",", 
     RowBox[{"UndirectedEdge", "[", 
      RowBox[{"4", ",", "6"}], "]"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"6", "\[TwoWayRule]", "2"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[108]:=",ExpressionUUID->"9b480e1b-f8d0-4f91-a039-fbe33a797b77"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {{{0, 0, 1, 0, 0, 0}, {0, 0, 0, 0, 0, 0}, {0, 
      0, 0, 0, 1, 0}, {0, 0, 0, 0, 0, 0}, {1, 0, 0, 0, 0, 0}, {0, 0, 0, 0, 0, 
      0}}, {{0, 1, 0, 0, 0, 1}, {0, 0, 1, 1, 0, 1}, {0, 0, 0, 1, 0, 0}, {0, 0,
       0, 0, 1, 1}, {0, 0, 0, 0, 0, 1}, {0, 0, 0, 0, 0, 0}}}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 
          0.5000000000000007}, {-0.8660254037844384, -0.4999999999999994}}, 
          0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
         3.8285686989269494`*^-16, -1.}}, 0.02261146496815286], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        ArrowBox[{{3.8285686989269494`*^-16, -1.}, {0.8660254037844386, 
         0.4999999999999993}}, 0.02261146496815286], 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844386, 0.4999999999999993}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        ArrowBox[{{0.8660254037844386, 
         0.4999999999999993}, {-0.8660254037844389, 0.5000000000000007}}, 
         0.02261146496815286]}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
          0.02261146496815286], 
         InsetBox["1", 
          Offset[{2, 2}, {-0.843413938816286, 0.5226114649681535}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.02261146496815286], 
         InsetBox["2", 
          Offset[{2, 2}, {-0.8434139388162856, -0.47738853503184653}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.8285686989269494*^-16, -1.}, 0.02261146496815286], 
         InsetBox["3", 
          Offset[{2, 2}, {0.022611464968153243, -0.9773885350318472}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.02261146496815286], 
         InsetBox["4", 
          Offset[{2, 2}, {0.8886368687525918, -0.47738853503184836}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844386, 0.4999999999999993}, 
          0.02261146496815286], 
         InsetBox["5", 
          Offset[{2, 2}, {0.8886368687525914, 0.5226114649681521}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.02261146496815286], 
         InsetBox["6", 
          Offset[{2, 2}, {0.022611464968153045, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[108]=",ExpressionUUID->"b71ede0e-2a61-40b9-b7b4-1004a3dd6161"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Edge Contraction", "Subsection",
 CellTags->"Section2i",ExpressionUUID->"7d548293-46b5-428a-ad06-4d80e1c772be"],

Cell[TextData[{
 "Recall that an edge contraction for an edge ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]],ExpressionUUID->
  "b148826b-a49c-43a7-a82d-5e35cb14a29c"],
 " with endpoints ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "5945d0d5-41a7-4d55-af1e-cbbb0a5b67a4"],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "01727374-5e71-48a3-b0eb-a1ebe91920c3"],
 " consists of deleting the edge, merging ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "fabfb28c-db8c-4463-a061-335dcbd1abf0"],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "801d410c-33f1-464f-b936-06ccc3a7cc93"],
 " into a new vertex ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "d7889f76-1aca-4dbf-bde4-4188f97d9421"],
 ", and preserving all edges (other than ",
 Cell[BoxData[
  FormBox["e", TraditionalForm]],ExpressionUUID->
  "32c5617b-7587-49fc-98d5-aa2a861def59"],
 ") which had ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "961a8d80-627e-4251-8a50-0328e1e5d635"],
 " or ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "5e25f0de-a4b1-4fd8-aea2-2b49cf6d7080"],
 " as an endpoint by setting ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "8de8a771-0a1f-4675-a262-da09d33737ea"],
 " as the new endpoint. As an illustration, consider the graph shown below."
}], "Text",ExpressionUUID->"0ec1baf5-2a90-45d8-8904-410c9092067c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleContraction", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6", ",", "7"}], "}"}],
     ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"Style", "[", 
       RowBox[{
        RowBox[{"3", "\[Rule]", "4"}], ",", 
        RowBox[{"{", 
         RowBox[{"Thick", ",", "Red"}], "}"}]}], "]"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"4", "\[Rule]", "7"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}], ",", 
      RowBox[{"6", "\[Rule]", "7"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[109]:=",ExpressionUUID->"a5f4038a-59e3-471e-b8e7-9e7edbac7078"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7}, {
      Null, {{1, 2}, {1, 3}, {2, 3}, {3, 4}, {4, 5}, {4, 7}, {5, 6}, {6, 
       7}}}, {EdgeStyle -> {UndirectedEdge[3, 4] -> {{
            Thickness[Large], 
            RGBColor[1, 0, 0]}}}, VertexLabels -> {"Name"}, 
       VertexCoordinates -> {{0, 1}, {0, 0}, {1, 0}, {2, 0}, {3, 0}, {3, 1}, {
        2, 1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 1.}, {0., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 1.}, {1., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {1., 0.}}, 0.030239520958083826`]}, 
        {RGBColor[1, 0, 0], Thickness[Large], 
         {Arrowheads[0.], 
          ArrowBox[{{1., 0.}, {2., 0.}}, 0.030239520958083826`]}}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 0.}, {3., 0.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2., 0.}, {2., 1.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 0.}, {3., 1.}}, 0.030239520958083826`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 1.}, {2., 1.}}, 0.030239520958083826`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 1.}, 0.030239520958083826], 
         InsetBox["1", 
          Offset[{2, 2}, {0.030239520958083826, 1.030239520958084}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.030239520958083826], 
         InsetBox["2", 
          Offset[{2, 2}, {0.030239520958083826, 0.030239520958083826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.030239520958083826], 
         InsetBox["3", 
          Offset[{2, 2}, {1.030239520958084, 0.030239520958083826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.030239520958083826], 
         InsetBox["4", 
          Offset[{2, 2}, {2.0302395209580837, 0.030239520958083826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.030239520958083826], 
         InsetBox["5", 
          Offset[{2, 2}, {3.0302395209580837, 0.030239520958083826}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 1.}, 0.030239520958083826], 
         InsetBox["6", 
          Offset[{2, 2}, {3.0302395209580837, 1.030239520958084}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.030239520958083826], 
         InsetBox["7", 
          Offset[{2, 2}, {2.0302395209580837, 1.030239520958084}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[109]=",ExpressionUUID->"d0c02628-a62b-4939-958f-f821ec2aae10"]
}, Open  ]],

Cell[TextData[{
 "Observe the use of the ",
 StyleBox[ButtonBox["Style",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Style"], "FunctionLink"],
 " wrapper on the edge between vertices 3 and 4 to highlight that edge."
}], "Text",ExpressionUUID->"719610d1-f686-45da-8c12-04703bfb32d8"],

Cell[TextData[{
 "The function ",
 StyleBox[ButtonBox["EdgeContract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeContract"], "FunctionLink"],
 " can be used to contract an edge. It takes two arguments: a graph and an \
edge or list of edges. "
}], "Text",ExpressionUUID->"19617917-bdbc-4353-9b5f-dbca0414bc5a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EdgeContract", "[", 
  RowBox[{"exampleContraction", ",", 
   RowBox[{"3", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellLabel->
  "In[110]:=",ExpressionUUID->"e5dfc16e-fe98-4eb7-afa2-c842ab5d59a0"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 5, 6, 7, 3}, {
      Null, {{1, 2}, {3, 4}, {4, 5}, {6, 1}, {6, 2}, {6, 3}, {6, 5}}}, {
      EdgeStyle -> {UndirectedEdge[3, 4] -> {{
            Thickness[Large], 
            RGBColor[1, 0, 0]}}}, 
       VertexLabels -> {"Name", 3 -> None, 4 -> None}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8702099711030229}, {0.0008859058870702707, 
          0.14529556736825255`}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8702099711030229}, {0.9878807933079587, 
          0.5080527706450798}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.0008859058870702707, 0.14529556736825255`}, {
          0.9878807933079587, 0.5080527706450798}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9879652931318, 1.016659114014502}, {2.7714501167475705`,
           0.5075247853981719}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9879652931318, 1.016659114014502}, {0.9878807933079587, 
          0.5080527706450798}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7714501167475705`, 0.5075247853981719}, {
          1.9880764379763445`, 0.}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9880764379763445`, 0.}, {0.9878807933079587, 
          0.5080527706450798}}, 0.028577782415983327`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.8702099711030229}, 0.028577782415983327], 
         InsetBox["1", 
          Offset[{2, 2}, {0.028577782415983327, 0.8987877535190062}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.0008859058870702707, 0.14529556736825255}, 
          0.028577782415983327], 
         InsetBox["2", 
          Offset[{2, 2}, {0.029463688303053598, 0.17387334978423588}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.9879652931318, 1.016659114014502}, 0.028577782415983327], 
         InsetBox["5", 
          Offset[{2, 2}, {2.016543075547783, 1.0452368964304855}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.7714501167475705, 0.5075247853981719}, 
          0.028577782415983327], 
         InsetBox["6", 
          Offset[{2, 2}, {2.800027899163554, 0.5361025678141552}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.9880764379763445, 0.}, 0.028577782415983327], 
         InsetBox["7", 
          Offset[{2, 2}, {2.016654220392328, 0.028577782415983327}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, 
        DiskBox[{0.9878807933079587, 0.5080527706450798}, 
         0.028577782415983327]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[110]=",ExpressionUUID->"7893cbf8-d4cd-4fdd-8e5b-ff4df1a13eaa"]
}, Open  ]],

Cell[TextData[{
 "The Wolfram Language also includes a similar function, ",
 StyleBox[ButtonBox["VertexContract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexContract"], "FunctionLink"],
 ". The second argument of ",
 StyleBox[ButtonBox["VertexContract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexContract"], "FunctionLink"],
 " is a list of vertices, and the result is the graph obtained by combining \
all the vertices in the list into a single vertex and removing all edges \
between the merged vertices. When the vertices are the endpoints of an edge, \
the effect is the same as ",
 StyleBox[ButtonBox["EdgeContract",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeContract"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"ca745a24-d234-4958-ba22-fc9e8c615cb6"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexContract", "[", 
  RowBox[{"exampleContraction", ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[111]:=",ExpressionUUID->"60613b9e-97f0-4f7d-a7a2-8a260fc4e5af"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 5, 6, 7, 3}, {
      Null, {{1, 2}, {3, 4}, {4, 5}, {6, 1}, {6, 2}, {6, 3}, {6, 5}}}, {
      EdgeStyle -> {UndirectedEdge[3, 4] -> {{
            Thickness[Large], 
            RGBColor[1, 0, 0]}}}, 
       VertexLabels -> {"Name", 3 -> None, 4 -> None}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8702099711030229}, {0.0008859058870702707, 
          0.14529556736825255`}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.8702099711030229}, {0.9878807933079587, 
          0.5080527706450798}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.0008859058870702707, 0.14529556736825255`}, {
          0.9878807933079587, 0.5080527706450798}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9879652931318, 1.016659114014502}, {2.7714501167475705`,
           0.5075247853981719}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9879652931318, 1.016659114014502}, {0.9878807933079587, 
          0.5080527706450798}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7714501167475705`, 0.5075247853981719}, {
          1.9880764379763445`, 0.}}, 0.028577782415983327`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.9880764379763445`, 0.}, {0.9878807933079587, 
          0.5080527706450798}}, 0.028577782415983327`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.8702099711030229}, 0.028577782415983327], 
         InsetBox["1", 
          Offset[{2, 2}, {0.028577782415983327, 0.8987877535190062}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.0008859058870702707, 0.14529556736825255}, 
          0.028577782415983327], 
         InsetBox["2", 
          Offset[{2, 2}, {0.029463688303053598, 0.17387334978423588}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.9879652931318, 1.016659114014502}, 0.028577782415983327], 
         InsetBox["5", 
          Offset[{2, 2}, {2.016543075547783, 1.0452368964304855}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.7714501167475705, 0.5075247853981719}, 
          0.028577782415983327], 
         InsetBox["6", 
          Offset[{2, 2}, {2.800027899163554, 0.5361025678141552}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.9880764379763445, 0.}, 0.028577782415983327], 
         InsetBox["7", 
          Offset[{2, 2}, {2.016654220392328, 0.028577782415983327}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, 
        DiskBox[{0.9878807933079587, 0.5080527706450798}, 
         0.028577782415983327]}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[111]=",ExpressionUUID->"b7db639d-ecb0-4083-9648-ff3e1eb62785"]
}, Open  ]],

Cell["\<\
The second argument, however, is not limited to two vertices, nor do they \
need to be adjacent, as the example below illustrates.\
\>", "Text",ExpressionUUID->"aaa81d69-0ecc-4959-bccf-836cc66af7a6"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleContraction2", "=", 
  RowBox[{"CirculantGraph", "[", 
   RowBox[{"9", ",", "3", ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexStyle", "\[Rule]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"i", "\[Rule]", "Red"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "9", ",", "8"}], "}"}]}], "}"}]}], "]"}]}]}], 
   "]"}]}]], "Input",
 CellLabel->
  "In[112]:=",ExpressionUUID->"8baf1ab6-499a-433d-8531-e4e9be10fb1d"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9}, {
      Null, {{1, 4}, {1, 7}, {2, 5}, {2, 8}, {3, 6}, {3, 9}, {4, 7}, {5, 8}, {
       6, 9}}}, {
      GraphLayout -> {
        "VertexLayout" -> "CircularEmbedding", "PackingLayout" -> None}, 
       VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       VertexStyle -> {
        1 -> RGBColor[1, 0, 0], 9 -> RGBColor[1, 0, 0], 8 -> 
         RGBColor[1, 0, 0]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.6427876096865399, 
          0.7660444431189786}, {-0.3420201433256684, -0.9396926207859079}}, 
          0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.6427876096865399, 0.7660444431189786}, {
          0.984807753012208, 0.17364817766692903`}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9848077530122081, 0.17364817766693114`}, {
          0.34202014332566916`, -0.9396926207859085}}, 
          0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9848077530122081, 0.17364817766693114`}, {
          0.6427876096865389, 0.7660444431189772}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, \
{-7.044813998280222*^-16, 1.}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.3420201433256684, -0.9396926207859079}, {
          0.984807753012208, 0.17364817766692903`}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.34202014332566916`, -0.9396926207859085}, {
          0.6427876096865389, 0.7660444431189772}}, 0.06840402866513366]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.8660254037844389, -0.5000000000000012}, {-7.044813998280222*^-16, 
          1.}}, 0.06840402866513366]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[1, 0, 0], 
          DiskBox[{-0.6427876096865399, 0.7660444431189786}, 
           0.06840402866513366]}, 
         InsetBox["1", {-0.6427876096865399, 0.7660444431189786},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9848077530122081, 0.17364817766693114}, 
          0.06840402866513366], 
         InsetBox["2", {-0.9848077530122081, 0.17364817766693114},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.06840402866513366], 
         InsetBox["3", {-0.8660254037844384, -0.4999999999999994},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.3420201433256684, -0.9396926207859079}, 
          0.06840402866513366], 
         InsetBox["4", {-0.3420201433256684, -0.9396926207859079},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.34202014332566916, -0.9396926207859085}, 
          0.06840402866513366], 
         InsetBox["5", {0.34202014332566916, -0.9396926207859085},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.06840402866513366], 
         InsetBox["6", {0.8660254037844389, -0.5000000000000012},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.984807753012208, 0.17364817766692903}, 
          0.06840402866513366], 
         InsetBox["7", {0.984807753012208, 0.17364817766692903},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 0, 0], 
          DiskBox[{0.6427876096865389, 0.7660444431189772}, 
           0.06840402866513366]}, 
         InsetBox["8", {0.6427876096865389, 0.7660444431189772},
          BaseStyle->"Graphics"]}, {
         {RGBColor[1, 0, 0], 
          DiskBox[{-7.044813998280222*^-16, 1.}, 0.06840402866513366]}, 
         InsetBox["9", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[112]=",ExpressionUUID->"41e3e4cc-0058-4105-b6af-1d11753dd963"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"VertexContract", "[", 
    RowBox[{"exampleContraction2", ",", 
     RowBox[{"{", 
      RowBox[{"9", ",", "1", ",", "8"}], "}"}]}], "]"}], ",", 
   RowBox[{"VertexStyle", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"9", "\[Rule]", "Red"}], "}"}]}]}], "]"}]], "Input",
 CellLabel->
  "In[113]:=",ExpressionUUID->"565355d4-e019-4386-a9e4-8053d5d25ff1"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{2, 3, 4, 5, 6, 7, 9}, {
      Null, {{1, 4}, {2, 5}, {3, 6}, {7, 1}, {7, 2}, {7, 3}, {7, 4}, {7, 5}, {
       7, 6}}}, {
      GraphLayout -> {
        "Dimension" -> 2, "PackingLayout" -> None, "VertexLayout" -> 
         "CircularEmbedding"}, VertexLabels -> {
         Placed["Name", Center], 1 -> None, 9 -> None, 8 -> None}, 
       VertexSize -> {Medium, 9 -> Automatic, 1 -> Automatic, 8 -> Automatic},
        VertexStyle -> {9 -> RGBColor[1, 0, 0]}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 0.6234898018587343}, {
          0.43388373911755884`, -0.9009688679024196}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7818314824680302, 
          0.6234898018587343}, {-7.044813998280222*^-16, 1.}}, 
          0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.9749279121818237, -0.2225209339563155}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.7818314824680294, 0.6234898018587327}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.43388373911755884`, -0.9009688679024196}, \
{-7.044813998280222*^-16, 1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.9749279121818237, -0.2225209339563155}, {-7.044813998280222*^-16, 
          1.}}, 0.08677674782351154]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7818314824680294, 
          0.6234898018587327}, {-7.044813998280222*^-16, 1.}}, 
          0.08677674782351154]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.7818314824680302, 0.6234898018587343}, 
          0.08677674782351154], 
         InsetBox["2", {-0.7818314824680302, 0.6234898018587343},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9749279121818235, -0.22252093395631342}, 
          0.08677674782351154], 
         InsetBox["3", {-0.9749279121818235, -0.22252093395631342},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.43388373911755757, -0.9009688679024187}, 
          0.08677674782351154], 
         InsetBox["4", {-0.43388373911755757, -0.9009688679024187},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.43388373911755884, -0.9009688679024196}, 
          0.08677674782351154], 
         InsetBox["5", {0.43388373911755884, -0.9009688679024196},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9749279121818237, -0.2225209339563155}, 
          0.08677674782351154], 
         InsetBox["6", {0.9749279121818237, -0.2225209339563155},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.7818314824680294, 0.6234898018587327}, 
          0.08677674782351154], 
         InsetBox["7", {0.7818314824680294, 0.6234898018587327},
          BaseStyle->"Graphics"]}, 
        {RGBColor[1, 0, 0], 
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.02220329551621769]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[113]=",ExpressionUUID->"a081c276-bf60-4928-8747-58589a95005f"]
}, Open  ]],

Cell[TextData[{
 "Note that when the vertices are merged, the new vertex is given the name of \
the vertex that appears first in the list. Observe above that we listed 9 \
first in the set of vertices, and so it is 9 that we style as red. That the \
new vertex is unnumbered and smaller than before illustrates that the result \
of the contraction function is to delete all the original vertices, along \
with all options attached to them. Also observe that the contraction \
functions do not accept the options available to ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 ", so we had to apply ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " to the result of the contraction."
}], "Text",ExpressionUUID->"891ceba0-f844-46bd-a3f9-525774ef5d8b"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unions and Complements of Graphs", "Subsection",
 CellTags->"Section2j",ExpressionUUID->"2cf5a3b0-20b2-4f16-b51f-bbc0c08de9a5"],

Cell["\<\
Recall that the union of two graphs is the graph obtained by taking the union \
of the sets of vertices and the sets of edges from the two graphs. \
\>", "Text",ExpressionUUID->"79ba4537-83f5-4569-aaed-c6e7fcba80f5"],

Cell["\<\
As an example, we will \[OpenCurlyDoubleQuote]fill in\[CloseCurlyDoubleQuote] \
a prism graph by computing the union of the prism with the complete graph on \
the vertices in one ring. We begin with a version of the prism we created \
above.\
\>", "Text",ExpressionUUID->"2a0b9033-0d47-466a-8ec2-2b548ddf4fd8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"unionExampleA", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "4"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}], ",", 
      RowBox[{"6", "\[Rule]", "1"}], ",", 
      RowBox[{"7", "\[Rule]", "8"}], ",", 
      RowBox[{"8", "\[Rule]", "9"}], ",", 
      RowBox[{"9", "\[Rule]", "10"}], ",", 
      RowBox[{"10", "\[Rule]", "11"}], ",", 
      RowBox[{"11", "\[Rule]", "12"}], ",", 
      RowBox[{"12", "\[Rule]", "7"}], ",", 
      RowBox[{"1", "\[Rule]", "7"}], ",", 
      RowBox[{"2", "\[Rule]", "8"}], ",", 
      RowBox[{"3", "\[Rule]", "9"}], ",", 
      RowBox[{"4", "\[Rule]", "10"}], ",", 
      RowBox[{"5", "\[Rule]", "11"}], ",", 
      RowBox[{"6", "\[Rule]", "12"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[114]:=",ExpressionUUID->"30b62c73-769b-4d49-81f6-641c75872aee"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}, {
      Null, {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}, {6, 1}, {7, 8}, {8, 9}, {
       9, 10}, {10, 11}, {11, 12}, {12, 7}, {1, 7}, {2, 8}, {3, 9}, {4, 10}, {
       5, 11}, {6, 12}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          0.9839435806937917, 2.4015105280929165`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          1.199176045955087, 0.4394265853678305}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {0., 
          0.9620430219770666}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          2.099708266496932, 2.3764856914994263`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          0.39587622224175434`, 2.005570642165643}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          2.7301971795384574`, 1.4401369864781073`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.333865160413755, 0.39609325735504175`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.199176045955087, 0.4394265853678305}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.746198062974075, 0.}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.199176045955087, 0.4394265853678305}, {
          0.6298912750595992, 0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {0.39587622224175434`, 
          2.005570642165643}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {0.6298912750595992, 
          0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5313638191412857`, 1.9620109994805106`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1323633389347414`, 1.0677650125596039`}, {
          1.746198062974075, 0.}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.746198062974075, 0.}, {0.6298912750595992, 
          0.025825868311962763`}}, 0.0701245984515375]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.5983109393792555, 1.333510780126811}, 0.0701245984515375],
          InsetBox["1", {0.5983109393792555, 1.333510780126811},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9839435806937917, 2.4015105280929165}, 
          0.0701245984515375], 
         InsetBox["2", {0.9839435806937917, 2.4015105280929165},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.099708266496932, 2.3764856914994263}, 0.0701245984515375],
          InsetBox["3", {2.099708266496932, 2.3764856914994263},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.7301971795384574, 1.4401369864781073}, 
          0.0701245984515375], 
         InsetBox["4", {2.7301971795384574, 1.4401369864781073},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.333865160413755, 0.39609325735504175}, 
          0.0701245984515375], 
         InsetBox["5", {2.333865160413755, 0.39609325735504175},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.199176045955087, 0.4394265853678305}, 0.0701245984515375],
          InsetBox["6", {1.199176045955087, 0.4394265853678305},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.9620430219770666}, 0.0701245984515375], 
         InsetBox["7", {0., 0.9620430219770666},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.39587622224175434, 2.005570642165643}, 
          0.0701245984515375], 
         InsetBox["8", {0.39587622224175434, 2.005570642165643},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5313638191412857, 1.9620109994805106}, 
          0.0701245984515375], 
         InsetBox["9", {1.5313638191412857, 1.9620109994805106},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.1323633389347414, 1.0677650125596039}, 
          0.0701245984515375], 
         InsetBox["10", {2.1323633389347414, 1.0677650125596039},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.746198062974075, 0.}, 0.0701245984515375], 
         InsetBox["11", {1.746198062974075, 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6298912750595992, 0.025825868311962763}, 
          0.0701245984515375], 
         InsetBox["12", {0.6298912750595992, 0.025825868311962763},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[114]=",ExpressionUUID->"ad491ac7-4a8e-44ef-8ef4-9ea33cdd8578"]
}, Open  ]],

Cell[TextData[{
 "We use the complete graph on 6 vertices as the second graph that will form \
part of the union. By default, ",
 StyleBox[ButtonBox["CompleteGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/CompleteGraph"], "FunctionLink"],
 " with argument 6 will form the complete graph on the vertices from 1 \
through 6. Suppose instead that we want the complete graph on the vertices \
from 7 through 12. To do this, apply the ",
 StyleBox[ButtonBox["VertexReplace",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexReplace"], "FunctionLink"],
 " function. This function accepts two arguments: a graph and a list of rules \
specifying how to modify the names of the vertices. In this instance, we will \
replace vertex 1 with 7, 2 with 8, and so on, so we need to use the list of \
rules ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "7"}], ",", 
     RowBox[{"2", "\[Rule]", "6"}], ",", "\[Ellipsis]", ",", 
     RowBox[{"6", "\[Rule]", "12"}]}], "}"}], TraditionalForm]],
  ExpressionUUID->"a0f5ca5e-f4dd-4584-a62d-b985dc6eaf26"],
 ", which we will create with a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"4f3121e5-fa07-4de3-b7ba-ce1773d52dfd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"unionExampleB", "=", 
  RowBox[{"VertexReplace", "[", 
   RowBox[{
    RowBox[{"CompleteGraph", "[", 
     RowBox[{"6", ",", 
      RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"i", "\[Rule]", 
       RowBox[{"i", "+", "6"}]}], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[115]:=",ExpressionUUID->"33b6cc2e-f0ea-400b-af6d-15aaef1e1749"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{7, 8, 9, 10, 11, 12}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 5, 10, 15, 20, 25, 30}, {{2}, {3}, {4}, {5}, {6}, {1}, {3}, {
          4}, {5}, {6}, {1}, {2}, {4}, {5}, {6}, {1}, {2}, {3}, {5}, {6}, {
          1}, {2}, {3}, {4}, {6}, {1}, {2}, {3}, {4}, {5}}}, Pattern}]}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 
          0.5000000000000007}, {-0.8660254037844384, -0.4999999999999994}}, 
          0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          3.8285686989269494`*^-16, -1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {
          0.8660254037844389, -0.5000000000000012}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {0.8660254037844386, 
          0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {1.8369701987210297`*^-16,
           1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          0.8660254037844386, 0.4999999999999993}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844386, 0.4999999999999993}, {
          1.8369701987210297`*^-16, 1.}}, 0.02261146496815286]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
          0.02261146496815286], 
         InsetBox["7", 
          Offset[{2, 2}, {-0.843413938816286, 0.5226114649681535}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.02261146496815286], 
         InsetBox["8", 
          Offset[{2, 2}, {-0.8434139388162856, -0.47738853503184653}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.8285686989269494*^-16, -1.}, 0.02261146496815286], 
         InsetBox["9", 
          Offset[{2, 2}, {0.022611464968153243, -0.9773885350318472}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.02261146496815286], 
         InsetBox["10", 
          Offset[{2, 2}, {0.8886368687525918, -0.47738853503184836}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844386, 0.4999999999999993}, 
          0.02261146496815286], 
         InsetBox["11", 
          Offset[{2, 2}, {0.8886368687525914, 0.5226114649681521}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.02261146496815286], 
         InsetBox["12", 
          Offset[{2, 2}, {0.022611464968153045, 1.0226114649681528}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[115]=",ExpressionUUID->"ae5c3c94-021d-4ce6-af33-5b74a69906d2"]
}, Open  ]],

Cell[TextData[{
 "Note that the same effect can be obtained with the function ",
 StyleBox[ButtonBox["IndexGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IndexGraph"], "FunctionLink"],
 ", with first argument a graph and second argument the smallest integer to \
be used (defaulting to 1 if the second argument is omitted). ",
 StyleBox[ButtonBox["VertexReplace",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexReplace"], "FunctionLink"],
 " is the more general function."
}], "Text",ExpressionUUID->"faf705e4-fb41-45ce-9713-d88db8137021"],

Cell[TextData[{
 "Also note that this is not the same as setting different labels for the \
vertices. The result of ",
 StyleBox[ButtonBox["VertexReplace",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexReplace"], "FunctionLink"],
 " is a graph with the internal names of the vertices changed."
}], "Text",ExpressionUUID->"8fd1f664-d508-48d5-bca3-8c872580015d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"VertexList", "[", "unionExampleB", "]"}]], "Input",
 CellLabel->
  "In[116]:=",ExpressionUUID->"4250574a-62ad-4185-91a4-33d6412e4c0c"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"7", ",", "8", ",", "9", ",", "10", ",", "11", ",", "12"}], 
  "}"}]], "Output",
 CellLabel->
  "Out[116]=",ExpressionUUID->"5dc07636-d37a-4452-90df-741f0daf241c"]
}, Open  ]],

Cell[TextData[{
 "To obtain the union of the graphs, we apply the ",
 StyleBox[ButtonBox["GraphUnion",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphUnion"], "FunctionLink"],
 " function, which simply takes the two (or more) graphs as arguments, along \
with the usual options."
}], "Text",ExpressionUUID->"9db377b1-d81e-4040-80ac-b67814ad460f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"unionExample", "=", 
  RowBox[{"GraphUnion", "[", 
   RowBox[{"unionExampleA", ",", "unionExampleB", ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[117]:=",ExpressionUUID->"f459f7af-1497-4b9e-aa8f-be938137807c"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}, {
      Null, {{1, 2}, {1, 7}, {2, 3}, {2, 8}, {3, 4}, {3, 9}, {4, 5}, {4, 
       10}, {5, 6}, {5, 11}, {6, 1}, {6, 12}, {7, 8}, {7, 9}, {7, 10}, {7, 
       11}, {7, 12}, {8, 9}, {8, 10}, {8, 11}, {8, 12}, {9, 10}, {9, 11}, {9, 
       12}, {10, 11}, {10, 12}, {11, 12}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.7071913354253746, 0.0017371171895790294`}, {
          2.123360656829433, 0.}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7071913354253746, 0.0017371171895790294`}, {0., 
          1.2278301842640489`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7071913354253746, 0.0017371171895790294`}, {
          1.1493073953353186`, 0.763001016404376}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.123360656829433, 0.}, {2.8328742308413517`, 
          1.2265585699814128`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.123360656829433, 0.}, {1.6836991600385602`, 
          0.7636387931506287}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.8328742308413517`, 1.2265585699814128`}, {
          2.1249909704264685`, 2.452895534992326}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.8328742308413517`, 1.2265585699814128`}, {
          1.952934675305475, 1.2266188040854957`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1249909704264685`, 2.452895534992326}, {
          0.7087043053910814, 2.4530810339704474`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1249909704264685`, 2.452895534992326}, {
          1.6840025605623565`, 1.6898280281048312`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7087043053910814, 2.4530810339704474`}, {0., 
          1.2278301842640489`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7087043053910814, 2.4530810339704474`}, {
          1.1488531370174286`, 1.690607776763056}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 1.2278301842640489`}, {0.8815955604571679, 
          1.2262384437093572`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1493073953353186`, 0.763001016404376}, {
          1.6836991600385602`, 0.7636387931506287}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1493073953353186`, 0.763001016404376}, {
          1.952934675305475, 1.2266188040854957`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1493073953353186`, 0.763001016404376}, {
          1.6840025605623565`, 1.6898280281048312`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1493073953353186`, 0.763001016404376}, {
          1.1488531370174286`, 1.690607776763056}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1493073953353186`, 0.763001016404376}, {
          0.8815955604571679, 1.2262384437093572`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.6836991600385602`, 0.7636387931506287}, {
          1.952934675305475, 1.2266188040854957`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.6836991600385602`, 0.7636387931506287}, {
          1.6840025605623565`, 1.6898280281048312`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.6836991600385602`, 0.7636387931506287}, {
          1.1488531370174286`, 1.690607776763056}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.6836991600385602`, 0.7636387931506287}, {
          0.8815955604571679, 1.2262384437093572`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.952934675305475, 1.2266188040854957`}, {
          1.6840025605623565`, 1.6898280281048312`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.952934675305475, 1.2266188040854957`}, {
          1.1488531370174286`, 1.690607776763056}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.952934675305475, 1.2266188040854957`}, {
          0.8815955604571679, 1.2262384437093572`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.6840025605623565`, 1.6898280281048312`}, {
          1.1488531370174286`, 1.690607776763056}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.6840025605623565`, 1.6898280281048312`}, {
          0.8815955604571679, 1.2262384437093572`}}, 0.05343921452845494]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.1488531370174286`, 1.690607776763056}, {
          0.8815955604571679, 1.2262384437093572`}}, 0.05343921452845494]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.7071913354253746, 0.0017371171895790294}, 
          0.05343921452845494], 
         InsetBox["1", {0.7071913354253746, 0.0017371171895790294},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.123360656829433, 0.}, 0.05343921452845494], 
         InsetBox["2", {2.123360656829433, 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.8328742308413517, 1.2265585699814128}, 
          0.05343921452845494], 
         InsetBox["3", {2.8328742308413517, 1.2265585699814128},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.1249909704264685, 2.452895534992326}, 
          0.05343921452845494], 
         InsetBox["4", {2.1249909704264685, 2.452895534992326},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.7087043053910814, 2.4530810339704474}, 
          0.05343921452845494], 
         InsetBox["5", {0.7087043053910814, 2.4530810339704474},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 1.2278301842640489}, 0.05343921452845494], 
         InsetBox["6", {0., 1.2278301842640489},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.1493073953353186, 0.763001016404376}, 
          0.05343921452845494], 
         InsetBox["7", {1.1493073953353186, 0.763001016404376},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.6836991600385602, 0.7636387931506287}, 
          0.05343921452845494], 
         InsetBox["8", {1.6836991600385602, 0.7636387931506287},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.952934675305475, 1.2266188040854957}, 
          0.05343921452845494], 
         InsetBox["9", {1.952934675305475, 1.2266188040854957},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.6840025605623565, 1.6898280281048312}, 
          0.05343921452845494], 
         InsetBox["10", {1.6840025605623565, 1.6898280281048312},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.1488531370174286, 1.690607776763056}, 
          0.05343921452845494], 
         InsetBox["11", {1.1488531370174286, 1.690607776763056},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8815955604571679, 1.2262384437093572}, 
          0.05343921452845494], 
         InsetBox["12", {0.8815955604571679, 1.2262384437093572},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[117]=",ExpressionUUID->"a99acdf7-2535-4fee-bccf-ef54c0341880"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has rearranged the locations of the vertices. If you prefer the \
three-dimensional appearance of the prism, you can impose those locations by \
setting the ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " property when performing the union. The ",
 StyleBox[ButtonBox["GraphEmbedding",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphEmbedding"], "FunctionLink"],
 " function applied to a graph returns a list of all the coordinates of the \
vertices. Since we did not add any new vertices here, we can simply apply ",
 StyleBox[ButtonBox["GraphEmbedding",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphEmbedding"], "FunctionLink"],
 " to the original graph."
}], "Text",ExpressionUUID->"797a8dea-9284-4aee-935d-b02210f23df0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"unionExample", "=", 
  RowBox[{"GraphUnion", "[", 
   RowBox[{"unionExampleA", ",", "unionExampleB", ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"GraphEmbedding", "[", "unionExampleA", "]"}]}]}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[118]:=",ExpressionUUID->"b981b866-1e49-4371-848c-02efe7cf0380"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12}, {
      Null, {{1, 2}, {1, 7}, {2, 3}, {2, 8}, {3, 4}, {3, 9}, {4, 5}, {4, 
       10}, {5, 6}, {5, 11}, {6, 1}, {6, 12}, {7, 8}, {7, 9}, {7, 10}, {7, 
       11}, {7, 12}, {8, 9}, {8, 10}, {8, 11}, {8, 12}, {9, 10}, {9, 11}, {9, 
       12}, {10, 11}, {10, 12}, {11, 12}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       VertexCoordinates -> {{0.5983109393792555, 1.333510780126811}, {
        0.9839435806937917, 2.4015105280929165`}, {2.099708266496932, 
        2.3764856914994263`}, {2.7301971795384574`, 1.4401369864781073`}, {
        2.333865160413755, 0.39609325735504175`}, {1.199176045955087, 
        0.4394265853678305}, {0., 0.9620430219770666}, {0.39587622224175434`, 
        2.005570642165643}, {1.5313638191412857`, 1.9620109994805106`}, {
        2.1323633389347414`, 1.0677650125596039`}, {1.746198062974075, 0.}, {
        0.6298912750595992, 0.025825868311962763`}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          0.9839435806937917, 2.4015105280929165`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {
          1.199176045955087, 0.4394265853678305}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5983109393792555, 1.333510780126811}, {0., 
          0.9620430219770666}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          2.099708266496932, 2.3764856914994263`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9839435806937917, 2.4015105280929165`}, {
          0.39587622224175434`, 2.005570642165643}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          2.7301971795384574`, 1.4401369864781073`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.099708266496932, 2.3764856914994263`}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.333865160413755, 0.39609325735504175`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.7301971795384574`, 1.4401369864781073`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.199176045955087, 0.4394265853678305}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.333865160413755, 0.39609325735504175`}, {
          1.746198062974075, 0.}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.199176045955087, 0.4394265853678305}, {
          0.6298912750595992, 0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {0.39587622224175434`, 
          2.005570642165643}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {1.5313638191412857`, 
          1.9620109994805106`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {2.1323633389347414`, 
          1.0677650125596039`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {1.746198062974075, 0.}}, 
          0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.9620430219770666}, {0.6298912750595992, 
          0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
          1.5313638191412857`, 1.9620109994805106`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
          1.746198062974075, 0.}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.39587622224175434`, 2.005570642165643}, {
          0.6298912750595992, 0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5313638191412857`, 1.9620109994805106`}, {
          2.1323633389347414`, 1.0677650125596039`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5313638191412857`, 1.9620109994805106`}, {
          1.746198062974075, 0.}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.5313638191412857`, 1.9620109994805106`}, {
          0.6298912750595992, 0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1323633389347414`, 1.0677650125596039`}, {
          1.746198062974075, 0.}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1323633389347414`, 1.0677650125596039`}, {
          0.6298912750595992, 0.025825868311962763`}}, 0.0701245984515375]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.746198062974075, 0.}, {0.6298912750595992, 
          0.025825868311962763`}}, 0.0701245984515375]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.5983109393792555, 1.333510780126811}, 0.0701245984515375],
          InsetBox["1", {0.5983109393792555, 1.333510780126811},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9839435806937917, 2.4015105280929165}, 
          0.0701245984515375], 
         InsetBox["2", {0.9839435806937917, 2.4015105280929165},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.099708266496932, 2.3764856914994263}, 0.0701245984515375],
          InsetBox["3", {2.099708266496932, 2.3764856914994263},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.7301971795384574, 1.4401369864781073}, 
          0.0701245984515375], 
         InsetBox["4", {2.7301971795384574, 1.4401369864781073},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.333865160413755, 0.39609325735504175}, 
          0.0701245984515375], 
         InsetBox["5", {2.333865160413755, 0.39609325735504175},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.199176045955087, 0.4394265853678305}, 0.0701245984515375],
          InsetBox["6", {1.199176045955087, 0.4394265853678305},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.9620430219770666}, 0.0701245984515375], 
         InsetBox["7", {0., 0.9620430219770666},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.39587622224175434, 2.005570642165643}, 
          0.0701245984515375], 
         InsetBox["8", {0.39587622224175434, 2.005570642165643},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.5313638191412857, 1.9620109994805106}, 
          0.0701245984515375], 
         InsetBox["9", {1.5313638191412857, 1.9620109994805106},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.1323633389347414, 1.0677650125596039}, 
          0.0701245984515375], 
         InsetBox["10", {2.1323633389347414, 1.0677650125596039},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.746198062974075, 0.}, 0.0701245984515375], 
         InsetBox["11", {1.746198062974075, 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.6298912750595992, 0.025825868311962763}, 
          0.0701245984515375], 
         InsetBox["12", {0.6298912750595992, 0.025825868311962763},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[118]=",ExpressionUUID->"5abba517-d2b1-4ea3-b651-911d1e4e624c"]
}, Open  ]],

Cell[TextData[{
 "Finally, we consider graph complements, described in Exercise 61 of Section \
10.2. The complement, ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["G", "_"], TraditionalForm]],ExpressionUUID->
  "f9fc2fbb-8c9a-411a-ab71-30e2182713cc"],
 ", of a graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "118f22d8-d5e9-42b8-a4bd-3ca799775ca1"],
 " is the graph whose vertex set is the same as that of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "171b97c9-6b46-40dd-8045-c2bf7044e797"],
 ", but whose edge set is the set of all pairs of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "b25c5a42-1320-4af1-b5fc-0a8e34adf426"],
 " that have no edge between them. In other words, if ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "cdea6dc9-a192-4b17-8ebd-8d4aa64467c0"],
 " has ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "908be30e-5144-4d61-b286-1d27360c50f8"],
 " vertices, then the edge set of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "16f1873a-3243-4d34-a7ff-fead4099ab12"],
 " is the complement of the edge set of ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "def22d8d-dbc7-40c6-8401-939840bca6a4"],
 " relative to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "n"], TraditionalForm]],ExpressionUUID->
  "66a5315f-fc66-4241-a09e-c4ea57d807d2"],
 ", the complete graph on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "891ca2e8-26ba-4805-80d8-e016f05f485b"],
 " vertices. The Wolfram Language has a function to compute the complement of \
a graph: ",
 StyleBox[ButtonBox["GraphComplement",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphComplement"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"934a6472-ff8e-4d97-b123-5de6bfc86662"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WheelGraph", "[", 
  RowBox[{"8", ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Small"}]}], "]"}]], "Input",
 CellLabel->
  "In[119]:=",ExpressionUUID->"80426a08-1d22-4bff-bef9-35b11ceab158"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 7, 10, 13, 16, 19, 22, 25, 28}, {{2}, {3}, {4}, {5}, {6}, {
          7}, {8}, {1}, {3}, {8}, {1}, {2}, {4}, {1}, {3}, {5}, {1}, {4}, {
          6}, {1}, {5}, {7}, {1}, {6}, {8}, {1}, {2}, {7}}}, Pattern}]}, {
      GraphLayout -> "StarEmbedding", VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Small}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {-0.9749279121818235, -0.22252093395631342`}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {-0.43388373911755757`, -0.9009688679024187}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {0.43388373911755884`, -0.9009688679024196}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {0.9749279121818237, -0.2225209339563155}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {0.7818314824680294, 0.6234898018587327}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {-7.044813998280222*^-16, 1.}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {-0.7818314824680302, 0.6234898018587351}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, \
{-0.43388373911755757`, -0.9009688679024187}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, \
{-0.7818314824680302, 0.6234898018587351}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.43388373911755884`, -0.9009688679024196}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.43388373911755884`, -0.9009688679024196}, {
          0.9749279121818237, -0.2225209339563155}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9749279121818237, -0.2225209339563155}, {
          0.7818314824680294, 0.6234898018587327}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7818314824680294, 
          0.6234898018587327}, {-7.044813998280222*^-16, 1.}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-7.044813998280222*^-16, 1.}, {-0.7818314824680302, 
          0.6234898018587351}}, 0.04338837391175577]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.}, 0.04338837391175577], 
         InsetBox["1", {0., 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9749279121818235, -0.22252093395631342}, 
          0.04338837391175577], 
         InsetBox["2", {-0.9749279121818235, -0.22252093395631342},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.43388373911755757, -0.9009688679024187}, 
          0.04338837391175577], 
         InsetBox["3", {-0.43388373911755757, -0.9009688679024187},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.43388373911755884, -0.9009688679024196}, 
          0.04338837391175577], 
         InsetBox["4", {0.43388373911755884, -0.9009688679024196},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9749279121818237, -0.2225209339563155}, 
          0.04338837391175577], 
         InsetBox["5", {0.9749279121818237, -0.2225209339563155},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.7818314824680294, 0.6234898018587327}, 
          0.04338837391175577], 
         InsetBox["6", {0.7818314824680294, 0.6234898018587327},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.04338837391175577], 
         InsetBox["7", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.7818314824680302, 0.6234898018587351}, 
          0.04338837391175577], 
         InsetBox["8", {-0.7818314824680302, 0.6234898018587351},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[119]=",ExpressionUUID->"f5ac5f79-f95d-4fe3-b627-7155b07261ba"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphComplement", "[", 
  RowBox[{
   RowBox[{"WheelGraph", "[", "8", "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Small"}]}], "]"}]], "Input",
 CellLabel->
  "In[120]:=",ExpressionUUID->"d6dad0b9-49fa-40a3-8ed1-c526f6b61f19"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 0, 4, 8, 12, 16, 20, 24, 28}, {{4}, {5}, {6}, {7}, {5}, {6}, {
          7}, {8}, {2}, {6}, {7}, {8}, {2}, {3}, {7}, {8}, {2}, {3}, {4}, {
          8}, {2}, {3}, {4}, {5}, {3}, {4}, {5}, {6}}}, Pattern}]}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Small}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-1., 1.0106430996148606`*^-15}, {
          6.049014748177263*^-16, -1.}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1., 1.0106430996148606`*^-15}, {
          0.7071067811865481, -0.7071067811865483}}, 
          0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1., 1.0106430996148606`*^-15}, {
          1., -1.133107779529596*^-15}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1., 1.0106430996148606`*^-15}, {0.707106781186547, 
          0.7071067811865467}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7071067811865471, -0.7071067811865468}, {
          0.7071067811865481, -0.7071067811865483}}, 
          0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7071067811865471, -0.7071067811865468}, {
          1., -1.133107779529596*^-15}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7071067811865471, -0.7071067811865468}, {
          0.707106781186547, 0.7071067811865467}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.7071067811865471, -0.7071067811865468}, \
{-7.044813998280222*^-16, 1.}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{6.049014748177263*^-16, -1.}, {
          1., -1.133107779529596*^-15}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{6.049014748177263*^-16, -1.}, {0.707106781186547, 
          0.7071067811865467}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{6.049014748177263*^-16, -1.}, {-7.044813998280222*^-16, 
          1.}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7071067811865481, -0.7071067811865483}, {
          0.707106781186547, 0.7071067811865467}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.7071067811865481, -0.7071067811865483}, {-7.044813998280222*^-16, 
          1.}}, 0.038268343236508955`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., -1.133107779529596*^-15}, {-7.044813998280222*^-16, 
          1.}}, 0.038268343236508955`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.707106781186548, 0.7071067811865482}, 
          0.038268343236508955], 
         InsetBox["1", {-0.707106781186548, 0.7071067811865482},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1., 1.0106430996148606*^-15}, 0.038268343236508955], 
         InsetBox["2", {-1., 1.0106430996148606*^-15},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.7071067811865471, -0.7071067811865468}, 
          0.038268343236508955], 
         InsetBox["3", {-0.7071067811865471, -0.7071067811865468},
          BaseStyle->"Graphics"]}, {
         DiskBox[{6.049014748177263*^-16, -1.}, 0.038268343236508955], 
         InsetBox["4", {6.049014748177263*^-16, -1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.7071067811865481, -0.7071067811865483}, 
          0.038268343236508955], 
         InsetBox["5", {0.7071067811865481, -0.7071067811865483},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1., -1.133107779529596*^-15}, 0.038268343236508955], 
         InsetBox["6", {1., -1.133107779529596*^-15},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.707106781186547, 0.7071067811865467}, 
          0.038268343236508955], 
         InsetBox["7", {0.707106781186547, 0.7071067811865467},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.038268343236508955], 
         InsetBox["8", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[120]=",ExpressionUUID->"e1049834-a189-45ca-a3f0-759d5022ef36"]
}, Open  ]],

Cell[TextData[{
 "Again, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has rearranged the vertices. We can impose the original locations as \
follows."
}], "Text",ExpressionUUID->"a268771e-f795-4c40-b9d4-323d57ca01fc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"complementExample", "=", 
  RowBox[{"GraphComplement", "[", 
   RowBox[{
    RowBox[{"WheelGraph", "[", "8", "]"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Small"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"GraphEmbedding", "[", 
      RowBox[{"WheelGraph", "[", "8", "]"}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[121]:=",ExpressionUUID->"52930a25-f467-4b99-afd8-bb4bc94ceac4"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 0, 4, 8, 12, 16, 20, 24, 28}, {{4}, {5}, {6}, {7}, {5}, {6}, {
          7}, {8}, {2}, {6}, {7}, {8}, {2}, {3}, {7}, {8}, {2}, {3}, {4}, {
          8}, {2}, {3}, {4}, {5}, {3}, {4}, {5}, {6}}}, Pattern}]}, {
      VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Small}, 
       VertexCoordinates -> {{0., 
        0.}, {-0.9749279121818235, -0.22252093395631342`}, \
{-0.43388373911755757`, -0.9009688679024187}, {
        0.43388373911755884`, -0.9009688679024196}, {
        0.9749279121818237, -0.2225209339563155}, {0.7818314824680294, 
        0.6234898018587327}, {-7.044813998280222*^-16, 
        1.}, {-0.7818314824680302, 0.6234898018587351}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.43388373911755884`, -0.9009688679024196}}, 
          0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.9749279121818237, -0.2225209339563155}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, {
          0.7818314824680294, 0.6234898018587327}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9749279121818235, -0.22252093395631342`}, \
{-7.044813998280222*^-16, 1.}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.9749279121818237, -0.2225209339563155}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, {
          0.7818314824680294, 0.6234898018587327}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, \
{-7.044813998280222*^-16, 1.}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.43388373911755757`, -0.9009688679024187}, \
{-0.7818314824680302, 0.6234898018587351}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.43388373911755884`, -0.9009688679024196}, {
          0.7818314824680294, 0.6234898018587327}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.43388373911755884`, -0.9009688679024196}, \
{-7.044813998280222*^-16, 1.}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.43388373911755884`, -0.9009688679024196}, {-0.7818314824680302, 
          0.6234898018587351}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.9749279121818237, -0.2225209339563155}, {-7.044813998280222*^-16, 
          1.}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.9749279121818237, -0.2225209339563155}, {-0.7818314824680302, 
          0.6234898018587351}}, 0.04338837391175577]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.7818314824680294, 
          0.6234898018587327}, {-0.7818314824680302, 0.6234898018587351}}, 
          0.04338837391175577]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.}, 0.04338837391175577], 
         InsetBox["1", {0., 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9749279121818235, -0.22252093395631342}, 
          0.04338837391175577], 
         InsetBox["2", {-0.9749279121818235, -0.22252093395631342},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.43388373911755757, -0.9009688679024187}, 
          0.04338837391175577], 
         InsetBox["3", {-0.43388373911755757, -0.9009688679024187},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.43388373911755884, -0.9009688679024196}, 
          0.04338837391175577], 
         InsetBox["4", {0.43388373911755884, -0.9009688679024196},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9749279121818237, -0.2225209339563155}, 
          0.04338837391175577], 
         InsetBox["5", {0.9749279121818237, -0.2225209339563155},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.7818314824680294, 0.6234898018587327}, 
          0.04338837391175577], 
         InsetBox["6", {0.7818314824680294, 0.6234898018587327},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.04338837391175577], 
         InsetBox["7", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.7818314824680302, 0.6234898018587351}, 
          0.04338837391175577], 
         InsetBox["8", {-0.7818314824680302, 0.6234898018587351},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[121]=",ExpressionUUID->"c2984c2a-cc47-4d85-9b58-f30c5c1dcb51"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.3 Representing Graphs and Graph Isomorphism", "Section",
 CellTags->"Section3",ExpressionUUID->"13a377f2-f887-483f-af22-7be649862fe0"],

Cell["\<\
In this section, we will see how to represent graphs in terms of adjacency \
matrices, adjacency lists, and incidence matrices. We will then use the \
adjacency matrix representation to help determine whether two graphs are \
isomorphic. \
\>", "Text",ExpressionUUID->"f3d1bc53-8eae-47cb-a8ab-65edbc029a30"],

Cell[CellGroupData[{

Cell["Adjacency Matrices", "Subsection",
 CellTags->"Section3a",ExpressionUUID->"bb97f9de-d74d-4260-bc5f-37c7990131ee"],

Cell[TextData[{
 "The adjacency matrix of a graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "4e223436-c9d7-4037-9b05-ff03fd5a6749"],
 " with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "ce327832-4ac4-424c-80d1-89dd27d9b658"],
 " vertices is the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Times]", "n"}], TraditionalForm]],ExpressionUUID->
  "7db9180e-1bed-428f-b3a1-a8d38a3b743c"],
 " matrix whose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"i", ",", "j"}], ")"}], TraditionalForm]],ExpressionUUID->
  "d63e6a39-3281-4b0e-89ed-24c2e457f2db"],
 " entry is 1 if there is an edge from vertex ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]],ExpressionUUID->
  "cb5a526f-ccc6-42c8-bd96-60f8541ca3f5"],
 " to vertex ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]],ExpressionUUID->
  "b9656fed-e3e6-4388-8b5f-66a2f4d75d51"],
 " and 0 if not. You can define a graph by passing an adjacency matrix, \
represented as a list of lists, to the function ",
 StyleBox[ButtonBox["AdjacencyGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyGraph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"106a8cce-33e6-4ea3-b750-83fc8a0553f4"],

Cell["As an example, we reproduce Example 4 from Section 10.3.", "Text",ExpressionUUID->"85c0c982-0756-4148-a26a-e7145a79b964"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"exampleAdjM", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "1", ",", "0"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"exampleAdjM", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[122]:=",ExpressionUUID->"66bd552e-7a55-4756-9563-2bf7b8e3903b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "1", "1", "0"},
     {"1", "0", "0", "1"},
     {"1", "0", "0", "1"},
     {"0", "1", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellLabel->
  "Out[123]//MatrixForm=",ExpressionUUID->"0b4948c0-8337-4986-867d-\
ca41aafea52b"]
}, Open  ]],

Cell[TextData[{
 "Recall that we must invoke ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 " in an expression separate from the definition of the symbol in order to \
avoid having the ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 " head permanently stored in the symbol."
}], "Text",ExpressionUUID->"039676a4-45d8-42bb-b8c6-ee4b0a0af1da"],

Cell[TextData[{
 "We now invoke the ",
 StyleBox[ButtonBox["AdjacencyGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyGraph"], "FunctionLink"],
 " function with this matrix."
}], "Text",ExpressionUUID->"df3eebde-344f-415d-ae96-4bd12885edc1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AdjacencyGraph", "[", "exampleAdjM", "]"}]], "Input",
 CellLabel->
  "In[124]:=",ExpressionUUID->"8233d738-1050-4749-8148-b2e1e472134f"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4}, {Null, 
       SparseArray[
       Automatic, {4, 4}, 0, {
        1, {{0, 2, 4, 6, 8}, {{2}, {3}, {1}, {4}, {1}, {4}, {2}, {3}}}, 
         Pattern}]}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-1., 1.0106430996148606`*^-15}, {
       6.049014748177263*^-16, -1.}, {
       1., -1.133107779529596*^-15}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.02261146496815286]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.02261146496815286]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02261146496815286], DiskBox[2, 0.02261146496815286], 
         DiskBox[3, 0.02261146496815286], DiskBox[4, 0.02261146496815286]}}]],
     
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[124]=",ExpressionUUID->"77baca62-abdf-4adf-9c29-36ede66cd67c"]
}, Open  ]],

Cell[TextData[{
 "In the textbook, the vertices for this graph were labeled as letters rather \
than numbers. The ",
 StyleBox[ButtonBox["AdjacencyGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyGraph"], "FunctionLink"],
 " function accepts a list of names for the vertices as an optional first \
argument. It also accepts the usual graph options."
}], "Text",ExpressionUUID->"0ad54285-a0bf-4e8a-98db-d11cb0e00adc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AdjacencyGraph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
    "}"}], ",", "exampleAdjM", ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Small"}]}], "]"}]], "Input",
 CellLabel->
  "In[125]:=",ExpressionUUID->"db4d2cb9-c97d-4e16-90ac-85214be4f6ae"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d"}, {Null, 
       SparseArray[
       Automatic, {4, 4}, 0, {
        1, {{0, 2, 4, 6, 8}, {{2}, {3}, {1}, {4}, {1}, {4}, {2}, {3}}}, 
         Pattern}]}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Small}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-1., 1.0106430996148606`*^-15}, {
          6.049014748177263*^-16, -1.}}, 0.0707106781186547]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1., 1.0106430996148606`*^-15}, {
          1., -1.133107779529596*^-15}}, 0.0707106781186547]}, 
        {Arrowheads[0.], 
         ArrowBox[{{6.049014748177263*^-16, -1.}, {-7.044813998280222*^-16, 
          1.}}, 0.0707106781186547]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., -1.133107779529596*^-15}, {-7.044813998280222*^-16, 
          1.}}, 0.0707106781186547]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{-1., 1.0106430996148606*^-15}, 0.0707106781186547], 
         InsetBox["\<\"a\"\>", {-1., 1.0106430996148606*^-15},
          BaseStyle->"Graphics"]}, {
         DiskBox[{6.049014748177263*^-16, -1.}, 0.0707106781186547], 
         InsetBox["\<\"b\"\>", {6.049014748177263*^-16, -1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1., -1.133107779529596*^-15}, 0.0707106781186547], 
         InsetBox["\<\"c\"\>", {1., -1.133107779529596*^-15},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.0707106781186547], 
         InsetBox["\<\"d\"\>", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[125]=",ExpressionUUID->"dfdc4e26-a174-4030-84b9-fce0ea930e26"]
}, Open  ]],

Cell["\<\
Notice that this is the same graph as is produced in the textbook, with the \
exception of the locations of the vertices. \
\>", "Text",ExpressionUUID->"8618e784-2a99-41f7-b878-2723e02f9b33"],

Cell[TextData[{
 "The Wolfram Language also provides a function, ",
 StyleBox[ButtonBox["AdjacencyMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyMatrix"], "FunctionLink"],
 ", for computing the adjacency matrix of a graph. The output of this \
function is always a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " object, which is more efficient at storing large arrays with many entries \
0. You can display a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " with ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 ", as usual, and you can convert it into the usual list of lists \
representation with the function ",
 StyleBox[ButtonBox["Normal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Normal"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"df397c3f-f3d1-47cc-a093-47045144ba6d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wheelAdjacency", "=", 
  RowBox[{"AdjacencyMatrix", "[", 
   RowBox[{"WheelGraph", "[", "7", "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[126]:=",ExpressionUUID->"d80d1742-a7f4-4cdf-a16c-88738ae20221"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   TagBox["SparseArray",
    "SummaryHead"], "[", 
   DynamicModuleBox[{Typeset`open$$ = False, Typeset`embedState$$ = "Ready"}, 
    
    TemplateBox[{PaneSelectorBox[{False -> GridBox[{{
            PaneBox[
             ButtonBox[
              DynamicBox[
               FEPrivate`FrontEndResource[
               "FEBitmaps", "SquarePlusIconMedium"]], 
              ButtonFunction :> (Typeset`open$$ = True), Appearance -> None, 
              Evaluator -> Automatic, Method -> "Preemptive"], 
             Alignment -> {Center, Center}, ImageSize -> 
             Dynamic[{
               Automatic, 
                3.5 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                 Magnification])}]], 
            GraphicsBox[
             RasterBox[CompressedData["
1:eJxTTMoPSmVmYGBgh2IQe2e3gevOLS/sb7gvX1cz7a39yYjcf/Ff39vjErcq
3x03S+mc/RzjRVvMf1yxf7FI8XHgqlcki5Nq/lBxD7XcSevwGWzqaR3OQyX9
DDb3DJQ4AJc7j+A=
              "], {{0, 0}, {7, 7}}, {0, 1}], AspectRatio -> 
             1, Axes -> False, Frame -> False, FrameLabel -> {None, None}, 
             FrameStyle -> Directive[
               Opacity[0.5], 
               Thickness[Tiny], 
               RGBColor[0.368417, 0.506779, 0.709798]], 
             FrameTicks -> {{None, None}, {None, None}}, GridLinesStyle -> 
             Directive[
               GrayLevel[0.5, 0.4]], ImageSize -> 
             Dynamic[{
               Automatic, 
                3.5 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                 Magnification])}], 
             Method -> {
              "AxisPadding" -> Scaled[0.02], "DefaultBoundaryStyle" -> 
               Automatic, "DefaultPlotStyle" -> Automatic, "DomainPadding" -> 
               Scaled[0.02], "RangePadding" -> Scaled[0.05]}], 
            GridBox[{{
               RowBox[{
                 TagBox["\"Specified elements: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["24", "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Dimensions: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox[
                  RowBox[{"{", 
                    RowBox[{"7", ",", "7"}], "}"}], "SummaryItem"]}]}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}}, AutoDelete -> 
             False, GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{2}}, "Rows" -> {{Automatic}}},
              BaseStyle -> {
              ShowStringCharacters -> False, NumberMarks -> False, 
               PrintPrecision -> 3, ShowSyntaxStyles -> False}]}}, 
          GridBoxAlignment -> {"Rows" -> {{Top}}}, AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          BaselinePosition -> {1, 1}], True -> GridBox[{{
            PaneBox[
             ButtonBox[
              DynamicBox[
               FEPrivate`FrontEndResource[
               "FEBitmaps", "SquareMinusIconMedium"]], 
              ButtonFunction :> (Typeset`open$$ = False), Appearance -> None, 
              Evaluator -> Automatic, Method -> "Preemptive"], 
             Alignment -> {Center, Center}, ImageSize -> 
             Dynamic[{
               Automatic, 
                3.5 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                 Magnification])}]], 
            GraphicsBox[
             RasterBox[CompressedData["
1:eJxTTMoPSmVmYGBgh2IQe2e3gevOLS/sb7gvX1cz7a39yYjcf/Ff39vjErcq
3x03S+mc/RzjRVvMf1yxf7FI8XHgqlcki5Nq/lBxD7XcSevwGWzqaR3OQyX9
DDb3DJQ4AJc7j+A=
              "], {{0, 0}, {7, 7}}, {0, 1}], AspectRatio -> 
             1, Axes -> False, Frame -> False, FrameLabel -> {None, None}, 
             FrameStyle -> Directive[
               Opacity[0.5], 
               Thickness[Tiny], 
               RGBColor[0.368417, 0.506779, 0.709798]], 
             FrameTicks -> {{None, None}, {None, None}}, GridLinesStyle -> 
             Directive[
               GrayLevel[0.5, 0.4]], ImageSize -> 
             Dynamic[{
               Automatic, 
                3.5 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                 Magnification])}], 
             Method -> {
              "AxisPadding" -> Scaled[0.02], "DefaultBoundaryStyle" -> 
               Automatic, "DefaultPlotStyle" -> Automatic, "DomainPadding" -> 
               Scaled[0.02], "RangePadding" -> Scaled[0.05]}], 
            GridBox[{{
               RowBox[{
                 TagBox["\"Specified elements: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["24", "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Dimensions: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox[
                  RowBox[{"{", 
                    RowBox[{"7", ",", "7"}], "}"}], "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Default: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["0", "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Density: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["0.4897959183673469`", "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Elements:\"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"\"", "SummaryItem"]}]}, {
               TagBox[
                TagBox[
                 GridBox[{{
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"1", ",", "2"}], "}"}], "\[Rule]", 
                    StyleBox[
                    PaneBox[
                    "1", ContentPadding -> False, FrameMargins -> 0, 
                    StripOnInput -> True, BaselinePosition -> Baseline, 
                    ImageSize -> {{1, 300}, Automatic}], LineBreakWithin -> 
                    False]}]}, {
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"1", ",", "3"}], "}"}], "\[Rule]", 
                    StyleBox[
                    PaneBox[
                    "1", ContentPadding -> False, FrameMargins -> 0, 
                    StripOnInput -> True, BaselinePosition -> Baseline, 
                    ImageSize -> {{1, 300}, Automatic}], LineBreakWithin -> 
                    False]}]}, {
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"1", ",", "4"}], "}"}], "\[Rule]", 
                    StyleBox[
                    PaneBox[
                    "1", ContentPadding -> False, FrameMargins -> 0, 
                    StripOnInput -> True, BaselinePosition -> Baseline, 
                    ImageSize -> {{1, 300}, Automatic}], LineBreakWithin -> 
                    False]}]}, {
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{"1", ",", "5"}], "}"}], "\[Rule]", 
                    StyleBox[
                    PaneBox[
                    "1", ContentPadding -> False, FrameMargins -> 0, 
                    StripOnInput -> True, BaselinePosition -> Baseline, 
                    ImageSize -> {{1, 300}, Automatic}], LineBreakWithin -> 
                    False]}]}, {
                    ItemBox[
                    "\"\[VerticalEllipsis]\"", Alignment -> Center, 
                    StripOnInput -> False]}}, 
                  GridBoxAlignment -> {"Columns" -> {{Left}}}, 
                  DefaultBaseStyle -> "Column", 
                  GridBoxItemSize -> {
                   "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], 
                 "Column"], "SummaryItem"]}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}}, AutoDelete -> 
             False, GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{2}}, "Rows" -> {{Automatic}}},
              BaseStyle -> {
              ShowStringCharacters -> False, NumberMarks -> False, 
               PrintPrecision -> 3, ShowSyntaxStyles -> False}]}}, 
          GridBoxAlignment -> {"Rows" -> {{Top}}}, AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          BaselinePosition -> {1, 1}]}, 
       Dynamic[Typeset`open$$], ImageSize -> Automatic]},
     "SummaryPanel"],
    DynamicModuleValues:>{}], "]"}],
  SparseArray[
  Automatic, {7, 7}, 0, {
   1, {{0, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {4}, {5}, {6}, {7}, {1}, {
     3}, {7}, {1}, {2}, {4}, {1}, {3}, {5}, {1}, {4}, {6}, {1}, {5}, {7}, {
     1}, {2}, {6}}}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1}}],
  Editable->False,
  SelectWithContents->True,
  Selectable->False]], "Output",
 CellLabel->
  "Out[126]=",ExpressionUUID->"4b1a988a-c119-4cc9-84cf-7c5d5f06141f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wheelAdjacency", "//", "MatrixForm"}]], "Input",
 CellLabel->
  "In[127]:=",ExpressionUUID->"88cb2818-5f9e-449f-9656-b681107a46ec"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "1", "1", "1", "1", "1", "1"},
     {"1", "0", "1", "0", "0", "0", "1"},
     {"1", "1", "0", "1", "0", "0", "0"},
     {"1", "0", "1", "0", "1", "0", "0"},
     {"1", "0", "0", "1", "0", "1", "0"},
     {"1", "0", "0", "0", "1", "0", "1"},
     {"1", "1", "0", "0", "0", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[
   SparseArray[
    Automatic, {7, 7}, 0, {
     1, {{0, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {4}, {5}, {6}, {7}, {1}, {
       3}, {7}, {1}, {2}, {4}, {1}, {3}, {5}, {1}, {4}, {6}, {1}, {5}, {7}, {
       1}, {2}, {6}}}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
      1, 1, 1, 1, 1, 1}}]]]]], "Output",
 CellLabel->
  "Out[127]//MatrixForm=",ExpressionUUID->"df466704-1064-4997-a7f8-\
c807c3e7ac21"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"wheelAdjacency", "//", "Normal"}]], "Input",
 CellLabel->
  "In[128]:=",ExpressionUUID->"ea58eef8-7ff5-405f-bda9-942ebda0d2a2"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "0", ",", "0", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "1", ",", "0", ",", "0", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "1", ",", "0", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "1", ",", "0", ",", "1", ",", "0"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "0", ",", "0", ",", "1", ",", "0", ",", "1"}],
     "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "0", ",", "0", ",", "0", ",", "1", ",", "0"}],
     "}"}]}], "}"}]], "Output",
 CellLabel->
  "Out[128]=",ExpressionUUID->"d72e5992-2c62-4318-8a25-7dd8bd42562b"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Adjacency Lists", "Subsection",
 CellTags->"Section3b",ExpressionUUID->"087546c0-0b7a-4730-b97e-d29b61b0d382"],

Cell["\<\
Recall that a representation of a graph as an adjacency list consists of the \
lists of neighbors of each vertex.\
\>", "Text",ExpressionUUID->"a63cc783-91f2-454e-94db-d45ae007bf02"],

Cell[TextData[{
 "Earlier, we saw the function ",
 StyleBox[ButtonBox["AdjacencyList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyList"], "FunctionLink"],
 " used to determine the list of vertices adjacent to a given vertex. For \
example, the following determines the vertices adjacent to vertex 2 in the \
wheel graph on 7 vertices."
}], "Text",ExpressionUUID->"1dd4b689-177e-4116-9eb3-afbee681cbf9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AdjacencyList", "[", 
  RowBox[{
   RowBox[{"WheelGraph", "[", "7", "]"}], ",", "2"}], "]"}]], "Input",
 CellLabel->
  "In[129]:=",ExpressionUUID->"d369e03e-d0ca-45cf-90fd-836bdd3eed6e"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "7"}], "}"}]], "Output",
 CellLabel->
  "Out[129]=",ExpressionUUID->"772b280b-6863-4307-a90a-a683c794eb56"]
}, Open  ]],

Cell[TextData[{
 "By looping over the vertex set, we create a function that returns an ",
 StyleBox[ButtonBox["Association",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"], "FunctionLink"],
 " containing the adjacency lists. Note that applying ",
 StyleBox[ButtonBox["Dataset",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Dataset"], "FunctionLink"],
 " to a single ",
 StyleBox[ButtonBox["Association",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"], "FunctionLink"],
 ", displays a table with the rows labeled by the keys."
}], "Text",ExpressionUUID->"57855267-76e8-446f-83d8-65e3615a7d41"],

Cell[BoxData[
 RowBox[{
  RowBox[{"adjacencyList", "[", "G_Graph", "]"}], ":=", 
  RowBox[{"Association", "@@", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"v", "\[Rule]", 
      RowBox[{"AdjacencyList", "[", 
       RowBox[{"G", ",", "v"}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"v", ",", 
       RowBox[{"VertexList", "[", "G", "]"}]}], "}"}]}], "]"}]}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[130]:=",ExpressionUUID->"5ea10e16-5dad-4a24-9cce-3e9f949b750f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"adjacencyList", "[", 
   RowBox[{"WheelGraph", "[", "7", "]"}], "]"}], "//", "Dataset"}]], "Input",
 CellLabel->
  "In[131]:=",ExpressionUUID->"e76f7788-7213-4252-a38b-5e0b4e12d532"],

Cell[BoxData[
 TemplateBox[{TagBox[
    DynamicModuleBox[{
     TypeSystem`NestedGrid`PackagePrivate`$state$$ = 
      Data`UnorderedAssociation[
       "RowCount" -> 7, "Length" -> 7, "InitialType" -> TypeSystem`Assoc[
          TypeSystem`Atom[Integer], 
          TypeSystem`Vector[
           TypeSystem`Atom[Integer], TypeSystem`AnyLength], 7], "Shape" -> 
        TypeSystem`PackageScope`Limited[
          TypeSystem`PackageScope`ColumnShape[
           TypeSystem`PackageScope`KeyValueShape[
            TypeSystem`PackageScope`AtomShape[34], 
            TypeSystem`PackageScope`UnknownShape]], 7, {}], "InitialData" -> 
        TypeSystem`PackageScope`CacheHandle[390989022], "Type" -> 
        TypeSystem`Assoc[
          TypeSystem`Atom[Integer], 
          TypeSystem`Vector[
           TypeSystem`Atom[Integer], TypeSystem`AnyLength], 7], "HandleQ" -> 
        True, "RowTarget" -> 20, "Data" -> 
        TypeSystem`PackageScope`CacheHandle[390989022]], 
      TypeSystem`NestedGrid`PackagePrivate`$path$$ = {}, 
      TypeSystem`NestedGrid`PackagePrivate`$pos$$ = 1, 
      TypeSystem`NestedGrid`PackagePrivate`$grid$$ = 
      DynamicModule[{
        TypeSystem`NestedGrid`PackagePrivate`renderedGrid = Deploy[
           Style[
            Grid[{{
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["1"], 
                    Style[
                    RawBoxes["1"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[1]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[1]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{2, 3, 4, 5, 6, 7}, ShowStringCharacters -> False], 
                    
                    Style[
                    Style[{2, 3, 4, 5, 6, 7}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[1]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[1]]], "Mouse"]]}, {
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["2"], 
                    Style[
                    RawBoxes["2"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[2]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[2]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{1, 3, 7}, ShowStringCharacters -> False], 
                    Style[
                    Style[{1, 3, 7}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[2]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[2]]], "Mouse"]]}, {
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["3"], 
                    Style[
                    RawBoxes["3"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[3]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[3]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{1, 2, 4}, ShowStringCharacters -> False], 
                    Style[
                    Style[{1, 2, 4}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[3]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[3]]], "Mouse"]]}, {
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["4"], 
                    Style[
                    RawBoxes["4"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[4]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[4]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{1, 3, 5}, ShowStringCharacters -> False], 
                    Style[
                    Style[{1, 3, 5}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[4]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[4]]], "Mouse"]]}, {
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["5"], 
                    Style[
                    RawBoxes["5"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[5]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[5]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{1, 4, 6}, ShowStringCharacters -> False], 
                    Style[
                    Style[{1, 4, 6}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[5]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[5]]], "Mouse"]]}, {
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["6"], 
                    Style[
                    RawBoxes["6"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[6]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[6]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{1, 5, 7}, ShowStringCharacters -> False], 
                    Style[
                    Style[{1, 5, 7}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[6]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[6]]], "Mouse"]]}, {
               Item[
                Pane[
                 Annotation[
                  EventHandler[
                   MouseAppearance[
                    Mouseover[
                    RawBoxes["7"], 
                    Style[
                    RawBoxes["7"], FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                   TypeSystem`NestedGrid`PackagePrivate`updateState[
                    TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[7]}]], 
                  TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                   GeneralUtilities`Slice[
                    Key[7]]], "Mouse"], ImageSize -> {{34, Full}, Automatic}],
                 Background -> GrayLevel[0.95], 
                Alignment -> {Left, Baseline}], 
               Item[
                Annotation[
                 EventHandler[
                  MouseAppearance[
                   Mouseover[
                    Style[{1, 2, 6}, ShowStringCharacters -> False], 
                    Style[
                    Style[{1, 2, 6}, ShowStringCharacters -> False], 
                    FontColor -> 
                    RGBColor[
                    0.27450980392156865`, 0.5372549019607843, 
                    0.792156862745098]]], "LinkHand"], {"MouseClicked", 1} :> 
                  TypeSystem`NestedGrid`PackagePrivate`updateState[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                    TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][{
                    Key[7]}]], 
                 TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
                  GeneralUtilities`Slice[
                   Key[7]]], "Mouse"]]}}, BaseStyle -> {ContextMenu -> {
                 MenuItem["Copy position to clipboard", 
                  KernelExecute[
                   TypeSystem`NestedGrid`PackagePrivate`toCurrentPosition[
                   TypeSystem`NestedGrid`PackagePrivate`copyClip]], 
                  MenuEvaluator -> Automatic], 
                 MenuItem["Copy data to clipboard", 
                  KernelExecute[
                   TypeSystem`NestedGrid`PackagePrivate`toCurrentData[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`copyClip]], 
                  MenuEvaluator -> Automatic], Delimiter, 
                 MenuItem["Paste position in new cell", 
                  KernelExecute[
                   TypeSystem`NestedGrid`PackagePrivate`toCurrentPosition[
                   TypeSystem`NestedGrid`PackagePrivate`cellPaste]], 
                  MenuEvaluator -> Automatic], 
                 MenuItem["Paste data in new cell", 
                  KernelExecute[
                   TypeSystem`NestedGrid`PackagePrivate`toCurrentData[
                   TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                    TypeSystem`NestedGrid`PackagePrivate`cellPaste]], 
                  MenuEvaluator -> Automatic]}, FontFamily -> "Verdana", 
               FontSize -> 12}, Alignment -> Left, RowMinHeight -> 1.5, 
             Dividers -> All, FrameStyle -> GrayLevel[0.85], 
             BaseStyle -> {FontFamily -> "Verdana", FontSize -> 12}], 
            LineBreakWithin -> False, ContextMenu -> {}, NumberMarks -> False,
             ShowAutoStyles -> False]], 
         TypeSystem`NestedGrid`PackagePrivate`initialQ = True, 
         TypeSystem`NestedGrid`PackagePrivate`self = $Failed}, 
        Dynamic[
         TypeSystem`NestedGrid`PackagePrivate`setupViewPath[
         TypeSystem`NestedGrid`PackagePrivate`$path$$, If[
            Not[TypeSystem`NestedGrid`PackagePrivate`initialQ], 
            
            Module[{TypeSystem`NestedGrid`PackagePrivate`tmpGrid$ = $Failed, 
              TypeSystem`NestedGrid`PackagePrivate`tmpData$ = 
              TypeSystem`NestedGrid`PackagePrivate`LookupCacheValue[
                TypeSystem`NestedGrid`PackagePrivate`$state$$["Data"]]}, 
             TypeSystem`NestedGrid`PackagePrivate`tmpGrid$ = 
              If[TypeSystem`NestedGrid`PackagePrivate`tmpData$ === 
                TypeSystem`NestedGrid`PackagePrivate`$NotCached, 
                TypeSystem`NestedGrid`PackagePrivate`renderedGrid, 
                TypeSystem`NestedGrid`PackagePrivate`renderGrid[
                TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                 TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                 TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
                 TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491][
                TypeSystem`NestedGrid`PackagePrivate`tmpData$]]; If[
               Not[
                FailureQ[TypeSystem`NestedGrid`PackagePrivate`tmpGrid$]], 
               TypeSystem`NestedGrid`PackagePrivate`renderedGrid = 
               TypeSystem`NestedGrid`PackagePrivate`tmpGrid$]; Null]]; 
          TypeSystem`NestedGrid`PackagePrivate`initialQ = False; 
          TypeSystem`NestedGrid`PackagePrivate`$pos$$; If[
            FailureQ[TypeSystem`NestedGrid`PackagePrivate`renderedGrid], 
            TypeSystem`SparseGrid[
             TypeSystem`H["(data no longer present)"]], 
            If[GeneralUtilities`$DebugMode, 
             Row[{TypeSystem`NestedGrid`PackagePrivate`renderedGrid, "  ", 
               TypeSystem`NestedGrid`PackagePrivate`formatState[
               TypeSystem`NestedGrid`PackagePrivate`$state$$, 
                TypeSystem`NestedGrid`PackagePrivate`$path$$, 
                TypeSystem`NestedGrid`PackagePrivate`$pos$$]}], 
             TypeSystem`NestedGrid`PackagePrivate`renderedGrid]]], 
         TrackedSymbols :> {TypeSystem`NestedGrid`PackagePrivate`$pos$$}], 
        DynamicModuleValues :> {}], 
      TypeSystem`NestedGrid`PackagePrivate`$posCell$$, 
      TypeSystem`NestedGrid`PackagePrivate`$topBar$$ = Dynamic[
        TypeSystem`NestedGrid`PackagePrivate`alignBar[
        TypeSystem`NestedGrid`PackagePrivate`$state$$][
         TypeSystem`NestedGrid`PackagePrivate`makeFramedBar[
          TypeSystem`PackageScope`Pathbar[
          TypeSystem`NestedGrid`PackagePrivate`$path$$, 
           TypeSystem`NestedGrid`PackagePrivate`updateState[
           TypeSystem`NestedGrid`PackagePrivate`$state$$, 
            TypeSystem`NestedGrid`PackagePrivate`$path$$, 
            TypeSystem`NestedGrid`PackagePrivate`$pos$$, 
            TypeSystem`NestedGrid`PackagePrivate`$grid$$, 5968491]]]], 
        TrackedSymbols :> {TypeSystem`NestedGrid`PackagePrivate`$path$$}], 
      TypeSystem`NestedGrid`PackagePrivate`$bottomBar$$ = Framed[
        Dynamic[
         Replace[
          MouseAnnotation[Null], {
          TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][Null] -> 
           "", TypeSystem`NestedGrid`PackagePrivate`$SliceMarker[5968491][
             Pattern[TypeSystem`NestedGrid`PackagePrivate`path, 
              Blank[]]] :> (
            TypeSystem`NestedGrid`PackagePrivate`$lastPath = 
             TypeSystem`NestedGrid`PackagePrivate`path; 
            TypeSystem`NestedGrid`PackagePrivate`makePathTrail[
             TypeSystem`NestedGrid`PackagePrivate`path, 
              TypeSystem`NestedGrid`PackagePrivate`makePathElements]), Null :> 
           Spacer[10], Blank[] :> Spacer[10]}], TrackedSymbols :> {}], 
        FrameStyle -> None, ImageMargins -> 0, FrameMargins -> 0, Alignment -> 
        Top, ImageSize -> {Automatic, 14}]}, 
     DynamicBox[
      ToBoxes[
       Column[
        If[TypeSystem`NestedGrid`PackagePrivate`$path$$ == {}, {
          TypeSystem`NestedGrid`PackagePrivate`displayScrollBar[
          TypeSystem`NestedGrid`PackagePrivate`$grid$$, 
           TypeSystem`NestedGrid`PackagePrivate`$state$$, 
           TypeSystem`NestedGrid`PackagePrivate`$pos$$], 
          TypeSystem`NestedGrid`PackagePrivate`onDesktopShow[
           TypeSystem`NestedGrid`PackagePrivate`alignBar[
           TypeSystem`NestedGrid`PackagePrivate`$state$$][
           TypeSystem`NestedGrid`PackagePrivate`$bottomBar$$]]}, {
         TypeSystem`NestedGrid`PackagePrivate`$topBar$$, 
          TypeSystem`NestedGrid`PackagePrivate`displayScrollBar[
          TypeSystem`NestedGrid`PackagePrivate`$grid$$, 
           TypeSystem`NestedGrid`PackagePrivate`$state$$, 
           TypeSystem`NestedGrid`PackagePrivate`$pos$$], 
          TypeSystem`NestedGrid`PackagePrivate`onDesktopShow[
           TypeSystem`NestedGrid`PackagePrivate`alignBar[
           TypeSystem`NestedGrid`PackagePrivate`$state$$][
           TypeSystem`NestedGrid`PackagePrivate`$bottomBar$$]]}], Spacings -> 
        If[TypeSystem`NestedGrid`PackagePrivate`$path$$ == {}, 
          0, {{}, {0, 0.05, 0}}]], StandardForm], 
      ImageSizeCache -> {174., {115.693115234375, 122.306884765625}}, 
      TrackedSymbols :> {
       TypeSystem`NestedGrid`PackagePrivate`$state$$, 
        TypeSystem`NestedGrid`PackagePrivate`$grid$$}], 
     BaseStyle -> {LineBreakWithin -> False}, Deinitialization :> 
     TypeSystem`NestedGrid`PackagePrivate`deleteState[
      TypeSystem`NestedGrid`PackagePrivate`$state$$], 
     DynamicModuleValues :> {}, Initialization :> 
     Block[{$ContextPath = $ContextPath}, 
       Needs["TypeSystem`"]]], Deploy, DefaultBaseStyle -> "Deploy"]},
  "CopyTag",
  DisplayFunction->(#& ),
  InterpretationFunction->("Dataset[<>]"& )]], "Output",
 CellLabel->
  "Out[131]=",ExpressionUUID->"4cd3f1e7-80c0-4908-b5f7-98ff687d53fd"]
}, Open  ]],

Cell[TextData[{
 "The Wolfram Language does not include a function to create a graph from an \
adjacency list. However, it is not difficult to create a function that \
transforms an adjacency list into a graph object. We will use the adjacency \
matrix as an intermediate in order to illustrate how to build a matrix as a \
",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 ". We will create a function ",
 StyleBox["adjacencyListGraph", "Code"],
 " that accepts as its argument an ",
 StyleBox[ButtonBox["Association",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"], "FunctionLink"],
 " whose keys are the vertices and with the lists of adjacent vertices as the \
corresponding values. We will assume that the graph does not contain multiple \
edges."
}], "Text",ExpressionUUID->"60ff907a-fe80-4473-803d-85cf9114eaa4"],

Cell["\<\
Our first step is to define a predicate that checks that the rules making up \
the association are of the correct form. Specifically, each rule in an \
association representing an adjacency list for a graph can have any object as \
the left-hand element, but the right-hand element must be a list.\
\>", "Text",ExpressionUUID->"df4996a2-2d4a-4c76-aa2e-7ebc50777a09"],

Cell[BoxData[
 RowBox[{
  RowBox[{"adjacencyRuleQ", "[", "r_Rule", "]"}], ":=", 
  RowBox[{"ListQ", "[", 
   RowBox[{"r", "[", 
    RowBox[{"[", "2", "]"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[135]:=",ExpressionUUID->"71941755-ec20-47d5-9d53-2c4b7e515386"],

Cell["\<\
With this predicate in place, the signature of our function will be:\
\>", "Text",ExpressionUUID->"413bbb3a-d8d6-4c6e-bcff-de01fddd58d9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"adjacencyListGraph", "[", 
   RowBox[{"A", ":", 
    RowBox[{"Association", "[", 
     RowBox[{"__", "?", "adjacencyRuleQ"}], "]"}]}], "]"}], ":="}]], "Input",
 Editable->False,
 Evaluatable->False,ExpressionUUID->"79d78e9c-52cc-49bf-af96-99b12f676444"],

Cell[TextData[{
 "This will ensure that the argument is an ",
 StyleBox[ButtonBox["Association",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"], "FunctionLink"],
 " representing an adjacency list of a graph."
}], "Text",ExpressionUUID->"999e4745-9e7b-4cd9-bb9f-bd0fd4df74e3"],

Cell[TextData[{
 "The main work of the function will be to transform the list into a matrix \
with 1s in the locations specified by the adjacency list. We mentioned above \
that a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " is particularly suitable for matrices with few non-zero entries. To \
further explore this type of object, we will have our ",
 StyleBox["adjacencyListGraph", "Code"],
 " function create a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " as part of its operation. There are several different syntax options for \
creating a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 ", but we will use the most descriptive: a list of rules identifying \
positions with values. For example, to create a ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "\[Times]", "4"}], TraditionalForm]],ExpressionUUID->
  "9078f6d1-6ba9-4790-a9c9-9ede87130172"],
 " matrix with 1s in positions ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"1", ",", "3"}], ")"}], TraditionalForm]],ExpressionUUID->
  "a92da11f-5d0a-4e43-9f12-a4438b5fe9fa"],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"2", ",", "4"}], ")"}], TraditionalForm]],ExpressionUUID->
  "eca6e9d4-0d95-4af5-b8ce-dfaaf55c08bc"],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"4", ",", "1"}], ")"}], TraditionalForm]],ExpressionUUID->
  "631a1630-7bc1-4b13-a02b-1e0beb6412f5"],
 ", you enter the following."
}], "Text",ExpressionUUID->"8d0d3b6f-3cfb-4cbe-99d5-3515b2dd6bde"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SparseArray", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}], "\[Rule]", "1"}], ",", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"2", ",", "4"}], "}"}], "\[Rule]", "1"}], ",", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"4", ",", "1"}], "}"}], "\[Rule]", "1"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "4"}], "}"}]}], "]"}], "//", "MatrixForm"}]], "Input",
 CellLabel->
  "In[133]:=",ExpressionUUID->"d8baa098-b2cb-40ff-b0dd-60478b6e39b0"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "1", "0"},
     {"0", "0", "0", "1"},
     {"0", "0", "0", "0"},
     {"1", "0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[
   SparseArray[
    Automatic, {4, 4}, 0, {
     1, {{0, 1, 2, 2, 3}, {{3}, {4}, {1}}}, {1, 1, 1}}]]]]], "Output",
 CellLabel->
  "Out[133]//MatrixForm=",ExpressionUUID->"0ed8831a-1727-4488-a6ca-\
6d6a561870b5"]
}, Open  ]],

Cell[TextData[{
 "Observe that the first argument is a list of rules with the positions \
within the matrix given as lists on the left hand of each rule, and the value \
that belongs in the position on the right. The second argument to ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " is a list specifying the dimensions of the matrix. If the second argument \
is omitted, the Wolfram Language will attempt to deduce the size of the \
matrix from the given entries."
}], "Text",ExpressionUUID->"4c648c11-c08f-4828-9314-16e458941e64"],

Cell[TextData[{
 "The ",
 StyleBox["adjacencyListGraph", "Code"],
 " function will begin by determining the vertex set of the graph. Consider \
the example adjacency list shown below."
}], "Text",ExpressionUUID->"d14f62a4-ee29-4e56-906c-ea9394e73d22"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleList", "=", 
  RowBox[{"<|", 
   RowBox[{
    RowBox[{"\"\<a\>\"", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"\"\<b\>\"", ",", "\"\<c\>\"", ",", "3"}], "}"}]}], ",", 
    RowBox[{"\"\<b\>\"", "\[Rule]", 
     RowBox[{"{", "\"\<c\>\"", "}"}]}], ",", 
    RowBox[{"\"\<c\>\"", "\[Rule]", 
     RowBox[{"{", "3", "}"}]}]}], "|>"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[134]:=",ExpressionUUID->"34a05393-f0bd-4c44-8d9f-26aa64899b96"],

Cell[BoxData[
 RowBox[{"\[LeftAssociation]", 
  RowBox[{
   RowBox[{"\<\"a\"\>", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "3"}], "}"}]}], ",", 
   RowBox[{"\<\"b\"\>", "\[Rule]", 
    RowBox[{"{", "\<\"c\"\>", "}"}]}], ",", 
   RowBox[{"\<\"c\"\>", "\[Rule]", 
    RowBox[{"{", "3", "}"}]}]}], "\[RightAssociation]"}]], "Output",
 CellLabel->
  "Out[134]=",ExpressionUUID->"bbbf2d0d-d710-4a73-a485-0a94a040ee90"]
}, Open  ]],

Cell[TextData[{
 "Note that the vertices are not of all the same type\[LongDash]three are \
strings and another an integer. Also note that one of the vertices does not \
appear as a key. While a properly formed adjacency list would include the \
rule ",
 StyleBox["3\[ThinSpace]\[Rule]\[ThinSpace]{}", "Code"],
 ", we will design our function to be sufficiently robust to handle such \
input. We use ",
 StyleBox[ButtonBox["Keys",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Keys"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Values",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Values"], "FunctionLink"],
 " to extract the keys and values. The result of ",
 StyleBox[ButtonBox["Values",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Values"], "FunctionLink"],
 " is a list of lists and so will need to be flattened."
}], "Text",ExpressionUUID->"9de9d7f6-3141-426a-8b74-878d601d1044"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleListVertices", "=", 
  RowBox[{"Union", "[", 
   RowBox[{
    RowBox[{"Keys", "[", "exampleList", "]"}], ",", 
    RowBox[{"Flatten", "@", 
     RowBox[{"Values", "[", "exampleList", "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[136]:=",ExpressionUUID->"411b34fd-6057-4fe6-8ad5-e28b8528c789"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], 
  "}"}]], "Output",
 CellLabel->
  "Out[136]=",ExpressionUUID->"13545100-04bd-4317-bc34-fcac7ae0e069"]
}, Open  ]],

Cell[TextData[{
 "In order to create an adjacency matrix, we need to translate the names of \
the vertices into positions. We will create an ",
 StyleBox[ButtonBox["Association",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Association"], "FunctionLink"],
 " with the vertex names as keys and their positions in the list of vertices \
as the values. The ",
 StyleBox[ButtonBox["MapThread",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MapThread"], "FunctionLink"],
 " function will accept a function of multiple arguments and a list of lists. \
The result will be a list in which each element will be the function applied \
to the elements in the corresponding positions. This is illustrated below \
with a undefined symbol in the first argument."
}], "Text",ExpressionUUID->"91f7c7c9-6949-40fa-8adf-e3a6a5df088d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MapThread", "[", 
  RowBox[{"f", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}]}], 
    "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[137]:=",ExpressionUUID->"4ce8d680-4f66-4bd7-8207-a240d4f0fb60"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"1", ",", "\<\"a\"\>"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"2", ",", "\<\"b\"\>"}], "]"}], ",", 
   RowBox[{"f", "[", 
    RowBox[{"3", ",", "\<\"c\"\>"}], "]"}]}], "}"}]], "Output",
 CellLabel->
  "Out[137]=",ExpressionUUID->"483f682b-60f5-4e6f-b180-6e0d0156ec9b"]
}, Open  ]],

Cell["\<\
This allows us to create a list of rules identifying the names of vertices \
with their position in the list.\
\>", "Text",ExpressionUUID->"c3940e75-1cd9-4d79-be3f-dc7bbde894d7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleDictionary", "=", 
  RowBox[{"Association", "[", 
   RowBox[{"MapThread", "[", 
    RowBox[{"Rule", ",", 
     RowBox[{"{", 
      RowBox[{"exampleListVertices", ",", 
       RowBox[{"Range", "[", "4", "]"}]}], "}"}]}], "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[138]:=",ExpressionUUID->"d5eb3f3e-b3f2-4445-924a-c81a61d92327"],

Cell[BoxData[
 RowBox[{"\[LeftAssociation]", 
  RowBox[{
   RowBox[{"3", "\[Rule]", "1"}], ",", 
   RowBox[{"\<\"a\"\>", "\[Rule]", "2"}], ",", 
   RowBox[{"\<\"b\"\>", "\[Rule]", "3"}], ",", 
   RowBox[{"\<\"c\"\>", "\[Rule]", "4"}]}], "\[RightAssociation]"}]], "Output",\

 CellLabel->
  "Out[138]=",ExpressionUUID->"69e06659-eedd-4997-8309-5a5a23a58bd9"]
}, Open  ]],

Cell["\<\
With this association in place, we can access the position of a vertex as \
follows:\
\>", "Text",ExpressionUUID->"9480b0e5-ccc6-4b57-bdc5-889bbbb4d6de"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleDictionary", "[", "\"\<a\>\"", "]"}]], "Input",
 CellLabel->
  "In[139]:=",ExpressionUUID->"e22ad285-3196-4008-9999-577513d1ca2e"],

Cell[BoxData["2"], "Output",
 CellLabel->
  "Out[139]=",ExpressionUUID->"5c51ff87-69a4-4469-bacf-03cfbc3089ad"]
}, Open  ]],

Cell[TextData[{
 "To build the rules that define the ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 ", we use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " with two table variables\[LongDash]first looping over the keys of the \
association and then over the lists of vertices adjacent to them."
}], "Text",ExpressionUUID->"631b3b50-51c8-49ae-a64b-a3eab037b747"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"exampleDictionary", "[", "v", "]"}], ",", 
      RowBox[{"exampleDictionary", "[", "w", "]"}]}], "}"}], "\[Rule]", "1"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", 
     RowBox[{"Keys", "[", "exampleList", "]"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"w", ",", 
     RowBox[{"exampleList", "[", "v", "]"}]}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[140]:=",ExpressionUUID->"5ecd273f-a546-4b27-bf39-831878dc2851"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "3"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "4"}], "}"}], "\[Rule]", "1"}], ",", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], "\[Rule]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"3", ",", "4"}], "}"}], "\[Rule]", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"4", ",", "1"}], "}"}], "\[Rule]", "1"}], "}"}]}], 
  "}"}]], "Output",
 CellLabel->
  "Out[140]=",ExpressionUUID->"a5c10bf2-069a-4ca5-a7b6-d9c82397896c"]
}, Open  ]],

Cell[TextData[{
 "This will need to be flattened in order to serve as the argument of ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 ". "
}], "Text",ExpressionUUID->"6baff404-2a23-48ae-98ce-59a7907e1636"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SparseArray", "[", 
   RowBox[{"Flatten", "[", 
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"exampleDictionary", "[", "v", "]"}], ",", 
         RowBox[{"exampleDictionary", "[", "w", "]"}]}], "}"}], "\[Rule]", 
       "1"}], ",", 
      RowBox[{"{", 
       RowBox[{"v", ",", 
        RowBox[{"Keys", "[", "exampleList", "]"}]}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"w", ",", 
        RowBox[{"exampleList", "[", "v", "]"}]}], "}"}]}], "]"}], "]"}], 
   "]"}], "//", "MatrixForm"}]], "Input",
 CellLabel->
  "In[141]:=",ExpressionUUID->"5f6f3438-e26e-4256-9f4e-8f4318922911"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "0", "0", "0"},
     {"1", "0", "1", "1"},
     {"0", "0", "0", "1"},
     {"1", "0", "0", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[
   SparseArray[
    Automatic, {4, 4}, 0, {
     1, {{0, 0, 3, 4, 5}, {{3}, {4}, {1}, {4}, {1}}}, {1, 1, 1, 1, 
      1}}]]]]], "Output",
 CellLabel->
  "Out[141]//MatrixForm=",ExpressionUUID->"d185ca26-03af-4660-adc8-\
f06e59d2af39"]
}, Open  ]],

Cell[TextData[{
 "We now build ",
 StyleBox["adjacencyListGraph", "Code"],
 ". We use a ",
 StyleBox[ButtonBox["BlankNullSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankNullSequence"], "FunctionLink"],
 " (",
 StyleBox["___", "Code"],
 "), which matches any number of argument, including 0, to pass options from \
this function to ",
 StyleBox[ButtonBox["AdjacencyGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyGraph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"0e19baf2-5173-4d06-a71c-e6fa2e0c5974"],

Cell[BoxData[
 RowBox[{
  RowBox[{"adjacencyListGraph", "[", 
   RowBox[{
    RowBox[{"A", ":", 
     RowBox[{"Association", "[", 
      RowBox[{"__", "?", "adjacencyRuleQ"}], "]"}]}], ",", "opts___"}], "]"}],
   ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"V", ",", "n", ",", "D", ",", "rules", ",", "v", ",", "w"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V", "=", 
      RowBox[{"Union", "[", 
       RowBox[{
        RowBox[{"Keys", "[", "A", "]"}], ",", 
        RowBox[{"Flatten", "@", 
         RowBox[{"Values", "[", "A", "]"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"n", "=", 
      RowBox[{"Length", "[", "V", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"D", "=", 
      RowBox[{"Association", "[", 
       RowBox[{"MapThread", "[", 
        RowBox[{"Rule", ",", 
         RowBox[{"{", 
          RowBox[{"V", ",", 
           RowBox[{"Range", "[", "n", "]"}]}], "}"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"rules", "=", 
      RowBox[{"Flatten", "[", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"D", "[", "v", "]"}], ",", 
            RowBox[{"D", "[", "w", "]"}]}], "}"}], "\[Rule]", "1"}], ",", 
         RowBox[{"{", 
          RowBox[{"v", ",", 
           RowBox[{"Keys", "[", "A", "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"w", ",", 
           RowBox[{"A", "[", "v", "]"}]}], "}"}]}], "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"AdjacencyGraph", "[", 
      RowBox[{
       RowBox[{"SparseArray", "[", 
        RowBox[{"rules", ",", 
         RowBox[{"{", 
          RowBox[{"n", ",", "n"}], "}"}]}], "]"}], ",", "opts"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[142]:=",ExpressionUUID->"bfd71859-adff-45db-8331-d00d901ec524"],

Cell["We apply this function to an example.", "Text",ExpressionUUID->"d4265b9a-35dc-4a88-b265-4779db428800"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleAL", "=", 
  RowBox[{"adjacencyListGraph", "[", 
   RowBox[{
    RowBox[{"<|", 
     RowBox[{
      RowBox[{"1", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], ",", 
      RowBox[{"2", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "3", ",", "4"}], "}"}]}], ",", 
      RowBox[{"3", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}]}], ",", 
      RowBox[{"4", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"2", ",", "5"}], "}"}]}], ",", 
      RowBox[{"5", "\[Rule]", 
       RowBox[{"{", "4", "}"}]}]}], "|>"}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[143]:=",ExpressionUUID->"1d20e2b4-0d8c-4bf5-b851-676e3e338545"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 2, 5, 7, 9, 10}, {{2}, {3}, {1}, {3}, {4}, {1}, {2}, {2}, {
          5}, {4}}}, Pattern}]}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {0.9035675208407604, 0.3783936291918627}}, 
          0.07567633483019226]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {0.0016365078580331538`, 0.7567615788180366}}, 
          0.07567633483019226]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9035675208407604, 0.3783936291918627}, {
          0.0016365078580331538`, 0.7567615788180366}}, 0.07567633483019226]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9035675208407604, 0.3783936291918627}, {
          2.1100767471814814`, 0.3782410387236667}}, 0.07567633483019226]}, 
        {Arrowheads[0.], 
         ArrowBox[{{2.1100767471814814`, 0.3782410387236667}, {
          3.1373969617796416`, 0.378122716930539}}, 0.07567633483019226]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.}, 0.07567633483019226], 
         InsetBox["1", {0., 0.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9035675208407604, 0.3783936291918627}, 
          0.07567633483019226], 
         InsetBox["2", {0.9035675208407604, 0.3783936291918627},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.0016365078580331538, 0.7567615788180366}, 
          0.07567633483019226], 
         InsetBox["3", {0.0016365078580331538, 0.7567615788180366},
          BaseStyle->"Graphics"]}, {
         DiskBox[{2.1100767471814814, 0.3782410387236667}, 
          0.07567633483019226], 
         InsetBox["4", {2.1100767471814814, 0.3782410387236667},
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.1373969617796416, 0.378122716930539}, 
          0.07567633483019226], 
         InsetBox["5", {3.1373969617796416, 0.378122716930539},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[143]=",ExpressionUUID->"e2c015dc-f526-4a47-8da8-c585b1a3f8c6"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Incidence Matrices", "Subsection",
 CellTags->"Section3c",ExpressionUUID->"84121ca9-c025-445e-a2c2-ac551fbded43"],

Cell[TextData[{
 "The third representation of graphs that we are considering is the incidence \
matrix. For a graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "02985a13-d86c-427f-b5ae-92f76e1da864"],
 " with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "d7ba6ef6-1231-4995-8993-731ed68b30bc"],
 " vertices and ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]],ExpressionUUID->
  "7fe09f27-b4d2-4efc-9a9e-a018e18c6f4d"],
 " edges, the associated incidence matrix is the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Times]", "m"}], TraditionalForm]],ExpressionUUID->
  "80ce3d90-fd75-4b24-ad8d-f0781d43e27f"],
 " matrix whose ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"i", ",", "j"}], ")"}], TraditionalForm]],ExpressionUUID->
  "08f2dc7b-d4f4-46f5-bbb4-67b803faa413"],
 " entry is 1 if vertex ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]],ExpressionUUID->
  "b91b89ca-2b91-4bc1-9196-d4901e58cee8"],
 " is an endpoint of edge ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]],ExpressionUUID->
  "80bec33a-be88-4b35-beba-a248963a72c0"],
 ", and 0 otherwise."
}], "Text",ExpressionUUID->"17c3ab9f-e050-445d-a9e0-c3c9c7f5f07a"],

Cell[TextData[{
 "The Wolfram Language includes functions for working with incidence \
matrices. Given an incidence matrix ",
 Cell[BoxData[
  FormBox["M", TraditionalForm]],ExpressionUUID->
  "3ca71ae2-287c-41b6-871f-f0752706f029"],
 ", the function ",
 StyleBox[ButtonBox["IncidenceGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IncidenceGraph"], "FunctionLink"],
 " will produce the associated graph. As an example, we reverse Example 6 \
from Section 10.3 and use the incidence matrix given in the solution in order \
to reproduce the graph."
}], "Text",ExpressionUUID->"3779fabf-1c2d-4097-b346-86e8dff3af3b"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"exampleIncidenceM", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "1", ",", "0", ",", "0", ",", "0", ",", "0"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0", ",", "1", ",", "1"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "0", ",", "1", ",", "0", ",", "0", ",", "0"}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1", ",", "0", ",", "1", ",", "1", ",", "0"}], 
      "}"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"exampleIncidenceM", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[144]:=",ExpressionUUID->"c77a919e-2ab0-4e2d-ba47-43489d4b000c"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", "0", "0", "0", "0"},
     {"0", "0", "1", "1", "0", "1"},
     {"0", "0", "0", "0", "1", "1"},
     {"1", "0", "1", "0", "0", "0"},
     {"0", "1", "0", "1", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellLabel->
  "Out[145]//MatrixForm=",ExpressionUUID->"0d1ca385-6e54-4781-b459-\
0e986ca935e2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleIncidenceG", "=", 
  RowBox[{"IncidenceGraph", "[", 
   RowBox[{"exampleIncidenceM", ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center", ",", 
       RowBox[{
        SubscriptBox["v", "#"], "&"}]}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.5", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1.5", ",", "0"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[146]:=",ExpressionUUID->"5049bccb-e104-47ef-86b2-3394e305eaf8"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {
      Null, {{1, 4}, {1, 5}, {2, 4}, {2, 5}, {3, 5}, {2, 3}}}, {VertexLabels -> {
         Placed["Name", Center, Subscript[$CellContext`v, #]& ]}, 
       VertexSize -> {Medium}, 
       VertexCoordinates -> {{0, 1}, {1, 1}, {2, 1}, {0.5, 0}, {1.5, 0}}}]]}, 
    
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {0.5, 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {1.5, 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {0.5, 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1.5, 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {1.5, 0.}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 1.}, 0.1], InsetBox[
          SubscriptBox["v", "1"], {0., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.1], InsetBox[
          SubscriptBox["v", "2"], {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.1], InsetBox[
          SubscriptBox["v", "3"], {2., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{0.5, 0.}, 0.1], InsetBox[
          SubscriptBox["v", "4"], {0.5, 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{1.5, 0.}, 0.1], InsetBox[
          SubscriptBox["v", "5"], {1.5, 0.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[146]=",ExpressionUUID->"2e333ab4-8934-4bcc-95c7-83ad910c1927"]
}, Open  ]],

Cell[TextData[{
 "Note that, in order to label the vertices with subscripted ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "d17af0bc-394f-4990-be84-14be5f20f4ce"],
 StyleBox["\[CloseCurlyQuote]s",
  IgnoreSpellCheck->True],
 ", we use ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " with a function as the third argument. This function applies to the label \
determined by the first argument to produce the displayed label. So these \
vertices are named 1 through 6 and those names become subscripts in their \
labels."
}], "Text",ExpressionUUID->"caff71ce-7c29-4c13-8750-d617c3c61109"],

Cell[TextData[{
 "For the reverse, the ",
 StyleBox[ButtonBox["IncidenceMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IncidenceMatrix"], "FunctionLink"],
 " function will produce the incidence matrix for a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. We apply this function to the previous graph. The output from this \
function is a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " object, so we apply ",
 StyleBox[ButtonBox["MatrixForm",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixForm"], "FunctionLink"],
 " to view it."
}], "Text",ExpressionUUID->"c9e2c968-3e33-4a99-8faf-0e78826b4923"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IncidenceMatrix", "[", "exampleIncidenceG", "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->
  "In[147]:=",ExpressionUUID->"613bd4a5-b273-4654-b808-438c1ebe693a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"1", "1", "0", "0", "0", "0"},
     {"0", "0", "1", "1", "0", "1"},
     {"0", "0", "0", "0", "1", "1"},
     {"1", "0", "1", "0", "0", "0"},
     {"0", "1", "0", "1", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[
   SparseArray[
    Automatic, {5, 6}, 0, {
     1, {{0, 2, 5, 7, 9, 12}, {{1}, {2}, {3}, {4}, {6}, {5}, {6}, {1}, {3}, {
       2}, {4}, {5}}}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}}]]]]], "Output",
 CellLabel->
  "Out[147]//MatrixForm=",ExpressionUUID->"ccb26a96-bca9-4460-83a1-\
605401bbd665"]
}, Open  ]],

Cell[TextData[{
 "For a directed graph, the ",
 StyleBox[ButtonBox["IncidenceMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IncidenceMatrix"], "FunctionLink"],
 " function returns a matrix with a 1 in position ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"i", ",", "j"}], ")"}], TraditionalForm]],ExpressionUUID->
  "805d4450-43ae-4c31-9a7b-6ed60add0214"],
 " indicating that the vertex ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]],ExpressionUUID->
  "b1e507e0-8a1f-44fa-8ab3-a9d9f1f357b2"],
 " is the head of edge ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]],ExpressionUUID->
  "08ac3bf6-4d9d-4814-a98b-7b40329155ca"],
 " and an entry of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", "1"}], TraditionalForm]],ExpressionUUID->
  "4c80a239-4598-4bb7-8fb7-179594773bad"],
 " indicating that the vertex is the tail of the edge."
}], "Text",ExpressionUUID->"bb63c743-9e11-4598-a012-dbd120fefa9e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"directedIncidenceG", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "1"}], ",", 
      RowBox[{"2", "\[Rule]", "4"}], ",", 
      RowBox[{"4", "\[Rule]", "1"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "True"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Small"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[148]:=",ExpressionUUID->"38e4408b-91fb-4df9-ade9-4b01f4707ce3"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4}, {{{1, 2}, {2, 3}, {3, 1}, {2, 4}, {4, 1}}, Null}, {
      VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Small}, 
       VertexCoordinates -> {{0, 0}, {1, 1}, {0, 1}, {1, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
        ArrowBox[{{{0., 0.}, {1., 1.}}, {{1., 1.}, {0., 1.}}, {{1., 1.}, {1., 
         0.}}, {{0., 1.}, {0., 0.}}, {{1., 0.}, {0., 0.}}}, 0.05]}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.}, 0.05], InsetBox["1", {0., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.05], 
         InsetBox["2", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 1.}, 0.05], 
         InsetBox["3", {0., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.05], 
         InsetBox["4", {1., 0.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[148]=",ExpressionUUID->"4bd5902a-9332-4682-9998-977916ddf07b"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"IncidenceMatrix", "[", "directedIncidenceG", "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->
  "In[149]:=",ExpressionUUID->"e2cfa573-7c83-4b70-b10a-bf5df16d409a"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "1", "0", "1"},
     {"1", 
      RowBox[{"-", "1"}], "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "1", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", "0", "1", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[
   SparseArray[
    Automatic, {4, 5}, 0, {
     1, {{0, 3, 6, 8, 10}, {{1}, {3}, {5}, {1}, {2}, {4}, {2}, {3}, {4}, {
       5}}}, {-1, 1, 1, 1, -1, -1, 1, -1, 1, -1}}]]]]], "Output",
 CellLabel->
  "Out[149]//MatrixForm=",ExpressionUUID->"fad1572f-6479-4037-b96f-\
1912c949657e"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Isomorphism of Graphs", "Subsection",
 CellTags->"Section3d",ExpressionUUID->"b384af5d-602f-4ad5-89a8-20097b4e5998"],

Cell["\<\
We conclude Section 10.3 with a brief discussion of isomorphism of graphs and \
graph invariants. Determining whether two graphs are isomorphic is a \
difficult problem. The naive approach (exhaustively checking each possible \
mapping) can require exponential time. \
\>", "Text",ExpressionUUID->"5b161143-3654-4a9c-9dab-f367ab3f7560"],

Cell["\<\
Graph invariants are useful tools for confirming that two graphs are not \
isomorphic. While there is no complete collection of graph invariants that \
will definitively conclude whether two graphs are or are not isomorphic, they \
can, for many pairs of graphs, quickly demonstrate the impossibility of an \
isomorphism. We will create a function that will check some of the basic \
invariants: number of vertices, number of edges, whether the graph is \
directed, and whether it is bipartite. We also introduce another invariant: \
the degree sequence.\
\>", "Text",ExpressionUUID->"da12f7e7-a740-465e-83d8-4ff3f582c4a8"],

Cell[TextData[{
 "For a graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "ee921c47-6a69-4bfe-8153-c8f0f535707c"],
 ", the ",
 StyleBox["degree sequence",
  FontSlant->"Italic"],
 " is the list of the degrees of the vertices of the graph sorted in \
ascending order. The Wolfram Language function ",
 StyleBox[ButtonBox["VertexDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDegree"], "FunctionLink"],
 " applied to a graph and a vertex returns the degree of the vertex. Applied \
to the graph with no second argument, it produces a list of the degrees of \
the vertices of a graph, listed in order of the vertices. Since this depends \
on the order in which ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " stores the vertices, it is not an invariant. However, applying the ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " function to the result of ",
 StyleBox[ButtonBox["VertexDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDegree"], "FunctionLink"],
 " returns the degree sequence for the graph, which is an invariant."
}], "Text",ExpressionUUID->"cabf7f87-d0f1-495f-8a15-f3861e3c3977"],

Cell["\<\
The function defined below checks, one at a time, the invariants we have \
mentioned. If any of the invariants indicate that the graphs are not \
isomorphic, the procedure prints a statement to that effect.\
\>", "Text",ExpressionUUID->"ba4ea62b-cc55-4d52-80e1-6948ccb58af2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"checkInvariants", "[", 
   RowBox[{"G1_Graph", ",", "G2_Graph"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"notIsomorphic", "=", "False"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"VertexCount", "[", "G1", "]"}], "\[NotEqual]", 
        RowBox[{"VertexCount", "[", "G2", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"notIsomorphic", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{
        "Print", "[", "\"\<Different numbers of vertices.\>\"", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"EdgeCount", "[", "G1", "]"}], "\[NotEqual]", 
        RowBox[{"EdgeCount", "[", "G2", "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"notIsomorphic", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{
        "Print", "[", "\"\<Different numbers of edges.\>\"", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"Equivalent", "[", 
         RowBox[{
          RowBox[{"DirectedGraphQ", "[", "G1", "]"}], ",", 
          RowBox[{"DirectedGraphQ", "[", "G2", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"notIsomorphic", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{
        "Print", "[", "\"\<One is directed, one is undirected.\>\"", 
         "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"Equivalent", "[", 
         RowBox[{
          RowBox[{"BipartiteGraphQ", "[", "G1", "]"}], ",", 
          RowBox[{"BipartiteGraphQ", "[", "G2", "]"}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"notIsomorphic", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{
        "Print", "[", "\"\<One is bipartite, one is not.\>\"", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Sort", "[", 
         RowBox[{"VertexDegree", "[", "G1", "]"}], "]"}], "\[NotEqual]", 
        RowBox[{"Sort", "[", 
         RowBox[{"VertexDegree", "[", "G2", "]"}], "]"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"notIsomorphic", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{
        "Print", "[", "\"\<Degree sequences do not match.\>\"", "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"notIsomorphic", ",", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", "\"\<The graphs are not isomorphic.\>\"", "]"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"Print", "[", "\"\<The graphs MAY be isomorphic.\>\"", "]"}]}],
       "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[150]:=",ExpressionUUID->"414eee9e-68f5-4d85-9b1a-b8c22d9db707"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkInvariants", "[", 
  RowBox[{"directedIncidenceG", ",", "exampleIncidenceG"}], "]"}]], "Input",
 CellLabel->
  "In[151]:=",ExpressionUUID->"d7f6f518-e553-4ac3-ac6a-8f696f57e507"],

Cell[CellGroupData[{

Cell[BoxData["\<\"Different numbers of vertices.\"\>"], "Print",
 CellLabel->
  "During evaluation of \
In[151]:=",ExpressionUUID->"60cf4436-4dfa-4195-aefc-366d5f055c39"],

Cell[BoxData["\<\"Different numbers of edges.\"\>"], "Print",
 CellLabel->
  "During evaluation of \
In[151]:=",ExpressionUUID->"20600cf5-1b98-4249-8528-48768cd69b96"],

Cell[BoxData["\<\"One is directed, one is undirected.\"\>"], "Print",
 CellLabel->
  "During evaluation of \
In[151]:=",ExpressionUUID->"963d1f1d-3203-4cd6-bbb1-8c6060440f39"],

Cell[BoxData["\<\"Degree sequences do not match.\"\>"], "Print",
 CellLabel->
  "During evaluation of \
In[151]:=",ExpressionUUID->"21bf77dc-f36e-46de-8683-5f1e747dfd65"],

Cell[BoxData["\<\"The graphs are not isomorphic.\"\>"], "Print",
 CellLabel->
  "During evaluation of \
In[151]:=",ExpressionUUID->"b1efc861-ef38-400c-8342-4c9f7c28c9b3"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"checkInvariants", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", "3", "]"}], ",", 
   RowBox[{"CycleGraph", "[", "3", "]"}]}], "]"}]], "Input",
 CellLabel->
  "In[152]:=",ExpressionUUID->"bab9f54a-ddda-40b3-acc7-5bb5c3d9f597"],

Cell[BoxData["\<\"The graphs MAY be isomorphic.\"\>"], "Print",
 CellLabel->
  "During evaluation of \
In[152]:=",ExpressionUUID->"7c5c53e9-7074-4725-ae96-4e5e5c2254a6"]
}, Open  ]],

Cell[TextData[{
 "The Wolfram Language provides a function, ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 ", for determining whether or not two graphs are isomorphic. This function \
applies to any ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. The ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 " function accepts two graphs as its arguments. It returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the graphs are isomorphic."
}], "Text",ExpressionUUID->"82b3eda0-dc98-44de-b6fa-934f3a01d6a7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IsomorphicGraphQ", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", "3", "]"}], ",", 
   RowBox[{"CycleGraph", "[", "3", "]"}]}], "]"}]], "Input",
 CellLabel->
  "In[153]:=",ExpressionUUID->"9580e130-8119-485c-87f0-a32bf14385b4"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[153]=",ExpressionUUID->"ddff1f31-ac73-46c5-98db-94363eeedeb9"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["FindGraphIsomorphism",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindGraphIsomorphism"], "FunctionLink"],
 " function can be used to determine an explicit isomorphism for a pair of \
graphs that are in fact isomorphic. Like ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 ", the only required arguments are the two graphs, although you can give a \
positive integer or the symbol ",
 StyleBox["All", "Code"],
 " as the third argument in order to attempt to produce multiple \
isomorphisms. If the two graphs are in fact isomorphic, the output is a list \
of associations composed of rules of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"v", "\[Rule]", "w"}], TraditionalForm]],ExpressionUUID->
  "dbb4b32c-4776-4b4f-b2c7-eb2572d7669b"],
 " indicating that vertex ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "f7d49cc1-e54b-4d8b-ae70-052fc80f25b4"],
 " in the first graph is mapped to vertex ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "6b5ef3cd-c7b7-49c0-b785-a653231ee3dd"],
 " in the second graph. If the graphs are not isomorphic, ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 " returns an empty list."
}], "Text",ExpressionUUID->"0984b386-cd3c-4427-9e11-e2d7a178d326"],

Cell["\<\
We illustrate by reproducing the graphs in Figure 12 of Section 10.3 of the \
textbook. \
\>", "Text",ExpressionUUID->"97057be5-5d0f-4680-99e7-078389af0c4a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure12G", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "4"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "6"}], ",", 
      RowBox[{"3", "\[Rule]", "4"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center", ",", 
       RowBox[{
        SubscriptBox["u", "#"], "&"}]}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[154]:=",ExpressionUUID->"97076ae9-4b96-4db3-ad3f-6ce35dd2c446"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{1, 2}, {1, 4}, {2, 3}, {2, 6}, {3, 4}, {4, 5}, {5, 6}}}, {
      VertexLabels -> {
         Placed["Name", Center, Subscript[$CellContext`u, #]& ]}, 
       VertexSize -> {Medium}, 
       VertexCoordinates -> {{0, 2}, {3, 2}, {3, 0}, {0, 0}, {1, 1}, {2, 
        1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {3., 2.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {0., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{3., 2.}, {3., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{3., 2.}, {2., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{3., 0.}, {0., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{0., 0.}, {1., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 2.}, 0.1], InsetBox[
          SubscriptBox["u", "1"], {0., 2.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 2.}, 0.1], InsetBox[
          SubscriptBox["u", "2"], {3., 2.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.1], InsetBox[
          SubscriptBox["u", "3"], {3., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.1], InsetBox[
          SubscriptBox["u", "4"], {0., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.1], InsetBox[
          SubscriptBox["u", "5"], {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.1], InsetBox[
          SubscriptBox["u", "6"], {2., 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[154]=",ExpressionUUID->"6ee52e94-cc29-4b91-9b84-c2e5bbbde2f6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure12H", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "5"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "4"}], ",", 
      RowBox[{"3", "\[Rule]", "6"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center", ",", 
       RowBox[{
        SubscriptBox["v", "#"], "&"}]}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1.3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[155]:=",ExpressionUUID->"ec2c24c3-7e72-4c51-9d5c-3465221cee65"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{1, 2}, {1, 5}, {2, 3}, {3, 4}, {3, 6}, {4, 5}, {5, 6}}}, {
      VertexLabels -> {
         Placed["Name", Center, Subscript[$CellContext`v, #]& ]}, 
       VertexSize -> {Medium}, 
       VertexCoordinates -> {{0, 2}, {1, 1.3}, {3, 2}, {3, 0}, {0, 0}, {2, 
         1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 2.}, {1., 1.3}}, 0.10440306508910552`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 2.}, {0., 0.}}, 0.10440306508910552`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 1.3}, {3., 2.}}, 0.10440306508910552`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {3., 0.}}, 0.10440306508910552`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {2., 1.}}, 0.10440306508910552`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 0.}, {0., 0.}}, 0.10440306508910552`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {2., 1.}}, 0.10440306508910552`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 2.}, 0.10440306508910552], InsetBox[
          SubscriptBox["v", "1"], {0., 2.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.3}, 0.10440306508910552], 
         InsetBox[
          SubscriptBox["v", "2"], {1., 1.3},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 2.}, 0.10440306508910552], 
         InsetBox[
          SubscriptBox["v", "3"], {3., 2.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.10440306508910552], 
         InsetBox[
          SubscriptBox["v", "4"], {3., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.10440306508910552], 
         InsetBox[
          SubscriptBox["v", "5"], {0., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.10440306508910552], 
         InsetBox[
          SubscriptBox["v", "6"], {2., 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[155]=",ExpressionUUID->"c77907d8-957a-48cc-bcfd-19f9a0b98cad"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 " confirms that the graphs are isomorphic."
}], "Text",ExpressionUUID->"9291b735-b1b2-4b2d-b3d8-56e740408616"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IsomorphicGraphQ", "[", 
  RowBox[{"figure12G", ",", "figure12H"}], "]"}]], "Input",
 CellLabel->
  "In[156]:=",ExpressionUUID->"1d2f0551-5aac-4a2d-a54e-3a25bf782838"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[156]=",ExpressionUUID->"392ea034-e308-4524-93bf-6a2e5f3b1f24"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["FindGraphIsomorphism",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindGraphIsomorphism"], "FunctionLink"],
 " determines the isomorphism."
}], "Text",ExpressionUUID->"f7cb6bac-08a3-4537-a39f-476cd7ffcff3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindGraphIsomorphism", "[", 
  RowBox[{"figure12G", ",", "figure12H"}], "]"}]], "Input",
 CellLabel->
  "In[157]:=",ExpressionUUID->"6ec6eacf-8944-4e05-8251-a054eecdf321"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\[LeftAssociation]", 
   RowBox[{
    RowBox[{"1", "\[Rule]", "4"}], ",", 
    RowBox[{"2", "\[Rule]", "3"}], ",", 
    RowBox[{"3", "\[Rule]", "6"}], ",", 
    RowBox[{"4", "\[Rule]", "5"}], ",", 
    RowBox[{"5", "\[Rule]", "1"}], ",", 
    RowBox[{"6", "\[Rule]", "2"}]}], "\[RightAssociation]"}], "}"}]], "Output",\

 CellLabel->
  "Out[157]=",ExpressionUUID->"a0b5a10b-9edb-41fc-97a6-4a23dc24cbe9"]
}, Open  ]],

Cell["Observe that there are four different isomorphisms.", "Text",ExpressionUUID->"8953e4be-2fc4-408d-b2fe-2e8ee5af17d3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindGraphIsomorphism", "[", 
  RowBox[{"figure12G", ",", "figure12H", ",", "All"}], "]"}]], "Input",
 CellLabel->
  "In[158]:=",ExpressionUUID->"f6c852b4-8c20-4379-89b7-a3dcada40d86"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\[LeftAssociation]", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "4"}], ",", 
     RowBox[{"2", "\[Rule]", "3"}], ",", 
     RowBox[{"3", "\[Rule]", "6"}], ",", 
     RowBox[{"4", "\[Rule]", "5"}], ",", 
     RowBox[{"5", "\[Rule]", "1"}], ",", 
     RowBox[{"6", "\[Rule]", "2"}]}], "\[RightAssociation]"}], ",", 
   RowBox[{"\[LeftAssociation]", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "4"}], ",", 
     RowBox[{"2", "\[Rule]", "5"}], ",", 
     RowBox[{"3", "\[Rule]", "6"}], ",", 
     RowBox[{"4", "\[Rule]", "3"}], ",", 
     RowBox[{"5", "\[Rule]", "2"}], ",", 
     RowBox[{"6", "\[Rule]", "1"}]}], "\[RightAssociation]"}], ",", 
   RowBox[{"\[LeftAssociation]", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "6"}], ",", 
     RowBox[{"2", "\[Rule]", "3"}], ",", 
     RowBox[{"3", "\[Rule]", "4"}], ",", 
     RowBox[{"4", "\[Rule]", "5"}], ",", 
     RowBox[{"5", "\[Rule]", "1"}], ",", 
     RowBox[{"6", "\[Rule]", "2"}]}], "\[RightAssociation]"}], ",", 
   RowBox[{"\[LeftAssociation]", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "6"}], ",", 
     RowBox[{"2", "\[Rule]", "5"}], ",", 
     RowBox[{"3", "\[Rule]", "4"}], ",", 
     RowBox[{"4", "\[Rule]", "3"}], ",", 
     RowBox[{"5", "\[Rule]", "2"}], ",", 
     RowBox[{"6", "\[Rule]", "1"}]}], "\[RightAssociation]"}]}], 
  "}"}]], "Output",
 CellLabel->
  "Out[158]=",ExpressionUUID->"2214e747-28ac-4521-af3b-c7ef7b2cb465"]
}, Open  ]],

Cell[TextData[{
 "We can make an isomorphism visible by coloring each vertex in the second \
graph and using the composition to then determine colors for the first graph. \
To choose the colors, we\[CloseCurlyQuote]ll make use of the Wolfram Language \
built-in color schemes. There are a number of different ways to use ",
 StyleBox[ButtonBox["ColorData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorData"], "FunctionLink"],
 "; we will simply use the indexed color schemes. If you apply ",
 StyleBox[ButtonBox["ColorData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorData"], "FunctionLink"],
 " to a positive integer up to 113, the result is a ",
 StyleBox[ButtonBox["ColorDataFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorDataFunction"], "FunctionLink"],
 ", which is a function that returns a color. "
}], "Text",ExpressionUUID->"24a659bf-9493-45e5-9fd1-b574743c04b7"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ColorData", "[", "10", "]"}], "[", "3", "]"}]], "Input",
 CellLabel->
  "In[159]:=",ExpressionUUID->"59db8d88-013a-4464-822b-677f52e0f72c"],

Cell[BoxData[
 InterpretationBox[
  ButtonBox[
   TooltipBox[
    GraphicsBox[{
      {GrayLevel[0], RectangleBox[{0, 0}]}, 
      {GrayLevel[0], RectangleBox[{1, -1}]}, 
      {RGBColor[0.9372549019607843, 0.6274509803921569, 0.16862745098039217`],
        RectangleBox[{0, -1}, {2, 1}]}},
     AspectRatio->1,
     DefaultBaseStyle->"ColorSwatchGraphics",
     Frame->True,
     FrameStyle->RGBColor[
      0.6248366013071895, 0.41830065359477125`, 0.11241830065359479`],
     FrameTicks->None,
     ImageSize->
      Dynamic[{Automatic, 
        1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
         Magnification])}],
     PlotRangePadding->None],
    StyleBox[
     RowBox[{"RGBColor", "[", 
       RowBox[{
        "0.9372549019607843`", ",", "0.6274509803921569`", ",", 
         "0.16862745098039217`"}], "]"}], NumberMarks -> False]],
   Appearance->None,
   BaseStyle->{},
   BaselinePosition->Baseline,
   ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
     If[
      Not[
       AbsoluteCurrentValue["Deployed"]], 
      SelectionMove[Typeset`box$, All, Expression]; 
      FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
      FrontEnd`Private`$ColorSelectorInitialColor = 
       RGBColor[0.9372549019607843, 0.6274509803921569, 0.16862745098039217`]; 
      FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
      MathLink`CallFrontEnd[
        FrontEnd`AttachCell[Typeset`box$, 
         FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
         Left, Top}, 
         "ClosingActions" -> {
          "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
   DefaultBaseStyle->{},
   Evaluator->Automatic,
   Method->"Preemptive"],
  RGBColor[0.9372549019607843, 0.6274509803921569, 0.16862745098039217`],
  Editable->False,
  Selectable->False]], "Output",
 CellLabel->
  "Out[159]=",ExpressionUUID->"7216b793-ce64-471d-9f2d-2e4ab377bef9"]
}, Open  ]],

Cell[TextData[{
 "Passing ",
 StyleBox[ButtonBox["ColorData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorData"], "FunctionLink"],
 " the second argument ",
 StyleBox["\[OpenCurlyDoubleQuote]Panel\[CloseCurlyDoubleQuote]", "Code"],
 " (including the quotation marks) will display the colors produced by the \
particular ",
 StyleBox[ButtonBox["ColorDataFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorDataFunction"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"5f7e7312-80bc-4f67-b77d-9133f8e434c8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ColorData", "[", 
  RowBox[{"87", ",", "\"\<Panel\>\""}], "]"}]], "Input",
 CellLabel->
  "In[160]:=",ExpressionUUID->"aa44d0ba-2e86-4320-b02a-e9d23a3a712a"],

Cell[BoxData[
 PanelBox[
  StyleBox[
   GraphicsBox[{
     {Hue[0.05, 1, 1], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{1, 0}, Offset[{-1, -1}, {2, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.05, 1, 1]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.1, 1, 0.95], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{2, 0}, Offset[{-1, -1}, {3, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.1, 1, 0.95]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0, 0.7, 0.9], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{3, 0}, Offset[{-1, -1}, {4, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0, 0.7, 0.9]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.15, 0.7, 0.85], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{4, 0}, Offset[{-1, -1}, {5, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.15, 0.7, 0.85]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.83, 0.33, 0.75], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{5, 0}, Offset[{-1, -1}, {6, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.83, 0.33, 0.75]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.07, 0.7, 0.9], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{6, 0}, Offset[{-1, -1}, {7, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.07, 0.7, 0.9]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.17, 0.7, 0.6], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{7, 0}, Offset[{-1, -1}, {8, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.17, 0.7, 0.6]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.08, 1, 1], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{8, 0}, Offset[{-1, -1}, {9, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.08, 1, 1]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.67, 0.4, 0.9], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{9, 0}, Offset[{-1, -1}, {10, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.67, 0.4, 0.9]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}, 
     {Hue[0.11, 1, 0.75], EdgeForm[Opacity[0.1]], 
      TagBox[RectangleBox[{10, 0}, Offset[{-1, -1}, {11, 1}]],
       EventHandlerTag[{"MouseClicked" :> NotebookWrite[
           InputNotebook[], 
           ToBoxes[
            Hue[0.11, 1, 0.75]]], Method -> "Preemptive", PassEventsDown -> 
         Automatic, PassEventsUp -> True}]]}},
    AspectRatio->NCache[
      Rational[1, 10], 0.1],
    ContentSelectable->False,
    ImageSize->250,
    PlotRangePadding->None],
   StripOnInput->False,
   CacheGraphics->False],
  FrameMargins->Automatic]], "Output",
 CellLabel->
  "Out[160]=",ExpressionUUID->"3ff83839-0c59-4eef-8ee4-20a2f25fa74a"]
}, Open  ]],

Cell[TextData[{
 "Provided the ",
 StyleBox[ButtonBox["ColorDataFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorDataFunction"], "FunctionLink"],
 " has enough colors, we can shade all of the vertices of a graph with \
different colors by forming a list of rules. "
}], "Text",ExpressionUUID->"7a400198-a3fa-4674-b98f-1af1e3f85fcc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"v", "->", 
    RowBox[{
     RowBox[{"ColorData", "[", "10", "]"}], "[", "v", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"v", ",", "6"}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[161]:=",ExpressionUUID->"5a85c005-372d-4933-a65b-fc02a87c3bf3"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "\[Rule]", 
    InterpretationBox[
     ButtonBox[
      TooltipBox[
       GraphicsBox[{
         {GrayLevel[0], RectangleBox[{0, 0}]}, 
         {GrayLevel[0], RectangleBox[{1, -1}]}, 
         {RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 
          RectangleBox[{0, -1}, {2, 1}]}},
        AspectRatio->1,
        DefaultBaseStyle->"ColorSwatchGraphics",
        Frame->True,
        FrameStyle->RGBColor[0.465359477124183, 0.010457516339869282`, 0.],
        FrameTicks->None,
        ImageSize->
         Dynamic[{
          Automatic, 
           1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
            Magnification])}],
        PlotRangePadding->None],
       StyleBox[
        RowBox[{"RGBColor", "[", 
          
          RowBox[{"0.6980392156862745`", ",", "0.01568627450980392`", ",", 
            "0.`"}], "]"}], NumberMarks -> False]],
      Appearance->None,
      BaseStyle->{},
      BaselinePosition->Baseline,
      ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
        If[
         Not[
          AbsoluteCurrentValue["Deployed"]], 
         SelectionMove[Typeset`box$, All, Expression]; 
         FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
         FrontEnd`Private`$ColorSelectorInitialColor = 
          RGBColor[0.6980392156862745, 0.01568627450980392, 0.]; 
         FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
         MathLink`CallFrontEnd[
           FrontEnd`AttachCell[Typeset`box$, 
            FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
            Left, Top}, 
            "ClosingActions" -> {
             "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
      DefaultBaseStyle->{},
      Evaluator->Automatic,
      Method->"Preemptive"],
     RGBColor[0.6980392156862745, 0.01568627450980392, 0.],
     Editable->False,
     Selectable->False]}], ",", 
   RowBox[{"2", "\[Rule]", 
    InterpretationBox[
     ButtonBox[
      TooltipBox[
       GraphicsBox[{
         {GrayLevel[0], RectangleBox[{0, 0}]}, 
         {GrayLevel[0], RectangleBox[{1, -1}]}, 
         {RGBColor[
          0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`], 
          RectangleBox[{0, -1}, {2, 1}]}},
        AspectRatio->1,
        DefaultBaseStyle->"ColorSwatchGraphics",
        Frame->True,
        FrameStyle->RGBColor[
         0.6143790849673203, 0.3294117647058824, 0.28758169934640526`],
        FrameTicks->None,
        ImageSize->
         Dynamic[{
          Automatic, 
           1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
            Magnification])}],
        PlotRangePadding->None],
       StyleBox[
        RowBox[{"RGBColor", "[", 
          
          RowBox[{"0.9215686274509803`", ",", "0.49411764705882355`", ",", 
            "0.43137254901960786`"}], "]"}], NumberMarks -> False]],
      Appearance->None,
      BaseStyle->{},
      BaselinePosition->Baseline,
      ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
        If[
         Not[
          AbsoluteCurrentValue["Deployed"]], 
         SelectionMove[Typeset`box$, All, Expression]; 
         FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
         FrontEnd`Private`$ColorSelectorInitialColor = 
          RGBColor[
           0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`]; 
         FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
         MathLink`CallFrontEnd[
           FrontEnd`AttachCell[Typeset`box$, 
            FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
            Left, Top}, 
            "ClosingActions" -> {
             "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
      DefaultBaseStyle->{},
      Evaluator->Automatic,
      Method->"Preemptive"],
     RGBColor[0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`],
     
     Editable->False,
     Selectable->False]}], ",", 
   RowBox[{"3", "\[Rule]", 
    InterpretationBox[
     ButtonBox[
      TooltipBox[
       GraphicsBox[{
         {GrayLevel[0], RectangleBox[{0, 0}]}, 
         {GrayLevel[0], RectangleBox[{1, -1}]}, 
         {RGBColor[
          0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
          RectangleBox[{0, -1}, {2, 1}]}},
        AspectRatio->1,
        DefaultBaseStyle->"ColorSwatchGraphics",
        Frame->True,
        FrameStyle->RGBColor[
         0.6248366013071895, 0.41830065359477125`, 0.11241830065359479`],
        FrameTicks->None,
        ImageSize->
         Dynamic[{
          Automatic, 
           1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
            Magnification])}],
        PlotRangePadding->None],
       StyleBox[
        RowBox[{"RGBColor", "[", 
          
          RowBox[{"0.9372549019607843`", ",", "0.6274509803921569`", ",", 
            "0.16862745098039217`"}], "]"}], NumberMarks -> False]],
      Appearance->None,
      BaseStyle->{},
      BaselinePosition->Baseline,
      ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
        If[
         Not[
          AbsoluteCurrentValue["Deployed"]], 
         SelectionMove[Typeset`box$, All, Expression]; 
         FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
         FrontEnd`Private`$ColorSelectorInitialColor = 
          RGBColor[
           0.9372549019607843, 0.6274509803921569, 0.16862745098039217`]; 
         FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
         MathLink`CallFrontEnd[
           FrontEnd`AttachCell[Typeset`box$, 
            FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
            Left, Top}, 
            "ClosingActions" -> {
             "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
      DefaultBaseStyle->{},
      Evaluator->Automatic,
      Method->"Preemptive"],
     RGBColor[0.9372549019607843, 0.6274509803921569, 0.16862745098039217`],
     Editable->False,
     Selectable->False]}], ",", 
   RowBox[{"4", "\[Rule]", 
    InterpretationBox[
     ButtonBox[
      TooltipBox[
       GraphicsBox[{
         {GrayLevel[0], RectangleBox[{0, 0}]}, 
         {GrayLevel[0], RectangleBox[{1, -1}]}, 
         {RGBColor[
          0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
          RectangleBox[{0, -1}, {2, 1}]}},
        AspectRatio->1,
        DefaultBaseStyle->"ColorSwatchGraphics",
        Frame->True,
        FrameStyle->RGBColor[
         0.6614379084967321, 0.5437908496732027, 0.32679738562091504`],
        FrameTicks->None,
        ImageSize->
         Dynamic[{
          Automatic, 
           1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
            Magnification])}],
        PlotRangePadding->None],
       StyleBox[
        RowBox[{"RGBColor", "[", 
          
          RowBox[{"0.9921568627450981`", ",", "0.8156862745098039`", ",", 
            "0.49019607843137253`"}], "]"}], NumberMarks -> False]],
      Appearance->None,
      BaseStyle->{},
      BaselinePosition->Baseline,
      ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
        If[
         Not[
          AbsoluteCurrentValue["Deployed"]], 
         SelectionMove[Typeset`box$, All, Expression]; 
         FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
         FrontEnd`Private`$ColorSelectorInitialColor = 
          RGBColor[
           0.9921568627450981, 0.8156862745098039, 0.49019607843137253`]; 
         FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
         MathLink`CallFrontEnd[
           FrontEnd`AttachCell[Typeset`box$, 
            FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
            Left, Top}, 
            "ClosingActions" -> {
             "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
      DefaultBaseStyle->{},
      Evaluator->Automatic,
      Method->"Preemptive"],
     RGBColor[0.9921568627450981, 0.8156862745098039, 0.49019607843137253`],
     Editable->False,
     Selectable->False]}], ",", 
   RowBox[{"5", "\[Rule]", 
    InterpretationBox[
     ButtonBox[
      TooltipBox[
       GraphicsBox[{
         {GrayLevel[0], RectangleBox[{0, 0}]}, 
         {GrayLevel[0], RectangleBox[{1, -1}]}, 
         {RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 
          RectangleBox[{0, -1}, {2, 1}]}},
        AspectRatio->1,
        DefaultBaseStyle->"ColorSwatchGraphics",
        Frame->True,
        FrameStyle->RGBColor[
         0.48366013071895425`, 0.5333333333333334, 0.047058823529411764`],
        FrameTicks->None,
        ImageSize->
         Dynamic[{
          Automatic, 
           1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
            Magnification])}],
        PlotRangePadding->None],
       StyleBox[
        RowBox[{"RGBColor", "[", 
          
          RowBox[{"0.7254901960784313`", ",", "0.8`", ",", 
            "0.07058823529411765`"}], "]"}], NumberMarks -> False]],
      Appearance->None,
      BaseStyle->{},
      BaselinePosition->Baseline,
      ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
        If[
         Not[
          AbsoluteCurrentValue["Deployed"]], 
         SelectionMove[Typeset`box$, All, Expression]; 
         FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
         FrontEnd`Private`$ColorSelectorInitialColor = 
          RGBColor[0.7254901960784313, 0.8, 0.07058823529411765]; 
         FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
         MathLink`CallFrontEnd[
           FrontEnd`AttachCell[Typeset`box$, 
            FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
            Left, Top}, 
            "ClosingActions" -> {
             "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
      DefaultBaseStyle->{},
      Evaluator->Automatic,
      Method->"Preemptive"],
     RGBColor[0.7254901960784313, 0.8, 0.07058823529411765],
     Editable->False,
     Selectable->False]}], ",", 
   RowBox[{"6", "\[Rule]", 
    InterpretationBox[
     ButtonBox[
      TooltipBox[
       GraphicsBox[{
         {GrayLevel[0], RectangleBox[{0, 0}]}, 
         {GrayLevel[0], RectangleBox[{1, -1}]}, 
         {RGBColor[
          0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
          RectangleBox[{0, -1}, {2, 1}]}},
        AspectRatio->1,
        DefaultBaseStyle->"ColorSwatchGraphics",
        Frame->True,
        FrameStyle->RGBColor[
         0.21176470588235294`, 0.32679738562091504`, 0.05228758169934641],
        FrameTicks->None,
        ImageSize->
         Dynamic[{
          Automatic, 
           1.35 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
            Magnification])}],
        PlotRangePadding->None],
       StyleBox[
        RowBox[{"RGBColor", "[", 
          
          RowBox[{"0.3176470588235294`", ",", "0.49019607843137253`", ",", 
            "0.0784313725490196`"}], "]"}], NumberMarks -> False]],
      Appearance->None,
      BaseStyle->{},
      BaselinePosition->Baseline,
      ButtonFunction:>With[{Typeset`box$ = EvaluationBox[]}, 
        If[
         Not[
          AbsoluteCurrentValue["Deployed"]], 
         SelectionMove[Typeset`box$, All, Expression]; 
         FrontEnd`Private`$ColorSelectorInitialAlpha = 1; 
         FrontEnd`Private`$ColorSelectorInitialColor = 
          RGBColor[
           0.3176470588235294, 0.49019607843137253`, 0.0784313725490196]; 
         FrontEnd`Private`$ColorSelectorUseMakeBoxes = True; 
         MathLink`CallFrontEnd[
           FrontEnd`AttachCell[Typeset`box$, 
            FrontEndResource["RGBColorValueSelector"], {0, {Left, Bottom}}, {
            Left, Top}, 
            "ClosingActions" -> {
             "SelectionDeparture", "ParentChanged", "EvaluatorQuit"}]]]],
      DefaultBaseStyle->{},
      Evaluator->Automatic,
      Method->"Preemptive"],
     RGBColor[0.3176470588235294, 0.49019607843137253`, 0.0784313725490196],
     Editable->False,
     Selectable->False]}]}], "}"}]], "Output",
 CellLabel->
  "Out[161]=",ExpressionUUID->"0cce4c55-1764-4996-9963-b30d18897705"]
}, Open  ]],

Cell[TextData[{
 "This list will be the value of ",
 StyleBox[ButtonBox["VertexStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexStyle"], "FunctionLink"],
 ". Here is the graph ",
 Cell[BoxData[
  FormBox["H", TraditionalForm]],ExpressionUUID->
  "251ff98f-0c14-40a3-8e21-a632ec3cf2f2"],
 " with colors added"
}], "Text",ExpressionUUID->"b7627b10-9a5a-4069-83ab-c2ca22d2c71d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure12Hcolor", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "5"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"3", "\[Rule]", "4"}], ",", 
      RowBox[{"3", "\[Rule]", "6"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center", ",", 
       RowBox[{
        SubscriptBox["v", "#"], "&"}]}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Large"}], ",", 
    RowBox[{"VertexStyle", "\[Rule]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"v", "->", 
        RowBox[{
         RowBox[{"ColorData", "[", "10", "]"}], "[", "v", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", "6"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1.3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellLabel->
  "In[162]:=",ExpressionUUID->"8f74ee77-b443-4380-af02-e6258820f67d"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{1, 2}, {1, 5}, {2, 3}, {3, 4}, {3, 6}, {4, 5}, {5, 6}}}, {
      VertexLabels -> {
         Placed["Name", Center, Subscript[$CellContext`v, #]& ]}, 
       VertexSize -> {Large}, 
       VertexStyle -> {
        6 -> RGBColor[
          0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 1 -> 
         RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 2 -> 
         RGBColor[
          0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`], 3 -> 
         RGBColor[
          0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 4 -> 
         RGBColor[
          0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 5 -> 
         RGBColor[0.7254901960784313, 0.8, 0.07058823529411765]}, 
       VertexCoordinates -> {{0, 2}, {1, 1.3}, {3, 2}, {3, 0}, {0, 0}, {2, 
         1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0., 2.}, {1., 1.3}}, 0.20880613017821104`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 2.}, {0., 0.}}, 0.20880613017821104`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 1.3}, {3., 2.}}, 0.20880613017821104`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {3., 0.}}, 0.20880613017821104`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 2.}, {2., 1.}}, 0.20880613017821104`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3., 0.}, {0., 0.}}, 0.20880613017821104`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.}, {2., 1.}}, 0.20880613017821104`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 
          DiskBox[{0., 2.}, 0.20880613017821104]}, InsetBox[
          SubscriptBox["v", "1"], {0., 2.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`], 
          DiskBox[{1., 1.3}, 0.20880613017821104]}, InsetBox[
          SubscriptBox["v", "2"], {1., 1.3},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
          DiskBox[{3., 2.}, 0.20880613017821104]}, InsetBox[
          SubscriptBox["v", "3"], {3., 2.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
          DiskBox[{3., 0.}, 0.20880613017821104]}, InsetBox[
          SubscriptBox["v", "4"], {3., 0.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 
          DiskBox[{0., 0.}, 0.20880613017821104]}, InsetBox[
          SubscriptBox["v", "5"], {0., 0.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
          DiskBox[{2., 1.}, 0.20880613017821104]}, InsetBox[
          SubscriptBox["v", "6"], {2., 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[162]=",ExpressionUUID->"7be131c6-1d6b-4fe2-a84e-2653e53ee24e"]
}, Open  ]],

Cell[TextData[{
 "We then color ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "b14d60ad-0189-4b85-9e76-77207e04e246"],
 " by applying one of the isomorphisms before accessing the color."
}], "Text",ExpressionUUID->"350390e6-324d-4a7f-a01f-0539764250f5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure12isomorphism", "=", 
  RowBox[{
   RowBox[{"FindGraphIsomorphism", "[", 
    RowBox[{"figure12G", ",", "figure12H", ",", "All"}], "]"}], "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellLabel->
  "In[163]:=",ExpressionUUID->"4105b7d4-14a2-4f4b-8531-dc1413007e33"],

Cell[BoxData[
 RowBox[{"\[LeftAssociation]", 
  RowBox[{
   RowBox[{"1", "\[Rule]", "4"}], ",", 
   RowBox[{"2", "\[Rule]", "3"}], ",", 
   RowBox[{"3", "\[Rule]", "6"}], ",", 
   RowBox[{"4", "\[Rule]", "5"}], ",", 
   RowBox[{"5", "\[Rule]", "1"}], ",", 
   RowBox[{"6", "\[Rule]", "2"}]}], "\[RightAssociation]"}]], "Output",
 CellLabel->
  "Out[163]=",ExpressionUUID->"be3c90c8-7caf-4836-9569-3b29254db1e8"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure12Gcolor", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5", ",", "6"}], "}"}], 
    ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "2"}], ",", 
      RowBox[{"1", "\[Rule]", "4"}], ",", 
      RowBox[{"2", "\[Rule]", "3"}], ",", 
      RowBox[{"2", "\[Rule]", "6"}], ",", 
      RowBox[{"3", "\[Rule]", "4"}], ",", 
      RowBox[{"4", "\[Rule]", "5"}], ",", 
      RowBox[{"5", "\[Rule]", "6"}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center", ",", 
       RowBox[{
        SubscriptBox["u", "#"], "&"}]}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Large"}], ",", 
    RowBox[{"VertexStyle", "\[Rule]", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"v", "->", 
        RowBox[{
         RowBox[{"ColorData", "[", "10", "]"}], "[", 
         RowBox[{"figure12isomorphism", "[", "v", "]"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"v", ",", "6"}], "}"}]}], "]"}]}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 CellLabel->
  "In[164]:=",ExpressionUUID->"b7335a0a-9298-4c77-97d2-73f2bb086a51"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6}, {
      Null, {{1, 2}, {1, 4}, {2, 3}, {2, 6}, {3, 4}, {4, 5}, {5, 6}}}, {
      VertexLabels -> {
         Placed["Name", Center, Subscript[$CellContext`u, #]& ]}, 
       VertexSize -> {Large}, 
       VertexStyle -> {
        6 -> RGBColor[
          0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`], 1 -> 
         RGBColor[
          0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 2 -> 
         RGBColor[
          0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 3 -> 
         RGBColor[
          0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 4 -> 
         RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 5 -> 
         RGBColor[0.6980392156862745, 0.01568627450980392, 0.]}, 
       VertexCoordinates -> {{0, 2}, {3, 2}, {3, 0}, {0, 0}, {1, 1}, {2, 
        1}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {3., 2.}}, 0.2]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {0., 0.}}, 0.2]}, 
        {Arrowheads[0.], ArrowBox[{{3., 2.}, {3., 0.}}, 0.2]}, 
        {Arrowheads[0.], ArrowBox[{{3., 2.}, {2., 1.}}, 0.2]}, 
        {Arrowheads[0.], ArrowBox[{{3., 0.}, {0., 0.}}, 0.2]}, 
        {Arrowheads[0.], ArrowBox[{{0., 0.}, {1., 1.}}, 0.2]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.2]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[
          0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
          DiskBox[{0., 2.}, 0.2]}, InsetBox[
          SubscriptBox["u", "1"], {0., 2.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
          DiskBox[{3., 2.}, 0.2]}, InsetBox[
          SubscriptBox["u", "2"], {3., 2.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
          DiskBox[{3., 0.}, 0.2]}, InsetBox[
          SubscriptBox["u", "3"], {3., 0.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 
          DiskBox[{0., 0.}, 0.2]}, InsetBox[
          SubscriptBox["u", "4"], {0., 0.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 
          DiskBox[{1., 1.}, 0.2]}, InsetBox[
          SubscriptBox["u", "5"], {1., 1.},
          BaseStyle->"Graphics"]}, {
         {RGBColor[
          0.9215686274509803, 0.49411764705882355`, 0.43137254901960786`], 
          DiskBox[{2., 1.}, 0.2]}, InsetBox[
          SubscriptBox["u", "6"], {2., 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[164]=",ExpressionUUID->"0fd3cfc7-9e29-4e9c-a882-9eaa1891aef1"]
}, Open  ]],

Cell[TextData[{
 "We put them side-by-side to compare with ",
 StyleBox[ButtonBox["GraphicsRow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphicsRow"], "FunctionLink"],
 ", which like ",
 StyleBox[ButtonBox["Row",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Row"], "FunctionLink"],
 " for more general display, takes a list of objects and displays them in a \
row."
}], "Text",ExpressionUUID->"a2f7f1dd-2b38-4eca-b074-a4d46f9da29e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphicsRow", "[", 
  RowBox[{"{", 
   RowBox[{"figure12Gcolor", ",", "figure12Hcolor"}], "}"}], "]"}]], "Input",
 CellLabel->
  "In[165]:=",ExpressionUUID->"4b153d66-e90e-4959-b995-4c5df522766d"],

Cell[BoxData[
 GraphicsBox[{{}, {InsetBox[
     FormBox[
      GraphicsBox[
       NamespaceBox["NetworkGraphics",
        DynamicModuleBox[{Typeset`graph = HoldComplete[
          
          Graph[{1, 2, 3, 4, 5, 6}, {
           Null, {{1, 2}, {1, 4}, {2, 3}, {2, 6}, {3, 4}, {4, 5}, {5, 6}}}, {
           VertexLabels -> {
              Placed["Name", Center, Subscript[$CellContext`u, #]& ]}, 
            VertexSize -> {Large}, 
            VertexStyle -> {
             6 -> RGBColor[
               0.9215686274509803, 0.49411764705882355`, 
                0.43137254901960786`], 1 -> 
              RGBColor[
               0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
              2 -> RGBColor[
               0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
              3 -> RGBColor[
               0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
              4 -> RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 5 -> 
              RGBColor[0.6980392156862745, 0.01568627450980392, 0.]}, 
            VertexCoordinates -> {{0, 2}, {3, 2}, {3, 0}, {0, 0}, {1, 1}, {2, 
             1}}}]]}, 
         TagBox[GraphicsGroupBox[{
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{{0., 2.}, {3., 2.}}, 0.2]}, 
             {Arrowheads[0.], ArrowBox[{{0., 2.}, {0., 0.}}, 0.2]}, 
             {Arrowheads[0.], ArrowBox[{{3., 2.}, {3., 0.}}, 0.2]}, 
             {Arrowheads[0.], ArrowBox[{{3., 2.}, {2., 1.}}, 0.2]}, 
             {Arrowheads[0.], ArrowBox[{{3., 0.}, {0., 0.}}, 0.2]}, 
             {Arrowheads[0.], ArrowBox[{{0., 0.}, {1., 1.}}, 0.2]}, 
             {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.2]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
              {RGBColor[
               0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
               DiskBox[{0., 2.}, 0.2]}, InsetBox[
               SubscriptBox["u", "1"], {0., 2.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
               DiskBox[{3., 2.}, 0.2]}, InsetBox[
               SubscriptBox["u", "2"], {3., 2.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
               DiskBox[{3., 0.}, 0.2]}, InsetBox[
               SubscriptBox["u", "3"], {3., 0.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 
               DiskBox[{0., 0.}, 0.2]}, InsetBox[
               SubscriptBox["u", "4"], {0., 0.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 
               DiskBox[{1., 1.}, 0.2]}, InsetBox[
               SubscriptBox["u", "5"], {1., 1.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.9215686274509803, 0.49411764705882355`, 
                0.43137254901960786`], DiskBox[{2., 1.}, 0.2]}, InsetBox[
               SubscriptBox["u", "6"], {2., 1.},
               BaseStyle->"Graphics"]}}}],
          MouseAppearanceTag["NetworkGraphics"]],
         AllowKernelInitialization->False]],
       DefaultBaseStyle->{
        "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
         Hue[0.8, 1., 0.6]},
       FormatType->TraditionalForm,
       FrameTicks->None],
      TraditionalForm], {192., -189.}, {Center, Center}, {360., 360.}], 
    InsetBox[
     FormBox[
      GraphicsBox[
       NamespaceBox["NetworkGraphics",
        DynamicModuleBox[{Typeset`graph = HoldComplete[
          
          Graph[{1, 2, 3, 4, 5, 6}, {
           Null, {{1, 2}, {1, 5}, {2, 3}, {3, 4}, {3, 6}, {4, 5}, {5, 6}}}, {
           VertexLabels -> {
              Placed["Name", Center, Subscript[$CellContext`v, #]& ]}, 
            VertexSize -> {Large}, 
            VertexStyle -> {
             6 -> RGBColor[
               0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
              1 -> RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 2 -> 
              RGBColor[
               0.9215686274509803, 0.49411764705882355`, 
                0.43137254901960786`], 3 -> 
              RGBColor[
               0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
              4 -> 
              RGBColor[
               0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
              5 -> RGBColor[0.7254901960784313, 0.8, 0.07058823529411765]}, 
            VertexCoordinates -> {{0, 2}, {1, 1.3}, {3, 2}, {3, 0}, {0, 0}, {
              2, 1}}}]]}, 
         TagBox[GraphicsGroupBox[{
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], 
              ArrowBox[{{0., 2.}, {1., 1.3}}, 0.20880613017821104`]}, 
             {Arrowheads[0.], 
              ArrowBox[{{0., 2.}, {0., 0.}}, 0.20880613017821104`]}, 
             {Arrowheads[0.], 
              ArrowBox[{{1., 1.3}, {3., 2.}}, 0.20880613017821104`]}, 
             {Arrowheads[0.], 
              ArrowBox[{{3., 2.}, {3., 0.}}, 0.20880613017821104`]}, 
             {Arrowheads[0.], 
              ArrowBox[{{3., 2.}, {2., 1.}}, 0.20880613017821104`]}, 
             {Arrowheads[0.], 
              ArrowBox[{{3., 0.}, {0., 0.}}, 0.20880613017821104`]}, 
             {Arrowheads[0.], 
              ArrowBox[{{0., 0.}, {2., 1.}}, 0.20880613017821104`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
              {RGBColor[0.6980392156862745, 0.01568627450980392, 0.], 
               DiskBox[{0., 2.}, 0.20880613017821104]}, InsetBox[
               SubscriptBox["v", "1"], {0., 2.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.9215686274509803, 0.49411764705882355`, 
                0.43137254901960786`], 
               DiskBox[{1., 1.3}, 0.20880613017821104]}, InsetBox[
               SubscriptBox["v", "2"], {1., 1.3},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.9372549019607843, 0.6274509803921569, 0.16862745098039217`], 
               DiskBox[{3., 2.}, 0.20880613017821104]}, InsetBox[
               SubscriptBox["v", "3"], {3., 2.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.9921568627450981, 0.8156862745098039, 0.49019607843137253`], 
               DiskBox[{3., 0.}, 0.20880613017821104]}, InsetBox[
               SubscriptBox["v", "4"], {3., 0.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[0.7254901960784313, 0.8, 0.07058823529411765], 
               DiskBox[{0., 0.}, 0.20880613017821104]}, InsetBox[
               SubscriptBox["v", "5"], {0., 0.},
               BaseStyle->"Graphics"]}, {
              {RGBColor[
               0.3176470588235294, 0.49019607843137253`, 0.0784313725490196], 
               DiskBox[{2., 1.}, 0.20880613017821104]}, InsetBox[
               SubscriptBox["v", "6"], {2., 1.},
               BaseStyle->"Graphics"]}}}],
          MouseAppearanceTag["NetworkGraphics"]],
         AllowKernelInitialization->False]],
       DefaultBaseStyle->{
        "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
         Hue[0.8, 1., 0.6]},
       FormatType->TraditionalForm,
       FrameTicks->None],
      TraditionalForm], {576., -189.}, {Center, Center}, {360., 360.}]}, {}},
  ContentSelectable->True,
  PlotRangePadding->{6, 5}]], "Output",
 CellLabel->
  "Out[165]=",ExpressionUUID->"0456f9fa-0d34-4571-9dbc-e94c09ec9356"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.4 Connectivity", "Section",
 CellTags->"Section4",ExpressionUUID->"295b098c-70f8-44f6-94d5-da31bbedd722"],

Cell["\<\
The Wolfram Language provides a number of functions related to connectivity \
of graphs.\
\>", "Text",ExpressionUUID->"bdd498ab-8796-4a7e-ba8b-3a74d2c58ef0"],

Cell[TextData[{
 "The first such function that we consider is ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 ". This function takes one argument, the name of the graph, and returns true \
or false. As an example, consider the complete bipartite graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"2", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "f1b530e6-a0db-4008-8362-b744beeaaf3b"],
 " and its complement."
}], "Text",ExpressionUUID->"737136be-1b47-441e-b3a6-98b69ee4e116"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteGraph", "[", 
  RowBox[{"{", 
   RowBox[{"2", ",", "3"}], "}"}], "]"}]], "Input",
 CellLabel->
  "In[166]:=",ExpressionUUID->"d926ec9b-92d8-48bc-8248-27ef63a7a871"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 3, 6, 8, 10, 12}, {{3}, {4}, {5}, {3}, {4}, {5}, {1}, {2}, {
          1}, {2}, {1}, {2}}}, Pattern}]}, {
      GraphLayout -> {"MultipartiteLayout", "VertexPartition" -> {2, 3}}}]]}, 
    
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0., -0.17950849718747378`}, {0., 
       0.17950849718747378`}, {1., -0.35901699437494755`}, {1., 0.}, {1., 
       0.35901699437494755`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.01273]}, 
         {Arrowheads[0.], ArrowBox[{2, 5}, 0.01273]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.01273], DiskBox[2, 0.01273], DiskBox[3, 0.01273], 
         DiskBox[4, 0.01273], DiskBox[5, 0.01273]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[166]=",ExpressionUUID->"800c182d-1f11-487e-a9b7-4f54634bc5d1"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConnectedGraphQ", "[", 
  RowBox[{"CompleteGraph", "[", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}]], "Input",
 CellLabel->
  "In[167]:=",ExpressionUUID->"5ac03750-2d01-4492-8e2f-3ebe31d854be"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[167]=",ExpressionUUID->"d5fe9294-304b-47d9-8aae-bc99ebc10707"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphComplement", "[", 
  RowBox[{"CompleteGraph", "[", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}]], "Input",
 CellLabel->
  "In[168]:=",ExpressionUUID->"b84b7b89-f09c-4ff6-8368-ad2cd36ac3d9"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 1, 2, 4, 6, 8}, {{2}, {1}, {4}, {5}, {3}, {5}, {3}, {4}}}, 
         Pattern}]}, {GraphLayout -> "CircularEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.9510565162951538, 
       0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
       0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
       0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{4, 5}, 0.021812234931106983`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.021812234931106983], DiskBox[2, 0.021812234931106983], 
         DiskBox[3, 0.021812234931106983], DiskBox[4, 0.021812234931106983], 
         DiskBox[5, 0.021812234931106983]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[168]=",ExpressionUUID->"05216e24-ef26-4b85-a5a8-0dd8843b9702"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConnectedGraphQ", "[", 
  RowBox[{"GraphComplement", "[", 
   RowBox[{"CompleteGraph", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}], "]"}]], "Input",
 CellLabel->
  "In[169]:=",ExpressionUUID->"a3478fe8-8798-4249-beba-84c3c3ed0f9f"],

Cell[BoxData["False"], "Output",
 CellLabel->
  "Out[169]=",ExpressionUUID->"02e5d97f-4561-479a-afc2-c018e9ea741e"]
}, Open  ]],

Cell[TextData[{
 "When used with a directed graph, ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 " returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the directed graph is strongly connected."
}], "Text",ExpressionUUID->"5009e94d-0c88-4fd3-b147-9898b7362822"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"stronglyConnectedEx", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[Rule]", "2"}], ",", 
     RowBox[{"2", "\[Rule]", "3"}], ",", 
     RowBox[{"3", "\[Rule]", "4"}], ",", 
     RowBox[{"4", "\[Rule]", "1"}], ",", 
     RowBox[{"1", "\[Rule]", "5"}], ",", 
     RowBox[{"5", "\[Rule]", "2"}], ",", 
     RowBox[{"3", "\[Rule]", "5"}], ",", 
     RowBox[{"5", "\[Rule]", "4"}]}], "}"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[170]:=",ExpressionUUID->"332e4ac2-b07d-450f-bf0a-e6d0fa1a56ac"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 
      5}, {{{1, 2}, {2, 3}, {3, 4}, {4, 1}, {1, 5}, {5, 2}, {3, 5}, {5, 4}}, 
       Null}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0., 1.1537108639737284`}, {0.004916287076213055, 
       0.}, {1.1607554929704147`, 0.0027887853451148414`}, {1.154988119841705,
        1.1570175257891293`}, {0.5792897237044975, 0.5778516807252814}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
         ArrowBox[{{1, 2}, {1, 5}, {2, 3}, {3, 4}, {3, 5}, {4, 1}, {5, 2}, {5,
           4}}, 0.015417340259311824`]}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.015417340259311824], DiskBox[2, 0.015417340259311824], 
         DiskBox[3, 0.015417340259311824], DiskBox[4, 0.015417340259311824], 
         DiskBox[5, 0.015417340259311824]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[170]=",ExpressionUUID->"fcae593d-76e5-4623-9b29-8b5cd978dc27"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 " reveals that the above graph is connected."
}], "Text",ExpressionUUID->"4be76e03-fe22-44a5-aa40-93445a59b795"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConnectedGraphQ", "[", "stronglyConnectedEx", "]"}]], "Input",
 CellLabel->
  "In[171]:=",ExpressionUUID->"6e9eae8e-e3da-45d0-871e-0cf0218eee89"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[171]=",ExpressionUUID->"a6cf5b98-3582-4eb3-aad9-acd6ca45faa9"]
}, Open  ]],

Cell[TextData[{
 "The following example, while weakly connected, is not strongly connected \
and thus ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 " will return ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"3f5c83f6-57a9-452b-a8b2-0d1b2a5f034d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"weaklyConnectedEx", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"4", "\[Rule]", "2"}], ",", 
     RowBox[{"2", "\[Rule]", "1"}], ",", 
     RowBox[{"1", "\[Rule]", "3"}], ",", 
     RowBox[{"3", "\[Rule]", "4"}], ",", 
     RowBox[{"4", "\[Rule]", "5"}], ",", 
     RowBox[{"6", "\[Rule]", "8"}], ",", 
     RowBox[{"8", "\[Rule]", "9"}], ",", 
     RowBox[{"9", "\[Rule]", "7"}], ",", 
     RowBox[{"7", "\[Rule]", "6"}], ",", 
     RowBox[{"6", "\[Rule]", "5"}]}], "}"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[172]:=",ExpressionUUID->"750d0bca-fba3-4ad7-92ea-d961af64289c"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{4, 2, 1, 3, 5, 6, 8, 9, 
      7}, {{{1, 2}, {2, 3}, {3, 4}, {4, 1}, {1, 5}, {6, 7}, {7, 8}, {8, 9}, {
       9, 6}, {6, 5}}, Null}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{1.6267075319767295`, 0.3921723219111086}, {
       0.7069903983693449, 0.7843695287371116}, {0., 0.3926757989121187}, {
       0.7072390316371968, 0.}, {2.928118570265649, 0.3921468659152118}, {
       4.229319840743651, 0.3923558385375609}, {5.148893514966267, 
       0.7850029976131969}, {5.855663518252539, 0.3921823793461117}, {
       5.149307991705942, 0.000026629052002513642`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
         ArrowBox[{{1, 2}, {1, 5}, {2, 3}, {3, 4}, {4, 1}, {6, 5}, {6, 7}, {7,
           8}, {8, 9}, {9, 6}}, 0.047727837353446295`]}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.047727837353446295], DiskBox[2, 0.047727837353446295], 
         DiskBox[3, 0.047727837353446295], DiskBox[4, 0.047727837353446295], 
         DiskBox[5, 0.047727837353446295], DiskBox[6, 0.047727837353446295], 
         DiskBox[7, 0.047727837353446295], DiskBox[8, 0.047727837353446295], 
         DiskBox[9, 0.047727837353446295]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[172]=",ExpressionUUID->"ab161af4-0c1e-4464-8312-3b26a1443b14"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConnectedGraphQ", "[", "weaklyConnectedEx", "]"}]], "Input",
 CellLabel->
  "In[173]:=",ExpressionUUID->"636786aa-def6-4fd9-8ebd-12d8b83a0a58"],

Cell[BoxData["False"], "Output",
 CellLabel->
  "Out[173]=",ExpressionUUID->"e6171fbd-a601-4526-9d1e-aa62f127118a"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["WeaklyConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WeaklyConnectedGraphQ"], "FunctionLink"],
 " function will determine whether a directed graph is weakly connected. A \
graph is weakly connected if it is connected as an undirected graph."
}], "Text",ExpressionUUID->"17fe7850-2ec2-4985-9523-5aeedb434450"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WeaklyConnectedGraphQ", "[", "weaklyConnectedEx", "]"}]], "Input",
 CellLabel->
  "In[174]:=",ExpressionUUID->"92d8a710-3803-4049-ac08-ab23598a4f47"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[174]=",ExpressionUUID->"2106bfa5-bb69-47b6-aa8f-1d4f29f5ecf4"]
}, Open  ]],

Cell[TextData[{
 "The Wolfram Language also has functions to extract the connected components \
of a graph that is not connected. The ",
 StyleBox[ButtonBox["ConnectedComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedComponents"], "FunctionLink"],
 " function takes a graph as input and returns a list of lists of vertices. "
}], "Text",ExpressionUUID->"7155f219-be2b-4b8e-ba5d-ae533e70eb41"],

Cell[TextData[{
 "As an example, consider the complement of the graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"2", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "4ad90717-7887-4707-8a85-e060b386489e"],
 "."
}], "Text",ExpressionUUID->"76b19247-320d-463e-b1a1-ce556b74160c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphComplement", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], "]"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}]}], "]"}]], "Input",
 CellLabel->
  "In[175]:=",ExpressionUUID->"99ea4a50-847f-444c-9a2e-88cb3a209dcf"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 1, 2, 4, 6, 8}, {{2}, {1}, {4}, {5}, {3}, {5}, {3}, {4}}}, 
         Pattern}]}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {"Name"}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}}, 
          0.021812234931106983`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
          0.9510565162951533, 0.30901699437494634`}}, 0.021812234931106983`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924737, -0.8090169943749481}, {-7.044813998280222*^-16, 
          1.}}, 0.021812234931106983`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951533, 
          0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
          0.021812234931106983`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
          0.021812234931106983], 
         InsetBox["1", 
          Offset[{2, 2}, {-0.9292442813640468, 0.3308292293060553}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
          0.021812234931106983], 
         InsetBox["2", 
          Offset[{2, 2}, {-0.5659730173613657, -0.7872047594438398}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924737, -0.8090169943749481}, 
          0.021812234931106983], 
         InsetBox["3", 
          Offset[{2, 2}, {0.6095974872235806, -0.7872047594438412}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9510565162951533, 0.30901699437494634}, 
          0.021812234931106983], 
         InsetBox["4", 
          Offset[{2, 2}, {0.9728687512262603, 0.3308292293060533}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.021812234931106983], 
         InsetBox["5", 
          Offset[{2, 2}, {0.02181223493110628, 1.021812234931107}], 
          ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[175]=",ExpressionUUID->"55e50d5b-6377-40cd-938e-f29d89c4252f"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConnectedComponents", "[", 
  RowBox[{"GraphComplement", "[", 
   RowBox[{"CompleteGraph", "[", 
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], "]"}], "]"}], "]"}]], "Input",
 CellLabel->
  "In[176]:=",ExpressionUUID->"fb08f507-216c-4896-9b95-25b3c5cab535"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}]}], "}"}]], "Output",
 CellLabel->
  "Out[176]=",ExpressionUUID->"8d02cf7b-2f52-4dd9-8ed6-7abcb317e088"]
}, Open  ]],

Cell[TextData[{
 "This output indicates that the complement of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"2", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "2047364d-3b1c-4789-bc06-eed90e2844ad"],
 " has two connected components, one with vertex set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]],ExpressionUUID->
  "8d517694-2100-4f56-882f-efea43e0443d"],
 " and the other with vertex set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "5"}], "}"}], TraditionalForm]],
  ExpressionUUID->"1cdc4891-88fd-4133-85cb-ddd41ae8651e"],
 "."
}], "Text",ExpressionUUID->"c46cc1f4-bb7a-40e5-b1f1-0ee97db44242"],

Cell[TextData[{
 "For directed graphs, ",
 StyleBox[ButtonBox["ConnectedComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedComponents"], "FunctionLink"],
 " will produce the strongly connected components. The ",
 StyleBox[ButtonBox["WeaklyConnectedComponents",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WeaklyConnectedComponents"], "FunctionLink"],
 " function will output the weakly connected components."
}], "Text",ExpressionUUID->"d170df3f-cd3d-4c29-aae2-001b953658c8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConnectedComponents", "[", "weaklyConnectedEx", "]"}]], "Input",
 CellLabel->
  "In[177]:=",ExpressionUUID->"cdbf4e12-6abc-4572-b855-2266e1fbf945"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "5", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "8", ",", "9", ",", "7"}], "}"}]}], "}"}]], "Output",
 CellLabel->
  "Out[177]=",ExpressionUUID->"eb24eb34-5a9b-423f-b257-9759659ab1db"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WeaklyConnectedComponents", "[", "weaklyConnectedEx", 
  "]"}]], "Input",
 CellLabel->
  "In[178]:=",ExpressionUUID->"84099db6-5870-419b-8c3c-9cf43bdf86fa"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
   "4", ",", "2", ",", "1", ",", "3", ",", "5", ",", "6", ",", "8", ",", "9", 
    ",", "7"}], "}"}], "}"}]], "Output",
 CellLabel->
  "Out[178]=",ExpressionUUID->"afb79e27-98e7-4c13-9518-6d2f40454089"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coloring the Components", "Subsection",
 CellTags->"Section4a",ExpressionUUID->"92db923f-3f31-43f5-b0c7-852336f274ae"],

Cell["\<\
We now present a function that will color code the connected components in a \
graph. (This function will color up to ten components before repeating \
colors.) \
\>", "Text",ExpressionUUID->"c03e6756-047d-455c-adb8-c4db36c8c158"],

Cell[TextData[{
 "As in the previous section, we use ",
 StyleBox[ButtonBox["ColorData",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorData"], "FunctionLink"],
 " to obtain the colors. Also note the use of ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " to set the ",
 StyleBox[ButtonBox["VertexStyle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexStyle"], "FunctionLink"],
 " property. We first saw ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " in the subsection on edge contraction in Section 10.2 of this manual. \
Finally, note the use of ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " with third argument 1, which causes the result of ",
 StyleBox[ButtonBox["Mod",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Mod"], "FunctionLink"],
 " to have minimum value 1. This allows us to cycle through the list of ten \
colors provided by the particular ",
 StyleBox[ButtonBox["ColorDataFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorDataFunction"], "FunctionLink"],
 " we chose."
}], "Text",ExpressionUUID->"cb5ae444-6e97-4798-b9b2-37cdf28a9215"],

Cell[BoxData[
 RowBox[{
  RowBox[{"highlightComponents", "[", 
   RowBox[{"G_Graph", ",", "opts___"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"components", ",", "c", ",", "i", ",", "v", ",", 
      RowBox[{"H", "=", "G"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"components", "=", 
      RowBox[{"ConnectedComponents", "[", "G", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"c", "=", "0"}], ";", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{"Length", "[", "components", "]"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"c", "=", 
         RowBox[{"Mod", "[", 
          RowBox[{
           RowBox[{"c", "+", "1"}], ",", "10", ",", "1"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"PropertyValue", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"H", ",", "v"}], "}"}], ",", "VertexStyle"}], "]"}], 
           "=", 
           RowBox[{
            RowBox[{"ColorData", "[", "14", "]"}], "[", "c", "]"}]}], 
          "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"v", ",", 
            RowBox[{"components", "[", 
             RowBox[{"[", "i", "]"}], "]"}]}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Graph", "[", 
      RowBox[{"H", ",", "opts"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[179]:=",ExpressionUUID->"4f92f03f-1352-4493-8074-853f8386c6d7"],

Cell["We apply this function to the weakly connected graph above.", "Text",ExpressionUUID->"481e0be2-6e71-438f-9dde-add8147d571b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"highlightComponents", "[", 
  RowBox[{"weaklyConnectedEx", ",", 
   RowBox[{"VertexSize", "\[Rule]", "Medium"}]}], "]"}]], "Input",
 CellLabel->
  "In[181]:=",ExpressionUUID->"0ce6255f-f8a5-4285-a8e1-50959d671e78"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{4, 2, 1, 3, 5, 6, 8, 9, 
      7}, {{{1, 2}, {2, 3}, {3, 4}, {4, 1}, {1, 5}, {6, 7}, {7, 8}, {8, 9}, {
       9, 6}, {6, 5}}, Null}, {
      GraphLayout -> {"Dimension" -> 2}, VertexSize -> {Medium}, 
       VertexStyle -> {
        6 -> RGBColor[1., 0.7333333333333333, 0.2], 1 -> 
         RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 9 -> 
         RGBColor[1., 0.7333333333333333, 0.2], 2 -> 
         RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 8 -> 
         RGBColor[1., 0.7333333333333333, 0.2], 3 -> 
         RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 4 -> 
         RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 7 -> 
         RGBColor[1., 0.7333333333333333, 0.2], 5 -> 
         RGBColor[
          0.8352941176470589, 0.5764705882352941, 0.5607843137254902]}}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{1.6267075319767295`, 0.3921723219111086}, {
       0.7069903983693449, 0.7843695287371116}, {0., 0.3926757989121187}, {
       0.7072390316371968, 0.}, {2.928118570265649, 0.3921468659152118}, {
       4.229319840743651, 0.3923558385375609}, {5.148893514966267, 
       0.7850029976131969}, {5.855663518252539, 0.3921823793461117}, {
       5.149307991705942, 0.000026629052002513642`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[Medium], 
         ArrowBox[{{1, 2}, {1, 5}, {2, 3}, {3, 4}, {4, 1}, {6, 5}, {6, 7}, {7,
           8}, {8, 9}, {9, 6}}, 0.07843695681436019]}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         {RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 
          DiskBox[1, 0.07843695681436019]}, 
         {RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 
          DiskBox[2, 0.07843695681436019]}, 
         {RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 
          DiskBox[3, 0.07843695681436019]}, 
         {RGBColor[
          0.9490196078431372, 0.29411764705882354`, 0.050980392156862744`], 
          DiskBox[4, 0.07843695681436019]}, 
         {RGBColor[
          0.8352941176470589, 0.5764705882352941, 0.5607843137254902], 
          DiskBox[5, 0.07843695681436019]}, 
         {RGBColor[1., 0.7333333333333333, 0.2], 
          DiskBox[6, 0.07843695681436019]}, 
         {RGBColor[1., 0.7333333333333333, 0.2], 
          DiskBox[7, 0.07843695681436019]}, 
         {RGBColor[1., 0.7333333333333333, 0.2], 
          DiskBox[8, 0.07843695681436019]}, 
         {RGBColor[1., 0.7333333333333333, 0.2], 
          DiskBox[9, 0.07843695681436019]}}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[181]=",ExpressionUUID->"3be8d9c2-159d-4cc4-8637-097c2d71b291"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Counting Paths Between Vertices", "Subsection",
 CellTags->"Section4b",ExpressionUUID->"57f070f6-2b90-4bde-857a-9713e6787c03"],

Cell[TextData[{
 "The last topic that we consider in this section is determining the number \
of paths between two vertices of a given length. As described in the \
textbook, if ",
 Cell[BoxData[
  FormBox["A", TraditionalForm]],ExpressionUUID->
  "9f0ed7d0-4bfc-4504-a0b6-65350d76ae98"],
 " is the adjacency matrix for a graph (which may be undirected or directed \
and may include loops and multiple edges), then the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"i", ",", "j"}], ")"}], TraditionalForm]],ExpressionUUID->
  "5edeb31b-f70f-4d2e-bdcd-076c1ac6550e"],
 " entry of the matrix ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["A", "r"], TraditionalForm]],ExpressionUUID->
  "8952052a-44d3-4b25-8572-1f8ab23a0802"],
 " is the number of paths of length ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]],ExpressionUUID->
  "6e34cb3d-8d02-429a-8cd7-469a2e003c31"],
 " from vertex ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]],ExpressionUUID->
  "6ef541ba-f701-4acb-b902-1d97212ebc9a"],
 " to vertex ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]],ExpressionUUID->
  "248b0eb5-29d6-449f-8d74-c38eb373db4e"],
 "."
}], "Text",ExpressionUUID->"82440408-505f-400c-8887-3409a2377c6e"],

Cell[TextData[{
 "As an example, consider the ",
 StyleBox["stronglyConnectedEx", "Code"],
 " graph from above. We can obtain its adjacency matrix by applying the ",
 StyleBox[ButtonBox["AdjacencyMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyMatrix"], "FunctionLink"],
 " function to the name of the graph. "
}], "Text",ExpressionUUID->"2ea29578-a7f4-4123-9d48-7005a98e0683"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"aMatrix", "=", 
   RowBox[{"AdjacencyMatrix", "[", "stronglyConnectedEx", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"aMatrix", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[182]:=",ExpressionUUID->"beea9479-6612-4ec2-b9f2-9ae8e10fda6b"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0", "1", "0", "0", "1"},
     {"0", "0", "1", "0", "0"},
     {"0", "0", "0", "1", "1"},
     {"1", "0", "0", "0", "0"},
     {"0", "1", "0", "1", "0"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[
   SparseArray[
    Automatic, {5, 5}, 0, {
     1, {{0, 2, 3, 5, 6, 8}, {{2}, {5}, {3}, {4}, {5}, {1}, {2}, {4}}}, {1, 1,
       1, 1, 1, 1, 1, 1}}]]]]], "Output",
 CellLabel->
  "Out[183]//MatrixForm=",ExpressionUUID->"8279410f-f428-4b5b-8a83-\
fd6e7fe4e390"]
}, Open  ]],

Cell[TextData[{
 "Recall that ",
 StyleBox[ButtonBox["AdjacencyMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyMatrix"], "FunctionLink"],
 " produces a ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 ". You could use ",
 StyleBox[ButtonBox["Normal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Normal"], "FunctionLink"],
 " to transform the ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 " object into a usual list of lists representation of the matrix. However, \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can compute more efficiently with the ",
 StyleBox[ButtonBox["SparseArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SparseArray"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"d3b11ced-ed61-4391-b73d-42c0c60a27ec"],

Cell["Next, compute some powers of the adjacency matrix.", "Text",ExpressionUUID->"5eb94f53-d1b1-4db4-80dc-8b01f1077c44"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"MatrixForm", "[", 
    RowBox[{"MatrixPower", "[", 
     RowBox[{"aMatrix", ",", "i"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "2", ",", "7"}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[184]:=",ExpressionUUID->"813b3d7f-f5e1-42eb-a2d1-f7302e5d07da"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"0", "1", "1", "1", "0"},
       {"0", "0", "0", "1", "1"},
       {"1", "1", "0", "1", "0"},
       {"0", "1", "0", "0", "1"},
       {"1", "0", "1", "0", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[
     SparseArray[
      Automatic, {5, 5}, 0, {
       1, {{0, 3, 5, 8, 10, 12}, {{3}, {2}, {4}, {4}, {5}, {1}, {2}, {4}, {
         2}, {5}, {3}, {1}}}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1}}]]]], ",", 
   
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0", "1", "1", "1"},
       {"1", "1", "0", "1", "0"},
       {"1", "1", "1", "0", "1"},
       {"0", "1", "1", "1", "0"},
       {"0", "1", "0", "1", "2"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[
     SparseArray[
      Automatic, {5, 5}, 0, {
       1, {{0, 4, 7, 11, 14, 17}, {{4}, {5}, {3}, {1}, {1}, {2}, {4}, {2}, {
         5}, {3}, {1}, {3}, {2}, {4}, {4}, {5}, {2}}}, {1, 1, 1, 1, 1, 1, 1, 
        1, 1, 1, 1, 1, 1, 1, 1, 2, 1}}]]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "2", "0", "2", "2"},
       {"1", "1", "1", "0", "1"},
       {"0", "2", "1", "2", "2"},
       {"1", "0", "1", "1", "1"},
       {"1", "2", "1", "2", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[
     SparseArray[
      Automatic, {5, 5}, 0, {
       1, {{0, 4, 8, 12, 16, 20}, {{1}, {2}, {4}, {5}, {2}, {5}, {3}, {1}, {
         3}, {2}, {4}, {5}, {4}, {5}, {3}, {1}, {1}, {2}, {4}, {3}}}, {1, 2, 
        2, 2, 1, 1, 1, 1, 1, 2, 2, 2, 1, 1, 1, 1, 1, 2, 2, 1}}]]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"2", "3", "2", "2", "1"},
       {"0", "2", "1", "2", "2"},
       {"2", "2", "2", "3", "1"},
       {"1", "2", "0", "2", "2"},
       {"2", "1", "2", "1", "2"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[
     SparseArray[
      Automatic, {5, 5}, 0, {
       1, {{0, 5, 9, 14, 18, 23}, {{2}, {5}, {3}, {1}, {4}, {3}, {2}, {4}, {
         5}, {4}, {5}, {3}, {1}, {2}, {1}, {2}, {4}, {5}, {2}, {5}, {3}, {
         1}, {4}}}, {3, 1, 2, 2, 2, 1, 2, 2, 2, 3, 1, 2, 2, 2, 1, 2, 2, 2, 1, 
        2, 2, 2, 1}}]]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"2", "3", "3", "3", "4"},
       {"2", "2", "2", "3", "1"},
       {"3", "3", "2", "3", "4"},
       {"2", "3", "2", "2", "1"},
       {"1", "4", "1", "4", "4"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[
     SparseArray[
      Automatic, {5, 5}, 0, {
       1, {{0, 5, 10, 15, 20, 25}, {{3}, {2}, {4}, {5}, {1}, {4}, {5}, {3}, {
         1}, {2}, {1}, {2}, {4}, {5}, {3}, {2}, {5}, {3}, {1}, {4}, {3}, {
         2}, {4}, {5}, {1}}}, {3, 3, 3, 4, 2, 3, 1, 2, 2, 2, 3, 3, 3, 4, 2, 3,
         1, 2, 2, 2, 1, 4, 4, 4, 1}}]]]], ",", 
   TagBox[
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"3", "6", "3", "7", "5"},
       {"3", "3", "2", "3", "4"},
       {"3", "7", "3", "6", "5"},
       {"2", "3", "3", "3", "4"},
       {"4", "5", "4", "5", "2"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
    Function[BoxForm`e$, 
     MatrixForm[
     SparseArray[
      Automatic, {5, 5}, 0, {
       1, {{0, 5, 10, 15, 20, 25}, {{4}, {5}, {3}, {1}, {2}, {1}, {2}, {4}, {
         5}, {3}, {2}, {5}, {3}, {1}, {4}, {3}, {2}, {4}, {5}, {1}, {4}, {
         5}, {3}, {1}, {2}}}, {7, 5, 3, 3, 6, 3, 3, 3, 4, 2, 7, 5, 3, 3, 6, 3,
         3, 3, 4, 2, 5, 2, 4, 4, 5}}]]]]}], "}"}]], "Output",
 CellLabel->
  "Out[184]=",ExpressionUUID->"7150565b-90ff-49ae-8e43-5a7d5e4e44cf"]
}, Open  ]],

Cell[TextData[{
 "Note that the ",
 StyleBox[ButtonBox["MatrixPower",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MatrixPower"], "FunctionLink"],
 " function is used to compute powers of matrices. Using the ",
 StyleBox[ButtonBox["Power",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Power"], "FunctionLink"],
 " (",
 StyleBox["^", "Code"],
 ") operator on a matrix computes element-wise."
}], "Text",ExpressionUUID->"2c5b6104-1c5d-4034-932e-3c7eafe00bb5"],

Cell[TextData[{
 "We see that there are 4 paths of length 6 from vertex 3 to vertex 5, since \
the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"3", ",", "5"}], ")"}], TraditionalForm]],ExpressionUUID->
  "294f24a5-f48e-4783-af56-82fa6465af8e"],
 " entry in the 6th power of the adjacency matrix is 4. We also see that \
there are cycles of length 3 for every vertex and there are no cycles of \
length less than 3. Finally, we know that the shortest path from vertex 2 to \
vertex 1 is of length 3, since the ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"2", ",", "1"}], ")"}], TraditionalForm]],ExpressionUUID->
  "0cd0ff1c-a8c7-44f0-a90d-c30c407ff0f3"],
 " entry is 0 for the first and second powers of the matrix."
}], "Text",ExpressionUUID->"f7e8e71a-1d42-4a23-abdb-aa82bb376d4c"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.5 Euler and Hamilton Paths", "Section",
 CellTags->"Section5",ExpressionUUID->"883be708-f882-4ed6-9e72-06db73a03569"],

Cell[TextData[{
 "In this section, we will show how to use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to solve two problems that seem closely related, but which are quite \
different in computational complexity. The two problems that will be analyzed \
are the problem of finding a simple circuit that contains every edge exactly \
once (an Euler circuit) and the problem of finding a simple circuit that \
contains every vertex exactly once (a Hamilton circuit). (Note that the \
textbook uses the term circuit while the Wolfram Language uses the word \
cycle. These two terms are synonymous.)"
}], "Text",ExpressionUUID->"006a7469-9117-451a-aec6-a72040fa14a2"],

Cell[CellGroupData[{

Cell["Euler Circuits", "Subsection",
 CellTags->"Section5a",ExpressionUUID->"16c74548-e882-4f80-aaf0-225dca35522e"],

Cell[TextData[{
 "The Wolfram Language comes equipped with a function to determine if a given \
simple graph has an Euler circuit or not. This function, ",
 StyleBox[ButtonBox["EulerianGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EulerianGraphQ"], "FunctionLink"],
 ", takes one argument, a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. As an example, we have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " check to see if ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "5"], TraditionalForm]],ExpressionUUID->
  "8fd40d47-8aaf-4616-8629-6b0388bc8427"],
 " is Eulerian, that is, has an Euler circuit."
}], "Text",ExpressionUUID->"ed36fb97-8000-434c-8550-809f1c8ccdd5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EulerianGraphQ", "[", 
  RowBox[{"CompleteGraph", "[", "5", "]"}], "]"}]], "Input",
 CellLabel->
  "In[185]:=",ExpressionUUID->"31a7a6a5-d4ab-4974-81d3-a08040343a51"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[185]=",ExpressionUUID->"3b7e974a-2482-46a2-9191-8551e8b24f7a"]
}, Open  ]],

Cell[TextData[{
 "To explicitly find an Euler circuit, we use the function ",
 StyleBox[ButtonBox["FindEulerianCycle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindEulerianCycle"], "FunctionLink"],
 ". The function accepts one or two arguments. The first argument should be a \
graph, although you can provide a list of rules specifying the edges rather \
than a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object. If the graph is the only argument, the function returns a list \
containing a single list of edges representing an Euler circuit. For example, \
the following identifies an Euler circuit on the complete graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "5"], TraditionalForm]],ExpressionUUID->
  "796185d6-254d-4252-affa-78f17133eb86"],
 "."
}], "Text",ExpressionUUID->"98632c14-6b9a-447e-a2ed-db65dc3946d8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindEulerianCycle", "[", 
  RowBox[{"CompleteGraph", "[", "5", "]"}], "]"}]], "Input",
 CellLabel->
  "In[186]:=",ExpressionUUID->"dd73c6a9-b18c-4ea0-a212-5c22efd31370"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "\[UndirectedEdge]", "5"}], ",", 
    RowBox[{"5", "\[UndirectedEdge]", "4"}], ",", 
    RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
    RowBox[{"3", "\[UndirectedEdge]", "5"}], ",", 
    RowBox[{"5", "\[UndirectedEdge]", "2"}], ",", 
    RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
    RowBox[{"4", "\[UndirectedEdge]", "1"}], ",", 
    RowBox[{"1", "\[UndirectedEdge]", "3"}], ",", 
    RowBox[{"3", "\[UndirectedEdge]", "2"}], ",", 
    RowBox[{"2", "\[UndirectedEdge]", "1"}]}], "}"}], "}"}]], "Output",
 CellLabel->
  "Out[186]=",ExpressionUUID->"4a0acf2f-a49c-4067-a257-31eef7d77648"]
}, Open  ]],

Cell[TextData[{
 "If you provide ",
 StyleBox[ButtonBox["FindEulerianCycle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindEulerianCycle"], "FunctionLink"],
 " with a positive integer as a second argument, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will attempt to find more than one Euler circuit, with the integer serving \
as a maximum number of cycles to return, provided they exist. In this case, \
the output will be a list of lists of edges, with each sublist representing a \
distinct circuit."
}], "Text",ExpressionUUID->"b8cd6e88-56e3-484c-9243-26365da06b81"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindEulerianCycle", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", "5", "]"}], ",", "3"}], "]"}]], "Input",
 CellLabel->
  "In[187]:=",ExpressionUUID->"cac7af16-2345-4133-8daa-3ee77412f625"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "1"}], ",", 
     RowBox[{"1", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "1"}], ",", 
     RowBox[{"1", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "1"}], ",", 
     RowBox[{"1", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "1"}]}], "}"}]}], "}"}]], "Output",
 CellLabel->
  "Out[187]=",ExpressionUUID->"1feacedc-d7f5-4997-9872-b4b992170ab5"]
}, Open  ]],

Cell[TextData[{
 "With the symbol ",
 StyleBox[ButtonBox["All",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/All"], "FunctionLink"],
 " as the second argument, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will determine all of the Euler circuits. We see below that the complete \
graph on 5 vertices has 132 Euler circuits."
}], "Text",ExpressionUUID->"3bf1bdce-ceaf-4658-b697-ff9fe1158226"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"FindEulerianCycle", "[", 
   RowBox[{
    RowBox[{"CompleteGraph", "[", "5", "]"}], ",", "All"}], "]"}], 
  "]"}]], "Input",
 CellLabel->
  "In[188]:=",ExpressionUUID->"3ce37e20-c092-4f8b-8cf4-799ecb3ea9ca"],

Cell[BoxData["132"], "Output",
 CellLabel->
  "Out[188]=",ExpressionUUID->"ed9b6ab0-763b-4797-a2c0-832f589f9100"]
}, Open  ]],

Cell[TextData[{
 "Now, we will visualize this path by creating an animation that successively \
highlights the edges in the path. To do this we will use the ",
 StyleBox[ButtonBox["Animate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Animate"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["Animate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Animate"], "FunctionLink"],
 " function takes two arguments, similar to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". The first argument is a Wolfram Language expression, typically one that \
generates an image and that is dependent on a variable. The second argument \
is a list describing the range of the variable. The structure of this list is \
similar to the second argument in a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". "
}], "Text",ExpressionUUID->"ff89fe66-b77f-44aa-997e-0588d6f12917"],

Cell[TextData[{
 "We will also be making use of two options. Setting the ",
 StyleBox[ButtonBox["AnimationRunning",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AnimationRunning"], "FunctionLink"],
 " option to ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " will prevent the animation from beginning until you explicitly click on \
the play button. Without this option, the animation would run as soon as ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has finished generating it. Setting the ",
 StyleBox[ButtonBox["AnimationRepetitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AnimationRepetitions"], "FunctionLink"],
 " option to 1 will cause the animation to stop once it has played through. \
This option defaults to ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 ", meaning it will automatically restart every time it reaches the end."
}], "Text",ExpressionUUID->"a382b5f7-28f8-47b5-a05a-ced8e9a37cf3"],

Cell[TextData[{
 "To create the animation, we need a graph and a circuit. We will use ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "5"], TraditionalForm]],ExpressionUUID->
  "2d7bf8e9-cded-4ba6-9093-d7805118af04"],
 " as the graph and we store the circuit as ",
 StyleBox["exampleCircuit", "Code"],
 ". Note that we apply ",
 StyleBox[ButtonBox["First",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/First"], "FunctionLink"],
 " since ",
 StyleBox[ButtonBox["FindEulerianCycle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindEulerianCycle"], "FunctionLink"],
 " returns a list of lists."
}], "Text",ExpressionUUID->"ea5d10d0-78e9-4161-96b6-5c2b96ad6e2e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleCircuit", "=", 
  RowBox[{"First", "[", 
   RowBox[{"FindEulerianCycle", "[", 
    RowBox[{"CompleteGraph", "[", "5", "]"}], "]"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[189]:=",ExpressionUUID->"9cadf90d-b27f-40f9-a0a2-6684bb932d21"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "\[UndirectedEdge]", "5"}], ",", 
   RowBox[{"5", "\[UndirectedEdge]", "4"}], ",", 
   RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
   RowBox[{"3", "\[UndirectedEdge]", "5"}], ",", 
   RowBox[{"5", "\[UndirectedEdge]", "2"}], ",", 
   RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
   RowBox[{"4", "\[UndirectedEdge]", "1"}], ",", 
   RowBox[{"1", "\[UndirectedEdge]", "3"}], ",", 
   RowBox[{"3", "\[UndirectedEdge]", "2"}], ",", 
   RowBox[{"2", "\[UndirectedEdge]", "1"}]}], "}"}]], "Output",
 CellLabel->
  "Out[189]=",ExpressionUUID->"818be0ce-b3d6-446e-af80-e86d2067ab51"]
}, Open  ]],

Cell[TextData[{
 "To display the path, we will apply the ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 " function. This function was first described in Section 10.2 in the section \
on bipartite graphs. It takes two arguments: a graph and a list containing \
vertices, edges, or subgraphs."
}], "Text",ExpressionUUID->"0a9db13a-17ab-4350-8405-71d264e5b73a"],

Cell[TextData[{
 "To draw the successive stages in the circuit, we will apply ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 " to the graph and to a sublist of ",
 StyleBox["exampleCircuit", "Code"],
 ". We will obtain the sublist by applying ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Code"],
 ") to a ",
 StyleBox[ButtonBox["Span",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Span"], "FunctionLink"],
 " (",
 StyleBox[";;", "Code"],
 ") from 1 to the current stage. For example, to display the path after three \
steps, we enter the following:"
}], "Text",ExpressionUUID->"b685e1bf-87a6-48b2-a731-cb101b8afc8b"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", "5", "]"}], ",", 
   RowBox[{"exampleCircuit", "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", "3"}], "]"}], "]"}]}], "]"}]], "Input",
 CellLabel->
  "In[190]:=",ExpressionUUID->"09e31f20-958c-47c6-8f6a-c39596b01cb7"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 4, 8, 12, 16, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {5}, {
          1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {GraphHighlight -> {
         UndirectedEdge[5, 4], 
         UndirectedEdge[1, 5], 
         UndirectedEdge[4, 3]}, GraphLayout -> "CircularEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.9510565162951538, 
       0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
       0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
       0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}},
            0.021812234931106983`]}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9510565162951538, 0.30901699437494834`}, {
           0.5877852522924737, -0.8090169943749481}}, 0.021812234931106983`]}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.9510565162951538, 0.30901699437494834`}, {
           0.9510565162951533, 0.30901699437494634`}}, 
           0.021812234931106983`]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{-0.9510565162951538, 
            0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
            0.021812234931106983`]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
           0.5877852522924737, -0.8090169943749481}}, 0.021812234931106983`]}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
           0.9510565162951533, 0.30901699437494634`}}, 
           0.021812234931106983`]}, 
         {Arrowheads[0.], 
          ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, \
{-7.044813998280222*^-16, 1.}}, 0.021812234931106983`]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 0.30901699437494634`}}, 
            0.021812234931106983`]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{
           0.5877852522924737, -0.8090169943749481}, {-7.044813998280222*^-16,
            1.}}, 0.021812234931106983`]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
            0.021812234931106983`]}}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.021812234931106983], DiskBox[2, 0.021812234931106983], 
         DiskBox[3, 0.021812234931106983], DiskBox[4, 0.021812234931106983], 
         DiskBox[5, 0.021812234931106983]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[190]=",ExpressionUUID->"052b5b34-b9a0-4ae3-abec-4c55d6d18d28"]
}, Open  ]],

Cell["\<\
Note that the span from 1 to 0 will result in the empty list and thus nothing \
highlighted.\
\>", "Text",ExpressionUUID->"9685c40f-3cc4-4a5b-8f55-57b715a011cf"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exampleCircuit", "[", 
  RowBox[{"[", 
   RowBox[{"1", ";;", "0"}], "]"}], "]"}]], "Input",
 CellLabel->
  "In[191]:=",ExpressionUUID->"9f2c5162-d701-402d-9640-b18e56333fbd"],

Cell[BoxData[
 RowBox[{"{", "}"}]], "Output",
 CellLabel->
  "Out[191]=",ExpressionUUID->"90f47ad7-c120-420a-b1d8-88612c697019"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", "5", "]"}], ",", 
   RowBox[{"exampleCircuit", "[", 
    RowBox[{"[", 
     RowBox[{"1", ";;", "0"}], "]"}], "]"}]}], "]"}]], "Input",
 CellLabel->
  "In[192]:=",ExpressionUUID->"33e9a646-7148-4254-afe8-6955e9743a2b"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 4, 8, 12, 16, 20}, {{2}, {3}, {4}, {5}, {1}, {3}, {4}, {5}, {
          1}, {2}, {4}, {5}, {1}, {2}, {3}, {5}, {1}, {2}, {3}, {4}}}, 
         Pattern}]}, {GraphLayout -> "CircularEmbedding"}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{-0.9510565162951538, 
       0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
       0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
       0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 3}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{2, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{3, 5}, 0.021812234931106983`]}, 
         {Arrowheads[0.], ArrowBox[{4, 5}, 0.021812234931106983`]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.021812234931106983], DiskBox[2, 0.021812234931106983], 
         DiskBox[3, 0.021812234931106983], DiskBox[4, 0.021812234931106983], 
         DiskBox[5, 0.021812234931106983]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[192]=",ExpressionUUID->"d1938e02-b6e6-4d53-94c2-27ecab80a879"]
}, Open  ]],

Cell[TextData[{
 "We produce the animation using ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 " as above, with the second argument to the ",
 StyleBox[ButtonBox["Span",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Span"], "FunctionLink"],
 " (",
 StyleBox[";;", "Code"],
 ") as a variable. In the second argument of ",
 StyleBox[ButtonBox["Animate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Animate"], "FunctionLink"],
 ", this variable will be set to run from 0 to the ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " of the path. Note that, unlike ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", the variables in an ",
 StyleBox[ButtonBox["Animate",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Animate"], "FunctionLink"],
 " are not assumed to be restricted to integers, so we must specify a step \
value of 1 in the variable specification."
}], "Text",ExpressionUUID->"eb863aef-0698-4bae-b4ac-c46f838bd590"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Animate", "[", 
  RowBox[{
   RowBox[{"HighlightGraph", "[", 
    RowBox[{
     RowBox[{"CompleteGraph", "[", "5", "]"}], ",", 
     RowBox[{"exampleCircuit", "[", 
      RowBox[{"[", 
       RowBox[{"1", ";;", "i"}], "]"}], "]"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", 
     RowBox[{"Length", "[", "exampleCircuit", "]"}], ",", "1"}], "}"}], ",", 
   RowBox[{"AnimationRunning", "\[Rule]", "False"}], ",", 
   RowBox[{"AnimationRepetitions", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellLabel->
  "In[193]:=",ExpressionUUID->"5a98ae5a-7162-461b-8161-97213356752a"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[$CellContext`i$$], 0, 10, 1}}, Typeset`size$$ = {
    450., {212., 217.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = True, $CellContext`i$6734$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$$, $CellContext`i$6734$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> HighlightGraph[
        CompleteGraph[5], 
        Part[$CellContext`exampleCircuit, 
         Span[1, $CellContext`i$$]]], 
      "Specifications" :> {{$CellContext`i$$, 0, 10, 1, AnimationRunning -> 
         False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{503., {256., 263.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellLabel->
  "Out[193]=",ExpressionUUID->"089a308f-4ad6-4f13-bdc0-027ccb427580"]
}, Open  ]],

Cell[TextData[{
 "We now turn this into a function. The only difference between the ",
 StyleBox["animatePath", "Code"],
 " function below and the example from above is that we replace the variable \
in the loop specification with the list ",
 StyleBox["{i,\[ThinSpace]0,\[ThinSpace]\"step\"}", "Code"],
 ". This syntax gives the variable ",
 StyleBox["i", "Code"],
 " the initial value 0 and labels it as \[OpenCurlyDoubleQuote]step\
\[CloseCurlyDoubleQuote] in the animation controller. "
}], "Text",ExpressionUUID->"5d9a9938-ee9c-4146-8cdd-a088536edef9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"animatePath", "[", 
   RowBox[{"g_Graph", ",", "p_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", "len"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"len", "=", 
      RowBox[{"Length", "[", "p", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"Animate", "[", 
      RowBox[{
       RowBox[{"HighlightGraph", "[", 
        RowBox[{"g", ",", 
         RowBox[{"p", "[", 
          RowBox[{"[", 
           RowBox[{"1", ";;", "i"}], "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"i", ",", "0", ",", "\"\<step\>\""}], "}"}], ",", "0", ",", 
         "len", ",", "1"}], "}"}], ",", 
       RowBox[{"AnimationRunning", "\[Rule]", "False"}], ",", 
       RowBox[{"AnimationRepetitions", "\[Rule]", "1"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[194]:=",ExpressionUUID->"615a8938-dc74-4988-ac6c-ab2570f08891"],

Cell["\<\
To use this function, we just pass it an Eulerian graph, a circuit, and any \
options for drawing the graph.\
\>", "Text",ExpressionUUID->"0edc27ce-1fea-4f87-876d-c7a34a1f6dd1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"animatePath", "[", 
  RowBox[{
   RowBox[{"CompleteGraph", "[", 
    RowBox[{"7", ",", 
     RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
     RowBox[{"VertexLabels", "\[Rule]", 
      RowBox[{"Placed", "[", 
       RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}]}], "]"}], ",", 
   RowBox[{"First", "[", 
    RowBox[{"FindEulerianCycle", "[", 
     RowBox[{"CompleteGraph", "[", "7", "]"}], "]"}], "]"}]}], "]"}]], "Input",\

 CellLabel->
  "In[195]:=",ExpressionUUID->"1a52f0ac-f6e6-4700-82c9-bf9f297590d3"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$6798$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`i$6798$$], 0, "step"}, 0, 21, 1}}, Typeset`size$$ = {
    450., {217., 222.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = True, $CellContext`i$6798$6799$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$6798$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$6798$$, $CellContext`i$6798$6799$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> HighlightGraph[
        Graph[{1, 2, 3, 4, 5, 6, 7}, {Null, 
          SparseArray[
          Automatic, {7, 7}, 0, {
           1, {{0, 6, 12, 18, 24, 30, 36, 42}, {{2}, {3}, {4}, {5}, {6}, {
             7}, {1}, {3}, {4}, {5}, {6}, {7}, {1}, {2}, {4}, {5}, {6}, {7}, {
             1}, {2}, {3}, {5}, {6}, {7}, {1}, {2}, {3}, {4}, {6}, {7}, {1}, {
             2}, {3}, {4}, {5}, {7}, {1}, {2}, {3}, {4}, {5}, {6}}}, 
            Pattern}]}, {
         GraphLayout -> "CircularEmbedding", VertexLabels -> {
            Placed["Name", Center]}, VertexSize -> {Medium}}], 
        Part[{
          UndirectedEdge[1, 7], 
          UndirectedEdge[7, 6], 
          UndirectedEdge[6, 5], 
          UndirectedEdge[5, 7], 
          UndirectedEdge[7, 4], 
          UndirectedEdge[4, 6], 
          UndirectedEdge[6, 3], 
          UndirectedEdge[3, 5], 
          UndirectedEdge[5, 4], 
          UndirectedEdge[4, 3], 
          UndirectedEdge[3, 7], 
          UndirectedEdge[7, 2], 
          UndirectedEdge[2, 6], 
          UndirectedEdge[6, 1], 
          UndirectedEdge[1, 5], 
          UndirectedEdge[5, 2], 
          UndirectedEdge[2, 4], 
          UndirectedEdge[4, 1], 
          UndirectedEdge[1, 3], 
          UndirectedEdge[3, 2], 
          UndirectedEdge[2, 1]}, 
         Span[1, $CellContext`i$6798$$]]], 
      "Specifications" :> {{{$CellContext`i$6798$$, 0, "step"}, 0, 21, 1, 
         AnimationRunning -> False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{503., {261., 268.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellLabel->
  "Out[195]=",ExpressionUUID->"cd250282-e810-48eb-89e5-3f40fc8d43b0"]
}, Open  ]],

Cell["\<\
You can see the Euler circuit traced out by clicking on the play button.\
\>", "Text",ExpressionUUID->"e5f716c5-2872-4db7-aa66-c43dea4443d0"],

Cell["\<\
Note that while our examples have all been undirected, the functions \
described here also apply to directed graphs, multigraphs, etc. Here, for \
example, is the bridges of K\[ODoubleDot]nigsberg graph.\
\>", "Text",ExpressionUUID->"484b3c74-d186-402a-aa01-bb10525be0d0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"konigsberg", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<B\>\""}], ",", 
      RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<B\>\""}], ",", 
      RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<C\>\""}], ",", 
      RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<C\>\""}], ",", 
      RowBox[{"\"\<A\>\"", "\[Rule]", "\"\<D\>\""}], ",", 
      RowBox[{"\"\<B\>\"", "\[Rule]", "\"\<D\>\""}], ",", 
      RowBox[{"\"\<C\>\"", "\[Rule]", "\"\<D\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}]}], "]"}]}]], "Input",
 CellLabel->
  "In[196]:=",ExpressionUUID->"99d7a1aa-2614-4e06-9e16-75d464cef9b5"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"A", "B", "C", "D"}, {
      Null, {{1, 2}, {1, 2}, {1, 3}, {1, 3}, {1, 4}, {2, 4}, {3, 4}}}, {
      VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9333992261410657, 0.8688665805294101}, {
          1.4721650633283785`, 0.8055746363293131}, {1.868189993467873, 
          0.43484730765149504`}}], 0.08688685691767564]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9333992261410657, 0.8688665805294101}, {
          1.3294241562805507`, 0.4981392518513735}, {1.868189993467873, 
          0.43484730765149504`}}], 0.08688685691767564]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9333992261410657, 0.8688665805294101}, {
          0.5382122372210536, 0.4979357926505048}, {0., 
          0.43398273710533725`}}], 0.08688685691767564]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0.9333992261410657, 0.8688665805294101}, {
          0.3951869889200116, 0.8049135249842548}, {0., 
          0.43398273710533725`}}], 0.08688685691767564]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9333992261410657, 0.8688665805294101}, {
          0.9352581888609788, 0.}}, 0.08688685691767564]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.868189993467873, 0.43484730765149504`}, {
          0.9352581888609788, 0.}}, 0.08688685691767564]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 0.43398273710533725`}, {0.9352581888609788, 0.}}, 
          0.08688685691767564]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{0.9333992261410657, 0.8688665805294101}, 
          0.08688685691767564], 
         InsetBox["\<\"A\"\>", {0.9333992261410657, 0.8688665805294101},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.868189993467873, 0.43484730765149504}, 
          0.08688685691767564], 
         InsetBox["\<\"B\"\>", {1.868189993467873, 0.43484730765149504},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0., 0.43398273710533725}, 0.08688685691767564], 
         InsetBox["\<\"C\"\>", {0., 0.43398273710533725},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9352581888609788, 0.}, 0.08688685691767564], 
         InsetBox["\<\"D\"\>", {0.9352581888609788, 0.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[196]=",ExpressionUUID->"bceee35f-a22a-4101-a433-1f30b846cced"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EulerianGraphQ", "[", "konigsberg", "]"}]], "Input",
 CellLabel->
  "In[197]:=",ExpressionUUID->"7ec61bdc-908f-42be-9841-9af830d85113"],

Cell[BoxData["False"], "Output",
 CellLabel->
  "Out[197]=",ExpressionUUID->"5f5c0291-82ce-4a90-a26f-8336a779c219"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Euler Paths", "Subsection",
 CellTags->"Section5b",ExpressionUUID->"f921781e-e24e-488e-b181-f9fd929304b2"],

Cell["\<\
While the Wolfram Language\[CloseCurlyQuote]s includes a built-in function \
for finding Eulerian circuits, it does not have a function for finding \
Eulerian paths that are not circuits. Moreover, building a function for \
finding Euler paths from scratch can be helpful in understanding the \
algorithm for finding both paths and circuits. We know, from Theorem 1 of \
Section 10.5, that a connected multigraph with at least two vertices has an \
Euler circuit if and only if the degree of every vertex is even. We also \
know, from Theorem 2, that if the graph has exactly two vertices of odd \
degree, then there is a Euler path which is not a circuit. \
\>", "Text",ExpressionUUID->"b133ff35-422e-4380-808b-34f7fc86bcb9"],

Cell[TextData[{
 "Using this fact, we can write a simple function for determining whether or \
not an undirected pseudograph (multiple edges and loops allowed) has an Euler \
path. The function first uses the Wolfram Language built-in functions to \
confirm that the graph given as the argument is undirected and connected. It \
uses ",
 StyleBox[ButtonBox["VertexDegree",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexDegree"], "FunctionLink"],
 " to obtain the list of the degrees of the vertices in the graph. We then ",
 StyleBox[ButtonBox["Map",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Map"], "FunctionLink"],
 " (",
 StyleBox["/@", "Code"],
 ") the ",
 StyleBox[ButtonBox["EvenQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EvenQ"], "FunctionLink"],
 " predicate over the list to transform the list of degrees into a list of \
true and false. Finally, we ",
 StyleBox[ButtonBox["Count",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Count"], "FunctionLink"],
 " those that are false and check whether that number, that is, the number of \
vertices of odd degree, is 0 or 2. We include 0, since an Euler circuit is a \
path. Note that ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " determines whether the second argument is a member of the first argument."
}], "Text",ExpressionUUID->"edcf8933-a80b-4a72-bc08-1d7e883c9fe4"],

Cell[BoxData[
 RowBox[{
  RowBox[{"eulerPathQ", "[", "g_Graph", "]"}], ":=", 
  RowBox[{
   RowBox[{"UndirectedGraphQ", "[", "g", "]"}], "&&", 
   RowBox[{"ConnectedGraphQ", "[", "g", "]"}], "&&", 
   RowBox[{"MemberQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "2"}], "}"}], ",", 
     RowBox[{"Count", "[", 
      RowBox[{
       RowBox[{"EvenQ", "/@", 
        RowBox[{"VertexDegree", "[", "g", "]"}]}], ",", "False"}], "]"}]}], 
    "]"}]}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[198]:=",ExpressionUUID->"47c4d6df-432e-4e89-a464-1346627fcc84"],

Cell[TextData[{
 "Consider the graph ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["G", "2"], TraditionalForm]],ExpressionUUID->
  "50a5a304-8d45-45a4-9936-a23274edaaf9"],
 " from Example 4, which has an Euler path, but not a circuit."
}], "Text",ExpressionUUID->"f6400ff5-5d8a-47f2-9690-a3f1bf219d0d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"figure7G2", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<g\>\"", ",", "\"\<f\>\"", ",", "\"\<e\>\"", ",", 
      "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<f\>\"", "\[Rule]", "\"\<g\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", "\[IndentingNewLine]", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", "\[IndentingNewLine]", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "0"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[199]:=",ExpressionUUID->"d16a03a2-7945-41c7-8ac7-244112fc514d"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "g", "f", "e", "b", "c", "d"}, {
      Null, {{1, 2}, {1, 5}, {5, 6}, {5, 2}, {6, 7}, {6, 2}, {6, 3}, {7, 4}, {
       7, 3}, {4, 3}, {3, 2}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       VertexCoordinates -> {{0, 1}, {1, 1}, {2, 1}, {3, 1}, {0, 0}, {1, 0}, {
        2, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {1., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {0., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {0., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {3., 1.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {1., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{3., 1.}, {2., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{0., 0.}, {1., 0.}}, 0.1]}, 
        {Arrowheads[0.], ArrowBox[{{1., 0.}, {2., 0.}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 1.}, 0.1], InsetBox["\<\"a\"\>", {0., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.1], 
         InsetBox["\<\"g\"\>", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.1], 
         InsetBox["\<\"f\"\>", {2., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 1.}, 0.1], 
         InsetBox["\<\"e\"\>", {3., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.1], 
         InsetBox["\<\"b\"\>", {0., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.1], 
         InsetBox["\<\"c\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.1], 
         InsetBox["\<\"d\"\>", {2., 0.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[199]=",ExpressionUUID->"de23d146-b9f4-4e53-8abf-f88422efb421"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EulerianGraphQ", "[", "figure7G2", "]"}]], "Input",
 CellLabel->
  "In[200]:=",ExpressionUUID->"6be58db2-b8b2-4ef1-af46-0793bc618536"],

Cell[BoxData["False"], "Output",
 CellLabel->
  "Out[200]=",ExpressionUUID->"c67cee85-113d-4336-b375-58483152610e"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"eulerPathQ", "[", "figure7G2", "]"}]], "Input",
 CellLabel->
  "In[201]:=",ExpressionUUID->"ce11be54-8a12-4546-a3c1-4ec07111f827"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[201]=",ExpressionUUID->"e984b709-2f9b-4a16-9e3c-ae2c9b694c66"]
}, Open  ]],

Cell["\<\
Now that we have a test that tells us if a path exists, we modify Algorithm 1 \
from Section 10.5 in order to find an Euler path, if it exists. The \
difference between this function and Algorithm 1 is that, if there is not an \
Euler circuit, we need to be sure to begin at one of the vertices of odd \
degree and end at the other. To do this, we begin by creating a new graph \
with an additional vertex adjacent to the two vertices of odd degree. We then \
follow Algorithm 1 to build an Euler circuit, and then cut out the temporary \
vertex.\
\>", "Text",ExpressionUUID->"20593bdd-9475-4a65-9948-1f54c642fde2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findEulerianPath", "[", "g_Graph", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "H", ",", "pathQ", ",", "oddvertices", ",", "newvertex", ",", "circuit", 
      ",", "subC", ",", "i", ",", "n", ",", "v", ",", "insertPoint", ",", "w",
       ",", "buildingSub", ",", "oldC", ",", "newVlocations"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"eulerPathQ", "[", "g", "]"}]}], ",", 
       RowBox[{"Return", "[", "$Failed", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"circuit", "=", 
      RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"H", "=", "g"}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Count", "[", 
         RowBox[{
          RowBox[{"EvenQ", "/@", 
           RowBox[{"VertexDegree", "[", "g", "]"}]}], ",", "False"}], "]"}], 
        "\[Equal]", "2"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"pathQ", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{"oddvertices", "=", 
         RowBox[{"Select", "[", 
          RowBox[{
           RowBox[{"VertexList", "[", "g", "]"}], ",", 
           RowBox[{
            RowBox[{"OddQ", "[", 
             RowBox[{"VertexDegree", "[", 
              RowBox[{"g", ",", "#"}], "]"}], "]"}], "&"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"H", "=", 
         RowBox[{"VertexAdd", "[", 
          RowBox[{"H", ",", "newvertex"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"H", "=", 
         RowBox[{"EdgeAdd", "[", 
          RowBox[{"H", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"newvertex", "\[Rule]", 
              RowBox[{"oddvertices", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], ",", 
             RowBox[{"newvertex", "\[Rule]", 
              RowBox[{"oddvertices", "[", 
               RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "]"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"pathQ", "=", "False"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"EdgeList", "[", "H", "]"}], "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"find", " ", "a", " ", "starting", " ", "point"}], " ", 
        "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"circuit", "\[Equal]", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"subC", "=", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"EdgeList", "[", "H", "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], "}"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"H", "=", 
            RowBox[{"EdgeDelete", "[", 
             RowBox[{"H", ",", 
              RowBox[{"subC", "[", 
               RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"insertPoint", "=", "0"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"i", "=", "1"}], ",", 
            RowBox[{"i", "\[LessEqual]", 
             RowBox[{"Length", "[", "circuit", "]"}]}], ",", 
            RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"v", "=", 
              RowBox[{"circuit", "[", 
               RowBox[{"[", 
                RowBox[{"i", ",", "2"}], "]"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"n", "=", 
              RowBox[{"neighbors", "[", 
               RowBox[{"H", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"n", "\[NotEqual]", 
                RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
               RowBox[{
                RowBox[{"w", "=", 
                 RowBox[{"n", "[", 
                  RowBox[{"[", "1", "]"}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"subC", "=", 
                 RowBox[{"{", 
                  RowBox[{"v", "\[Rule]", "w"}], "}"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"insertPoint", "=", "i"}], ";", "\[IndentingNewLine]", 
                RowBox[{"H", "=", 
                 RowBox[{"EdgeDelete", "[", 
                  RowBox[{"H", ",", 
                   RowBox[{"UndirectedEdge", "[", 
                    RowBox[{"v", ",", "w"}], "]"}]}], "]"}]}], ";", 
                "\[IndentingNewLine]", 
                RowBox[{"Break", "[", "]"}]}]}], "\[IndentingNewLine]", 
              "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"build", " ", "a", " ", "subcircuit"}], " ", "*)"}], 
        "\[IndentingNewLine]", 
        RowBox[{"buildingSub", "=", "True"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"buildingSub", "&&", 
           RowBox[{
            RowBox[{"EdgeList", "[", "H", "]"}], "\[NotEqual]", 
            RowBox[{"{", "}"}]}]}], ",", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"v", "=", 
            RowBox[{"subC", "[", 
             RowBox[{"[", 
              RowBox[{
               RowBox[{"-", "1"}], ",", "2"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"w", "=", 
            RowBox[{"First", "[", 
             RowBox[{"neighbors", "[", 
              RowBox[{"H", ",", "v"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"H", "=", 
            RowBox[{"EdgeDelete", "[", 
             RowBox[{"H", ",", 
              RowBox[{"UndirectedEdge", "[", 
               RowBox[{"v", ",", "w"}], "]"}]}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"subC", ",", 
             RowBox[{"UndirectedEdge", "[", 
              RowBox[{"v", ",", "w"}], "]"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"If", "[", 
            RowBox[{
             RowBox[{"w", "\[Equal]", 
              RowBox[{"subC", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ",", 
             RowBox[{"buildingSub", "=", "False"}]}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{
         "splice", " ", "the", " ", "subcircuit", " ", "into", " ", "the", 
          " ", "main", " ", "circuit"}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"circuit", "\[Equal]", 
           RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"circuit", "=", "subC"}], ",", "\[IndentingNewLine]", 
          RowBox[{"circuit", "=", 
           RowBox[{"Flatten", "[", 
            RowBox[{"Insert", "[", 
             RowBox[{"circuit", ",", "subC", ",", 
              RowBox[{"insertPoint", "+", "1"}]}], "]"}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{"pathQ", ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"newVlocations", "=", 
         RowBox[{
          RowBox[{"Position", "[", 
           RowBox[{"circuit", ",", "newvertex"}], "]"}], "[", 
          RowBox[{"[", 
           RowBox[{"All", ",", "1"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Join", "[", 
         RowBox[{
          RowBox[{"circuit", "[", 
           RowBox[{"[", 
            RowBox[{
             RowBox[{
              RowBox[{"Max", "[", "newVlocations", "]"}], "+", "1"}], ";;", 
             RowBox[{"-", "1"}]}], "]"}], "]"}], ",", 
          RowBox[{"circuit", "[", 
           RowBox[{"[", 
            RowBox[{"1", ";;", 
             RowBox[{
              RowBox[{"Min", "[", "newVlocations", "]"}], "-", "1"}]}], "]"}],
            "]"}]}], "]"}]}], ",", "\[IndentingNewLine]", "circuit"}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[213]:=",ExpressionUUID->"b0459a0f-33d4-4b01-8f15-7eb26f76e8b4"],

Cell[TextData[{
 "The function begins with a use of ",
 StyleBox["eulerPathQ", "Code"],
 " in order to avoid searching for a path that cannot exist. It then assigns \
to the symbol ",
 StyleBox["H", "Code"],
 " a copy of the graph. It is this copy that is used throughout the rest of \
the function, rather than the input that was passed to the algorithm. The \
benefit of using a copy is that the function will be able to manipulate it as \
the algorithm proceeds, for example, by deleting edges of ",
 StyleBox["H", "Code"],
 " once they are included in the circuit so that those edges are not reused."
}], "Text",ExpressionUUID->"ca37a71c-7611-42b8-b115-6734e47feef2"],

Cell[TextData[{
 "After copying the input graph, the function tests to see whether it has two \
vertices of odd degree. If it does, then the graph ",
 StyleBox["H", "Code"],
 " is modified to include a new vertex and edges between this new vertex and \
the two with odd degree. The result is that ",
 StyleBox["H", "Code"],
 " has vertices of all even degree, and thus an Euler circuit must exist. \
Note that the symbol used for the name of the new vertex is localized to the \
module. This means that, in practice, the symbol actually being used is \
something along the lines of ",
 StyleBox["newvertex", "Code",
  IgnoreSpellCheck->True],
 StyleBox["$371", "Code"],
 ", so it is unlikely to already exist in the graph. The function also sets a \
flag, ",
 StyleBox["pathQ", "Code"],
 ", to true or false, depending on whether the graph has a path or circuit."
}], "Text",ExpressionUUID->"3151628e-31d4-45fb-bbee-5965d224e077"],

Cell[TextData[{
 "Once ",
 StyleBox["H", "Code"],
 " has been modified, if needed, the function follows Algorithm 1 from \
Section 10.5. There are two key ideas at the heart of this algorithm. The \
first is that, for a graph whose vertices all have even degree, if you pick \
any vertex to start at and follow edges at random but without repetition, you \
will definitely return to the original vertex and create a circuit. The \
second key idea is that (for a connected graph), if your circuit does not \
include all of the edges of the graph, then some vertex used in the existing \
circuit can be made the starting point for a new ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 ". This ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 " can then be spliced into the main circuit. This will eventually use all \
the edges and the result will be a Euler circuit."
}], "Text",ExpressionUUID->"dd14f1ff-10bc-43d6-9cdb-efd7fa11c79d"],

Cell[TextData[{
 "The symbol ",
 StyleBox["circuit", "Code"],
 " will hold the main circuit that the function finds in ",
 StyleBox["H", "Code"],
 ". The circuit will be stored as a list of the edges through which the \
circuit passes and is initialized to the empty list. The main ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop consists of three parts: (1) determining the starting point for the \
",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 " (named ",
 StyleBox["subC", "Code"],
 "); (2) building the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 "; and (3) splicing the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 " into the main circuit."
}], "Text",ExpressionUUID->"12cafa15-6b96-474a-87a0-ef6e494b743f"],

Cell[TextData[{
 "The first step, finding the starting point for the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 ", depends on the state of the main circuit. If ",
 StyleBox["circuit", "Code"],
 " is the empty list (i.e., this is the first pass through the main loop), \
then the starting point is the first edge in the graph. If the main circuit \
is not empty, then the else clause looks at the vertices in the main circuit \
to find one that has neighbors (since edges are deleted from ",
 StyleBox["H", "Code"],
 " as they are added to the circuit, only vertices that are an endpoint of an \
unused edge have neighbors). The first vertex that has a neighbor is used as \
the starting point for the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 ". The ",
 StyleBox["insertPoint", "Code"],
 " variable is used to keep track of the index, relative to ",
 StyleBox["circuit", "Code"],
 ", of the starting vertex for the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 ". This is used when the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 " is spliced into the main circuit."
}], "Text",ExpressionUUID->"ac93333b-ca6e-4401-9e2e-5f5afad82d0a"],

Cell[TextData[{
 "The second step is to build ",
 StyleBox["subC", "Code"],
 ". The ",
 StyleBox["buildingSub", "Code"],
 " symbol is used to control the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop. It is initialized to true and is set to false once ",
 StyleBox["subC", "Code"],
 " has returned to its starting vertex and is thus a circuit. The variable ",
 StyleBox["v", "Code"],
 " is set to the last vertex currently included as part of the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 " and ",
 StyleBox["w", "Code"],
 " represents a neighbor of ",
 StyleBox["v", "Code"],
 ". To remove the edge between ",
 StyleBox["v", "Code"],
 " and ",
 StyleBox["w", "Code"],
 " from ",
 StyleBox["H", "Code"],
 ", we use ",
 StyleBox[ButtonBox["EdgeDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeDelete"], "FunctionLink"],
 ". "
}], "Text",ExpressionUUID->"60283a27-91ef-47e9-a5fd-7af2e29535a5"],

Cell[TextData[{
 "After deleting the edge from ",
 StyleBox["H", "Code"],
 ", the newest vertex is compared with the starting vertex to determine if \
the circuit has been closed. If the new vertex closes the circuit, then the \
",
 StyleBox["buildingSub", "Code"],
 " variable is set to false, which causes the loop to terminate. Otherwise, \
the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop continues building the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 "."
}], "Text",ExpressionUUID->"065b237e-51e8-4d2d-9287-a4a629eeaf28"],

Cell[TextData[{
 "The third step, once the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 " has been built, is to splice it into the main circuit. If ",
 StyleBox["circuit", "Code"],
 " is empty, it is merely set to ",
 StyleBox["subC", "Code"],
 ". Otherwise, ",
 StyleBox[ButtonBox["Insert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Insert"], "FunctionLink"],
 " is used to add the ",
 StyleBox["subcircuit",
  IgnoreSpellCheck->True],
 ". ",
 StyleBox[ButtonBox["Insert",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Insert"], "FunctionLink"],
 " takes a list as the first argument and adds the expression given as the \
second argument in the position specified by the third argument. The elements \
previously in that position and later and pushed down to make room. We give \
the third argument as ",
 Cell[BoxData[
  RowBox[{"insertPoint", "+", "1"}]], "myCode",ExpressionUUID->
  "afb263c8-6567-436a-ae0b-ed9067fdc353"],
 " to insert the sublist between the edges at position ",
 StyleBox["insertPoint", "Code"],
 " and position ",
 Cell[BoxData[
  RowBox[{"insertPoint", "+", "1"}]], "myCode",ExpressionUUID->
  "652c9591-0b53-4d13-b281-9fb3a6731644"],
 "."
}], "Text",ExpressionUUID->"ed662172-c26b-4ae7-ada5-8476a8a5b26d"],

Cell[TextData[{
 "The main ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop continues until all the edges of the graph have been included in the \
circuit, making circuit an Euler circuit for ",
 StyleBox["H", "Code"],
 ". Then the final step is to either output the circuit, if the original \
graph has an Euler circuit, or to cut the temporary vertex and corresponding \
edges out of the circuit and produce the Euler path. For this, we use ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 ", with first argument a list and second argument a pattern to search for, \
to find the locations of the temporary vertex in the circuit. The vertex will \
appear twice and its location will be reported as a pair consisting of the \
position within ",
 StyleBox["circuit", "Code"],
 " and a 1 or 2 corresponding to its location on the edge. The ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " specification ",
 StyleBox["[[All,1]]", "Code"],
 " returns the first element in all subexpressions and thus extracts the \
locations of the edges within the ",
 StyleBox["circuit", "Code"],
 " list containing the temporary vertex. Once the locations of the temporary \
vertex have been found, the function uses ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " to combine the part of the circuit after it has visited the temporary \
vertex with the part before. The result is the Euler path beginning at one of \
the vertices of odd degree and ending at the other."
}], "Text",ExpressionUUID->"34bc06c3-30b2-4f05-a00a-b9d7d6df16c3"],

Cell["As an example, consider Exercise 5 from Section 10.5.", "Text",ExpressionUUID->"43a997b6-3a00-48bc-a8a6-bc4d99098401"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise3", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0.5", ",", "0.5"}], "}"}]}], "}"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[210]:=",ExpressionUUID->"b0dc2f9c-0565-4886-97e8-37f582367267"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e"}, {
      Null, {{1, 2}, {1, 5}, {1, 3}, {2, 4}, {2, 5}, {2, 5}, {3, 4}, {3, 5}, {
       3, 5}, {4, 5}}}, {VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       VertexCoordinates -> {{0, 1}, {1, 1}, {0, 0}, {1, 0}, {0.5, 0.5}}}]]}, 
    
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {1., 1.}}, 0.07071067811865477]}, 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {0., 0.}}, 0.07071067811865477]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0., 1.}, {0.5, 0.5}}, 0.07071067811865477]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 0.}}, 0.07071067811865477]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1., 1.}, {0.8322203735915615, 
          0.6677796264087318}, {0.5, 0.5}}], 0.07071067811865477]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{1., 1.}, {0.6677796264084451, 
          0.8322203735915052}, {0.5, 0.5}}], 0.07071067811865477]}, 
        {Arrowheads[0.], ArrowBox[{{0., 0.}, {1., 0.}}, 0.07071067811865477]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0., 0.}, {0.16777962640844168`, 
          0.33222037359156953`}, {0.5, 0.5}}], 0.07071067811865477]}, 
        {Arrowheads[0.], 
         ArrowBox[BezierCurveBox[{{0., 0.}, {0.33222037359155826`, 
          0.16777962640845023`}, {0.5, 0.5}}], 0.07071067811865477]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1., 0.}, {0.5, 0.5}}, 0.07071067811865477]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 1.}, 0.07071067811865477], 
         InsetBox["\<\"a\"\>", {0., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.07071067811865477], 
         InsetBox["\<\"b\"\>", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.07071067811865477], 
         InsetBox["\<\"c\"\>", {0., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.07071067811865477], 
         InsetBox["\<\"d\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{0.5, 0.5}, 0.07071067811865477], 
         InsetBox["\<\"e\"\>", {0.5, 0.5},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[210]=",ExpressionUUID->"f223d859-6c94-4d74-8fda-61fc7baeec9d"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise3EulerPath", "=", 
  RowBox[{"findEulerianPath", "[", "exercise3", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[211]:=",ExpressionUUID->"97e29389-a4f9-410c-ae1f-a4d804c3ae75"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\"d\"\>", "\[UndirectedEdge]", "\<\"e\"\>"}], ",", 
   RowBox[{"\<\"e\"\>", "\[UndirectedEdge]", "\<\"b\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[UndirectedEdge]", "\<\"d\"\>"}], ",", 
   RowBox[{"\<\"d\"\>", "\[UndirectedEdge]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"c\"\>", "\[UndirectedEdge]", "\<\"a\"\>"}], ",", 
   RowBox[{"\<\"a\"\>", "\[UndirectedEdge]", "\<\"b\"\>"}], ",", 
   RowBox[{"\<\"b\"\>", "\[Rule]", "\<\"e\"\>"}], ",", 
   RowBox[{"\<\"e\"\>", "\[Rule]", "\<\"c\"\>"}], ",", 
   RowBox[{"\<\"c\"\>", "\[UndirectedEdge]", "\<\"e\"\>"}], ",", 
   RowBox[{"\<\"e\"\>", "\[UndirectedEdge]", "\<\"a\"\>"}]}], "}"}]], "Output",\

 CellLabel->
  "Out[211]=",ExpressionUUID->"c0f242f9-d9c0-4c12-9dc4-c907b11cb9be"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Hamilton Circuits", "Subsection",
 CellTags->"Section5c",ExpressionUUID->"41acec3b-72f3-45a8-854c-131b2698b7a4"],

Cell[TextData[{
 "Turning our attention to Hamilton circuits, the Wolfram Language provides \
the function ",
 StyleBox[ButtonBox["HamiltonianGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HamiltonianGraphQ"], "FunctionLink"],
 " for determining whether or not the graph contains a Hamilton circuit. This \
function, like ",
 StyleBox[ButtonBox["EulerianGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EulerianGraphQ"], "FunctionLink"],
 ", accepts a graph as the sole argument. It returns true or false depending \
on whether the graph has a Hamilton circuit."
}], "Text",ExpressionUUID->"1bfa9832-e019-4b42-a23a-d552a8d779e9"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hcGraphExample", "=", 
  RowBox[{"HypercubeGraph", "[", "3", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[214]:=",ExpressionUUID->"cf96aa4d-da29-42f3-8fab-baac583d6b1c"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 3, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {5}, {1}, {4}, {6}, {
          1}, {4}, {7}, {2}, {3}, {8}, {1}, {6}, {7}, {2}, {5}, {8}, {3}, {
          5}, {8}, {4}, {6}, {7}}}, Pattern}]}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.6411318907234969, 1.6256350616332038`}, {
       1.5431761475992014`, 1.6258514889129208`}, {1.3641521541128723`, 
       0.8124188351574709}, {2.186420636755495, 0.814439144539747}, {0., 
       0.8116524116085829}, {0.8207122623609914, 0.8136876580070856}, {
       0.6431241846256299, 0.}, {1.5449243126722068`, 
       0.0005338415834358345}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{1, 3}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{2, 6}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{3, 7}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{4, 8}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{5, 7}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{6, 8}, 0.02410631722965237]}, 
         {Arrowheads[0.], ArrowBox[{7, 8}, 0.02410631722965237]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02410631722965237], DiskBox[2, 0.02410631722965237], 
         DiskBox[3, 0.02410631722965237], DiskBox[4, 0.02410631722965237], 
         DiskBox[5, 0.02410631722965237], DiskBox[6, 0.02410631722965237], 
         DiskBox[7, 0.02410631722965237], DiskBox[8, 0.02410631722965237]}}]],
     
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[214]=",ExpressionUUID->"b3cd140f-1e77-4181-8d9a-98d3f30ebfd7"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HamiltonianGraphQ", "[", "hcGraphExample", "]"}]], "Input",
 CellLabel->
  "In[215]:=",ExpressionUUID->"c16d3428-85ea-4973-bb5f-2233a79fd33d"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[215]=",ExpressionUUID->"27091af3-4586-4e6f-86dc-f0c6aed8dfda"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["FindHamiltonianCycle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindHamiltonianCycle"], "FunctionLink"],
 " function, similar to ",
 StyleBox[ButtonBox["FindEulerianCycle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindEulerianCycle"], "FunctionLink"],
 ", accepts a graph as the argument and returns a list containing a \
Hamiltonian circuit. The optional second argument can be used to find more \
than one cycle."
}], "Text",ExpressionUUID->"71e4931d-89cd-4661-8ef3-6b692816f573"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindHamiltonianCycle", "[", "hcGraphExample", "]"}]], "Input",
 CellLabel->
  "In[216]:=",ExpressionUUID->"93daa322-dd6a-4761-9b5d-4b9580b33331"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
    RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
    RowBox[{"4", "\[UndirectedEdge]", "8"}], ",", 
    RowBox[{"8", "\[UndirectedEdge]", "6"}], ",", 
    RowBox[{"6", "\[UndirectedEdge]", "5"}], ",", 
    RowBox[{"5", "\[UndirectedEdge]", "7"}], ",", 
    RowBox[{"7", "\[UndirectedEdge]", "3"}], ",", 
    RowBox[{"3", "\[UndirectedEdge]", "1"}]}], "}"}], "}"}]], "Output",
 CellLabel->
  "Out[216]=",ExpressionUUID->"83b89b59-0c8d-439d-b159-12d4ea169070"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindHamiltonianCycle", "[", 
  RowBox[{"hcGraphExample", ",", "All"}], "]"}]], "Input",
 CellLabel->
  "In[217]:=",ExpressionUUID->"8a0beea2-2818-4038-b812-6b62109cfbc5"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "7"}], ",", 
     RowBox[{"7", "\[UndirectedEdge]", "8"}], ",", 
     RowBox[{"8", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "6"}], ",", 
     RowBox[{"6", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "6"}], ",", 
     RowBox[{"6", "\[UndirectedEdge]", "8"}], ",", 
     RowBox[{"8", "\[UndirectedEdge]", "7"}], ",", 
     RowBox[{"7", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "6"}], ",", 
     RowBox[{"6", "\[UndirectedEdge]", "8"}], ",", 
     RowBox[{"8", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "7"}], ",", 
     RowBox[{"7", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "7"}], ",", 
     RowBox[{"7", "\[UndirectedEdge]", "8"}], ",", 
     RowBox[{"8", "\[UndirectedEdge]", "6"}], ",", 
     RowBox[{"6", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "6"}], ",", 
     RowBox[{"6", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "7"}], ",", 
     RowBox[{"7", "\[UndirectedEdge]", "8"}], ",", 
     RowBox[{"8", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "1"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
     RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
     RowBox[{"4", "\[UndirectedEdge]", "8"}], ",", 
     RowBox[{"8", "\[UndirectedEdge]", "6"}], ",", 
     RowBox[{"6", "\[UndirectedEdge]", "5"}], ",", 
     RowBox[{"5", "\[UndirectedEdge]", "7"}], ",", 
     RowBox[{"7", "\[UndirectedEdge]", "3"}], ",", 
     RowBox[{"3", "\[UndirectedEdge]", "1"}]}], "}"}]}], "}"}]], "Output",
 CellLabel->
  "Out[217]=",ExpressionUUID->"d56e735b-0e9b-489c-b840-62c7fcb737bc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"hcGraphExampleCircuit", "=", 
  RowBox[{"First", "[", 
   RowBox[{"FindHamiltonianCycle", "[", "hcGraphExample", "]"}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[218]:=",ExpressionUUID->"15d6a140-8c43-4b21-a99a-fa686bf66ef3"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"1", "\[UndirectedEdge]", "2"}], ",", 
   RowBox[{"2", "\[UndirectedEdge]", "4"}], ",", 
   RowBox[{"4", "\[UndirectedEdge]", "8"}], ",", 
   RowBox[{"8", "\[UndirectedEdge]", "6"}], ",", 
   RowBox[{"6", "\[UndirectedEdge]", "5"}], ",", 
   RowBox[{"5", "\[UndirectedEdge]", "7"}], ",", 
   RowBox[{"7", "\[UndirectedEdge]", "3"}], ",", 
   RowBox[{"3", "\[UndirectedEdge]", "1"}]}], "}"}]], "Output",
 CellLabel->
  "Out[218]=",ExpressionUUID->"7cb600ad-0784-449e-af5a-b682077ef3c8"]
}, Open  ]],

Cell[TextData[{
 "We can use ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 " to illustrate the path statically."
}], "Text",ExpressionUUID->"336255e2-e663-467f-8fed-269c00af8469"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"hcGraphExample", ",", "hcGraphExampleCircuit"}], "]"}]], "Input",
 CellLabel->
  "In[219]:=",ExpressionUUID->"1ebbf5fc-75b8-487a-999f-11b33d7e9fdb"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
       SparseArray[
       Automatic, {8, 8}, 0, {
        1, {{0, 3, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {5}, {1}, {4}, {6}, {
          1}, {4}, {7}, {2}, {3}, {8}, {1}, {6}, {7}, {2}, {5}, {8}, {3}, {
          5}, {8}, {4}, {6}, {7}}}, Pattern}]}, {GraphHighlight -> {
         UndirectedEdge[7, 3], 
         UndirectedEdge[8, 6], 
         UndirectedEdge[4, 8], 
         UndirectedEdge[2, 4], 
         UndirectedEdge[5, 7], 
         UndirectedEdge[6, 5], 
         UndirectedEdge[3, 1], 
         UndirectedEdge[1, 2]}}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.6411318907234969, 1.6256350616332038`}, {
       1.5431761475992014`, 1.6258514889129208`}, {1.3641521541128723`, 
       0.8124188351574709}, {2.186420636755495, 0.814439144539747}, {0., 
       0.8116524116085829}, {0.8207122623609914, 0.8136876580070856}, {
       0.6431241846256299, 0.}, {1.5449243126722068`, 
       0.0005338415834358345}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {
            1.5431761475992014`, 1.6258514889129208`}}, 
            0.02410631722965237]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {
            1.3641521541128723`, 0.8124188351574709}}, 0.02410631722965237]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{0.6411318907234969, 1.6256350616332038`}, {0., 
           0.8116524116085829}}, 0.02410631722965237]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{1.5431761475992014`, 1.6258514889129208`}, {
            2.186420636755495, 0.814439144539747}}, 0.02410631722965237]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{1.5431761475992014`, 1.6258514889129208`}, {
           0.8207122623609914, 0.8136876580070856}}, 0.02410631722965237]}, 
         {Arrowheads[0.], 
          ArrowBox[{{1.3641521541128723`, 0.8124188351574709}, {
           2.186420636755495, 0.814439144539747}}, 0.02410631722965237]}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{1.3641521541128723`, 0.8124188351574709}, {
            0.6431241846256299, 0.}}, 0.02410631722965237]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{2.186420636755495, 0.814439144539747}, {
            1.5449243126722068`, 0.0005338415834358345}}, 
            0.02410631722965237]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0., 0.8116524116085829}, {0.8207122623609914, 
            0.8136876580070856}}, 0.02410631722965237]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0., 0.8116524116085829}, {0.6431241846256299, 0.}}, 
            0.02410631722965237]}}, 
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], 
           ArrowBox[{{0.8207122623609914, 0.8136876580070856}, {
            1.5449243126722068`, 0.0005338415834358345}}, 
            0.02410631722965237]}}, 
         {Arrowheads[0.], 
          ArrowBox[{{0.6431241846256299, 0.}, {1.5449243126722068`, 
           0.0005338415834358345}}, 0.02410631722965237]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02410631722965237], DiskBox[2, 0.02410631722965237], 
         DiskBox[3, 0.02410631722965237], DiskBox[4, 0.02410631722965237], 
         DiskBox[5, 0.02410631722965237], DiskBox[6, 0.02410631722965237], 
         DiskBox[7, 0.02410631722965237], DiskBox[8, 0.02410631722965237]}}]],
     
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[219]=",ExpressionUUID->"c0c703c2-e2dd-40df-9322-75d00e06378b"]
}, Open  ]],

Cell[TextData[{
 "Moreover, our animation function ",
 StyleBox["animatePath", "Code"],
 ", created above, works equally well here."
}], "Text",ExpressionUUID->"05515278-3695-4b0f-a8b3-9cb1c2156d1f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"animatePath", "[", 
  RowBox[{"hcGraphExample", ",", "hcGraphExampleCircuit"}], "]"}]], "Input",
 CellLabel->
  "In[220]:=",ExpressionUUID->"2f7d8df1-6e8b-4054-aa2a-dc2da60e7a6c"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$12225$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`i$12225$$], 0, "step"}, 0, 8, 1}}, Typeset`size$$ = {
    450., {168., 173.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = True, $CellContext`i$12225$12226$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$12225$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$12225$$, $CellContext`i$12225$12226$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> HighlightGraph[
        Graph[{1, 2, 3, 4, 5, 6, 7, 8}, {Null, 
          SparseArray[
          Automatic, {8, 8}, 0, {
           1, {{0, 3, 6, 9, 12, 15, 18, 21, 24}, {{2}, {3}, {5}, {1}, {4}, {
             6}, {1}, {4}, {7}, {2}, {3}, {8}, {1}, {6}, {7}, {2}, {5}, {8}, {
             3}, {5}, {8}, {4}, {6}, {7}}}, Pattern}]}], 
        Part[{
          UndirectedEdge[1, 2], 
          UndirectedEdge[2, 4], 
          UndirectedEdge[4, 8], 
          UndirectedEdge[8, 6], 
          UndirectedEdge[6, 5], 
          UndirectedEdge[5, 7], 
          UndirectedEdge[7, 3], 
          UndirectedEdge[3, 1]}, 
         Span[1, $CellContext`i$12225$$]]], 
      "Specifications" :> {{{$CellContext`i$12225$$, 0, "step"}, 0, 8, 1, 
         AnimationRunning -> False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{503., {212., 219.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellLabel->
  "Out[220]=",ExpressionUUID->"4220d439-1d7d-450a-9382-0f27f48e42fe"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.6 Shortest-Path Problems", "Section",
 CellTags->"Section6",ExpressionUUID->"ba88e25b-5334-48e1-91be-9c29178d2dee"],

Cell["\<\
Among the most common problems in graph theory are shortest path problems. \
Generally, in shortest path problems, we wish to determine a path between two \
vertices of a weighted graph that is minimal in terms of the total weight of \
the edges in the path.\
\>", "Text",ExpressionUUID->"12834ff3-6eaf-4f4c-b773-bf2a91666e99"],

Cell[TextData[{
 "To define a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object with weighted edges, use the ",
 StyleBox[ButtonBox["EdgeWeight",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeWeight"], "FunctionLink"],
 " option. The value associated with the option is the list of the weights of \
each edge. The weights must appear in the same order as they are displayed in \
the output by ",
 StyleBox[ButtonBox["EdgeList",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeList"], "FunctionLink"],
 ". For graphs you define by listing the edges, this is identical to the \
order you give in the definition."
}], "Text",ExpressionUUID->"44b94d3b-9d6a-42d4-bf97-4f5cc3b21038"],

Cell["\<\
We reproduce Exercise 2 from Section 10.6 of the textbook to use as an \
example. \
\>", "Text",ExpressionUUID->"6758678c-a7f2-4856-a9e3-12561728afd8"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise2", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\"", ",", "\"\<z\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<z\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<z\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"EdgeWeight", "->", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "3", ",", "5", ",", "2", ",", "5", ",", "1", ",", "2", ",", 
       "4"}], "}"}]}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", ".5"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", ".5"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
    RowBox[{"EdgeLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<EdgeWeight\>\"", ",", 
       RowBox[{"{", 
        RowBox[{".5", ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}], 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[221]:=",ExpressionUUID->"694369cd-3628-403a-a573-996ae2f87d50"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "z"}, {
      Null, {{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 5}, {4, 5}, {4, 6}, {5, 
       6}}}, {EdgeLabels -> {
         Placed["EdgeWeight", {0.5, {-1, 0}}]}, VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       EdgeWeight -> {2, 3, 5, 2, 5, 1, 2, 4}, 
       VertexCoordinates -> {{0, 0.5}, {1, 1}, {1, 0}, {2, 1}, {2, 0}, {
         3, 0.5}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], {
         {Arrowheads[0.], ArrowBox[{{0., 0.5}, {1., 1.}}, 0.1]}, 
         InsetBox["2", Offset[{0, 2}, {0.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{0., 0.5}, {1., 0.}}, 0.1]}, 
         InsetBox["3", Offset[{0, 2}, {0.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}, 
         InsetBox["5", Offset[{0, 2}, {1.5, 1.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 0.}}, 0.1]}, 
         InsetBox["2", Offset[{0, 2}, {1.5, 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 0.}, {2., 0.}}, 0.1]}, 
         InsetBox["5", Offset[{0, 2}, {1.5, 0.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 0.}}, 0.1]}, 
         InsetBox["1", Offset[{0, 2}, {2., 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 1.}, {3., 0.5}}, 0.1]}, 
         InsetBox["2", Offset[{0, 2}, {2.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 0.}, {3., 0.5}}, 0.1]}, 
         InsetBox["4", Offset[{0, 2}, {2.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.5}, 0.1], InsetBox["\<\"a\"\>", {0., 0.5},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.1], 
         InsetBox["\<\"b\"\>", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.1], 
         InsetBox["\<\"c\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.1], 
         InsetBox["\<\"d\"\>", {2., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.1], 
         InsetBox["\<\"e\"\>", {2., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.5}, 0.1], 
         InsetBox["\<\"z\"\>", {3., 0.5},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[221]=",ExpressionUUID->"6bfa0ba6-69ca-46a2-9a0b-ba08938409d2"]
}, Open  ]],

Cell[TextData[{
 "Note the use of the ",
 StyleBox[ButtonBox["EdgeLabels",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeLabels"], "FunctionLink"],
 " option to display the edge weights. The second argument of ",
 StyleBox[ButtonBox["Placed",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Placed"], "FunctionLink"],
 " indicates that the label should appear halfway between the vertices and \
should be shifted to the right. The form of the second argument is ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox["s", "CodeVar"], ",", 
    RowBox[{"{", 
     RowBox[{
      StyleBox[
       SubscriptBox["l", "x"], "CodeVar"], ",", 
      StyleBox[
       SubscriptBox["l", "y"], "CodeVar"]}], "}"}]}], "}"}]], "Input",
  ExpressionUUID->"4b7f8a7f-ae7b-46a4-83a1-fd4e9c46ca39"],
 ", where ",
 StyleBox["s", "CodeVar"],
 " indicates fraction along the edge that the label should appear and ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox[
     SubscriptBox["l", "x"], "CodeVar"], ",", 
    StyleBox[
     SubscriptBox["l", "y"], "CodeVar"]}], "}"}]], "Input",ExpressionUUID->
  "fd14a04c-4d11-4473-aa09-21065f09ff89"],
 ", indicates what position within the label is to be located at that \
position on the line. For example, ",
 StyleBox["{0,0}", "Code"],
 " indicates that the lower left corner of the label should appear on the \
edge, while ",
 StyleBox["{-1,0}", "Code"],
 " means that a point to the left of the label should be placed at the edge. \
You can imagine the label surrounded by a box, and ",
 Cell[BoxData[
  RowBox[{"{", 
   RowBox[{
    StyleBox[
     SubscriptBox["l", "x"], "CodeVar"], ",", 
    StyleBox[
     SubscriptBox["l", "y"], "CodeVar"]}], "}"}]], "Input",ExpressionUUID->
  "5412246f-cdb3-48ab-8e79-1d9c7a645acb"],
 " indicates, relative to the lower left corner of that box, where the label \
should be anchored to the edge. Also, the values are relative to the size of \
the label, so ",
 StyleBox["{-1,0}", "Code"],
 " refers to a point to the left of the label at a distance equal to the \
width of the label."
}], "Text",ExpressionUUID->"25d20cbc-4208-4452-be30-bf2ddd2cb5f4"],

Cell[TextData[{
 "A second approach to setting edge weights is by using the ",
 StyleBox[ButtonBox["EdgeWeight",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeWeight"], "FunctionLink"],
 " property. Rather than listing all of the edges and then listing the \
weights separately using the option as shown above, we can wrap the edge \
definitions in the ",
 StyleBox[ButtonBox["Property",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Property"], "FunctionLink"],
 " wrapper, setting the edge weights at the same time the edges are described."
}], "Text",ExpressionUUID->"373960ea-0967-4e9a-a09e-4e7ff78fa8a1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Graph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\"", ",", "\"\<z\>\""}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "2"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "3"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "5"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "2"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "5"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "1"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<z\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "2"}]}], "]"}], ",", 
     RowBox[{"Property", "[", 
      RowBox[{
       RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<z\>\""}], ",", 
       RowBox[{"EdgeWeight", "\[Rule]", "4"}]}], "]"}]}], "}"}], ",", 
   RowBox[{"VertexCoordinates", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", ".5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"3", ",", ".5"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
   RowBox[{"EdgeLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<EdgeWeight\>\"", ",", 
      RowBox[{"{", 
       RowBox[{".5", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}], 
  "]"}]], "Input",
 CellLabel->
  "In[222]:=",ExpressionUUID->"d0c5d656-cc5c-48db-89ca-f913d75a5353"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{
      "a", "b", "c", "d", "e", "z"}, {{{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 
       5}, {4, 5}, {4, 6}, {5, 6}}, Null}, {EdgeLabels -> {
         Placed["EdgeWeight", {0.5, {-1, 0}}]}, VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       EdgeWeight -> {2, 3, 5, 2, 5, 1, 2, 4}, 
       VertexCoordinates -> {{0, 0.5}, {1, 1}, {1, 0}, {2, 1}, {2, 0}, {
         3, 0.5}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], Arrowheads[
        Medium], {ArrowBox[{{0., 0.5}, {1., 1.}}, 0.1], 
         InsetBox["2", Offset[{0, 2}, {0.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{0., 0.5}, {1., 0.}}, 0.1], 
         InsetBox["3", Offset[{0, 2}, {0.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{1., 1.}, {2., 1.}}, 0.1], 
         InsetBox["5", Offset[{0, 2}, {1.5, 1.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{1., 1.}, {2., 0.}}, 0.1], 
         InsetBox["2", Offset[{0, 2}, {1.5, 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{1., 0.}, {2., 0.}}, 0.1], 
         InsetBox["5", Offset[{0, 2}, {1.5, 0.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{2., 1.}, {2., 0.}}, 0.1], 
         InsetBox["1", Offset[{0, 2}, {2., 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{2., 1.}, {3., 0.5}}, 0.1], 
         InsetBox["2", Offset[{0, 2}, {2.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {ArrowBox[{{2., 0.}, {3., 0.5}}, 0.1], 
         InsetBox["4", Offset[{0, 2}, {2.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.5}, 0.1], InsetBox["\<\"a\"\>", {0., 0.5},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.1], 
         InsetBox["\<\"b\"\>", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.1], 
         InsetBox["\<\"c\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.1], 
         InsetBox["\<\"d\"\>", {2., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.1], 
         InsetBox["\<\"e\"\>", {2., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.5}, 0.1], 
         InsetBox["\<\"z\"\>", {3., 0.5},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[222]=",ExpressionUUID->"f5e26aef-3608-4aad-b440-22b4f58e5bfd"]
}, Open  ]],

Cell[TextData[{
 "Note in the above that we omitted the ",
 StyleBox[ButtonBox["DirectedEdges",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdges"], "FunctionLink"],
 " option to illustrate that there is no difference with a directed graph."
}], "Text",ExpressionUUID->"d3d63b7b-8668-4027-a038-81caf6c9c8b3"],

Cell[TextData[{
 "You can also define a weighted graph using the ",
 StyleBox[ButtonBox["WeightedAdjacencyGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WeightedAdjacencyGraph"], "FunctionLink"],
 " function and an adjacency matrix. Unlike with ",
 StyleBox[ButtonBox["AdjacencyGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AdjacencyGraph"], "FunctionLink"],
 ", the adjacency matrix must use ",
 StyleBox[ButtonBox["Infinity",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Infinity"], "FunctionLink"],
 ", or \[Infinity] (\[EscapeKey]inf\[EscapeKey]) to indicate that there is no \
edge between the corresponding vertices. In the example below, we give the \
list of vertices as the first argument, otherwise ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will automatically use positive integers to name the vertices. Note that \
",
 StyleBox[ButtonBox["WeightedAdjacencyMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/WeightedAdjacencyMatrix"], "FunctionLink"],
 " applied to a weighted graph returns the adjacency matrix with weights."
}], "Text",ExpressionUUID->"a6208d09-1348-4ffc-8a37-087397a6ed88"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"exercise2matrix", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "\[Infinity]", ",", "2", ",", "3", ",", "\[Infinity]", ",", 
       "\[Infinity]", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "2", ",", "\[Infinity]", ",", "\[Infinity]", ",", "5", ",", "2", ",", 
       "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "3", ",", "\[Infinity]", ",", "\[Infinity]", ",", "\[Infinity]", ",", 
       "5", ",", "\[Infinity]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\[Infinity]", ",", "5", ",", "\[Infinity]", ",", "\[Infinity]", ",", 
       "1", ",", "2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\[Infinity]", ",", "2", ",", "5", ",", "1", ",", "\[Infinity]", ",", 
       "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
      "\[Infinity]", ",", "\[Infinity]", ",", "\[Infinity]", ",", "2", ",", 
       "4", ",", "\[Infinity]"}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"exercise2matrix", "//", "MatrixForm"}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[223]:=",ExpressionUUID->"44f090e2-6145-44ed-b505-d452c4ae44c9"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\[Infinity]", "2", "3", "\[Infinity]", "\[Infinity]", "\[Infinity]"},
     {"2", "\[Infinity]", "\[Infinity]", "5", "2", "\[Infinity]"},
     {"3", "\[Infinity]", "\[Infinity]", "\[Infinity]", "5", "\[Infinity]"},
     {"\[Infinity]", "5", "\[Infinity]", "\[Infinity]", "1", "2"},
     {"\[Infinity]", "2", "5", "1", "\[Infinity]", "4"},
     {"\[Infinity]", "\[Infinity]", "\[Infinity]", "2", "4", "\[Infinity]"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellLabel->
  "Out[224]//MatrixForm=",ExpressionUUID->"754dd8b1-ca95-42a8-81f3-\
c333ab4aaf96"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"WeightedAdjacencyGraph", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\"", ",", "\"\<z\>\""}], "}"}], ",", "exercise2matrix", ",", 
   RowBox[{"VertexCoordinates", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0", ",", ".5"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"1", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "1"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"2", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"3", ",", ".5"}], "}"}]}], "}"}]}], ",", 
   RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
   RowBox[{"VertexLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}], ",", 
   RowBox[{"EdgeLabels", "\[Rule]", 
    RowBox[{"Placed", "[", 
     RowBox[{"\"\<EdgeWeight\>\"", ",", 
      RowBox[{"{", 
       RowBox[{".5", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "0"}], "}"}]}], "}"}]}], "]"}]}]}], 
  "]"}]], "Input",
 CellLabel->
  "In[225]:=",ExpressionUUID->"77c5bb74-a620-4732-a673-002dea20527f"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "z"}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 2, 5, 7, 10, 14, 16}, {{2}, {3}, {1}, {4}, {5}, {1}, {5}, {
          2}, {5}, {6}, {2}, {3}, {4}, {6}, {4}, {5}}}, Pattern}]}, {
      EdgeLabels -> {
         Placed["EdgeWeight", {0.5, {-1, 0}}]}, VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       EdgeWeight -> {2, 3, 5, 2, 5, 1, 2, 4}, 
       VertexCoordinates -> {{0, 0.5}, {1, 1}, {1, 0}, {2, 1}, {2, 0}, {
         3, 0.5}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], {
         {Arrowheads[0.], ArrowBox[{{0., 0.5}, {1., 1.}}, 0.1]}, 
         InsetBox["2", Offset[{0, 2}, {0.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{0., 0.5}, {1., 0.}}, 0.1]}, 
         InsetBox["3", Offset[{0, 2}, {0.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}, 
         InsetBox["5", Offset[{0, 2}, {1.5, 1.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 0.}}, 0.1]}, 
         InsetBox["2", Offset[{0, 2}, {1.5, 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 0.}, {2., 0.}}, 0.1]}, 
         InsetBox["5", Offset[{0, 2}, {1.5, 0.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 0.}}, 0.1]}, 
         InsetBox["1", Offset[{0, 2}, {2., 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 1.}, {3., 0.5}}, 0.1]}, 
         InsetBox["2", Offset[{0, 2}, {2.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 0.}, {3., 0.5}}, 0.1]}, 
         InsetBox["4", Offset[{0, 2}, {2.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 0.5}, 0.1], InsetBox["\<\"a\"\>", {0., 0.5},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.1], 
         InsetBox["\<\"b\"\>", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.1], 
         InsetBox["\<\"c\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.1], 
         InsetBox["\<\"d\"\>", {2., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.1], 
         InsetBox["\<\"e\"\>", {2., 0.},
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.5}, 0.1], 
         InsetBox["\<\"z\"\>", {3., 0.5},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[225]=",ExpressionUUID->"5098c2d1-841b-44f4-a443-52d482df581c"]
}, Open  ]],

Cell[TextData[{
 "Now, we will make use of the Wolfram Language\[CloseCurlyQuote]s \
implementation of path finding algorithms to compute the shortest path \
between ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]],ExpressionUUID->
  "b9ea74e0-b2ea-44fd-af18-06daafc7caa2"],
 " and ",
 Cell[BoxData[
  FormBox["z", TraditionalForm]],ExpressionUUID->
  "1c9c590c-d0f4-4817-b1e5-f614f09e8f04"],
 ". To do this, we simply call the ",
 StyleBox[ButtonBox["FindShortestPath",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindShortestPath"], "FunctionLink"],
 " function with three arguments: the graph and the names of the starting and \
ending vertices. The output will be a list of vertices beginning with the \
starting vertex and ending with the final vertex through which the path \
runs."
}], "Text",ExpressionUUID->"f1baf928-9272-4246-9f1b-ecaf67353c21"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FindShortestPath", "[", 
  RowBox[{"exercise2", ",", "\"\<a\>\"", ",", "\"\<z\>\""}], "]"}]], "Input",
 CellLabel->
  "In[226]:=",ExpressionUUID->"5ec39d6b-f8fc-4035-ab68-7ad3acfcaf21"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"e\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"z\"\>"}], "}"}]], "Output",
 CellLabel->
  "Out[226]=",ExpressionUUID->"8999e3ff-65f9-43af-90c6-45044b9175da"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["PathGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PathGraph"], "FunctionLink"],
 " function can be used to help visualize this path. ",
 StyleBox[ButtonBox["PathGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PathGraph"], "FunctionLink"],
 " accepts a list of vertices as input and produces the graph obtained by \
connecting successive vertices with an edge. By default, the graph produced \
is undirected. To obtain a directed path, assign the ",
 StyleBox[ButtonBox["DirectedEdges",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DirectedEdges"], "FunctionLink"],
 " option to ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". Here, we will display the path in the graph by applying ",
 StyleBox[ButtonBox["HighlightGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/HighlightGraph"], "FunctionLink"],
 ", using ",
 StyleBox[ButtonBox["PathGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PathGraph"], "FunctionLink"],
 " in the second argument."
}], "Text",ExpressionUUID->"b73e878f-d705-4408-b59d-08228b5ca7bd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HighlightGraph", "[", 
  RowBox[{"exercise2", ",", 
   RowBox[{"PathGraph", "@", 
    RowBox[{"FindShortestPath", "[", 
     RowBox[{"exercise2", ",", "\"\<a\>\"", ",", "\"\<z\>\""}], "]"}]}]}], 
  "]"}]], "Input",
 CellLabel->
  "In[227]:=",ExpressionUUID->"f247b74e-d576-4dae-ae62-e7b1451e4951"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "z"}, {
      Null, {{1, 2}, {1, 3}, {2, 4}, {2, 5}, {3, 5}, {4, 5}, {4, 6}, {5, 
       6}}}, {EdgeLabels -> {
         Placed["EdgeWeight", {0.5, {-1, 0}}]}, VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, GraphHighlight -> {
         UndirectedEdge["d", "z"], 
         UndirectedEdge["b", "e"], "d", "e", "a", 
         UndirectedEdge["e", "d"], "b", 
         UndirectedEdge["a", "b"], "z"}, 
       EdgeWeight -> {2, 3, 5, 2, 5, 1, 2, 4}, 
       VertexCoordinates -> {{0, 0.5}, {1, 1}, {1, 0}, {2, 1}, {2, 0}, {
         3, 0.5}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], {
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], ArrowBox[{{0., 0.5}, {1., 1.}}, 0.1]}}, 
         InsetBox["2", Offset[{0, 2}, {0.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{0., 0.5}, {1., 0.}}, 0.1]}, 
         InsetBox["3", Offset[{0, 2}, {0.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 1.}}, 0.1]}, 
         InsetBox["5", Offset[{0, 2}, {1.5, 1.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 0.}}, 0.1]}}, 
         InsetBox["2", Offset[{0, 2}, {1.5, 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{1., 0.}, {2., 0.}}, 0.1]}, 
         InsetBox["5", Offset[{0, 2}, {1.5, 0.}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 0.}}, 0.1]}}, 
         InsetBox["1", Offset[{0, 2}, {2., 0.5}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], Opacity[1], 
          {Arrowheads[0.], ArrowBox[{{2., 1.}, {3., 0.5}}, 0.1]}}, 
         InsetBox["2", Offset[{0, 2}, {2.5, 0.75}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}, {
         {Arrowheads[0.], ArrowBox[{{2., 0.}, {3., 0.5}}, 0.1]}, 
         InsetBox["4", Offset[{0, 2}, {2.5, 0.25}], ImageScaled[{-1, 0}],
          BaseStyle->"Graphics"]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{0., 0.5}, 0.1]}, InsetBox["\<\"a\"\>", {0., 0.5},
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{1., 1.}, 0.1]}, InsetBox["\<\"b\"\>", {1., 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.1], 
         InsetBox["\<\"c\"\>", {1., 0.},
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{2., 1.}, 0.1]}, InsetBox["\<\"d\"\>", {2., 1.},
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{2., 0.}, 0.1]}, InsetBox["\<\"e\"\>", {2., 0.},
          BaseStyle->"Graphics"]}, {
         {Hue[1, 1, 0.7], EdgeForm[{Hue[1, 1, 0.7], Opacity[1]}], 
          DiskBox[{3., 0.5}, 0.1]}, InsetBox["\<\"z\"\>", {3., 0.5},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[227]=",ExpressionUUID->"412fc3c8-2d4a-4990-9816-2682009e23d4"]
}, Open  ]],

Cell[TextData[{
 "The length of the shortest path can be determined with the ",
 StyleBox[ButtonBox["GraphDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistance"], "FunctionLink"],
 " function and the same arguments."
}], "Text",ExpressionUUID->"bbaafb7c-c115-4ce5-af65-01c9e373b44d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphDistance", "[", 
  RowBox[{"exercise2", ",", "\"\<a\>\"", ",", "\"\<z\>\""}], "]"}]], "Input",
 CellLabel->
  "In[228]:=",ExpressionUUID->"210bd2b7-f78e-4bce-9ff1-41cc486ca865"],

Cell[BoxData["7.`"], "Output",
 CellLabel->
  "Out[228]=",ExpressionUUID->"3e4acd26-809f-4d46-bc7d-65871b55ebf8"]
}, Open  ]],

Cell[TextData[{
 "If the final argument, the destination vertex, is omitted from ",
 StyleBox[ButtonBox["GraphDistance",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistance"], "FunctionLink"],
 ", the result will be a list of the lengths of the shortest paths to each \
vertex of the graph."
}], "Text",ExpressionUUID->"04e39816-5b40-4bcd-9e76-c226b25e8651"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GraphDistance", "[", 
  RowBox[{"exercise2", ",", "\"\<a\>\""}], "]"}]], "Input",
 CellLabel->
  "In[229]:=",ExpressionUUID->"f1163dbe-a2de-4806-9709-1a7422722872"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"0", ",", "2.`", ",", "3.`", ",", "5.`", ",", "4.`", ",", "7.`"}], 
  "}"}]], "Output",
 CellLabel->
  "Out[229]=",ExpressionUUID->"171d7ae8-8b57-4341-b0f0-a81e79c115cc"]
}, Open  ]],

Cell[TextData[{
 "To determine the shortest path from every vertex to every other vertex, use \
the ",
 StyleBox[ButtonBox["GraphDistanceMatrix",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphDistanceMatrix"], "FunctionLink"],
 " function. Algorithm 2 in the Exercises of Section 10.6 describes the Floyd\
\[Dash]Warshall algorithm (also known as simply the Floyd algorithm), which \
is one method for computing this matrix."
}], "Text",ExpressionUUID->"b3a70403-28fa-4eb2-b8cf-4f90f2105113"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GraphDistanceMatrix", "[", "exercise2", "]"}], "//", 
  "MatrixForm"}]], "Input",
 CellLabel->
  "In[230]:=",ExpressionUUID->"9539013f-b09e-4ebe-9382-df1ddc22c3c5"],

Cell[BoxData[
 TagBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"0.`", "2.`", "3.`", "5.`", "4.`", "7.`"},
     {"2.`", "0.`", "5.`", "3.`", "2.`", "5.`"},
     {"3.`", "5.`", "0.`", "6.`", "5.`", "8.`"},
     {"5.`", "3.`", "6.`", "0.`", "1.`", "2.`"},
     {"4.`", "2.`", "5.`", "1.`", "0.`", "3.`"},
     {"7.`", "5.`", "8.`", "2.`", "3.`", "0.`"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
  Function[BoxForm`e$, 
   MatrixForm[BoxForm`e$]]]], "Output",
 CellLabel->
  "Out[230]//MatrixForm=",ExpressionUUID->"e41e20b9-6790-491a-ad30-\
14f0c62fc4d2"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.7 Planar Graphs", "Section",
 CellTags->"Section7",ExpressionUUID->"d5724981-81d0-4aeb-a1fd-f528100e9e21"],

Cell[TextData[{
 "This section explains how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to explore the question of whether a graph is planar by \
manipulating graphs in order to produce homeomorphic graphs and applying \
Kuratowski\[CloseCurlyQuote]s Theorem. We consider only undirected simple \
graphs in this section. The question of planarity for a directed graph can be \
answered by considering the underlying undirected graph, which can be \
obtained by applying the function ",
 StyleBox[ButtonBox["UndirectedGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UndirectedGraph"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"10c48b9e-c6e6-4450-9343-7a98fb6274cc"],

Cell[TextData[{
 "Before looking at how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can help you manipulate graphs to apply Kuratowski\[CloseCurlyQuote]s \
theorem manually, note that the function ",
 StyleBox[ButtonBox["PlanarGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PlanarGraphQ"], "FunctionLink"],
 ", applied to a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object, will determine whether or not the graph is planar."
}], "Text",ExpressionUUID->"409ddd99-40e1-4ed8-b977-0ccd7e51991e"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlanarGraphQ", "[", 
  RowBox[{"HypercubeGraph", "[", "3", "]"}], "]"}]], "Input",
 CellLabel->
  "In[231]:=",ExpressionUUID->"4428afac-d3f4-4e98-a6a2-1569b4a58c3d"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[231]=",ExpressionUUID->"b24fd399-05dd-4565-9bf2-838ca09cac60"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlanarGraphQ", "[", 
  RowBox[{"CompleteGraph", "[", "5", "]"}], "]"}]], "Input",
 CellLabel->
  "In[232]:=",ExpressionUUID->"25ab1454-9e9f-483a-be08-2469a5d05ec2"],

Cell[BoxData["False"], "Output",
 CellLabel->
  "Out[232]=",ExpressionUUID->"3b45621f-4314-436e-acdb-d48931c7c826"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Elementary Subdivisions, Smoothing, and Homeomorphic Graphs", \
"Subsection",
 CellTags->"Section7a",ExpressionUUID->"e2e7f02a-e5a9-43bd-8115-46e50f8fe350"],

Cell[TextData[{
 "Recall that an elementary subdivision refers to the process of modifying a \
graph by removing an edge ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"u", ",", "v"}], "}"}], TraditionalForm]],ExpressionUUID->
  "e45634e2-843b-45d6-8a40-669eb6308469"],
 " and replacing it with a vertex ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "71a66bc8-a313-42d5-8db8-6f283368c3aa"],
 " and new edges ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}], TraditionalForm]],ExpressionUUID->
  "b73419cb-5c2b-4d3e-a018-9544e391f492"],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"w", ",", "v"}], "}"}], TraditionalForm]],ExpressionUUID->
  "b5a3bc0c-2642-4648-9e84-fd41f4a9731f"],
 ". Effectively, this splits the original edge into two by inserting a vertex \
in the middle of it. It is not difficult to perform an elementary \
subdivision. We will use a cycle graph as an example."
}], "Text",ExpressionUUID->"07c7e560-211f-4343-be38-833d111633d5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subdivideExample", "=", 
  RowBox[{"CycleGraph", "[", 
   RowBox[{"5", ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[233]:=",ExpressionUUID->"076c9628-2206-47a9-9e2b-8221bad39437"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {Null, 
       SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 2, 4, 6, 8, 10}, {{2}, {5}, {1}, {3}, {2}, {4}, {3}, {5}, {
          1}, {4}}}, Pattern}]}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}}, 
          0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
          0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
          0.5877852522924737, -0.8090169943749481}}, 0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
          0.9510565162951533, 0.30901699437494634`}}, 
          0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951533, 
          0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
          0.11755705045849454`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
          0.11755705045849454], 
         InsetBox["1", {-0.9510565162951538, 0.30901699437494834},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
          0.11755705045849454], 
         InsetBox["2", {-0.5877852522924727, -0.8090169943749468},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924737, -0.8090169943749481}, 
          0.11755705045849454], 
         InsetBox["3", {0.5877852522924737, -0.8090169943749481},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9510565162951533, 0.30901699437494634}, 
          0.11755705045849454], 
         InsetBox["4", {0.9510565162951533, 0.30901699437494634},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.11755705045849454], 
         InsetBox["5", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[233]=",ExpressionUUID->"0db12089-2614-471b-989b-21360bcf16ba"]
}, Open  ]],

Cell[TextData[{
 "We will subdivide the edge ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]],ExpressionUUID->
  "0d1a1de1-eb4b-45c9-839a-206dd57e26bd"],
 ". To do this, we first need to introduce a vertex. We will give this vertex \
the name \[OpenCurlyDoubleQuote]1-2\[CloseCurlyDoubleQuote]. We apply ",
 StyleBox[ButtonBox["ToString",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ToString"], "FunctionLink"],
 " to the integers to create strings and combine them and the hyphen with the \
",
 StyleBox[ButtonBox["StringJoin",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StringJoin"], "FunctionLink"],
 " (",
 StyleBox["<>", "Code"],
 ") operator. We use the ",
 StyleBox[ButtonBox["VertexAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexAdd"], "FunctionLink"],
 " function to add the vertex to the graph."
}], "Text",ExpressionUUID->"05fd51c2-b17a-40ab-ac92-e417ffcd2dc5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subdivideExampleB", "=", 
  RowBox[{"VertexAdd", "[", 
   RowBox[{"subdivideExample", ",", 
    RowBox[{
     RowBox[{"ToString", "[", "1", "]"}], "<>", "\"\<-\>\"", "<>", 
     RowBox[{"ToString", "[", "2", "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[234]:=",ExpressionUUID->"7afbd1f6-537d-447e-b355-59b1f2afd176"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, "1-2"}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 2, 4, 6, 8, 10, 10}, {{2}, {5}, {1}, {3}, {2}, {4}, {3}, {
          5}, {1}, {4}}}, Pattern}]}, {
      GraphLayout -> "CircularEmbedding", VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 
          0.5000000000000007}, {-0.8660254037844384, -0.4999999999999994}}, 
          0.09999999999999994]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844389, 0.5000000000000007}, {
          0.8660254037844386, 0.4999999999999993}}, 0.09999999999999994]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.8660254037844384, -0.4999999999999994}, {
          3.8285686989269494`*^-16, -1.}}, 0.09999999999999994]}, 
        {Arrowheads[0.], 
         ArrowBox[{{3.8285686989269494`*^-16, -1.}, {
          0.8660254037844389, -0.5000000000000012}}, 0.09999999999999994]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.8660254037844389, -0.5000000000000012}, {
          0.8660254037844386, 0.4999999999999993}}, 0.09999999999999994]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.8660254037844389, 0.5000000000000007}, 
          0.09999999999999994], 
         InsetBox["1", {-0.8660254037844389, 0.5000000000000007},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.8660254037844384, -0.4999999999999994}, 
          0.09999999999999994], 
         InsetBox["2", {-0.8660254037844384, -0.4999999999999994},
          BaseStyle->"Graphics"]}, {
         DiskBox[{3.8285686989269494*^-16, -1.}, 0.09999999999999994], 
         InsetBox["3", {3.8285686989269494*^-16, -1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844389, -0.5000000000000012}, 
          0.09999999999999994], 
         InsetBox["4", {0.8660254037844389, -0.5000000000000012},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.8660254037844386, 0.4999999999999993}, 
          0.09999999999999994], 
         InsetBox["5", {0.8660254037844386, 0.4999999999999993},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.8369701987210297*^-16, 1.}, 0.09999999999999994], 
         InsetBox["\<\"1-2\"\>", {1.8369701987210297*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[234]=",ExpressionUUID->"3b7cd094-a7a1-4083-b46b-294ec3897d0a"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has moved the original vertices. We can put them back in place by \
assigning the ",
 StyleBox[ButtonBox["VertexCoordinates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexCoordinates"], "FunctionLink"],
 " option in the new graph to the positions of the original vertices obtained \
from ",
 StyleBox[ButtonBox["GraphEmbedding",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/GraphEmbedding"], "FunctionLink"],
 ". We just need to append a location for the new vertex, which will be at \
the end of the list of vertices. Note that we need to apply ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " to the result of the ",
 StyleBox[ButtonBox["VertexAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/VertexAdd"], "FunctionLink"],
 ", since that function will not accept the options normally available to \
graphs."
}], "Text",ExpressionUUID->"a756ac14-dae4-4c4c-8853-af254efaaacd"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subdivideExampleB", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"VertexAdd", "[", 
     RowBox[{"subdivideExample", ",", 
      RowBox[{
       RowBox[{"ToString", "[", "1", "]"}], "<>", "\"\<-\>\"", "<>", 
       RowBox[{"ToString", "[", "2", "]"}]}]}], "]"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"Append", "[", 
      RowBox[{
       RowBox[{"GraphEmbedding", "[", "subdivideExample", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}]}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[235]:=",ExpressionUUID->"e84fd38d-03ea-408f-ae65-a539a16be5f5"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, "1-2"}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 2, 4, 6, 8, 10, 10}, {{2}, {5}, {1}, {3}, {2}, {4}, {3}, {
          5}, {1}, {4}}}, Pattern}]}, {
      GraphLayout -> {
        "Dimension" -> 2, "VertexLayout" -> "CircularEmbedding"}, 
       VertexCoordinates -> {{-0.9510565162951538, 
         0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
         0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
         0.30901699437494634`}, {-7.044813998280222*^-16, 1.}, {0, 0}}, 
       VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}}, 
          0.09999999999999992]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
          0.09999999999999992]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
          0.5877852522924737, -0.8090169943749481}}, 0.09999999999999992]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
          0.9510565162951533, 0.30901699437494634`}}, 0.09999999999999992]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951533, 
          0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
          0.09999999999999992]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
          0.09999999999999992], 
         InsetBox["1", {-0.9510565162951538, 0.30901699437494834},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
          0.09999999999999992], 
         InsetBox["2", {-0.5877852522924727, -0.8090169943749468},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924737, -0.8090169943749481}, 
          0.09999999999999992], 
         InsetBox["3", {0.5877852522924737, -0.8090169943749481},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9510565162951533, 0.30901699437494634}, 
          0.09999999999999992], 
         InsetBox["4", {0.9510565162951533, 0.30901699437494634},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.09999999999999992], 
         InsetBox["5", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.09999999999999992], 
         InsetBox["\<\"1-2\"\>", {0., 0.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[235]=",ExpressionUUID->"88e40ee2-879f-45d7-ae4c-743387e5cefc"]
}, Open  ]],

Cell[TextData[{
 "Now, we move the new vertex along the edge which is to be deleted. This is \
done by using ",
 StyleBox[ButtonBox["PropertyValue",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PropertyValue"], "FunctionLink"],
 " and averaging the positions of the existing vertices."
}], "Text",ExpressionUUID->"6ca3e411-f350-49eb-89b2-bb6bac5e1011"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PropertyValue", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"subdivideExampleB", ",", "\"\<1-2\>\""}], "}"}], ",", 
    "VertexCoordinates"}], "]"}], "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"subdivideExample", ",", "1"}], "}"}], ",", 
       "VertexCoordinates"}], "]"}], "+", 
     RowBox[{"PropertyValue", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"subdivideExample", ",", "2"}], "}"}], ",", 
       "VertexCoordinates"}], "]"}]}], ")"}], "/", "2"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[236]:=",ExpressionUUID->"c3f3d529-920a-4666-8fd6-9d1a5012ef5b"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"-", "0.7694208842938133`"}], ",", 
   RowBox[{"-", "0.24999999999999922`"}]}], "}"}]], "Output",
 CellLabel->
  "Out[236]=",ExpressionUUID->"c9387b13-149c-492c-924b-2108d1cd0fd6"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["subdivideExampleB"], "Input",
 CellLabel->
  "In[237]:=",ExpressionUUID->"ea8c95e7-00a6-443d-a93f-44a522a68695"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, "1-2"}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 2, 4, 6, 8, 10, 10}, {{2}, {5}, {1}, {3}, {2}, {4}, {3}, {
          5}, {1}, {4}}}, Pattern}]}, {
      GraphLayout -> {
        "Dimension" -> 2, "VertexLayout" -> "CircularEmbedding"}, 
       VertexCoordinates -> {{-0.9510565162951538, 
        0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
        0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
        0.30901699437494634`}, {-7.044813998280222*^-16, 
        1.}, {-0.7694208842938133, -0.24999999999999922`}}, VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}}, 
          0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
          0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
          0.5877852522924737, -0.8090169943749481}}, 0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
          0.9510565162951533, 0.30901699437494634`}}, 0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951533, 
          0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
          0.05877852522924734]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
          0.05877852522924734], 
         InsetBox["1", {-0.9510565162951538, 0.30901699437494834},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
          0.05877852522924734], 
         InsetBox["2", {-0.5877852522924727, -0.8090169943749468},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924737, -0.8090169943749481}, 
          0.05877852522924734], 
         InsetBox["3", {0.5877852522924737, -0.8090169943749481},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9510565162951533, 0.30901699437494634}, 
          0.05877852522924734], 
         InsetBox["4", {0.9510565162951533, 0.30901699437494634},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.05877852522924734], 
         InsetBox["5", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.7694208842938133, -0.24999999999999922}, 
          0.05877852522924734], 
         InsetBox["\<\"1-2\"\>", {-0.7694208842938133, -0.24999999999999922},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[237]=",ExpressionUUID->"c6e8f253-376d-4ca5-8e0d-77e6bc35d427"]
}, Open  ]],

Cell[TextData[{
 "Finally, we simply remove the original edge with ",
 StyleBox[ButtonBox["EdgeDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeDelete"], "FunctionLink"],
 " and add the new ones with ",
 StyleBox[ButtonBox["EdgeAdd",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeAdd"], "FunctionLink"],
 ". Recall that the second argument of ",
 StyleBox[ButtonBox["EdgeDelete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EdgeDelete"], "FunctionLink"],
 " must be given using the special symbols \[UndirectedEdge] (\[EscapeKey]",
 StyleBox["ue",
  IgnoreSpellCheck->True],
 "\[EscapeKey]) or \[DirectedEdge] (\[EscapeKey]de\[EscapeKey]), not as a ",
 StyleBox[ButtonBox["Rule",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Rule"], "FunctionLink"],
 " (",
 StyleBox["->", "Code"],
 "). "
}], "Text",ExpressionUUID->"31c38212-7a00-4b9f-8536-be942158aaae"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"subdivideExampleB", "=", 
   RowBox[{"EdgeDelete", "[", 
    RowBox[{"subdivideExampleB", ",", 
     RowBox[{"1", "\[Rule]", "2"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"subdivideExampleB", "=", 
  RowBox[{"EdgeAdd", "[", 
   RowBox[{"subdivideExampleB", ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"1", "\[Rule]", "\"\<1-2\>\""}], ",", 
      RowBox[{"2", "\[Rule]", "\"\<1-2\>\""}]}], "}"}]}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[238]:=",ExpressionUUID->"dcf6bb99-5ace-4fe7-aa4b-24865434cd55"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, "1-2"}, {Null, 
       SparseArray[
       Automatic, {6, 6}, 0, {
        1, {{0, 2, 5, 7, 9, 11, 13}, {{5}, {6}, {1}, {3}, {6}, {2}, {4}, {
          3}, {5}, {1}, {4}, {1}, {2}}}, Pattern}]}, {
      GraphLayout -> {
        "Dimension" -> 2, "VertexLayout" -> "CircularEmbedding"}, 
       VertexCoordinates -> {{-0.9510565162951538, 
        0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
        0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
        0.30901699437494634`}, {-7.044813998280222*^-16, 
        1.}, {-0.7694208842938133, -0.24999999999999922`}}, VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
          0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.7694208842938133, \
-0.24999999999999922`}}, 0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
          0.5877852522924737, -0.8090169943749481}}, 0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, \
{-0.7694208842938133, -0.24999999999999922`}}, 0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
          0.9510565162951533, 0.30901699437494634`}}, 0.05877852522924734]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951533, 
          0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
          0.05877852522924734]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
          0.05877852522924734], 
         InsetBox["1", {-0.9510565162951538, 0.30901699437494834},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
          0.05877852522924734], 
         InsetBox["2", {-0.5877852522924727, -0.8090169943749468},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924737, -0.8090169943749481}, 
          0.05877852522924734], 
         InsetBox["3", {0.5877852522924737, -0.8090169943749481},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9510565162951533, 0.30901699437494634}, 
          0.05877852522924734], 
         InsetBox["4", {0.9510565162951533, 0.30901699437494634},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.05877852522924734], 
         InsetBox["5", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.7694208842938133, -0.24999999999999922}, 
          0.05877852522924734], 
         InsetBox["\<\"1-2\"\>", {-0.7694208842938133, -0.24999999999999922},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[239]=",ExpressionUUID->"49abb31b-62dd-4942-9036-b706d68efcbd"]
}, Open  ]],

Cell["Based on this example, we create the following function.", "Text",ExpressionUUID->"e6e69d3e-d91c-44d1-9378-11ae272a41d3"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"subdivideGraph", "::", "nonedge"}], "=", 
   "\"\<Second argument must be an edge in the graph.\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"subdivideGraph", "[", 
    RowBox[{"G_Graph", ",", 
     RowBox[{"E_Rule", "|", "E_UndirectedEdge"}]}], "]"}], "/;", 
   RowBox[{"UndirectedGraphQ", "[", "G", "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"H", ",", "e", ",", "newV", ",", "v"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Head", "[", "E", "]"}], "===", "Rule"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"e", "=", 
        RowBox[{"UndirectedEdge", "@@", "E"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"e", "=", "E"}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"EdgeQ", "[", 
         RowBox[{"G", ",", "e"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"subdivideGraph", "::", "nonedge"}], "]"}], ";", 
        RowBox[{"Return", "[", "$Failed", "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"newV", "=", 
      RowBox[{
       RowBox[{"ToString", "[", 
        RowBox[{"e", "[", 
         RowBox[{"[", "1", "]"}], "]"}], "]"}], "<>", "\"\<-\>\"", "<>", 
       RowBox[{"ToString", "[", 
        RowBox[{"e", "[", 
         RowBox[{"[", "2", "]"}], "]"}], "]"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"H", "=", 
      RowBox[{"Graph", "[", 
       RowBox[{
        RowBox[{"VertexAdd", "[", 
         RowBox[{"G", ",", "newV"}], "]"}], ",", 
        RowBox[{"VertexCoordinates", "\[Rule]", 
         RowBox[{"Append", "[", 
          RowBox[{
           RowBox[{"GraphEmbedding", "[", "G", "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"0", ",", "0"}], "}"}]}], "]"}]}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"PropertyValue", "[", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"H", ",", "newV"}], "}"}], ",", "VertexCoordinates"}], "]"}],
       "=", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"H", ",", 
             RowBox[{"e", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
           "VertexCoordinates"}], "]"}], "+", 
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"H", ",", 
             RowBox[{"e", "[", 
              RowBox[{"[", "2", "]"}], "]"}]}], "}"}], ",", 
           "VertexCoordinates"}], "]"}]}], ")"}], "/", "2"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"H", "=", 
      RowBox[{"EdgeDelete", "[", 
       RowBox[{"H", ",", "e"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"H", "=", 
      RowBox[{"EdgeAdd", "[", 
       RowBox[{"H", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"e", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "\[Rule]", "newV"}], ",", 
          RowBox[{"newV", "\[Rule]", 
           RowBox[{"e", "[", 
            RowBox[{"[", "2", "]"}], "]"}]}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", "H"}]}], "\[IndentingNewLine]", "]"}]}]}], "Input",\

 InitializationCell->True,
 CellLabel->
  "In[240]:=",ExpressionUUID->"ccd77f77-2b72-4c59-965f-87b4994759b2"],

Cell[TextData[{
 "The inverse operation of elementary subdivision is referred to as \
smoothing. To be precise, let ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "40c1ffa6-111d-4f30-9f9e-9ad3b7a93b9f"],
 " be a vertex of degree 2 with neighbors ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "be57f002-1284-4dfc-a443-d86fc12f53c0"],
 " and ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "9bc798ba-4564-4deb-b22c-46b9f540896f"],
 " and such that ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "9414e331-3567-4b18-ad26-8cc598827397"],
 " and ",
 Cell[BoxData[
  FormBox["w", TraditionalForm]],ExpressionUUID->
  "eab73d01-e1df-4dc1-8d02-c56b1a9ef355"],
 " are not adjacent. We smooth the vertex ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "c2c8b864-1768-412f-97d6-2bae16754a75"],
 " by deleting ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "ba4f72d6-9f12-42f8-92e9-95c8d46a24eb"],
 " and the edges incident to it and adding the edge ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"u", ",", "w"}], "}"}], TraditionalForm]],ExpressionUUID->
  "dfff140f-d7a8-4fbd-a810-50782e297fb7"],
 ". Below, we have created a function to implement smoothing. "
}], "Text",ExpressionUUID->"d765ac0d-895f-4287-99ef-d21eba4ff8c7"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"smoothGraph", "::", "vertx"}], "=", 
   "\"\<Cannot smooth this vertex.\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"smoothGraph", "[", 
    RowBox[{"G_Graph", ",", "v_"}], "]"}], "/;", 
   RowBox[{"UndirectedGraphQ", "[", "G", "]"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"N", ",", "H", ",", "e"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"N", "=", 
      RowBox[{"AdjacencyList", "[", 
       RowBox[{"G", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Length", "[", "N", "]"}], "\[NotEqual]", "2"}], "||", 
        RowBox[{"EdgeQ", "[", 
         RowBox[{"G", ",", 
          RowBox[{"UndirectedEdge", "[", 
           RowBox[{
            RowBox[{"N", "[", 
             RowBox[{"[", "1", "]"}], "]"}], ",", 
            RowBox[{"N", "[", 
             RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"Message", "[", 
         RowBox[{"smoothGraph", "::", "vertx"}], "]"}], ";", 
        RowBox[{"Return", "[", "$Failed", "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"H", "=", 
      RowBox[{"VertexDelete", "[", 
       RowBox[{"G", ",", "v"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"e", "=", 
      RowBox[{"UndirectedEdge", "[", 
       RowBox[{
        RowBox[{"N", "[", 
         RowBox[{"[", "1", "]"}], "]"}], ",", 
        RowBox[{"N", "[", 
         RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"H", "=", 
      RowBox[{"EdgeAdd", "[", 
       RowBox[{"H", ",", "e"}], "]"}]}], ";", "\[IndentingNewLine]", "H"}]}], 
   "\[IndentingNewLine]", "]"}]}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[242]:=",ExpressionUUID->"fa505704-dc1c-4a32-be58-75eea4f2bce9"],

Cell["As an example, we can smooth the vertex \[OpenCurlyDoubleQuote]1-2\
\[CloseCurlyDoubleQuote] added above.", "Text",ExpressionUUID->"40313240-af38-\
484d-8460-5fe9d886b69d"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"smoothGraph", "[", 
  RowBox[{"subdivideExampleB", ",", "\"\<1-2\>\""}], "]"}]], "Input",
 CellLabel->
  "In[244]:=",ExpressionUUID->"71b628ee-3997-4dab-87c3-0cc31d20114a"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5}, {
      Null, SparseArray[
       Automatic, {5, 5}, 0, {
        1, {{0, 2, 4, 6, 8, 10}, {{5}, {2}, {1}, {3}, {2}, {4}, {3}, {5}, {
          1}, {4}}}, {1, 1, 2, 1, 1, 1, 1, 1, 1, 1}}]}, {
      GraphLayout -> {
        "Dimension" -> 2, "VertexLayout" -> "CircularEmbedding"}, 
       VertexCoordinates -> {{-0.9510565162951538, 
        0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, {
        0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
        0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, VertexLabels -> {
         Placed["Name", Center], "1-2" -> None}, 
       VertexSize -> {Medium, "1-2" -> Automatic}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-7.044813998280222*^-16, 1.}}, 
          0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951538, 
          0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}}, 
          0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.5877852522924727, -0.8090169943749468}, {
          0.5877852522924737, -0.8090169943749481}}, 0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924737, -0.8090169943749481}, {
          0.9510565162951533, 0.30901699437494634`}}, 
          0.11755705045849454`]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951533, 
          0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, 
          0.11755705045849454`]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {
         DiskBox[{-0.9510565162951538, 0.30901699437494834}, 
          0.11755705045849454], 
         InsetBox["1", {-0.9510565162951538, 0.30901699437494834},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.5877852522924727, -0.8090169943749468}, 
          0.11755705045849454], 
         InsetBox["2", {-0.5877852522924727, -0.8090169943749468},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924737, -0.8090169943749481}, 
          0.11755705045849454], 
         InsetBox["3", {0.5877852522924737, -0.8090169943749481},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.9510565162951533, 0.30901699437494634}, 
          0.11755705045849454], 
         InsetBox["4", {0.9510565162951533, 0.30901699437494634},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-7.044813998280222*^-16, 1.}, 0.11755705045849454], 
         InsetBox["5", {-7.044813998280222*^-16, 1.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[244]=",ExpressionUUID->"b2ac4811-a0b0-47e8-8fb0-a368e33d8050"]
}, Open  ]],

Cell[TextData[{
 "The textbook defines graphs to be homeomorphic if they can be obtained from \
the same graph from a sequence of elementary subdivisions. It is clear that \
if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "1"], ",", 
    SubscriptBox["G", "2"], ",", 
    SubscriptBox["G", "3"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["G", "n"]}], TraditionalForm]],ExpressionUUID->
  "20c4e752-4f0e-45b3-bcdf-bb8929a1bd63"],
 " is a sequence of graphs, each of which can be obtained from the previous \
by an elementary subdivision, then ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["G", "n"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["G", "3"], ",", 
    SubscriptBox["G", "2"], ",", 
    SubscriptBox["G", "1"]}], TraditionalForm]],ExpressionUUID->
  "b347aaae-4d68-4247-9caf-e530675b65e9"],
 " is a sequence of graphs, each of which can be obtained from the previous \
by a smoothing. Hence, we can say that two graphs are homeomorphic if one can \
be transformed into the other by a sequence of elementary subdivisions and ",
 StyleBox["smoothings",
  IgnoreSpellCheck->True],
 "."
}], "Text",ExpressionUUID->"8a137350-f30a-45f9-a760-dd00605a85da"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applying Kuratowski\[CloseCurlyQuote]s Theorem", "Subsection",
 CellTags->"Section7b",ExpressionUUID->"759c9208-1e0c-4d49-b19d-bf97a15ff664"],

Cell[TextData[{
 "Recall that Kuratowski\[CloseCurlyQuote]s theorem asserts that a graph is \
nonplanar if and only if it contains a subgraph homeomorphic to either ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"3", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "d1121aa9-fefb-47b6-8e27-ddc52dea039d"],
 " or ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", "5"], TraditionalForm]],ExpressionUUID->
  "06756c11-a614-45fa-9158-26a96e127296"],
 ". Using the functions above and those for creating subgraphs, we can use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to manipulate a graph and confirm that it is nonplanar using Kuratowski\
\[CloseCurlyQuote]s theorem. We will illustrate this with the Petersen \
graph."
}], "Text",ExpressionUUID->"71ad8f7a-5fa2-4f56-9ef3-06bcdbe70609"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"petersen", "=", 
  RowBox[{"PetersenGraph", "[", 
   RowBox[{"5", ",", "2", ",", 
    RowBox[{"VertexSize", "\[Rule]", "Medium"}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", 
     RowBox[{"Placed", "[", 
      RowBox[{"\"\<Name\>\"", ",", "Center"}], "]"}]}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[245]:=",ExpressionUUID->"178efa70-3afa-4690-a51f-60488a2dc353"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}, {
      Null, {{1, 3}, {1, 4}, {1, 6}, {2, 4}, {2, 5}, {2, 7}, {3, 5}, {3, 8}, {
       4, 9}, {5, 10}, {6, 7}, {6, 10}, {7, 8}, {8, 9}, {9, 10}}}, {
      VertexLabels -> {
         Placed["Name", Center]}, VertexSize -> {Medium}, 
       VertexCoordinates -> {{0.9510565162951535, 0.30901699437494745`}, {
        0.5877852522924732, -0.8090169943749473}, {-0.587785252292473, \
-0.8090169943749475}, {-0.9510565162951536, 
        0.30901699437494723`}, {-2.4492935982947064`*^-16, 1.}, {
        1.902113032590307, 0.6180339887498949}, {
        1.1755705045849465`, -1.6180339887498947`}, {-1.175570504584946, \
-1.618033988749895}, {-1.9021130325903073`, 
        0.6180339887498945}, {-4.898587196589413*^-16, 2.}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951535, 
          0.30901699437494745`}, {-0.587785252292473, -0.8090169943749475}}, 
          0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951535, 
          0.30901699437494745`}, {-0.9510565162951536, 0.30901699437494723`}},
           0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951535, 0.30901699437494745`}, {
          1.902113032590307, 0.6180339887498949}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, {-0.9510565162951536, 
          0.30901699437494723`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, \
{-2.4492935982947064`*^-16, 1.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924732, -0.8090169943749473}, {
          1.1755705045849465`, -1.6180339887498947`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-2.4492935982947064`*^-16, 1.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-1.175570504584946, -1.618033988749895}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951536, 
          0.30901699437494723`}, {-1.9021130325903073`, 0.6180339887498945}}, 
          0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-2.4492935982947064`*^-16, 1.}, {-4.898587196589413*^-16, 
          2.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.902113032590307, 0.6180339887498949}, {
          1.1755705045849465`, -1.6180339887498947`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{1.902113032590307, 
          0.6180339887498949}, {-4.898587196589413*^-16, 2.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          1.1755705045849465`, -1.6180339887498947`}, {-1.175570504584946, \
-1.618033988749895}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1.175570504584946, -1.618033988749895}, \
{-1.9021130325903073`, 0.6180339887498945}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1.9021130325903073`, 
          0.6180339887498945}, {-4.898587196589413*^-16, 2.}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.9510565162951535, 0.30901699437494745}, 0.1], 
         InsetBox["1", {0.9510565162951535, 0.30901699437494745},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924732, -0.8090169943749473}, 0.1], 
         InsetBox["2", {0.5877852522924732, -0.8090169943749473},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.587785252292473, -0.8090169943749475}, 0.1], 
         InsetBox["3", {-0.587785252292473, -0.8090169943749475},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9510565162951536, 0.30901699437494723}, 0.1], 
         InsetBox["4", {-0.9510565162951536, 0.30901699437494723},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-2.4492935982947064*^-16, 1.}, 0.1], 
         InsetBox["5", {-2.4492935982947064*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.902113032590307, 0.6180339887498949}, 0.1], 
         InsetBox["6", {1.902113032590307, 0.6180339887498949},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.1755705045849465, -1.6180339887498947}, 0.1], 
         InsetBox["7", {1.1755705045849465, -1.6180339887498947},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1.175570504584946, -1.618033988749895}, 0.1], 
         InsetBox["8", {-1.175570504584946, -1.618033988749895},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1.9021130325903073, 0.6180339887498945}, 0.1], 
         InsetBox["9", {-1.9021130325903073, 0.6180339887498945},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-4.898587196589413*^-16, 2.}, 0.1], 
         InsetBox["10", {-4.898587196589413*^-16, 2.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[245]=",ExpressionUUID->"890c03fd-5607-4831-9d30-4f0d20734d16"]
}, Open  ]],

Cell["\<\
First, we form the subgraph of the Petersen graph obtained by removing vertex \
2 and the three edges incident to it.\
\>", "Text",ExpressionUUID->"04a437a9-47fe-477a-b318-a18d5b86456f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"petersen1", "=", 
  RowBox[{"VertexDelete", "[", 
   RowBox[{"petersen", ",", "6"}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[246]:=",ExpressionUUID->"804ac305-53b7-46e9-ba8d-540c0e0b5920"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 7, 8, 9, 10}, {
      Null, {{1, 3}, {1, 4}, {2, 4}, {2, 5}, {2, 6}, {3, 5}, {3, 7}, {4, 8}, {
       5, 9}, {6, 7}, {7, 8}, {8, 9}}}, {VertexLabels -> {
         Placed["Name", Center], 6 -> None}, 
       VertexSize -> {Medium, 6 -> Automatic}, 
       VertexCoordinates -> {{0.9510565162951535, 0.30901699437494745`}, {
        0.5877852522924732, -0.8090169943749473}, {-0.587785252292473, \
-0.8090169943749475}, {-0.9510565162951536, 
        0.30901699437494723`}, {-2.4492935982947064`*^-16, 1.}, {
        1.1755705045849465`, -1.6180339887498947`}, {-1.175570504584946, \
-1.618033988749895}, {-1.9021130325903073`, 
        0.6180339887498945}, {-4.898587196589413*^-16, 2.}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951535, 
          0.30901699437494745`}, {-0.587785252292473, -0.8090169943749475}}, 
          0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.9510565162951535, 
          0.30901699437494745`}, {-0.9510565162951536, 0.30901699437494723`}},
           0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, {-0.9510565162951536, 
          0.30901699437494723`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, \
{-2.4492935982947064`*^-16, 1.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{0.5877852522924732, -0.8090169943749473}, {
          1.1755705045849465`, -1.6180339887498947`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-2.4492935982947064`*^-16, 1.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-1.175570504584946, -1.618033988749895}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951536, 
          0.30901699437494723`}, {-1.9021130325903073`, 0.6180339887498945}}, 
          0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-2.4492935982947064`*^-16, 1.}, {-4.898587196589413*^-16, 
          2.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          1.1755705045849465`, -1.6180339887498947`}, {-1.175570504584946, \
-1.618033988749895}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1.175570504584946, -1.618033988749895}, \
{-1.9021130325903073`, 0.6180339887498945}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1.9021130325903073`, 
          0.6180339887498945}, {-4.898587196589413*^-16, 2.}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.9510565162951535, 0.30901699437494745}, 0.1], 
         InsetBox["1", {0.9510565162951535, 0.30901699437494745},
          BaseStyle->"Graphics"]}, {
         DiskBox[{0.5877852522924732, -0.8090169943749473}, 0.1], 
         InsetBox["2", {0.5877852522924732, -0.8090169943749473},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.587785252292473, -0.8090169943749475}, 0.1], 
         InsetBox["3", {-0.587785252292473, -0.8090169943749475},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9510565162951536, 0.30901699437494723}, 0.1], 
         InsetBox["4", {-0.9510565162951536, 0.30901699437494723},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-2.4492935982947064*^-16, 1.}, 0.1], 
         InsetBox["5", {-2.4492935982947064*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{1.1755705045849465, -1.6180339887498947}, 0.1], 
         InsetBox["7", {1.1755705045849465, -1.6180339887498947},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1.175570504584946, -1.618033988749895}, 0.1], 
         InsetBox["8", {-1.175570504584946, -1.618033988749895},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1.9021130325903073, 0.6180339887498945}, 0.1], 
         InsetBox["9", {-1.9021130325903073, 0.6180339887498945},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-4.898587196589413*^-16, 2.}, 0.1], 
         InsetBox["10", {-4.898587196589413*^-16, 2.},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[246]=",ExpressionUUID->"1d9c672d-cb50-45cc-9536-88acdb1f5d48"]
}, Open  ]],

Cell[TextData[{
 "Now, we notice that there are three vertices that are ",
 StyleBox["smoothable",
  IgnoreSpellCheck->True],
 ": 1, 7, and 10. That is to say, those three vertices have degree 2 and \
their neighbors are not adjacent. "
}], "Text",ExpressionUUID->"45b555ab-5acc-4477-8b67-0898fc43757c"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"petersen2", "=", 
   RowBox[{"smoothGraph", "[", 
    RowBox[{"petersen1", ",", "1"}], "]"}]}], ";", 
  RowBox[{"petersen3", "=", 
   RowBox[{"smoothGraph", "[", 
    RowBox[{"petersen2", ",", "7"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"petersen4", "=", 
  RowBox[{"smoothGraph", "[", 
   RowBox[{"petersen3", ",", "10"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellLabel->
  "In[247]:=",ExpressionUUID->"8379a1bc-5ec6-4863-ac9b-3050236f939e"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{2, 3, 4, 5, 8, 9}, {
      Null, {{1, 3}, {1, 4}, {2, 4}, {2, 5}, {3, 6}, {5, 6}, {2, 3}, {1, 5}, {
       4, 6}}}, {VertexLabels -> {
         Placed["Name", Center], 6 -> None, 1 -> None, 7 -> None, 10 -> None},
        VertexSize -> {
        Medium, 1 -> Automatic, 10 -> Automatic, 7 -> Automatic, 6 -> 
         Automatic}, 
       VertexCoordinates -> {{
        0.5877852522924732, -0.8090169943749473}, {-0.587785252292473, \
-0.8090169943749475}, {-0.9510565162951536, 
        0.30901699437494723`}, {-2.4492935982947064`*^-16, 
        1.}, {-1.175570504584946, -1.618033988749895}, {-1.9021130325903073`, 
        0.6180339887498945}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, {-0.9510565162951536, 
          0.30901699437494723`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, \
{-2.4492935982947064`*^-16, 1.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{
          0.5877852522924732, -0.8090169943749473}, {-1.175570504584946, \
-1.618033988749895}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-0.9510565162951536, 0.30901699437494723`}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-2.4492935982947064`*^-16, 1.}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.587785252292473, -0.8090169943749475}, \
{-1.175570504584946, -1.618033988749895}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-0.9510565162951536, 
          0.30901699437494723`}, {-1.9021130325903073`, 0.6180339887498945}}, 
          0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-2.4492935982947064`*^-16, 1.}, {-1.9021130325903073`, 
          0.6180339887498945}}, 0.1]}, 
        {Arrowheads[0.], 
         ArrowBox[{{-1.175570504584946, -1.618033988749895}, \
{-1.9021130325903073`, 0.6180339887498945}}, 0.1]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0.5877852522924732, -0.8090169943749473}, 0.1], 
         InsetBox["2", {0.5877852522924732, -0.8090169943749473},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.587785252292473, -0.8090169943749475}, 0.1], 
         InsetBox["3", {-0.587785252292473, -0.8090169943749475},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-0.9510565162951536, 0.30901699437494723}, 0.1], 
         InsetBox["4", {-0.9510565162951536, 0.30901699437494723},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-2.4492935982947064*^-16, 1.}, 0.1], 
         InsetBox["5", {-2.4492935982947064*^-16, 1.},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1.175570504584946, -1.618033988749895}, 0.1], 
         InsetBox["8", {-1.175570504584946, -1.618033988749895},
          BaseStyle->"Graphics"]}, {
         DiskBox[{-1.9021130325903073, 0.6180339887498945}, 0.1], 
         InsetBox["9", {-1.9021130325903073, 0.6180339887498945},
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[248]=",ExpressionUUID->"7c097e18-e454-4bd1-afd6-62f32616954d"]
}, Open  ]],

Cell[TextData[{
 "We now observe that this graph has 6 vertices, each of which has degree 3, \
just like ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"3", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "5e7a4d5c-62d8-4463-b6c0-15bf7a53b741"],
 ". Therefore, there is a definite possibility that this graph is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"3", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "6be99b66-275d-437f-a5d6-5a071ce3da92"],
 ". We confirm that is ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"3", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "44a7b456-4bf8-4819-b9ca-04213ca72886"],
 " with ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 "."
}], "Text",ExpressionUUID->"28fe7105-5903-4c03-a466-d215b2f5fb0a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IsomorphicGraphQ", "[", 
  RowBox[{"petersen4", ",", 
   RowBox[{"CompleteGraph", "[", 
    RowBox[{"{", 
     RowBox[{"3", ",", "3"}], "}"}], "]"}]}], "]"}]], "Input",
 CellLabel->
  "In[249]:=",ExpressionUUID->"5f00f345-7c35-4de2-a6a4-19f51bbde32d"],

Cell[BoxData["True"], "Output",
 CellLabel->
  "Out[249]=",ExpressionUUID->"8de4f50c-cbc9-47f9-9801-84f131ad24a1"]
}, Open  ]],

Cell[TextData[{
 "This demonstrates that the Petersen graph has a subgraph that is \
homeomorphic to ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["K", 
    RowBox[{"3", ",", "3"}]], TraditionalForm]],ExpressionUUID->
  "4abaa229-b1f4-498d-87e5-33ff9650b26e"],
 " and hence is nonplanar."
}], "Text",ExpressionUUID->"63e66462-150b-44b8-8853-527c303a2ca1"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["10.8 Graph Coloring", "Section",
 CellTags->"Section8",ExpressionUUID->"d784a58b-4e77-458f-aa87-baf1c76553df"],

Cell["\<\
In this section, we consider the problem of how to properly color a graph; \
that is, how to assign to each vertex of a graph a color such that no vertex \
has the same color as any of its neighbors. \
\>", "Text",ExpressionUUID->"2bb935e0-3645-446d-b19f-da5a7a01efb8"],

Cell["\<\
It is worth noting that, in terms of computational complexity, Hamilton \
circuits and graph coloring are equivalently difficult problems.\
\>", "Text",ExpressionUUID->"b483a67b-b29f-4325-9de7-3d870fef70df"],

Cell["\<\
We will create a function based on the algorithm described in the preface to \
Exercise 29 in Section 10.8 of the text. It can be shown that this algorithm \
will color a graph using at most one more color than the maximal degree of \
the graph. It is considered a greedy algorithm because it makes optimal \
choices at each step but never reconsiders its choices. That is to say, it \
does the best it can at every step but never backtracks to make improvements. \
Greedy algorithms often lead to good, but non-optimal, solutions.\
\>", "Text",ExpressionUUID->"8c9f33a1-8fc9-4449-8c7e-a235be4a975d"],

Cell["\<\
The algorithm proceeds as follows. First, the vertices are sorted in order of \
descending degree. The first color is assigned to the first vertex in the \
list. Also assign color 1 to the first vertex in the list not adjacent to \
vertex 1, to the next vertex not adjacent to those already colored, etc. \
Then, move on to the second color. The first uncolored vertex in the list is \
assigned color 2, as are vertices further down the list not adjacent to ones \
previously assigned the second color. This continues until all of the \
vertices have been given a color.\
\>", "Text",ExpressionUUID->"39244808-5f1a-459a-a4ea-46b76a98ed64"],

Cell[TextData[{
 "Our first step in implementing this function will be to sort the list of \
vertices in decreasing order of degree. For this, we will make use of the \
Wolfram Language\[CloseCurlyQuote]s very flexible ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " function. With no additional instructions, this function will sort a list \
of numbers in increasing numerical order and a list of strings in \
lexicographical order. The ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " function can accept an optional argument that allows us to specify the way \
in which the list is sorted. Specifically, ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " takes as an argument a Boolean-valued function on two arguments and \
returns true if the first argument precedes the second."
}], "Text",ExpressionUUID->"5ff16e5e-4b91-4276-a97b-77d027b374a3"],

Cell[TextData[{
 "For our graph coloring procedure, we will create a helper function that \
takes a ",
 StyleBox[ButtonBox["Graph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Graph"], "FunctionLink"],
 " object and returns the sorted list of vertices."
}], "Text",ExpressionUUID->"5a18d6c8-7668-42aa-aef1-9efd1f77f572"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sortVertices", "[", "G_Graph", "]"}], ":=", 
  RowBox[{"Sort", "[", 
   RowBox[{
    RowBox[{"VertexList", "[", "G", "]"}], ",", 
    RowBox[{
     RowBox[{
      RowBox[{"VertexDegree", "[", 
       RowBox[{"G", ",", "#1"}], "]"}], "\[GreaterEqual]", 
      RowBox[{"VertexDegree", "[", 
       RowBox[{"G", ",", "#2"}], "]"}]}], "&"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[250]:=",ExpressionUUID->"36086765-076c-4c0d-9183-ba0ce1605b34"],

Cell["\<\
In order for our algorithm to color the vertices of a graph, we need to \
decide on what colors to use. Here, we will use the first of the indexed \
color schemes.\
\>", "Text",ExpressionUUID->"f4c654e7-6e28-4e47-a096-92d87afe2e5a"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ColorData", "[", "1", "]"}]], "Input",
 CellLabel->
  "In[251]:=",ExpressionUUID->"4ea52fb9-a3a1-4346-96b6-2da6b3d3bad0"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{
   TagBox["ColorDataFunction",
    "SummaryHead"], "[", 
   DynamicModuleBox[{Typeset`open$$ = False, Typeset`embedState$$ = "Ready"}, 
    
    TemplateBox[{PaneSelectorBox[{False -> GridBox[{{
            PaneBox[
             ButtonBox[
              DynamicBox[
               FEPrivate`FrontEndResource[
               "FEBitmaps", "SquarePlusIconMedium"]], 
              ButtonFunction :> (Typeset`open$$ = True), Appearance -> None, 
              Evaluator -> Automatic, Method -> "Preemptive"], 
             Alignment -> {Center, Center}, ImageSize -> 
             Dynamic[{
               Automatic, 
                3.5 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                 Magnification])}]], 
            GridBox[{{
               RowBox[{
                 TagBox["\"Index: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["1", "SummaryItem"]}], 
               RowBox[{
                 TagBox["\"Colors: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\[Infinity]", "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Palette: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox[
                  StyleBox[
                   GraphicsBox[{{
                    RGBColor[0.24720000000000014`, 0.24, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{1, 0}, 
                    Offset[{-1, -1}, {2, 1}]]}, {
                    RGBColor[0.6, 0.24, 0.4428931686004542], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{2, 0}, 
                    Offset[{-1, -1}, {3, 1}]]}, {
                    RGBColor[0.6, 0.5470136627990908, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{3, 0}, 
                    Offset[{-1, -1}, {4, 1}]]}, {
                    RGBColor[0.24, 0.6, 0.33692049419863584`], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{4, 0}, 
                    Offset[{-1, -1}, {5, 1}]]}, {
                    RGBColor[0.24, 0.3531726744018182, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{5, 0}, 
                    Offset[{-1, -1}, {6, 1}]]}, {
                    RGBColor[0.6, 0.24, 0.5632658430022722], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{6, 0}, 
                    Offset[{-1, -1}, {7, 1}]]}, {
                    RGBColor[0.6, 0.42664098839727194`, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{7, 0}, 
                    Offset[{-1, -1}, {8, 1}]]}, {
                    RGBColor[0.2634521802031821, 0.6, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{8, 0}, 
                    Offset[{-1, -1}, {9, 1}]]}, {
                    RGBColor[0.24, 0.47354534880363613`, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{9, 0}, 
                    Offset[{-1, -1}, {10, 1}]]}, {
                    RGBColor[0.5163614825959097, 0.24, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{10, 0}, 
                    Offset[{-1, -1}, {11, 1}]]}, {
                    RGBColor[0.6, 0.3062683139954558, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{11, 0}, 
                    Offset[{-1, -1}, {12, 1}]]}, {
                    RGBColor[0.3838248546049982, 0.6, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{12, 0}, 
                    Offset[{-1, -1}, {13, 1}]]}, {
                    RGBColor[0.24, 0.5939180232054561, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{13, 0}, 
                    Offset[{-1, -1}, {14, 1}]]}, {
                    RGBColor[0.39598880819409377`, 0.24, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{14, 0}, 
                    Offset[{-1, -1}, {15, 1}]]}, {
                    RGBColor[0.6, 0.24, 0.2941043604063603], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{15, 0}, 
                    Offset[{-1, -1}, {16, 1}]]}}, {
                    ImageSize -> 65, BaselinePosition -> Bottom, 
                    PlotRangePadding -> None, AspectRatio -> NCache[
                    Rational[1, 10], 0.1]}], Selectable -> False, 
                   StripOnInput -> False], "SummaryItem"]}], 
               "\[SpanFromLeft]"}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}}, AutoDelete -> 
             False, GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{2}}, "Rows" -> {{Automatic}}},
              BaseStyle -> {
              ShowStringCharacters -> False, NumberMarks -> False, 
               PrintPrecision -> 3, ShowSyntaxStyles -> False}]}}, 
          GridBoxAlignment -> {"Rows" -> {{Top}}}, AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          BaselinePosition -> {1, 1}], True -> GridBox[{{
            PaneBox[
             ButtonBox[
              DynamicBox[
               FEPrivate`FrontEndResource[
               "FEBitmaps", "SquareMinusIconMedium"]], 
              ButtonFunction :> (Typeset`open$$ = False), Appearance -> None, 
              Evaluator -> Automatic, Method -> "Preemptive"], 
             Alignment -> {Center, Center}, ImageSize -> 
             Dynamic[{
               Automatic, 
                3.5 (CurrentValue["FontCapHeight"]/AbsoluteCurrentValue[
                 Magnification])}]], 
            GridBox[{{
               RowBox[{
                 TagBox["\"Index: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["1", "SummaryItem"]}], 
               RowBox[{
                 TagBox["\"Colors: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\[Infinity]", "SummaryItem"]}]}, {
               RowBox[{
                 TagBox["\"Palette: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox[
                  StyleBox[
                   GraphicsBox[{{
                    RGBColor[0.24720000000000014`, 0.24, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{1, 0}, 
                    Offset[{-1, -1}, {2, 1}]]}, {
                    RGBColor[0.6, 0.24, 0.4428931686004542], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{2, 0}, 
                    Offset[{-1, -1}, {3, 1}]]}, {
                    RGBColor[0.6, 0.5470136627990908, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{3, 0}, 
                    Offset[{-1, -1}, {4, 1}]]}, {
                    RGBColor[0.24, 0.6, 0.33692049419863584`], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{4, 0}, 
                    Offset[{-1, -1}, {5, 1}]]}, {
                    RGBColor[0.24, 0.3531726744018182, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{5, 0}, 
                    Offset[{-1, -1}, {6, 1}]]}, {
                    RGBColor[0.6, 0.24, 0.5632658430022722], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{6, 0}, 
                    Offset[{-1, -1}, {7, 1}]]}, {
                    RGBColor[0.6, 0.42664098839727194`, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{7, 0}, 
                    Offset[{-1, -1}, {8, 1}]]}, {
                    RGBColor[0.2634521802031821, 0.6, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{8, 0}, 
                    Offset[{-1, -1}, {9, 1}]]}, {
                    RGBColor[0.24, 0.47354534880363613`, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{9, 0}, 
                    Offset[{-1, -1}, {10, 1}]]}, {
                    RGBColor[0.5163614825959097, 0.24, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{10, 0}, 
                    Offset[{-1, -1}, {11, 1}]]}, {
                    RGBColor[0.6, 0.3062683139954558, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{11, 0}, 
                    Offset[{-1, -1}, {12, 1}]]}, {
                    RGBColor[0.3838248546049982, 0.6, 0.24], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{12, 0}, 
                    Offset[{-1, -1}, {13, 1}]]}, {
                    RGBColor[0.24, 0.5939180232054561, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{13, 0}, 
                    Offset[{-1, -1}, {14, 1}]]}, {
                    RGBColor[0.39598880819409377`, 0.24, 0.6], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{14, 0}, 
                    Offset[{-1, -1}, {15, 1}]]}, {
                    RGBColor[0.6, 0.24, 0.2941043604063603], 
                    EdgeForm[
                    Opacity[0.1]], 
                    RectangleBox[{15, 0}, 
                    Offset[{-1, -1}, {16, 1}]]}}, {
                    ImageSize -> 65, BaselinePosition -> Bottom, 
                    PlotRangePadding -> None, AspectRatio -> NCache[
                    Rational[1, 10], 0.1]}], Selectable -> False, 
                   StripOnInput -> False], "SummaryItem"]}], 
               "\[SpanFromLeft]"}, {
               RowBox[{
                 TagBox["\"Class: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"Indexed\"", "SummaryItem"]}], "\[SpanFromLeft]"}, {
              
               RowBox[{
                 TagBox["\"Alternate name: \"", "SummaryItemAnnotation"], 
                 "\[InvisibleSpace]", 
                 TagBox["\"GoldenRatio\"", "SummaryItem"]}], 
               "\[SpanFromLeft]"}}, 
             GridBoxAlignment -> {
              "Columns" -> {{Left}}, "Rows" -> {{Automatic}}}, AutoDelete -> 
             False, GridBoxItemSize -> {
              "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
             GridBoxSpacings -> {"Columns" -> {{2}}, "Rows" -> {{Automatic}}},
              BaseStyle -> {
              ShowStringCharacters -> False, NumberMarks -> False, 
               PrintPrecision -> 3, ShowSyntaxStyles -> False}]}}, 
          GridBoxAlignment -> {"Rows" -> {{Top}}}, AutoDelete -> False, 
          GridBoxItemSize -> {
           "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}, 
          BaselinePosition -> {1, 1}]}, 
       Dynamic[Typeset`open$$], ImageSize -> Automatic]},
     "SummaryPanel"],
    DynamicModuleValues:>{}], "]"}],
  ColorDataFunction[1, "Indexed", {1, 
    DirectedInfinity[1], 1}, (ToColor[
    Hue[
     N[
      FractionalPart[0.67 + (2/GoldenRatio) (# - 1)]], 0.6, 0.6], 
    RGBColor]& )[Floor[# - 1, 1] + 1]& ],
  Editable->False,
  SelectWithContents->True,
  Selectable->False]], "Output",
 CellLabel->
  "Out[251]=",ExpressionUUID->"45e680ca-b6cf-4bf4-a520-fb0a9ea16201"]
}, Open  ]],

Cell[TextData[{
 "The assertion that this ",
 StyleBox[ButtonBox["ColorDataFunction",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ColorDataFunction"], "FunctionLink"],
 " has infinitely many colors is true in the sense that it will produce a \
color for any positive integer, although there is, of course, a limit on how \
many of them a person will be able to distinguish."
}], "Text",ExpressionUUID->"7c5ab2eb-f138-4166-8b9e-cbbbefaf5603"],

Cell["We now implement the greedy coloring algorithm.", "Text",ExpressionUUID->"1bbbcaa3-25cf-415c-86d4-d5b69e8994b9"],

Cell[BoxData[
 RowBox[{
  RowBox[{"greedyColorer", "[", "G_Graph", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"H", "=", "G"}], ",", "V", ",", 
      RowBox[{"currentColor", "=", "0"}], ",", 
      RowBox[{"colorFunction", "=", 
       RowBox[{"ColorData", "[", "1", "]"}]}], ",", "excludeSet", ",", "i"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"V", "=", 
      RowBox[{"sortVertices", "[", "H", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"V", "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", 
       RowBox[{
        RowBox[{"currentColor", "++"}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"PropertyValue", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"H", ",", 
             RowBox[{"V", "[", 
              RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", "VertexStyle"}], 
          "]"}], "=", 
         RowBox[{"colorFunction", "[", "currentColor", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"excludeSet", "=", 
         RowBox[{"VertexList", "[", 
          RowBox[{"NeighborhoodGraph", "[", 
           RowBox[{"H", ",", 
            RowBox[{"V", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"V", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"V", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"i", "=", "1"}], ";", "\[IndentingNewLine]", 
        RowBox[{"While", "[", 
         RowBox[{
          RowBox[{"i", "\[LessEqual]", 
           RowBox[{"Length", "[", "V", "]"}]}], ",", "\[IndentingNewLine]", 
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{"!", 
             RowBox[{"MemberQ", "[", 
              RowBox[{"excludeSet", ",", 
               RowBox[{"V", "[", 
                RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], ",", 
            "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"PropertyValue", "[", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"H", ",", 
                  RowBox[{"V", "[", 
                   RowBox[{"[", "i", "]"}], "]"}]}], "}"}], ",", 
                "VertexStyle"}], "]"}], "=", 
              RowBox[{"colorFunction", "[", "currentColor", "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"excludeSet", "=", 
              RowBox[{"Union", "[", 
               RowBox[{"excludeSet", ",", 
                RowBox[{"VertexList", "[", 
                 RowBox[{"NeighborhoodGraph", "[", 
                  RowBox[{"H", ",", 
                   RowBox[{"V", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], "]"}]}], "]"}]}],
              ";", "\[IndentingNewLine]", 
             RowBox[{"V", "=", 
              RowBox[{"Delete", "[", 
               RowBox[{"V", ",", "i"}], "]"}]}]}], ",", "\[IndentingNewLine]", 
            RowBox[{"i", "++"}]}], "\[IndentingNewLine]", "]"}]}], 
         "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", "H"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[252]:=",ExpressionUUID->"3e20a829-05e9-49e3-b751-22b99f0bfcd4"],

Cell[TextData[{
 "Note that the list ",
 StyleBox["V", "Code"],
 ", which is initialized to the list of vertices, sorted in decreasing order \
of degree, is used to track which vertices still need to be assigned a color. \
When a vertex has been assigned a color, it is deleted from the list ",
 StyleBox["V", "Code"],
 " using ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 ". The ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " function removes from the list in the first argument the element at the \
position specified by the second argument."
}], "Text",ExpressionUUID->"c57d94b6-2d3e-4ea1-a6d7-3282378f933c"],

Cell[TextData[{
 "The ",
 StyleBox["excludeSet", "Code"],
 " variable is used to store all vertices which cannot be assigned the \
current color. Each time a vertex is assigned a color, it and all of its \
neighbors are added to the ",
 StyleBox["excludeSet", "Code"],
 ". As the function looks down the list of vertices that still need to have a \
color assigned, it checks to see if they are in this set."
}], "Text",ExpressionUUID->"f01159e6-c3bf-4267-89d3-4b3d78727052"],

Cell[TextData[{
 "The index ",
 StyleBox["i", "Code"],
 ", which controls the inner ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop, is incremented in the else clause of the ",
 StyleBox[ButtonBox["If",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/If"], "FunctionLink"],
 " statement that tests to see if a vertex can be assigned the color. If the \
vertex at index ",
 StyleBox["i", "Code"],
 " is assigned the color, then it is removed from the list ",
 StyleBox["V", "Code"],
 ", and thus the index ",
 StyleBox["i", "Code"],
 " refers to a different vertex (the vertex previously in position ",
 Cell[BoxData[
  FormBox[
   RowBox[{"i", "+", "1"}], TraditionalForm]],ExpressionUUID->
  "31d4ffa8-70e6-4048-980a-13109d0048b6"],
 ")."
}], "Text",ExpressionUUID->"72ed1039-3d6b-48e2-93ad-07774f1fec72"],

Cell["As an example, we solve Exercise 29 of Section 10.8.", "Text",ExpressionUUID->"e3c98c7e-500a-4d08-a595-b78e19d9cf48"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"exercise29", "=", 
  RowBox[{"Graph", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
      "\"\<e\>\"", ",", "\"\<f\>\"", ",", "\"\<g\>\"", ",", "\"\<h\>\"", ",", 
      "\"\<i\>\"", ",", "\"\<j\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<b\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<d\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<a\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<c\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<b\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<e\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<f\>\""}], ",", 
      RowBox[{"\"\<c\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
      RowBox[{"\"\<d\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<g\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<h\>\""}], ",", 
      RowBox[{"\"\<e\>\"", "\[Rule]", "\"\<i\>\""}], ",", 
      RowBox[{"\"\<f\>\"", "\[Rule]", "\"\<i\>\""}], ",", 
      RowBox[{"\"\<f\>\"", "\[Rule]", "\"\<j\>\""}], ",", 
      RowBox[{"\"\<h\>\"", "\[Rule]", "\"\<i\>\""}], ",", 
      RowBox[{"\"\<i\>\"", "\[Rule]", "\"\<j\>\""}]}], "}"}], ",", 
    RowBox[{"DirectedEdges", "\[Rule]", "False"}], ",", 
    RowBox[{"VertexCoordinates", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "2"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "1"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "0"}], "}"}]}], "}"}]}], ",", 
    RowBox[{"VertexLabels", "\[Rule]", "\"\<Name\>\""}], ",", 
    RowBox[{"VertexSize", "\[Rule]", "Small"}]}], "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[253]:=",ExpressionUUID->"eabed605-e01c-4ec5-a50e-dbdde6abff99"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {
      Null, {{1, 2}, {1, 4}, {1, 5}, {1, 8}, {2, 3}, {2, 5}, {2, 6}, {3, 5}, {
       3, 6}, {3, 8}, {4, 7}, {5, 7}, {5, 8}, {5, 9}, {6, 9}, {6, 10}, {8, 
       9}, {9, 10}}}, {
      VertexLabels -> {"Name"}, VertexSize -> {Small}, 
       VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {2, 1}, {
        0, 0}, {1, 0}, {2, 0}, {3, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {1., 2.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {0., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {1., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {1., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {2., 2.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {1., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {2., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {1., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {2., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {1., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {0., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {0., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {3., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 0.}, {2., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 0.}, {3., 0.}}, 0.05]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[
        0.7]}], {DiskBox[{0., 2.}, 0.05], 
         InsetBox["\<\"a\"\>", Offset[{2, 2}, {0.05, 2.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 2.}, 0.05], 
         InsetBox["\<\"b\"\>", Offset[{2, 2}, {1.05, 2.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 2.}, 0.05], 
         InsetBox["\<\"c\"\>", Offset[{2, 2}, {2.05, 2.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{0., 1.}, 0.05], 
         InsetBox["\<\"d\"\>", Offset[{2, 2}, {0.05, 1.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 1.}, 0.05], 
         InsetBox["\<\"e\"\>", Offset[{2, 2}, {1.05, 1.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 1.}, 0.05], 
         InsetBox["\<\"f\"\>", Offset[{2, 2}, {2.05, 1.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{0., 0.}, 0.05], 
         InsetBox["\<\"g\"\>", Offset[{2, 2}, {0.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{1., 0.}, 0.05], 
         InsetBox["\<\"h\"\>", Offset[{2, 2}, {1.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{2., 0.}, 0.05], 
         InsetBox["\<\"i\"\>", Offset[{2, 2}, {2.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {DiskBox[{3., 0.}, 0.05], 
         InsetBox["\<\"j\"\>", Offset[{2, 2}, {3.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[253]=",ExpressionUUID->"8ce3fc75-f6b1-43ba-a0e6-a187fa48e281"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"greedyColorer", "[", "exercise29", "]"}]], "Input",
 CellLabel->
  "In[262]:=",ExpressionUUID->"8ab82144-85d6-4325-a322-c41d038cbe8d"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{"a", "b", "c", "d", "e", "f", "g", "h", "i", "j"}, {
      Null, {{1, 2}, {1, 4}, {1, 5}, {1, 8}, {2, 3}, {2, 5}, {2, 6}, {3, 5}, {
       3, 6}, {3, 8}, {4, 7}, {5, 7}, {5, 8}, {5, 9}, {6, 9}, {6, 10}, {8, 
       9}, {9, 10}}}, {
      VertexLabels -> {"Name"}, VertexSize -> {Small}, 
       VertexStyle -> {
        "g" -> RGBColor[0.6, 0.24, 0.4428931686004542], "i" -> 
         RGBColor[0.6, 0.24, 0.4428931686004542], "c" -> 
         RGBColor[0.6, 0.24, 0.4428931686004542], "f" -> 
         RGBColor[0.24720000000000014`, 0.24, 0.6], "b" -> 
         RGBColor[0.6, 0.5470136627990908, 0.24], "e" -> 
         RGBColor[0.24720000000000014`, 0.24, 0.6], "j" -> 
         RGBColor[0.6, 0.5470136627990908, 0.24], "d" -> 
         RGBColor[0.24720000000000014`, 0.24, 0.6], "h" -> 
         RGBColor[0.6, 0.5470136627990908, 0.24], "a" -> 
         RGBColor[0.6, 0.24, 0.4428931686004542]}, 
       VertexCoordinates -> {{0, 2}, {1, 2}, {2, 2}, {0, 1}, {1, 1}, {2, 1}, {
        0, 0}, {1, 0}, {2, 0}, {3, 0}}}]]}, 
    TagBox[GraphicsGroupBox[{
       {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {1., 2.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {0., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {1., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 2.}, {1., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {2., 2.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {1., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 2.}, {2., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {1., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {2., 1.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 2.}, {1., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{0., 1.}, {0., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {0., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {1., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 1.}, {2., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {2., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 1.}, {3., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{1., 0.}, {2., 0.}}, 0.05]}, 
        {Arrowheads[0.], ArrowBox[{{2., 0.}, {3., 0.}}, 0.05]}}, 
       {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], {
         {RGBColor[0.6, 0.24, 0.4428931686004542], DiskBox[{0., 2.}, 0.05]}, 
         InsetBox["\<\"a\"\>", Offset[{2, 2}, {0.05, 2.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6, 0.5470136627990908, 0.24], DiskBox[{1., 2.}, 0.05]}, 
         InsetBox["\<\"b\"\>", Offset[{2, 2}, {1.05, 2.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6, 0.24, 0.4428931686004542], DiskBox[{2., 2.}, 0.05]}, 
         InsetBox["\<\"c\"\>", Offset[{2, 2}, {2.05, 2.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.24720000000000014`, 0.24, 0.6], DiskBox[{0., 1.}, 0.05]},
          InsetBox["\<\"d\"\>", Offset[{2, 2}, {0.05, 1.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.24720000000000014`, 0.24, 0.6], DiskBox[{1., 1.}, 0.05]},
          InsetBox["\<\"e\"\>", Offset[{2, 2}, {1.05, 1.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.24720000000000014`, 0.24, 0.6], DiskBox[{2., 1.}, 0.05]},
          InsetBox["\<\"f\"\>", Offset[{2, 2}, {2.05, 1.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6, 0.24, 0.4428931686004542], DiskBox[{0., 0.}, 0.05]}, 
         InsetBox["\<\"g\"\>", Offset[{2, 2}, {0.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6, 0.5470136627990908, 0.24], DiskBox[{1., 0.}, 0.05]}, 
         InsetBox["\<\"h\"\>", Offset[{2, 2}, {1.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6, 0.24, 0.4428931686004542], DiskBox[{2., 0.}, 0.05]}, 
         InsetBox["\<\"i\"\>", Offset[{2, 2}, {2.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}, {
         {RGBColor[0.6, 0.5470136627990908, 0.24], DiskBox[{3., 0.}, 0.05]}, 
         InsetBox["\<\"j\"\>", Offset[{2, 2}, {3.05, 0.05}], ImageScaled[{0, 0}],
          BaseStyle->"Graphics"]}}}],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[262]=",ExpressionUUID->"5cd1809d-d833-4c9c-bac7-173593d80cfd"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions",ExpressionUUID->"cb288fcc-7132-4aa8-b178-1230335363a2"],

Cell[CellGroupData[{

Cell["Computations and Explorations 1", "Subsection",
 CellTags->
  "SolutionsCE1",ExpressionUUID->"64f25afc-0aed-47e8-a139-c15c3085465c"],

Cell["Display all simple graphs with four vertices.", "Problem",ExpressionUUID->"f46125cf-6439-4b1b-9561-f66bd70461d7"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To solve this problem, we will generate all possible edge sets and then \
construct the graphs based on these edge sets. The possible edge sets are all \
of the subsets of the set of all possible edges, which we obtain from the \
complete graph on the vertices. We generalize the question and have our \
function create all the simple graphs on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "ea5cc948-3021-41c3-b4db-f7f94af1c195"],
 " vertices."
}], "Text",ExpressionUUID->"98a5630f-64b8-4055-befd-8a13b0258f3d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"allGraphs", "[", "n_Integer", "]"}], "/;", 
   RowBox[{"n", ">", "0"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"cg", ",", "v", ",", 
      RowBox[{"A", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"V", "=", 
       RowBox[{"Range", "[", "n", "]"}]}], ",", "powerE", ",", "vCoords", ",",
       "E"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"cg", "=", 
      RowBox[{"CompleteGraph", "[", "n", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"powerE", "=", 
      RowBox[{"Subsets", "[", 
       RowBox[{"EdgeList", "[", "cg", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vCoords", "=", 
      RowBox[{"GraphEmbedding", "[", "cg", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"A", ",", 
         RowBox[{"Graph", "[", 
          RowBox[{"V", ",", "E", ",", 
           RowBox[{"VertexCoordinates", "\[Rule]", "vCoords"}]}], "]"}]}], 
        "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"E", ",", "powerE"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", "A"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True,
 CellLabel->
  "In[254]:=",ExpressionUUID->"75e78af1-fa41-417b-a69e-2734f48c5fa5"],

Cell[TextData[{
 "Recall that the complete graph on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "e51cb4c6-0e97-4af4-a5e3-ccc781d63d02"],
 " vertices has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"n", ",", "2"}], ")"}], TraditionalForm]],ExpressionUUID->
  "c8a0534b-715f-433f-b72d-d3cb68d96f5e"],
 " edges, so there are ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", 
    RowBox[{"C", "(", 
     RowBox[{"n", ",", "2"}], ")"}]], TraditionalForm]],ExpressionUUID->
  "3f089a64-61ff-4afb-bc2b-6c8e7cb56f9e"],
 " graphs on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "0dafff6e-eb35-4020-ab97-941fdf78e39e"],
 " vertices. Thus, on 4 vertices, there are 64 graphs. For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", " ", "=", " ", "3"}], TraditionalForm]],ExpressionUUID->
  "e574b305-8321-48b5-99fa-40c7444dabb8"],
 ", there are only 8 graphs, which is more manageable."
}], "Text",ExpressionUUID->"8912dec1-3a22-4f9d-b2eb-5ba9ede804ce"],

Cell[TextData[{
 "We use the ",
 StyleBox[ButtonBox["Partition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Partition"], "FunctionLink"],
 " function to break the list of all graphs into triples and then apply ",
 StyleBox[ButtonBox["Grid",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Grid"], "FunctionLink"],
 " with the ",
 StyleBox[ButtonBox["Frame",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Frame"], "FunctionLink"],
 " option in order to display the results in a useful way. The second \
argument to ",
 StyleBox[ButtonBox["Partition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Partition"], "FunctionLink"],
 " is the size of the sublists (the rows) and the last specifies padding, \
which is needed to create sublists of equal size. The third argument is the \
offset, which equals the second in order to not repeat elements. The fourth \
argument allows for the last sublist to be padded."
}], "Text",ExpressionUUID->"ace56ef7-d8eb-4c4e-8443-de94bfe85655"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{
   RowBox[{"Partition", "[", 
    RowBox[{
     RowBox[{"allGraphs", "[", "3", "]"}], ",", "3", ",", "3", ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}], ",", "Null"}], "]"}], ",", 
   RowBox[{"Frame", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellLabel->
  "In[255]:=",ExpressionUUID->"8c66bca9-369f-444b-bfe1-5c28d872a483"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {{}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None,
      ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztlE1LAlEUhm+o9Tf8NbWIpDDQPiwGC3QR5kd+QNDOotW40MhFUNmqVpKR
i/5BELhI/QN+7NzoRKJv5xzuCEq0DxyYec593/fce+cOzJqRCB0eGYlw0HCv
xIzjUDgYdy9HYiQ5FpRSFbpv3IprUKkfFdBF5hScAhdGoxGq1SqSyTRM00S3
21X32qNaNPY4Q1ndp7Eo8urGNjLZa+yFUohE4xj0+7faplIktjjCUWqZmSSZ
SiNzXsTTWx33z+/weHfRbDbsSaiEZ3NXLI5kskVumZnENHOyzB2lLgpP8PoC
aLda9iRUisQWRzhKLc7p0+h02rJfXm6T0uVyeXIaVIvGu4tEE5y1T8MlcMCy
LDQbDV5MPWiNF2aNPJ1Tv36QOeaY4/9hS+BAr9dDo15nak35BS7UajXsG4f0
VwkIaaw95RMsYTj8xlEkitiZidLLh5DHpOvMJPplDeDf2Ufu9hWPb59CHpM+
Gx2Px8jnC1j3B3F6eUM8QL5wxbodtfdoDQYolUqIJ06ENLb3+Nfbq4UfPpRV
aw==\
\>"]], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{1, 2}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None,
      ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztlMtKQlEUhreY9Ro+TVFRGQVqVw4W6CAsNS8QNKtodBrYDSpMa1JNxCKh
hhFREThIe4HKmSNPF/RvrcU2MKJ54IFzvrX/9a19LoJ9RsQ/PWNEAj7D2RUy
Zv0BX9jZGQxRZLcppU7p3HUqrkGlvpyCDmo2oU3gQK1WQz6fRzQah2maKJfL
Kq17VEvGPXbI1XMa7RL3Do4gsbSFCX8Ms6EwPt7fUrpNpUTcYoVVGmnapAPR
WByJ5W0cXxaRzt2je8CLncMcLq6f1B7yV4/kUC0Z99hhN7G0zbPqx3amuSY3
3M/dYWXzBD0uN5LpMxzkbmm7dPaGHKol6xlwi8Muz9Bs83YOvL6+IDgXRv/w
GIbc48hms99fiGrJ+ofGyImw2xh3COywLAtPpRJenp/Vgc6olox62lO//kgt
tNDC/4NXYEelUkGpWGTqTHkEDhQKBUwa0/SvMi6kte4pt6ADn58fmAnOIbRo
InP2IOQ15dr5Vt+sKjyjk1hLnePo8lHIa8p/qvV6HcnkOlweHxZWd4lTSK5v
cN5QG89oVavIZDIIR+aFtG48419vr2xfO45eCQ==\
\>"]], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{1, 3}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None,
      ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]},
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{2, 3}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{2, 3}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{1, 2}, {1, 3}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.020399597244776385`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{1, 2}, {2, 3}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.020399597244776385`]}, 
             {Arrowheads[0.], ArrowBox[{2, 3}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None]},
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{1, 3}, {2, 3}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.020399597244776385`]}, 
             {Arrowheads[0.], ArrowBox[{2, 3}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3}, {Null, {{1, 2}, {1, 3}, {2, 3}}}, {
          VertexCoordinates -> {{-0.8660254037844384, -0.4999999999999994}, {
            0.8660254037844389, -0.5000000000000012}, {
            1.8369701987210297`*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.8660254037844384, -0.4999999999999994}, {
           0.8660254037844389, -0.5000000000000012}, {
           1.8369701987210297`*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.020399597244776385`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.020399597244776385`]}, 
             {Arrowheads[0.], ArrowBox[{2, 3}, 0.020399597244776385`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.020399597244776385], 
             DiskBox[2, 0.020399597244776385], 
             DiskBox[3, 0.020399597244776385]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], ""}
   },
   AutoDelete->False,
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellLabel->
  "Out[255]=",ExpressionUUID->"9612f1f9-3505-4a23-beb5-28e929e653e7"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Computations and Explorations 2", "Subsection",
 CellTags->
  "SolutionsCE2",ExpressionUUID->"8a238ba0-33e6-438e-9880-b9f82caf5bc8"],

Cell["\<\
Display a full set of nonisomorphic simple graphs with six vertices.\
\>", "Problem",ExpressionUUID->"8ff022e5-18a9-4e3f-8e32-6e9430a74522"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " The solution to this exercise is very similar to the previous question. \
The only difference is that, once the list of graphs is generated, we remove \
those that are isomorphic to others by applying ",
 StyleBox[ButtonBox["DeleteDuplicates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DeleteDuplicates"], "FunctionLink"],
 " with second argument ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 ". This uses ",
 StyleBox[ButtonBox["IsomorphicGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IsomorphicGraphQ"], "FunctionLink"],
 " to determine whether two elements are to be considered duplicates or not."
}], "Text",ExpressionUUID->"ca919fa7-5f65-49c0-84a4-62743f36372d"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nonIsoGraphs", "[", "n_Integer", "]"}], "/;", 
   RowBox[{"n", ">", "0"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"cg", ",", "v", ",", 
      RowBox[{"A", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"V", "=", 
       RowBox[{"Range", "[", "n", "]"}]}], ",", "powerE", ",", "vCoords", ",",
       "E", ",", "i", ",", "G", ",", "j"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"cg", "=", 
      RowBox[{"CompleteGraph", "[", "n", "]"}]}], ";", "\[IndentingNewLine]", 
     
     RowBox[{"powerE", "=", 
      RowBox[{"Subsets", "[", 
       RowBox[{"EdgeList", "[", "cg", "]"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"vCoords", "=", 
      RowBox[{"GraphEmbedding", "[", "cg", "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Do", "[", 
      RowBox[{
       RowBox[{"AppendTo", "[", 
        RowBox[{"A", ",", 
         RowBox[{"Graph", "[", 
          RowBox[{"V", ",", "E", ",", 
           RowBox[{"VertexCoordinates", "\[Rule]", "vCoords"}]}], "]"}]}], 
        "]"}], "\[IndentingNewLine]", ",", 
       RowBox[{"{", 
        RowBox[{"E", ",", "powerE"}], "}"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"DeleteDuplicates", "[", 
      RowBox[{"A", ",", "IsomorphicGraphQ"}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[256]:=",ExpressionUUID->"a27180cd-6386-4dc3-9fa6-21e70885f062"],

Cell["\<\
We apply this to five vertices, since six takes a bit more time to compute.\
\>", "Text",ExpressionUUID->"d5e574df-bf46-4d3f-aee1-89e15056e563"],

Cell[BoxData[
 RowBox[{
  RowBox[{"nonIso5", "=", 
   RowBox[{"nonIsoGraphs", "[", "5", "]"}]}], ";"}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[257]:=",ExpressionUUID->"cbbb1237-d81f-42e6-a769-7cef0d3511a2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "nonIso5", "]"}]], "Input",
 CellLabel->
  "In[258]:=",ExpressionUUID->"7acf9c26-231f-4e20-b65d-563fe4d90333"],

Cell[BoxData["34"], "Output",
 CellLabel->
  "Out[258]=",ExpressionUUID->"dc7ff650-0a7b-45e5-9985-a05b8f83fb4a"]
}, Open  ]],

Cell["\<\
We see that there are 34 nonisomorphic graphs on 5 vertices. Here are the \
first nine.\
\>", "Text",ExpressionUUID->"2d158777-c79c-4a55-953a-3957a96176c1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Grid", "[", 
  RowBox[{
   RowBox[{"Partition", "[", 
    RowBox[{
     RowBox[{"nonIso5", "[", 
      RowBox[{"[", 
       RowBox[{"1", ";;", "9"}], "]"}], "]"}], ",", "3"}], "]"}], ",", 
   RowBox[{"Frame", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellLabel->
  "In[259]:=",ExpressionUUID->"067835ea-8a74-4ce2-8d92-a416926deff3"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {{}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None,
      ImageCache->GraphicsData["CompressedBitmap", "\<\
eJztVd9rkmEUfofO/g0v1r9S0A/Y8EJHtu2baSqtzSkiUdsqcrB2sbZFFES6
igw2Yw0JvKiuhLrzboLi1bpY0BB/oFN7OufwfoIyuoxg+0Cfc57nOYf3O9/7
ft9VI+L3zhiRgMewX5w35vwBT9h+YXaeKMuQUipDv2274hgU6r8M6CKxD0YE
htHtdpHNZrG4tITk1hZq1aryaI1i4VhjD3l1nQYbMpkMLo26EHmwAYfbj0fx
ZXQ6HUPLFArFElvYSiUDTULhCO6uvMTOl308S32FwzWBn4eHZhMKhWKJLWyl
koEmj1dXMRmI4dXuN8TizzHl8aFarZpNKBSKJbawlUpG+qfx4+AAs3NhjDkn
4J6cRi6X602DYuFYYw95zWmcF7Cg2WyiVCri6OiX8mqOYuFI0z514gM5gzP4
h2ATsKBWq6Hdbkv2RnOcMw/Z92K36qTRaGB9fQPj7ikEb91GPp9Xr7VGsXCs
sYe8uk6DDalUCqPOaTxJfkJoYU1OZKVSSWqZQqFYYgtbqWSgyb2FRUQfPsU2
vQsSu9/lQJaKRbMJhUKxxBa2UslAk729j7g8dg3xzffwhe7DH5xBvV43m1Ao
FEtsYSuVWPun0W4f4x3dks8flFWVy+XeNCgWjjX2kNechjn7brcDfam3J3DW
/2GjnGYY1w+FDwO/wwk1p1wCwygUCrhxMyA7zktIudaUU+CcfASjsTsIRpdl
UzJyTrz29KzHrRYMOgArLz5g5/O+IOfED1qB30gkkrjiuC5bnJFz4k2rucYW
dU2n07IdGSk31/i3u1dDfwAgX5/y\
\>"]], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None,
      ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {1, 3}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None,
      ImageCache->GraphicsData["CompressedBitmap", "\<\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\
\>"]]},
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {3, 4}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{3, 4}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {1, 3}, {1, 4}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 4}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {1, 3}, {2, 3}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{2, 3}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None]},
    {
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {1, 3}, {2, 4}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{2, 4}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {1, 3}, {4, 5}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{4, 5}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None], 
     GraphicsBox[
      NamespaceBox["NetworkGraphics",
       DynamicModuleBox[{Typeset`graph = HoldComplete[
         Graph[{1, 2, 3, 4, 5}, {Null, {{1, 2}, {1, 3}, {1, 4}, {1, 5}}}, {
          VertexCoordinates -> {{-0.9510565162951538, 
            0.30901699437494834`}, {-0.5877852522924727, \
-0.8090169943749468}, {0.5877852522924737, -0.8090169943749481}, {
            0.9510565162951533, 
            0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}}]]}, 
        TagBox[
         GraphicsGroupBox[
          GraphicsComplexBox[{{-0.9510565162951538, 
           0.30901699437494834`}, {-0.5877852522924727, -0.8090169943749468}, \
{0.5877852522924737, -0.8090169943749481}, {0.9510565162951533, 
           0.30901699437494634`}, {-7.044813998280222*^-16, 1.}}, {
            {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
             {Arrowheads[0.], ArrowBox[{1, 2}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 3}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 4}, 0.021812234931106983`]}, 
             {Arrowheads[0.], ArrowBox[{1, 5}, 0.021812234931106983`]}}, 
            {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
             DiskBox[1, 0.021812234931106983], 
             DiskBox[2, 0.021812234931106983], 
             DiskBox[3, 0.021812234931106983], 
             DiskBox[4, 0.021812234931106983], 
             DiskBox[5, 0.021812234931106983]}}]],
         MouseAppearanceTag["NetworkGraphics"]],
        AllowKernelInitialization->False]],
      DefaultBaseStyle->{
       "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> 
        Hue[0.8, 1., 0.6]},
      FormatType->TraditionalForm,
      FrameTicks->None]}
   },
   AutoDelete->False,
   GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Grid"]], "Output",
 CellLabel->
  "Out[259]=",ExpressionUUID->"ca20aa90-6e10-40f5-b9d0-a06e127eebb6"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Computations and Explorations 9", "Subsection",
 CellTags->
  "SolutionsCE9",ExpressionUUID->"e88e5896-4992-4eee-b4c3-c3b982466197"],

Cell["\<\
Generate at random simple graphs with 10 vertices. Stop when you have \
constructed one with an Euler circuit. Display an Euler circuit in this graph.\
\>", "Problem",ExpressionUUID->"4690db05-14ed-4f87-8d3c-3573baed1567"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To generate the random graphs, we will use the ",
 StyleBox[ButtonBox["RandomGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomGraph"], "FunctionLink"],
 " function. By passing this function a list containing a number of vertices \
and a number of edges, it produces a graph with that number of vertices and \
edges. To display a random graph on 10 vertices and 15 edges, we enter the \
following:"
}], "Text",ExpressionUUID->"47697946-1bac-4c48-abb3-272270d45f38"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomGraph", "[", 
  RowBox[{"{", 
   RowBox[{"10", ",", "15"}], "}"}], "]"}]], "Input",
 CellLabel->
  "In[260]:=",ExpressionUUID->"dbbae5db-d405-44ce-9a0b-99bc4d5e8eda"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}, {Null, 
       SparseArray[
       Automatic, {10, 10}, 0, {
        1, {{0, 4, 8, 9, 13, 17, 18, 22, 23, 27, 30}, {{2}, {4}, {5}, {7}, {
          1}, {4}, {7}, {8}, {7}, {1}, {2}, {9}, {10}, {1}, {7}, {9}, {10}, {
          9}, {1}, {2}, {3}, {5}, {2}, {4}, {5}, {6}, {10}, {4}, {5}, {9}}}, 
         Pattern}]}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{1.4584234994874699`, 1.2115086935942498`}, {
       0.895897779341186, 0.7093506432265823}, {0.2613120737200121, 
       2.666068696689398}, {2.105575410469783, 0.7186974660890803}, {
       2.2115967194300605`, 1.5616582064493092`}, {4.351285059024006, 
       1.382292853880037}, {1.012872351383508, 1.8052801497715665`}, {0., 
       0.}, {3.2174338904118684`, 1.2486219196277746`}, {2.8248358212060496`, 
       0.8103450377366158}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{1, 4}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{1, 7}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{2, 7}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{2, 8}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{3, 7}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{4, 9}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{4, 10}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{5, 7}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{5, 9}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{5, 10}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{6, 9}, 0.03920456391551018]}, 
         {Arrowheads[0.], ArrowBox[{9, 10}, 0.03920456391551018]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.03920456391551018], DiskBox[2, 0.03920456391551018], 
         DiskBox[3, 0.03920456391551018], DiskBox[4, 0.03920456391551018], 
         DiskBox[5, 0.03920456391551018], DiskBox[6, 0.03920456391551018], 
         DiskBox[7, 0.03920456391551018], DiskBox[8, 0.03920456391551018], 
         DiskBox[9, 0.03920456391551018], DiskBox[10, 0.03920456391551018]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[260]=",ExpressionUUID->"337076b6-8672-428f-9456-d44f901a2a16"]
}, Open  ]],

Cell[TextData[{
 "Alternatively, the argument to ",
 StyleBox[ButtonBox["RandomGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomGraph"], "FunctionLink"],
 " can be a graph distribution, which is a probability distribution on the \
sample space consisting of graphs. There are six built-in graph \
distributions. We will use the ",
 StyleBox[ButtonBox["BernoulliGraphDistribution",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BernoulliGraphDistribution"], "FunctionLink"],
 ", which allows you to specify a number of vertices and a probability. The \
probability indicates the independent probability that an edge will appear \
between any two vertices. It requires two parameters, the number of vertices \
and the probability. The following constructs a random graph with 10 vertices \
and with each edge as likely to appear as not."
}], "Text",ExpressionUUID->"783a5267-adbf-414f-8b25-9fc7fdda5349"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomGraph", "[", 
  RowBox[{"BernoulliGraphDistribution", "[", 
   RowBox[{"10", ",", ".5"}], "]"}], "]"}]], "Input",
 CellLabel->
  "In[261]:=",ExpressionUUID->"a9135198-bd48-4de8-a39a-8865c44c13ee"],

Cell[BoxData[
 GraphicsBox[
  NamespaceBox["NetworkGraphics",
   DynamicModuleBox[{Typeset`graph = HoldComplete[
     Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}, {
      Null, SparseArray[
       Automatic, {10, 10}, 0, {
        1, {{0, 5, 8, 12, 15, 17, 19, 21, 21, 21, 21}, {{2}, {5}, {6}, {8}, {
          9}, {4}, {7}, {10}, {4}, {6}, {8}, {9}, {6}, {8}, {10}, {6}, {9}, {
          9}, {10}, {9}, {10}}}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
          1, 1, 1, 1, 1, 1}}]}]]}, 
    TagBox[
     GraphicsGroupBox[
      GraphicsComplexBox[{{0.65510852876723, 0.863563391484395}, {
       1.8203435495930906`, 1.5172250113331576`}, {0.5631045596452354, 
       1.5236972621055749`}, {1.1128843698367334`, 2.1499810006463798`}, {
       0.8331149791373379, 0.}, {1.2338723880145073`, 1.1726772545764401`}, {
       2.582298619898271, 1.0428802797845673`}, {0., 1.9316426327254388`}, {
       1.4441452342416552`, 0.540052688517476}, {2.1748124945041023`, 
       1.9753789487454965`}}, {
        {Hue[0.6, 0.7, 0.5], Opacity[0.7], 
         {Arrowheads[0.], ArrowBox[{1, 2}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{1, 5}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{1, 6}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{1, 8}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{1, 9}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{2, 4}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{2, 7}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{2, 10}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{3, 4}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{3, 6}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{3, 8}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{3, 9}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{4, 6}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{4, 8}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{4, 10}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{5, 6}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{5, 9}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{6, 9}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{6, 10}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{7, 9}, 0.02716722026238727]}, 
         {Arrowheads[0.], ArrowBox[{7, 10}, 0.02716722026238727]}}, 
        {Hue[0.6, 0.2, 0.8], EdgeForm[{GrayLevel[0], Opacity[0.7]}], 
         DiskBox[1, 0.02716722026238727], DiskBox[2, 0.02716722026238727], 
         DiskBox[3, 0.02716722026238727], DiskBox[4, 0.02716722026238727], 
         DiskBox[5, 0.02716722026238727], DiskBox[6, 0.02716722026238727], 
         DiskBox[7, 0.02716722026238727], DiskBox[8, 0.02716722026238727], 
         DiskBox[9, 0.02716722026238727], DiskBox[10, 0.02716722026238727]}}]],
     MouseAppearanceTag["NetworkGraphics"]],
    AllowKernelInitialization->False]],
  DefaultBaseStyle->{
   "NetworkGraphics", FrontEnd`GraphicsHighlightColor -> Hue[0.8, 1., 0.6]},
  FormatType->TraditionalForm,
  FrameTicks->None]], "Output",
 CellLabel->
  "Out[261]=",ExpressionUUID->"2e8c5d74-9e60-4e34-b929-80be3057c19b"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["RandomGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomGraph"], "FunctionLink"],
 " also accepts an optional second argument, a positive integer, causing it \
to produce a list of that many random graphs."
}], "Text",ExpressionUUID->"f3e22802-0bdb-4cba-b326-d1118970080c"],

Cell[TextData[{
 "Recall the description of the ",
 StyleBox[ButtonBox["EulerianGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EulerianGraphQ"], "FunctionLink"],
 " function. When this function is given a graph, it returns true or false \
depending on the existence of an Euler circuit."
}], "Text",ExpressionUUID->"3aa2ad82-0ebc-4ff7-b127-d8762675671e"],

Cell[TextData[{
 "To satisfy the requirements of this problem, we use ",
 StyleBox[ButtonBox["RandomGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomGraph"], "FunctionLink"],
 " to generate a random graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "d418b358-8b1d-47a6-a1aa-2106085a9863"],
 ". Then, we test it for an Euler circuit using ",
 StyleBox[ButtonBox["EulerianGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/EulerianGraphQ"], "FunctionLink"],
 ". As long as the randomly generated graph does not have an Euler circuit, \
we continue generating new random graphs. We display the path using ",
 StyleBox[ButtonBox["FindEulerianCycle",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FindEulerianCycle"], "FunctionLink"],
 " and the ",
 StyleBox["animatePath", "Code"],
 " function we created in Section 10.5."
}], "Text",ExpressionUUID->"99d0f0bb-e9f7-4d32-a5cf-bc7dcc1bd0b1"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"generateEulerian", "[", "n_Integer", "]"}], "/;", 
   RowBox[{"n", ">", "0"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"G", ",", "path"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"!", 
        RowBox[{"EulerianGraphQ", "[", "G", "]"}]}], ",", 
       RowBox[{"G", "=", 
        RowBox[{"RandomGraph", "[", 
         RowBox[{"BernoulliGraphDistribution", "[", 
          RowBox[{"n", ",", ".5"}], "]"}], "]"}]}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"animatePath", "[", 
      RowBox[{"G", ",", 
       RowBox[{"First", "[", 
        RowBox[{"FindEulerianCycle", "[", "G", "]"}], "]"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[262]:=",ExpressionUUID->"ca97698f-90b3-4a62-86ee-841bdcc5fc32"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"generateEulerian", "[", "10", "]"}]], "Input",
 CellLabel->
  "In[263]:=",ExpressionUUID->"f3bc9153-5223-45b8-ab2a-ee3c0221b33c"],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`i$14273$$ = 0, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{{
       Hold[$CellContext`i$14273$$], 0, "step"}, 0, 19, 1}}, 
    Typeset`size$$ = {360., {146., 150.}}, Typeset`update$$ = 0, 
    Typeset`initDone$$, Typeset`skipInitDone$$ = 
    True, $CellContext`i$14273$14274$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, "Variables" :> {$CellContext`i$14273$$ = 0}, 
      "ControllerVariables" :> {
        Hold[$CellContext`i$14273$$, $CellContext`i$14273$14274$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> HighlightGraph[
        Graph[{1, 2, 3, 4, 5, 6, 7, 8, 9, 10}, {
         Null, SparseArray[
          Automatic, {10, 10}, 0, {
           1, {{0, 6, 9, 13, 14, 16, 18, 18, 19, 19, 19}, {{2}, {5}, {7}, {
             8}, {9}, {10}, {4}, {5}, {6}, {7}, {8}, {9}, {10}, {7}, {6}, {
             7}, {8}, {9}, {9}}}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 
            1, 1, 1, 1, 1}}]}], 
        Part[{
          UndirectedEdge[1, 10], 
          UndirectedEdge[10, 3], 
          UndirectedEdge[3, 9], 
          UndirectedEdge[9, 8], 
          UndirectedEdge[8, 6], 
          UndirectedEdge[6, 9], 
          UndirectedEdge[9, 1], 
          UndirectedEdge[1, 8], 
          UndirectedEdge[8, 3], 
          UndirectedEdge[3, 7], 
          UndirectedEdge[7, 5], 
          UndirectedEdge[5, 6], 
          UndirectedEdge[6, 2], 
          UndirectedEdge[2, 5], 
          UndirectedEdge[5, 1], 
          UndirectedEdge[1, 7], 
          UndirectedEdge[7, 4], 
          UndirectedEdge[4, 2], 
          UndirectedEdge[2, 1]}, 
         Span[1, $CellContext`i$14273$$]]], 
      "Specifications" :> {{{$CellContext`i$14273$$, 0, "step"}, 0, 19, 1, 
         AnimationRunning -> False, AnimationRepetitions -> 1, 
         AppearanceElements -> {
          "ProgressSlider", "PlayPauseButton", "FasterSlowerButtons", 
           "DirectionButton"}}}, 
      "Options" :> {
       ControlType -> Animator, AppearanceElements -> None, DefaultBaseStyle -> 
        "Animate", DefaultLabelStyle -> "AnimateLabel", SynchronousUpdating -> 
        True, ShrinkingDelay -> 10.}, "DefaultOptions" :> {}],
     ImageSizeCache->{403., {182., 187.}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    SynchronousInitialization->True,
    UndoTrackedVariables:>{Typeset`show$$, Typeset`bookmarkMode$$},
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Animate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellLabel->
  "Out[263]=",ExpressionUUID->"f22e02d4-99d3-4471-9b84-fc74d226000f"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Computations and Explorations 13", "Subsection",
 CellTags->
  "SolutionsCE13",ExpressionUUID->"f3eda7c2-7e9d-4229-b39f-fddd85708cd4"],

Cell[TextData[{
 "Estimate the probability that a randomly generated simple graph with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "5fbd03f4-c99e-4ec0-a65f-c5ef58db9389"],
 " vertices is connected for each possible integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "22688678-a093-4a81-aedb-6c3861b9de09"],
 " not exceeding ten by generating a set of random simple graphs and \
determining whether each is connected."
}], "Problem",ExpressionUUID->"8eaa4f02-8fc8-457c-93b4-bf33cd166ecd"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " To solve this problem we will create a function that generates a number of \
random graphs of the specified size and counts the number that are connected. \
We use the ",
 StyleBox[ButtonBox["RandomGraph",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomGraph"], "FunctionLink"],
 " function to create the random graphs and the ",
 StyleBox[ButtonBox["ConnectedGraphQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConnectedGraphQ"], "FunctionLink"],
 " function to test them for connectivity."
}], "Text",ExpressionUUID->"7c93f37c-d834-46a1-9c17-5283109519c6"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"connectedProbability", "[", 
    RowBox[{"n_Integer", ",", "max_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"n", ">", "0"}], "&&", 
    RowBox[{"max", ">", "0"}]}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"G", ",", "i", ",", 
      RowBox[{"count", "=", "0"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", "max"}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"G", "=", 
         RowBox[{"RandomGraph", "[", 
          RowBox[{"BernoulliGraphDistribution", "[", 
           RowBox[{"n", ",", ".5"}], "]"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"ConnectedGraphQ", "[", "G", "]"}], ",", 
          RowBox[{"count", "++"}]}], "]"}]}]}], "\[IndentingNewLine]", "]"}], 
     ";", "\[IndentingNewLine]", 
     RowBox[{"count", "/", "max"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True,
 CellLabel->
  "In[264]:=",ExpressionUUID->"06acd114-54be-40b8-835c-c69bac74592f"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"connectedProbability", "[", 
    RowBox[{"i", ",", "100"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "10"}], "}"}]}], "]"}]], "Input",
 CellLabel->
  "In[265]:=",ExpressionUUID->"d94443d0-1d45-4d72-abb5-da63b3c926fb"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", 
   FractionBox["9", "20"], ",", 
   FractionBox["53", "100"], ",", 
   FractionBox["13", "25"], ",", 
   FractionBox["18", "25"], ",", 
   FractionBox["81", "100"], ",", 
   FractionBox["87", "100"], ",", 
   FractionBox["24", "25"], ",", 
   FractionBox["97", "100"], ",", 
   FractionBox["97", "100"]}], "}"}]], "Output",
 CellLabel->
  "Out[265]=",ExpressionUUID->"8ce3c9c5-5686-49ec-9e94-5941c3db78ee"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises",ExpressionUUID->"44306461-a2f0-47bc-88cd-2a26cd3eaa77"],

Cell[CellGroupData[{

Cell[TextData[{
 "Write a function in the Wolfram Language to find ",
 StyleBox["all",
  FontSlant->"Italic"],
 " maximal matchings for a bipartite graph."
}], "Item1Exercise",ExpressionUUID->"ba4f6430-4eda-4d3f-9e83-76adda5cf51c"],

Cell["\<\
Write functions in the Wolfram Language for calculating the adjacency and \
incidence matrices for a pseudograph.\
\>", "Item1Exercise",ExpressionUUID->"58c8bc2d-77fe-473b-b4a2-644c295ce261"],

Cell["\<\
Write a function in the Wolfram Language for creating a pseudograph from an \
incidence matrix.\
\>", "Item1Exercise",ExpressionUUID->"604a5d99-4463-41b3-bfcd-b3da9ba0d164"],

Cell["\<\
Write a function in the Wolfram Language to automate the creation of \
vertex-colored graphs to illustrate graph isomorphisms, as was done with the \
graphs from Example 11 at the end of Section 10.3 of this manual.\
\>", "Item1Exercise",ExpressionUUID->"8975beb5-ce52-41ca-95fd-f2d21be33e1b"],

Cell["\<\
Write a function in the Wolfram Language to find all of the minimal edge cuts \
of a given graph.\
\>", "Item1Exercise",ExpressionUUID->"943cc907-946e-4d6c-8ce0-36e56ef9c27d"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to construct all regular graphs of degree ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "a4b90197-b29b-4f74-9fcc-6feb0fa6a1d6"],
 ", given a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "6a8ecc6c-80bb-4862-b087-fe34f9b532e7"],
 ". (Regular graph is defined in the Exercises for Section 10.2.)"
}], "Item1Exercise",ExpressionUUID->"11230cba-f1d8-4c1a-a62b-eb1bfff04c8b"],

Cell[TextData[{
 "For vertices ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "e33ca3ac-2594-4205-a881-523ae3a0c853"],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "a726c9d7-10ab-49a9-9416-048308dd5f25"],
 " in a simple, undirected and connected graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "44eea94d-4406-4935-a3e1-a8a184077a5a"],
 ", the local vertex connectivity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Kappa]", "(", 
    RowBox[{"u", ",", "v"}], ")"}], TraditionalForm]],ExpressionUUID->
  "27390dec-c58f-4a70-a840-9ec139aa2e10"],
 " is defined to be the minimum number of vertices that must be removed so \
that there is no path between vertex ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "71b8a190-e65c-4749-91ec-18b1c3a3ffc2"],
 " and vertex ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "3018217d-a9ee-438a-8e03-9d74c72205f6"],
 ". Write a function in the Wolfram Language that calculates the local vertex \
connectivity of a graph and a pair of its vertices."
}], "Item1Exercise",ExpressionUUID->"6090ebc2-b451-4aee-a5a6-178437eedefd"],

Cell[TextData[{
 "For vertices ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "29413147-b6f8-4215-aeae-b9fa35d2cf4e"],
 " and ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "14270983-a567-4d4c-93e1-c4af4b2cb340"],
 " in a simple, undirected and connected graph ",
 Cell[BoxData[
  FormBox["G", TraditionalForm]],ExpressionUUID->
  "7ab9d0e0-d643-4a6c-a5e9-70559dcc3aae"],
 ", the local edge connectivity ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Lambda]", "(", 
    RowBox[{"u", ",", "v"}], ")"}], TraditionalForm]],ExpressionUUID->
  "60686f0d-424f-433c-9b39-c24c08b46814"],
 " is defined to be the minimum number of edges that must be removed so that \
there is no path between vertex ",
 Cell[BoxData[
  FormBox["u", TraditionalForm]],ExpressionUUID->
  "52f976e4-9e8c-4b9c-9d32-c5659193391b"],
 " and vertex ",
 Cell[BoxData[
  FormBox["v", TraditionalForm]],ExpressionUUID->
  "5578ca7f-70b0-40cf-939e-8774bc7f3337"],
 ". Write a function in the Wolfram Language that calculates the local edge \
connectivity of a graph and a pair of its vertices."
}], "Item1Exercise",ExpressionUUID->"04714530-c022-4425-b79a-b533d7d6a7d9"],

Cell["\<\
Write a function in the Wolfram Language that computes the thickness of a \
nonplanar simple graph (see the Exercises in Section 10.7 for a definition of \
thickness).\
\>", "Item1Exercise",ExpressionUUID->"15c4ee4d-be01-4a8f-9b1a-3149a0ae7e89"],

Cell["\<\
Write a function in the Wolfram Language for finding an orientation of a \
simple graph. (An orientation of a graph is defined in the Supplementary \
Exercises of Chapter 10.)\
\>", "Item1Exercise",ExpressionUUID->"eb58b103-6f19-4f89-861c-1e8a81786e7b"],

Cell["\<\
Write a function in the Wolfram Language for finding the bandwidth of a \
simple graph. (The bandwidth of a graph is defined in the Supplementary \
Exercises of Chapter 10.)\
\>", "Item1Exercise",ExpressionUUID->"aec3641c-1743-4b95-9822-35f3d5f4fcd9"],

Cell["\<\
Write a function in the Wolfram Language for finding the radius and diameter \
of a simple graph. (The radius and diameter of a graph are defined in the \
Supplementary Exercises of Chapter 10.)\
\>", "Item1Exercise",ExpressionUUID->"045ab48d-89a2-460d-95ea-3a6072055599"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to find the minimum number of queens controlling an ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "\[Times]", "n"}], TraditionalForm]],ExpressionUUID->
  "6093c8ad-3df7-4e2e-be53-1c591d2ab9c3"],
 " chessboard for as many values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "ef8bde70-2b6e-4359-b327-c2cb2c3819d5"],
 " as you can. Make use of the concept of a dominating set, described in the \
Supplementary Exercises of Chapter 10."
}], "Item1Exercise",ExpressionUUID->"3cc0e99a-3ad0-4656-8f29-3c5b4ac1d0f0"],

Cell[TextData[{
 "Write a function in the Wolfram Language for finding all self-complementary \
graphs on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "6a562dac-553f-4110-8b65-dffa8c0906a0"],
 " vertices. (A self-complementary graph is a graph which is isomorphic to \
its own complement.) Use your function to display the self-complementary \
graphs for as large a ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]],ExpressionUUID->
  "ed826f29-cb7c-451b-b25b-1ae45f10d283"],
 " as possible."
}], "Item1Exercise",ExpressionUUID->"7b2552cf-5c77-4c0c-92ba-8091b94e7488"],

Cell["\<\
Write a function in the Wolfram Language that finds a total coloring for a \
graph. A total coloring of a graph is an assignment of a color to each vertex \
and each edge such that: (a) no pair of adjacent vertices have the same \
color; (b) no two edges with a common endpoint have the same color; and (c) \
no edge has the same color as either of its endpoints.\
\>", "Item1Exercise",ExpressionUUID->"b3878cd3-da7f-45c5-b51f-9d892938562b"],

Cell[TextData[{
 "A sequence of positive integers is called ",
 StyleBox["graphic",
  FontSlant->"Italic"],
 " if there is a simple graph that has this sequence as its degree sequence. \
In this context, the degree sequence of a graph is the nondecreasing sequence \
made up of the degrees of the vertices of the graph. Develop a function in \
the Wolfram Language for determining whether a sequence of positive integers \
is graphic and, if it is, to construct a graph with this degree sequence."
}], "Item1Exercise",ExpressionUUID->"923f43c7-6651-43bf-a133-1a5616a37b2f"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{Automatic, 135}, {Automatic, 0}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section1d", "Section2a",
                "Section2b", "Section2c", "Section2d", "Section2e", 
               "Section2f", "Section2g", "Section2h", "Section2i", 
               "Section2j", "Section3a", "Section3b", "Section3c", 
               "Section3d", "Section4a", "Section4b", "Section5a", 
               "Section5b", "Section5c", "Section7a", "Section7b", 
               "SolutionsCE1", "SolutionsCE13", "SolutionsCE2", 
               "SolutionsCE9", "Section1", "Section2", "Section3", "Section4",
                "Section5", "Section6", "Section7", "Section8", "Solutions", 
               "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section5"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section6\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section6"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section7\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section7"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section8\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Section8"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Section7", "Section8", "Solutions", 
                "Exercises"}}] NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section1d", "Section2", "Section2a", "Section2b", "Section2c",
                "Section2d", "Section2e", "Section2f", "Section2g", 
               "Section2h", "Section2i", "Section2j", "Section3", "Section3a",
                "Section3b", "Section3c", "Section3d", "Section4", 
               "Section4a", "Section4b", "Section5", "Section5a", "Section5b",
                "Section5c", "Section6", "Section7", "Section7a", "Section7b",
                "Section8", "Solutions", "SolutionsCE1", "SolutionsCE13", 
               "SolutionsCE2", "SolutionsCE9"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification->1,
FrontEndVersion->"11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, \
2018)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], 
    CellMargins -> {{100, 0}, {20, 20}}, CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{12, 12}, {20, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 27, 
    FontWeight -> "Bold", Background -> RGBColor[0.09375, 0.570313, 0.816406],
     CounterBoxOptions -> {
     CounterFunction :> Identity}, $CellContext`DefaultReturnCreatedCellStyle -> 
    "Section"], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, AutoQuoteCharacters -> {}, 
    PasteAutoQuoteCharacters -> {}, LanguageCategory -> "Mathematica", 
    FormatType -> InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> True], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, CellFrameMargins -> {{6, 6}, {3, 5}},
        CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], FontFamily -> "Helvetica", 
       FontSize -> 11, FontWeight -> "Bold", FontSlant -> "Italic"]}, Open]], 
   
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Numbered"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Numbered"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, 
         TextJustification -> (FontWeight -> "Plain")], 
       CellMargins -> {{36, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 15000}, 
       ConversionRules :> {{"TeX" -> {"\n\\item ", ""}}, {
         "HTML" -> {"<li>", "</li>"}}}, 
       TaggingRules :> {
        "ExtraConversionRules" -> {
          "General" -> {"NestingLevel" -> 30}, 
           "TeX" -> {"CellGroupEnvironment" -> "enumerate"}, 
           "HTML" -> {"CellGroupEnvironment" -> "ol"}}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, TextJustification -> 1, CounterIncrements -> "Item1Numbered", 
       CounterAssignments -> {{"Item2Numbered", 0}, {"Item3Numbered", 0}}], 
      Cell[
       StyleData["Item1Numbered", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Numbered"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Plain"], 
       CellMargins -> {{24, 2}, {1, 2}}, CellFrameLabelMargins -> 3, 
       TextJustification -> 1]}, Open]], 
   Cell[
    StyleData["InitializationCell"], Background -> GrayLevel[1]], 
   Cell[
    StyleData["Item1"], TextJustification -> 1], 
   Cell[
    StyleData["Item1Printout"], TextJustification -> 1, MenuSortingValue -> 
    10000], 
   Cell[
    StyleData["Code"], FormatType -> InputForm]}, Visible -> False, 
  FrontEndVersion -> 
  "11.3 for Mac OS X x86 (32-bit, 64-bit Kernel) (March 5, 2018)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1184, 42, 125, 1, 55, "Section",ExpressionUUID->"7e1d154e-2090-47d3-8bc4-b8c9bc44c92c",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[21252, 580, 183, 4, 35, "Subsection",ExpressionUUID->"edbfee8f-9a39-43d0-bfd2-9599e02df672",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[64537, 1618, 117, 1, 32, "Subsection",ExpressionUUID->"ba8b3a85-5f88-4759-baa1-259a619565f1",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[97084, 2364, 118, 1, 32, "Subsection",ExpressionUUID->"1fec1932-2ece-43ae-b25a-86b027486969",
   CellTags->"Section1c"]},
 "Section1d"->{
  Cell[203248, 4798, 110, 1, 32, "Subsection",ExpressionUUID->"f6a00912-707c-4f69-aada-8b4fdc0c602f",
   CellTags->"Section1d"]},
 "Section2"->{
  Cell[220455, 5262, 147, 1, 55, "Section",ExpressionUUID->"1c5a4943-84e4-4bdc-9f5d-f582dc295695",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[221071, 5277, 107, 1, 32, "Subsection",ExpressionUUID->"80010f42-936c-49f0-9b1b-b46fe8d617f5",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[229992, 5509, 127, 1, 32, "Subsection",ExpressionUUID->"6beef6ed-30dc-4a7d-9036-5c578f4a00cc",
   CellTags->"Section2b"]},
 "Section2c"->{
  Cell[240061, 5739, 111, 1, 32, "Subsection",ExpressionUUID->"be3bc5df-3392-45e8-b163-ef14416610cd",
   CellTags->"Section2c"]},
 "Section2d"->{
  Cell[258919, 6176, 117, 1, 32, "Subsection",ExpressionUUID->"fa76909e-faac-40c7-9008-9f04388b75f0",
   CellTags->"Section2d"]},
 "Section2e"->{
  Cell[335291, 8045, 131, 1, 32, "Subsection",ExpressionUUID->"ea23e0b8-564d-4c56-9d82-6481f39dc5da",
   CellTags->"Section2e"]},
 "Section2f"->{
  Cell[350350, 8408, 132, 1, 32, "Subsection",ExpressionUUID->"e237cf2e-31dc-49d5-b7ea-299bee614096",
   CellTags->"Section2f"]},
 "Section2g"->{
  Cell[394976, 9435, 153, 2, 32, "Subsection",ExpressionUUID->"f15c3815-6b55-43f0-a8c0-a3e9d8e3122a",
   PageBreakWithin->False,
   CellTags->"Section2g"]},
 "Section2h"->{
  Cell[407198, 9741, 151, 2, 32, "Subsection",ExpressionUUID->"3e726959-9a34-4b84-b7d9-03ef53fda0a7",
   PageBreakWithin->False,
   CellTags->"Section2h"]},
 "Section2i"->{
  Cell[421421, 10077, 117, 1, 32, "Subsection",ExpressionUUID->"7d548293-46b5-428a-ad06-4d80e1c772be",
   CellTags->"Section2i"]},
 "Section2j"->{
  Cell[446781, 10685, 133, 1, 32, "Subsection",ExpressionUUID->"2cf5a3b0-20b2-4f16-b51f-bbc0c08de9a5",
   CellTags->"Section2j"]},
 "Section3"->{
  Cell[499015, 11831, 143, 1, 55, "Section",ExpressionUUID->"13a377f2-f887-483f-af22-7be649862fe0",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[499503, 11843, 119, 1, 32, "Subsection",ExpressionUUID->"bb97f9de-d74d-4260-bc5f-37c7990131ee",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[520987, 12382, 116, 1, 32, "Subsection",ExpressionUUID->"087546c0-0b7a-4730-b97e-d29b61b0d382",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[569489, 13557, 119, 1, 32, "Subsection",ExpressionUUID->"84121ca9-c025-445e-a2c2-ac551fbded43",
   CellTags->"Section3c"]},
 "Section3d"->{
  Cell[583371, 13947, 122, 1, 32, "Subsection",ExpressionUUID->"b384af5d-602f-4ad5-89a8-20097b4e5998",
   CellTags->"Section3d"]},
 "Section4"->{
  Cell[644677, 15507, 114, 1, 55, "Section",ExpressionUUID->"295b098c-70f8-44f6-94d5-da31bbedd722",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[663406, 16025, 124, 1, 32, "Subsection",ExpressionUUID->"92db923f-3f31-43f5-b0c7-852336f274ae",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[670487, 16199, 132, 1, 32, "Subsection",ExpressionUUID->"57f070f6-2b90-4bde-857a-9713e6787c03",
   CellTags->"Section4b"]},
 "Section5"->{
  Cell[682380, 16529, 126, 1, 55, "Section",ExpressionUUID->"883be708-f882-4ed6-9e72-06db73a03569",
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[683208, 16547, 115, 1, 32, "Subsection",ExpressionUUID->"16c74548-e882-4f80-aaf0-225dca35522e",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[715433, 17359, 112, 1, 32, "Subsection",ExpressionUUID->"f921781e-e24e-488e-b181-f9fd929304b2",
   CellTags->"Section5b"]},
 "Section5c"->{
  Cell[747451, 18140, 118, 1, 32, "Subsection",ExpressionUUID->"41acec3b-72f3-45a8-854c-131b2698b7a4",
   CellTags->"Section5c"]},
 "Section6"->{
  Cell[764333, 18547, 124, 1, 55, "Section",ExpressionUUID->"ba88e25b-5334-48e1-91be-9c29178d2dee",
   CellTags->"Section6"]},
 "Section7"->{
  Cell[797101, 19350, 115, 1, 55, "Section",ExpressionUUID->"d5724981-81d0-4aeb-a1fd-f528100e9e21",
   CellTags->"Section7"]},
 "Section7a"->{
  Cell[799213, 19413, 162, 2, 32, "Subsection",ExpressionUUID->"e2e7f02a-e5a9-43bd-8115-46e50f8fe350",
   CellTags->"Section7a"]},
 "Section7b"->{
  Cell[834308, 20296, 147, 1, 32, "Subsection",ExpressionUUID->"759c9208-1e0c-4d49-b19d-bf97a15ff664",
   CellTags->"Section7b"]},
 "Section8"->{
  Cell[852161, 20720, 116, 1, 55, "Section",ExpressionUUID->"d784a58b-4e77-458f-aa87-baf1c76553df",
   CellTags->"Section8"]},
 "Solutions"->{
  Cell[885719, 21459, 170, 3, 55, "Section",ExpressionUUID->"cb288fcc-7132-4aa8-b178-1230335363a2",
   CellTags->"Solutions"]},
 "SolutionsCE1"->{
  Cell[885914, 21466, 138, 2, 32, "Subsection",ExpressionUUID->"64f25afc-0aed-47e8-a139-c15c3085465c",
   CellTags->"SolutionsCE1"]},
 "SolutionsCE2"->{
  Cell[904661, 21887, 138, 2, 32, "Subsection",ExpressionUUID->"8a238ba0-33e6-438e-9880-b9f82caf5bc8",
   CellTags->"SolutionsCE2"]},
 "SolutionsCE9"->{
  Cell[930190, 22422, 138, 2, 32, "Subsection",ExpressionUUID->"e88e5896-4992-4eee-b4c3-c3b982466197",
   CellTags->"SolutionsCE9"]},
 "SolutionsCE13"->{
  Cell[944480, 22740, 140, 2, 32, "Subsection",ExpressionUUID->"f3eda7c2-7e9d-4229-b39f-fddd85708cd4",
   CellTags->"SolutionsCE13"]},
 "Exercises"->{
  Cell[947867, 22841, 107, 1, 55, "Section",ExpressionUUID->"44306461-a2f0-47bc-88cd-2a26cd3eaa77",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 975867, 23410},
 {"Section1a", 976005, 23413},
 {"Section1b", 976149, 23416},
 {"Section1c", 976294, 23419},
 {"Section1d", 976439, 23422},
 {"Section2", 976584, 23425},
 {"Section2a", 976726, 23428},
 {"Section2b", 976872, 23431},
 {"Section2c", 977018, 23434},
 {"Section2d", 977164, 23437},
 {"Section2e", 977310, 23440},
 {"Section2f", 977456, 23443},
 {"Section2g", 977602, 23446},
 {"Section2h", 977775, 23450},
 {"Section2i", 977948, 23454},
 {"Section2j", 978095, 23457},
 {"Section3", 978241, 23460},
 {"Section3a", 978384, 23463},
 {"Section3b", 978531, 23466},
 {"Section3c", 978678, 23469},
 {"Section3d", 978825, 23472},
 {"Section4", 978971, 23475},
 {"Section4a", 979114, 23478},
 {"Section4b", 979261, 23481},
 {"Section5", 979407, 23484},
 {"Section5a", 979550, 23487},
 {"Section5b", 979697, 23490},
 {"Section5c", 979844, 23493},
 {"Section6", 979990, 23496},
 {"Section7", 980132, 23499},
 {"Section7a", 980275, 23502},
 {"Section7b", 980422, 23505},
 {"Section8", 980568, 23508},
 {"Solutions", 980711, 23511},
 {"SolutionsCE1", 980858, 23514},
 {"SolutionsCE2", 981011, 23517},
 {"SolutionsCE9", 981164, 23520},
 {"SolutionsCE13", 981318, 23523},
 {"Exercises", 981469, 23526}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 144, 3, 108, "BookChapterTitle",ExpressionUUID->"1491c7f9-4f9a-4a9e-9c29-c96188c7b0da",
 CounterAssignments->{{"ChapterNumber", 10}}],
Cell[CellGroupData[{
Cell[749, 29, 86, 0, 55, "Section",ExpressionUUID->"66dd4a72-0321-46bb-8367-82f921295cf2"],
Cell[838, 31, 309, 6, 35, "Text",ExpressionUUID->"1e7096ef-be0f-4fd5-a7fb-d384f5c0101e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1184, 42, 125, 1, 55, "Section",ExpressionUUID->"7e1d154e-2090-47d3-8bc4-b8c9bc44c92c",
 CellTags->"Section1"],
Cell[1312, 45, 958, 22, 79, "Text",ExpressionUUID->"91ac5a55-9f9a-4232-aac0-270f33f8635c"],
Cell[CellGroupData[{
Cell[2295, 71, 163, 3, 31, "Input",ExpressionUUID->"d3b511e6-087a-4929-8d51-e510d059bcee"],
Cell[2461, 76, 301, 10, 42, "Output",ExpressionUUID->"64261b8f-8cd4-4421-bef7-9ffe99a70b64"]
}, Open  ]],
Cell[2777, 89, 1416, 36, 69, "Text",ExpressionUUID->"bd8d4154-8723-4815-bb0b-e23b48170a40"],
Cell[4196, 127, 687, 14, 55, "Text",ExpressionUUID->"eb36443a-e5fa-4028-8ee0-86f771de608a"],
Cell[4886, 143, 186, 3, 20, "Text",ExpressionUUID->"45faebc0-0474-45de-a7cb-9578a1eab641"],
Cell[CellGroupData[{
Cell[5097, 150, 465, 11, 31, "Input",ExpressionUUID->"9892c346-ced9-4372-a8d6-e23182e27da3",
 InitializationCell->True],
Cell[5565, 163, 1542, 30, 447, "Output",ExpressionUUID->"d3db0b7b-8f80-4def-9422-03f77a9801af"]
}, Open  ]],
Cell[7122, 196, 383, 10, 21, "Text",ExpressionUUID->"08e46637-b7d5-41cf-831d-ed8325c581b5"],
Cell[CellGroupData[{
Cell[7530, 210, 532, 12, 31, "Input",ExpressionUUID->"c9956d2e-2233-4057-9fd3-9706cdca4f45",
 InitializationCell->True],
Cell[8065, 224, 1302, 26, 178, "Output",ExpressionUUID->"0e42c780-3d52-4b85-b19c-64c4ff2e76ab"]
}, Open  ]],
Cell[9382, 253, 349, 5, 35, "Text",ExpressionUUID->"4617c9bf-0eec-47fd-8445-3255a62524f1"],
Cell[9734, 260, 679, 21, 39, "Text",ExpressionUUID->"131a2c41-3fd8-41a8-b8ab-bfd6df1bb55f"],
Cell[CellGroupData[{
Cell[10438, 285, 418, 9, 31, "Input",ExpressionUUID->"88d3a644-34d2-4668-bcd2-dd1fff27c14f"],
Cell[10859, 296, 1302, 26, 178, "Output",ExpressionUUID->"8f67ba99-a7e0-4fc0-bef9-493a4516519d"]
}, Open  ]],
Cell[12176, 325, 1171, 29, 92, "Text",ExpressionUUID->"03a29ee4-2656-493a-a9fa-5b7b153390ef"],
Cell[13350, 356, 395, 8, 36, "Text",ExpressionUUID->"655bbf22-af78-41a3-aa9c-eacb87db9877"],
Cell[CellGroupData[{
Cell[13770, 368, 491, 12, 31, "Input",ExpressionUUID->"04cc0727-f9a4-4dbc-81ad-5ec1dcae5a5f"],
Cell[14264, 382, 1429, 27, 447, "Output",ExpressionUUID->"ad012f9b-243a-478f-b57c-48affe5545d4"]
}, Open  ]],
Cell[15708, 412, 517, 14, 21, "Text",ExpressionUUID->"128bdea1-c1bb-4bcf-9a63-9df6f548f883"],
Cell[CellGroupData[{
Cell[16250, 430, 149, 2, 31, "Input",ExpressionUUID->"a89caa0d-de96-4233-954c-7cb5cfa5ea54"],
Cell[16402, 434, 206, 4, 31, "Output",ExpressionUUID->"c44a7b70-49cf-45dd-9d06-43c556264835"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16645, 443, 147, 2, 31, "Input",ExpressionUUID->"f235a7d0-f96b-48ae-a368-24e5202cd162"],
Cell[16795, 447, 397, 8, 31, "Output",ExpressionUUID->"aab3addd-63f0-4d3c-96c6-c4cc1a2e300a"]
}, Open  ]],
Cell[17207, 458, 983, 24, 53, "Text",ExpressionUUID->"f61db585-31e9-4f29-918e-185c0d028be8"],
Cell[CellGroupData[{
Cell[18215, 486, 178, 3, 31, "Input",ExpressionUUID->"4d79ae5c-7fee-4917-a989-d67b494537ca"],
Cell[18396, 491, 665, 18, 62, "Output",ExpressionUUID->"f67ec78c-57ea-4ddc-b89a-9a6178a2cd77"]
}, Open  ]],
Cell[19076, 512, 300, 7, 21, "Text",ExpressionUUID->"00519f47-789f-4070-b1c7-673859a63c79"],
Cell[CellGroupData[{
Cell[19401, 523, 155, 2, 31, "Input",ExpressionUUID->"bcdc60e9-c18a-4b56-bd6a-c6edfba23f93"],
Cell[19559, 527, 387, 7, 31, "Output",ExpressionUUID->"a016a09b-a2ab-467d-a756-0f92e767f465"]
}, Open  ]],
Cell[CellGroupData[{
Cell[19983, 539, 190, 4, 31, "Input",ExpressionUUID->"54e86a64-6b9f-4bd8-a39e-a541aa1c8b47"],
Cell[20176, 545, 658, 18, 62, "Output",ExpressionUUID->"adaff00c-bc64-4814-9941-839d58ddda74"]
}, Open  ]],
Cell[20849, 566, 378, 10, 21, "Text",ExpressionUUID->"cb73f302-ccf5-4530-bcea-16c63f4e5552"],
Cell[CellGroupData[{
Cell[21252, 580, 183, 4, 35, "Subsection",ExpressionUUID->"edbfee8f-9a39-43d0-bfd2-9599e02df672",
 CellTags->"Section1a"],
Cell[21438, 586, 526, 11, 37, "Text",ExpressionUUID->"0ed072f2-3a7c-44a0-9116-5edef16453b8"],
Cell[21967, 599, 542, 10, 54, "Text",ExpressionUUID->"e45996c5-bf35-41a0-bbca-a3ad1d8e8c77"],
Cell[CellGroupData[{
Cell[22534, 613, 529, 11, 51, "Input",ExpressionUUID->"0a09f387-735a-45a4-a6bb-2bfe5ae96556"],
Cell[23066, 626, 2326, 50, 183, "Output",ExpressionUUID->"b17f4419-2b8d-4552-b9f1-d49f7259c25b"]
}, Open  ]],
Cell[25407, 679, 505, 9, 51, "Text",ExpressionUUID->"d07f2506-e100-47dc-89ca-e5bd9d7051bd"],
Cell[CellGroupData[{
Cell[25937, 692, 659, 15, 51, "Input",ExpressionUUID->"5d11ba58-4fbf-4530-a6b2-b980edc260c8"],
Cell[26599, 709, 1988, 42, 178, "Output",ExpressionUUID->"c669670e-f87b-4ed9-9ce4-8e39d4249183"]
}, Open  ]],
Cell[28602, 754, 1156, 32, 70, "Text",ExpressionUUID->"38daf13f-184c-433b-a174-99c4fa8cdb2c"],
Cell[CellGroupData[{
Cell[29783, 790, 594, 13, 51, "Input",ExpressionUUID->"c2144ee1-9ce7-496e-85a8-0273128b1157"],
Cell[30380, 805, 2334, 50, 183, "Output",ExpressionUUID->"11ff6366-de35-4065-a94b-ae09ca1da85b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32751, 860, 796, 19, 51, "Input",ExpressionUUID->"d4685fdc-d947-497d-9201-1454f507da40"],
Cell[33550, 881, 2034, 43, 183, "Output",ExpressionUUID->"cd0b7eb6-c159-4464-a092-5b9e9d76c970"]
}, Open  ]],
Cell[35599, 927, 193, 3, 20, "Text",ExpressionUUID->"0a80cc13-6060-4389-890a-c7c78ce58dc8"],
Cell[CellGroupData[{
Cell[35817, 934, 770, 19, 51, "Input",ExpressionUUID->"4a4d5d27-9fbb-49a8-a9da-00602306e219"],
Cell[36590, 955, 2362, 50, 188, "Output",ExpressionUUID->"ef8f17d6-31f5-4082-bc16-f96207900bfe"]
}, Open  ]],
Cell[38967, 1008, 402, 8, 39, "Text",ExpressionUUID->"1f7d8997-696e-4eb4-a79c-7395e58b7bb8"],
Cell[CellGroupData[{
Cell[39394, 1020, 527, 11, 51, "Input",ExpressionUUID->"5033f8ba-04c0-48c1-b168-7225d9706985"],
Cell[39924, 1033, 2416, 46, 178, "Output",ExpressionUUID->"1b844450-ccb0-42fa-98a4-b1ff2d969695"]
}, Open  ]],
Cell[42355, 1082, 778, 18, 56, "Text",ExpressionUUID->"77cb1f23-515d-47a7-8aa4-7346588e0bf6"],
Cell[CellGroupData[{
Cell[43158, 1104, 658, 15, 51, "Input",ExpressionUUID->"c30db9d3-2516-43ff-aca9-be07cc001c7b"],
Cell[43819, 1121, 1856, 38, 178, "Output",ExpressionUUID->"aea6637e-a4da-4458-9f64-354e11a39e30"]
}, Open  ]],
Cell[CellGroupData[{
Cell[45712, 1164, 671, 16, 51, "Input",ExpressionUUID->"57c5bbe1-4c7b-4bf5-bf19-d799122f8d11"],
Cell[46386, 1182, 1856, 38, 178, "Output",ExpressionUUID->"a61f3acb-0e8b-4f18-945a-b2be40a309f6"]
}, Open  ]],
Cell[48257, 1223, 827, 20, 52, "Text",ExpressionUUID->"e74bd55a-3dd2-40cc-8729-cfbc077c60dd"],
Cell[CellGroupData[{
Cell[49109, 1247, 798, 19, 51, "Input",ExpressionUUID->"2bc7ba42-88e2-4334-b50b-2f14fc4e362a"],
Cell[49910, 1268, 2200, 46, 178, "Output",ExpressionUUID->"9650bbe3-bdb2-476f-836e-ebc6429b9bb9"]
}, Open  ]],
Cell[52125, 1317, 789, 20, 59, "Text",ExpressionUUID->"332c89da-7e34-4b44-8b84-ce598c224f54"],
Cell[CellGroupData[{
Cell[52939, 1341, 649, 14, 51, "Input",ExpressionUUID->"5545b2a7-1cbe-4d8f-9aa7-3ab979fc13d4"],
Cell[53591, 1357, 2081, 42, 189, "Output",ExpressionUUID->"9b5e51b4-3522-4c05-9a4e-992892c93689"]
}, Open  ]],
Cell[55687, 1402, 884, 21, 52, "Text",ExpressionUUID->"b545811d-046b-40a6-9cde-be940bb47a09"],
Cell[CellGroupData[{
Cell[56596, 1427, 639, 14, 51, "Input",ExpressionUUID->"ea30b30e-6cab-4fa9-9078-a2f7799a4d5c"],
Cell[57238, 1443, 1464, 30, 178, "Output",ExpressionUUID->"38e20e89-a218-4fe9-98be-75da7717c9ff"]
}, Open  ]],
Cell[58717, 1476, 1901, 47, 102, "Text",ExpressionUUID->"a2948603-61fe-4a57-9431-5daaa8468d4b"],
Cell[CellGroupData[{
Cell[60643, 1527, 997, 26, 70, "Input",ExpressionUUID->"04997973-1027-4639-8fb6-9c0088a19692"],
Cell[61643, 1555, 1697, 33, 380, "Output",ExpressionUUID->"c119a6df-26f0-40bd-be0b-65b417b3bc46"]
}, Open  ]],
Cell[63355, 1591, 1145, 22, 130, "Text",ExpressionUUID->"b1e93fde-a9e9-4889-929f-7ba385b41e36"]
}, Open  ]],
Cell[CellGroupData[{
Cell[64537, 1618, 117, 1, 32, "Subsection",ExpressionUUID->"ba8b3a85-5f88-4759-baa1-259a619565f1",
 CellTags->"Section1b"],
Cell[64657, 1621, 359, 7, 36, "Text",ExpressionUUID->"306ff252-4c10-412e-bb4b-4aa9c3fdf4cf"],
Cell[CellGroupData[{
Cell[65041, 1632, 1403, 34, 89, "Input",ExpressionUUID->"8f8bea1f-8afe-4673-9755-a2ade768a75b",
 InitializationCell->True],
Cell[66447, 1668, 3258, 60, 296, "Output",ExpressionUUID->"5c439ffb-e128-4c59-bd7b-9c951356f056"]
}, Open  ]],
Cell[69720, 1731, 293, 6, 21, "Text",ExpressionUUID->"0923ac16-f6fc-4454-885d-98c528516e4a"],
Cell[CellGroupData[{
Cell[70038, 1741, 1131, 27, 89, "Input",ExpressionUUID->"4075caa7-818b-473a-b14b-b4f5cfac54b7",
 InitializationCell->True],
Cell[71172, 1770, 2256, 45, 374, "Output",ExpressionUUID->"1f6c9bd5-c21f-4755-b03d-c9e397157ee6"]
}, Open  ]],
Cell[73443, 1818, 993, 23, 67, "Text",ExpressionUUID->"6ed9357f-6779-4e40-b9cf-84ff26975b59"],
Cell[74439, 1843, 289, 6, 21, "Text",ExpressionUUID->"94f96250-df0d-41f1-8a1a-909900f11189"],
Cell[CellGroupData[{
Cell[74753, 1853, 304, 7, 31, "Input",ExpressionUUID->"87877f43-721e-4565-acf5-19feb048ef54",
 InitializationCell->True],
Cell[75060, 1862, 3934, 87, 424, "Output",ExpressionUUID->"a25ce52c-96cc-4123-9dcf-0e76f3388233"]
}, Open  ]],
Cell[79009, 1952, 282, 4, 35, "Text",ExpressionUUID->"dff2e508-e3be-4eef-8ab6-f5cc831c9bf8"],
Cell[CellGroupData[{
Cell[79316, 1960, 453, 11, 31, "Input",ExpressionUUID->"d081112e-eecf-4317-81ff-3a7d70f461cf",
 InitializationCell->True],
Cell[79772, 1973, 4159, 86, 166, "Output",ExpressionUUID->"397b7933-5e50-41ce-92ab-2af8255c5cee"]
}, Open  ]],
Cell[83946, 2062, 689, 19, 40, "Text",ExpressionUUID->"131fe72e-4f04-48e7-9953-1a5d965282b8"],
Cell[CellGroupData[{
Cell[84660, 2085, 297, 6, 31, "Input",ExpressionUUID->"37c54b7f-3464-4df1-aacf-82b2900216dd",
 InitializationCell->True],
Cell[84960, 2093, 4001, 83, 235, "Output",ExpressionUUID->"75f1d671-b03d-43e4-bdfd-ac76e439b7cf"]
}, Open  ]],
Cell[88976, 2179, 178, 3, 20, "Text",ExpressionUUID->"dcecc9ec-2a92-4213-8fdf-b17a24e26d2f"],
Cell[CellGroupData[{
Cell[89179, 2186, 297, 6, 31, "Input",ExpressionUUID->"b1b09ca4-7091-4eb2-87ac-d608918d7772",
 InitializationCell->True],
Cell[89479, 2194, 3841, 80, 235, "Output",ExpressionUUID->"54fb9206-c67d-47c9-9b94-179c1a29ddbe"]
}, Open  ]],
Cell[93335, 2277, 172, 3, 20, "Text",ExpressionUUID->"ed2e50a7-7d14-418c-9dda-930db03ffc74"],
Cell[CellGroupData[{
Cell[93532, 2284, 254, 5, 31, "Input",ExpressionUUID->"030327b4-09aa-427a-b737-4be333ec58b0",
 InitializationCell->True],
Cell[93789, 2291, 3246, 67, 227, "Output",ExpressionUUID->"5ebe3d16-f7a4-4333-9cc1-853956272406"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[97084, 2364, 118, 1, 32, "Subsection",ExpressionUUID->"1fec1932-2ece-43ae-b25a-86b027486969",
 CellTags->"Section1c"],
Cell[97205, 2367, 494, 8, 50, "Text",ExpressionUUID->"afd2f856-2e5e-47e1-a9f0-1bb8a3b62325"],
Cell[97702, 2377, 254, 6, 21, "Text",ExpressionUUID->"c03231c1-9912-4d24-9dea-40c39c4367de"],
Cell[CellGroupData[{
Cell[97981, 2387, 154, 2, 31, "Input",ExpressionUUID->"9666764d-665e-470c-95b1-a7e759c6a538"],
Cell[98138, 2391, 1297, 33, 89, "Output",ExpressionUUID->"937cbe1b-5e98-47bb-909d-4127d249449e"]
}, Open  ]],
Cell[99450, 2427, 328, 5, 35, "Text",ExpressionUUID->"c7b02994-f421-44fb-850c-45e41db2df35"],
Cell[99781, 2434, 886, 24, 59, "Text",ExpressionUUID->"1286ee83-71c2-4084-af46-561105a36c05"],
Cell[CellGroupData[{
Cell[100692, 2462, 196, 3, 31, "Input",ExpressionUUID->"b56f5cfb-5f7a-4f03-a341-e5b83ad7c581"],
Cell[100891, 2467, 2539, 66, 203, "Output",ExpressionUUID->"4bf354e3-eb83-4e8b-b2cc-b7faf3afb4e5"]
}, Open  ]],
Cell[103445, 2536, 810, 13, 55, "Text",ExpressionUUID->"f75aabf0-d580-4347-bd37-5af958d12fbb"],
Cell[CellGroupData[{
Cell[104280, 2553, 223, 5, 31, "Input",ExpressionUUID->"171339b7-97de-41d1-a6f7-0dc0e9fe84da"],
Cell[104506, 2560, 780, 12, 89, "Output",ExpressionUUID->"deb21d51-1223-4a70-b968-2c564d5e7e5a"]
}, Open  ]],
Cell[105301, 2575, 330, 7, 21, "Text",ExpressionUUID->"0ba9e9ff-96a6-4b7b-8b9c-681060da4495"],
Cell[CellGroupData[{
Cell[105656, 2586, 383, 10, 31, "Input",ExpressionUUID->"0a8fcf84-1177-4154-9bf1-e5d3a536775f"],
Cell[106042, 2598, 1931, 35, 146, "Output",ExpressionUUID->"9189ab00-58b9-4ec7-b998-eadeb971bacd"]
}, Open  ]],
Cell[107988, 2636, 735, 15, 52, "Text",ExpressionUUID->"723b6e50-143d-4483-9e6f-a7037f22d241"],
Cell[CellGroupData[{
Cell[108748, 2655, 698, 19, 51, "Input",ExpressionUUID->"74ecd67e-a5f1-4703-a049-591cbc42ca16"],
Cell[109449, 2676, 1437, 35, 127, "Output",ExpressionUUID->"9a55580a-c740-41bd-8adc-bcfb51da1f11"]
}, Open  ]],
Cell[110901, 2714, 737, 15, 52, "Text",ExpressionUUID->"89bdfffc-4187-4da3-9016-64b4a36cad81"],
Cell[CellGroupData[{
Cell[111663, 2733, 1272, 34, 146, "Input",ExpressionUUID->"49a9ac19-67c1-499d-a95b-804d01ddcba8"],
Cell[112938, 2769, 63834, 1384, 298, "Output",ExpressionUUID->"1e16a6b7-09c7-4e87-b8d6-a969ee232e8a"]
}, Open  ]],
Cell[176787, 4156, 1020, 19, 71, "Text",ExpressionUUID->"b0e28b42-a635-4ef5-85cd-a5468de8bd28"],
Cell[CellGroupData[{
Cell[177832, 4179, 451, 13, 31, "Input",ExpressionUUID->"7947dba6-209f-4286-8211-344cc1a19067"],
Cell[178286, 4194, 1070, 29, 70, "Output",ExpressionUUID->"fea158fc-8744-46fb-b239-a35dc01a7a18"]
}, Open  ]],
Cell[179371, 4226, 505, 7, 35, "Text",ExpressionUUID->"522d75aa-baad-46df-880b-558b5e3e0124"],
Cell[179879, 4235, 662, 14, 55, "Text",ExpressionUUID->"5dfb8445-9f02-42ac-8b7d-1a5bb546bef7"],
Cell[CellGroupData[{
Cell[180566, 4253, 277, 6, 31, "Input",ExpressionUUID->"c4b353e0-6231-4630-acdc-2d29d15f7c2b"],
Cell[180846, 4261, 316, 7, 31, "Output",ExpressionUUID->"4c80c592-262e-4aec-bbd2-35471eced936"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181199, 4273, 282, 6, 31, "Input",ExpressionUUID->"d34f46ba-1b23-46bd-9b2e-7f9c522742f8"],
Cell[181484, 4281, 300, 7, 31, "Output",ExpressionUUID->"11e1d858-0294-41d3-9161-f80ad93b792b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181821, 4293, 279, 6, 31, "Input",ExpressionUUID->"68a4b272-fbba-42d2-9995-b485fd214415"],
Cell[182103, 4301, 334, 8, 31, "Output",ExpressionUUID->"bb6944a9-ab1e-4f93-8aa4-39737cdc97e9"]
}, Open  ]],
Cell[182452, 4312, 487, 9, 51, "Text",ExpressionUUID->"3bf252ad-07ec-46e0-8112-95b8da8144cc"],
Cell[CellGroupData[{
Cell[182964, 4325, 1269, 34, 108, "Input",ExpressionUUID->"25859ede-b01e-44ad-adfe-6862a9131198"],
Cell[184236, 4361, 18963, 431, 346, "Output",ExpressionUUID->"ad7ca542-edc1-45fe-9a84-0a2ab036a10a"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[203248, 4798, 110, 1, 32, "Subsection",ExpressionUUID->"f6a00912-707c-4f69-aada-8b4fdc0c602f",
 CellTags->"Section1d"],
Cell[203361, 4801, 1377, 35, 69, "Text",ExpressionUUID->"d2fde6e1-9fb7-4117-8656-0ca56df5b40b"],
Cell[204741, 4838, 963, 27, 53, "Text",ExpressionUUID->"84105a22-81a2-410b-ae71-7e7e8c6d4c37"],
Cell[205707, 4867, 781, 21, 40, "Text",ExpressionUUID->"530d5180-098c-4d82-bc39-4865b3ec76e5"],
Cell[206491, 4890, 124, 0, 20, "Text",ExpressionUUID->"6f08354c-478e-45d7-bf56-e0a3b69739a4"],
Cell[CellGroupData[{
Cell[206640, 4894, 396, 8, 31, "Input",ExpressionUUID->"d97fa745-225a-482c-891c-7ddd91e0c829"],
Cell[207039, 4904, 1276, 35, 166, "Output",ExpressionUUID->"1161b772-5bb0-44d5-9c05-f5ff7fcd8042"]
}, Open  ]],
Cell[208330, 4942, 907, 24, 53, "Text",ExpressionUUID->"6f63391b-0cea-4be0-becd-c194e19192a2"],
Cell[CellGroupData[{
Cell[209262, 4970, 470, 10, 31, "Input",ExpressionUUID->"124b6c89-ed3f-407b-90d5-e13c39d11c64"],
Cell[209735, 4982, 1354, 36, 166, "Output",ExpressionUUID->"c78482f3-6b11-4546-ab9d-c6ed9547ed38"]
}, Open  ]],
Cell[211104, 5021, 911, 21, 67, "Text",ExpressionUUID->"5adf40d0-d023-4300-92b9-a6adc6b1823b"],
Cell[CellGroupData[{
Cell[212040, 5046, 471, 10, 31, "Input",ExpressionUUID->"bf76a05e-750e-4f16-9881-25e945c19df1"],
Cell[212514, 5058, 1336, 31, 166, "Output",ExpressionUUID->"ec0e17fa-f307-4305-86ec-9d2a4102446c"]
}, Open  ]],
Cell[213865, 5092, 1793, 44, 99, "Text",ExpressionUUID->"611c7dbb-6361-404b-ab12-418469681e92"],
Cell[CellGroupData[{
Cell[215683, 5140, 994, 25, 51, "Input",ExpressionUUID->"f54ef256-0798-49b1-8172-521cace7964e"],
Cell[216680, 5167, 1112, 27, 375, "Output",ExpressionUUID->"f7065ca8-5c3b-4b82-b11e-553bd33ecc4c"]
}, Open  ]],
Cell[217807, 5197, 682, 12, 66, "Text",ExpressionUUID->"a711a089-ce17-4172-aa45-9c58b3e99b74"],
Cell[218492, 5211, 471, 10, 31, "Input",ExpressionUUID->"51b88f06-76d5-499e-ab13-49f042f6a0e7"],
Cell[218966, 5223, 1440, 33, 166, "Input",ExpressionUUID->"abc874c8-bf09-4ad4-ad71-d0a6ed0cbfaf"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[220455, 5262, 147, 1, 55, "Section",ExpressionUUID->"1c5a4943-84e4-4bdc-9f5d-f582dc295695",
 CellTags->"Section2"],
Cell[220605, 5265, 441, 8, 50, "Text",ExpressionUUID->"b827afbd-b785-4ffc-bdfd-daaa6bb09e22"],
Cell[CellGroupData[{
Cell[221071, 5277, 107, 1, 32, "Subsection",ExpressionUUID->"80010f42-936c-49f0-9b1b-b46fe8d617f5",
 CellTags->"Section2a"],
Cell[221181, 5280, 1005, 26, 56, "Text",ExpressionUUID->"a4e5b323-0924-4c19-97be-6ec03d438ffd"],
Cell[CellGroupData[{
Cell[222211, 5310, 114, 1, 31, "Input",ExpressionUUID->"9888b51f-1f9c-4776-90b1-53134b96e0be"],
Cell[222328, 5313, 3258, 60, 296, "Output",ExpressionUUID->"a8d074fa-cacb-43b4-99c1-18bfa7f54fb0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[225623, 5378, 183, 3, 31, "Input",ExpressionUUID->"ee7088a3-0122-41ad-a5a2-f8ddd090d5a0"],
Cell[225809, 5383, 107, 1, 31, "Output",ExpressionUUID->"de20d504-7af5-4888-ac8e-e7440fef6791"]
}, Open  ]],
Cell[CellGroupData[{
Cell[225953, 5389, 183, 3, 31, "Input",ExpressionUUID->"7d265ecc-326b-4779-8f7d-bff20709ac6f"],
Cell[226139, 5394, 107, 1, 31, "Output",ExpressionUUID->"74094687-0731-4228-b056-feb1b36202cb"]
}, Open  ]],
Cell[226261, 5398, 1032, 24, 67, "Text",ExpressionUUID->"d974460a-3af0-41c1-84e3-e7cd13e50543"],
Cell[CellGroupData[{
Cell[227318, 5426, 185, 3, 31, "Input",ExpressionUUID->"6556ff99-b52a-4332-97d5-3506de95f981"],
Cell[227506, 5431, 107, 1, 31, "Output",ExpressionUUID->"039149dd-6465-4f5e-ae6c-6bb239515f4b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[227650, 5437, 186, 3, 31, "Input",ExpressionUUID->"260b5094-3385-42ae-a985-c44959574935"],
Cell[227839, 5442, 107, 1, 31, "Output",ExpressionUUID->"10572701-1ebf-4c48-8093-526ec03f0eb2"]
}, Open  ]],
Cell[227961, 5446, 451, 9, 36, "Text",ExpressionUUID->"cf0161a8-d5a9-415e-9f3b-d1ff15eb8fd2"],
Cell[CellGroupData[{
Cell[228437, 5459, 150, 2, 31, "Input",ExpressionUUID->"127b7a10-8f57-480d-9e9e-a28a744b6d44"],
Cell[228590, 5463, 226, 4, 31, "Output",ExpressionUUID->"b1e22162-cfa7-4246-994d-1bbec43c3411"]
}, Open  ]],
Cell[CellGroupData[{
Cell[228853, 5472, 152, 2, 31, "Input",ExpressionUUID->"dddc3e77-e054-4e71-aa4d-cfd90849dbb6"],
Cell[229008, 5476, 182, 3, 31, "Output",ExpressionUUID->"b502cbf2-5394-4a18-bf00-88a499e65332"]
}, Open  ]],
Cell[CellGroupData[{
Cell[229227, 5484, 154, 2, 31, "Input",ExpressionUUID->"8c32150a-1ea1-47f2-88ba-18372100cd28"],
Cell[229384, 5488, 182, 3, 31, "Output",ExpressionUUID->"8ed727af-b786-4ee7-a2cc-3f24d21e0512"]
}, Open  ]],
Cell[CellGroupData[{
Cell[229603, 5496, 155, 2, 31, "Input",ExpressionUUID->"c01d315c-3b74-4915-bf4e-2b45c898bfb7"],
Cell[229761, 5500, 182, 3, 31, "Output",ExpressionUUID->"606d37e1-34cd-4e47-b7a6-4e0cb6fa6db5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[229992, 5509, 127, 1, 32, "Subsection",ExpressionUUID->"6beef6ed-30dc-4a7d-9036-5c578f4a00cc",
 CellTags->"Section2b"],
Cell[230122, 5512, 273, 4, 35, "Text",ExpressionUUID->"d276b7ff-cd86-4e74-8a45-45d4482d35bc"],
Cell[230398, 5518, 1197, 30, 66, "Text",ExpressionUUID->"8f0fb936-1de9-4acb-9ba8-d1782e590900"],
Cell[CellGroupData[{
Cell[231620, 5552, 145, 2, 31, "Input",ExpressionUUID->"f774a154-01c0-4493-af9c-5e8b7ad392f8"],
Cell[231768, 5556, 2041, 37, 358, "Output",ExpressionUUID->"bc718e80-a019-46fd-b981-7769209dcf47"]
}, Open  ]],
Cell[233824, 5596, 392, 11, 21, "Text",ExpressionUUID->"3067d43c-93d6-4b1b-a521-444a25e45a93"],
Cell[CellGroupData[{
Cell[234241, 5611, 142, 2, 31, "Input",ExpressionUUID->"f66d73b8-80bb-4b3e-9e24-8dd7d902a9ef"],
Cell[234386, 5615, 2343, 42, 369, "Output",ExpressionUUID->"ed3c2d2f-124b-4970-9197-c0fddfe0d5dc"]
}, Open  ]],
Cell[236744, 5660, 992, 26, 51, "Text",ExpressionUUID->"e2b5e3d8-89d3-4536-8d6b-1abc793afdbd"],
Cell[CellGroupData[{
Cell[237761, 5690, 142, 2, 31, "Input",ExpressionUUID->"72ba3ef9-5a37-4d56-8ec7-516d4d7a8344"],
Cell[237906, 5694, 2106, 39, 358, "Output",ExpressionUUID->"70b51613-0389-4c09-916f-5d56dfd3c000"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[240061, 5739, 111, 1, 32, "Subsection",ExpressionUUID->"be3bc5df-3392-45e8-b163-ef14416610cd",
 CellTags->"Section2c"],
Cell[240175, 5742, 1258, 32, 66, "Text",ExpressionUUID->"5acc6a22-cc88-4ea9-a86f-458dc95a45af"],
Cell[CellGroupData[{
Cell[241458, 5778, 146, 2, 31, "Input",ExpressionUUID->"cdecb6ac-3f12-487c-85d1-d8534e1ee3d4"],
Cell[241607, 5782, 2387, 43, 288, "Output",ExpressionUUID->"eb2db52c-5e46-4fd9-bbe5-29b4379a5667"]
}, Open  ]],
Cell[244009, 5828, 769, 20, 53, "Text",ExpressionUUID->"4212af1c-6e17-4ed9-b5f3-6fe08cc4ab77"],
Cell[CellGroupData[{
Cell[244803, 5852, 254, 5, 31, "Input",ExpressionUUID->"7e5831be-2ae7-4d73-b915-4268d07931f2"],
Cell[245060, 5859, 4793, 101, 293, "Output",ExpressionUUID->"d3a1a97d-c2fe-42a7-9163-33d4b66dda83"]
}, Open  ]],
Cell[249868, 5963, 556, 10, 55, "Text",ExpressionUUID->"f78e5175-78a5-4c8d-9e81-d5ca513af52a"],
Cell[250427, 5975, 953, 19, 67, "Text",ExpressionUUID->"e179eb9b-5885-4257-958c-ee9c9d947d1c"],
Cell[251383, 5996, 291, 7, 21, "Text",ExpressionUUID->"eba22082-6658-4847-b0d6-5af65d530cc5"],
Cell[CellGroupData[{
Cell[251699, 6007, 344, 9, 31, "Input",ExpressionUUID->"e8f98f8a-bc40-48be-97b7-7d1f6bba87b1"],
Cell[252046, 6018, 532, 11, 31, "Output",ExpressionUUID->"34ca43b9-bf11-475f-a975-8b86e0ed46d9"]
}, Open  ]],
Cell[252593, 6032, 852, 20, 53, "Text",ExpressionUUID->"2b49b231-4008-4c17-9024-abe47220b789"],
Cell[CellGroupData[{
Cell[253470, 6056, 646, 17, 70, "Input",ExpressionUUID->"33643236-a592-4494-8364-833dca074f7f"],
Cell[254119, 6075, 4751, 95, 294, "Output",ExpressionUUID->"f29c1b54-62be-43ff-91f7-503af7be7fbb"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[258919, 6176, 117, 1, 32, "Subsection",ExpressionUUID->"fa76909e-faac-40c7-9008-9f04388b75f0",
 CellTags->"Section2d"],
Cell[259039, 6179, 985, 24, 50, "Text",ExpressionUUID->"7e6066c1-6d1d-4fa4-b5a6-aaf9d60a39af"],
Cell[260027, 6205, 1874, 57, 54, "Text",ExpressionUUID->"22293f23-7cb2-4f3b-abde-4b3ecbded1c9"],
Cell[CellGroupData[{
Cell[261926, 6266, 192, 4, 31, "Input",ExpressionUUID->"534bd48a-b27a-4e04-a880-abd2fdbbc718"],
Cell[262121, 6272, 2033, 41, 296, "Output",ExpressionUUID->"4fb25386-10b3-4a6a-a9a7-acc4a82c8a1d"]
}, Open  ]],
Cell[264169, 6316, 637, 15, 51, "Text",ExpressionUUID->"4250a82b-5775-40eb-be0d-6b5ff8b61d13"],
Cell[CellGroupData[{
Cell[264831, 6335, 736, 22, 51, "Input",ExpressionUUID->"4095066a-70f6-47d3-9b35-edb20ce87511"],
Cell[265570, 6359, 4141, 88, 299, "Output",ExpressionUUID->"04667f52-069f-4820-b8cc-3469bda605e9"]
}, Open  ]],
Cell[269726, 6450, 541, 11, 35, "Text",ExpressionUUID->"a076328c-884d-49ca-a705-29dacf5de6dd"],
Cell[CellGroupData[{
Cell[270292, 6465, 202, 4, 31, "Input",ExpressionUUID->"41b17fac-cc62-42b0-b896-f62ad061401e"],
Cell[270497, 6471, 3526, 60, 271, "Output",ExpressionUUID->"5c70b557-9947-48e2-bcc7-afa995923266"]
}, Open  ]],
Cell[274038, 6534, 471, 11, 37, "Text",ExpressionUUID->"6e77f061-9231-4efd-82a6-87a291f3a9c3"],
Cell[CellGroupData[{
Cell[274534, 6549, 188, 3, 31, "Input",ExpressionUUID->"e75d0c96-346a-4fa9-b0e7-f9c7d4357f4d"],
Cell[274725, 6554, 110, 1, 31, "Output",ExpressionUUID->"2fc92568-ab90-4be5-9151-6a2d5d137f9b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[274872, 6560, 138, 2, 18, "Subsubsection",ExpressionUUID->"3030be26-376a-490f-8066-bc368840d52f"],
Cell[275013, 6564, 821, 16, 85, "Text",ExpressionUUID->"2305fe89-c5f8-4a78-9242-d51bd3317681"],
Cell[275837, 6582, 505, 11, 36, "Text",ExpressionUUID->"9ed6fe55-b130-48c1-9f44-da974ef2b21f"],
Cell[CellGroupData[{
Cell[276367, 6597, 498, 12, 51, "Input",ExpressionUUID->"b91cbd09-faf5-46ca-bb94-734d6be02815"],
Cell[276868, 6611, 1938, 38, 380, "Output",ExpressionUUID->"59d658f4-e23b-42cc-9d51-f05d9b7c5c1e"]
}, Open  ]],
Cell[278821, 6652, 553, 10, 51, "Text",ExpressionUUID->"9a38d6c8-7283-4d6f-b6e5-5a0bd2278841"],
Cell[CellGroupData[{
Cell[279399, 6666, 886, 21, 89, "Input",ExpressionUUID->"ff5d60e4-fd93-44f7-8e19-0f528f782fa3"],
Cell[280288, 6689, 2964, 57, 366, "Output",ExpressionUUID->"fa69acdb-745b-4176-8bcc-351ca0d99a6f"]
}, Open  ]],
Cell[283267, 6749, 757, 16, 52, "Text",ExpressionUUID->"b0882cd8-0c3c-4ebc-8751-ce53b29f065c"],
Cell[284027, 6767, 838, 17, 67, "Text",ExpressionUUID->"7ecd9b2a-cacd-4751-a818-29e88cfbfbce"],
Cell[284868, 6786, 824, 19, 52, "Text",ExpressionUUID->"7f34ac59-f386-405e-89fd-3fd655d4a7b8"],
Cell[CellGroupData[{
Cell[285717, 6809, 114, 1, 31, "Input",ExpressionUUID->"45073772-66f8-4519-a0f5-8cd9cd72f484"],
Cell[285834, 6812, 1303, 26, 178, "Output",ExpressionUUID->"937da144-75ea-41b9-8e37-0e8308cb88b3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[287174, 6843, 179, 3, 31, "Input",ExpressionUUID->"e2b00adb-ef54-4b45-815a-8a87e905ea78"],
Cell[287356, 6848, 666, 18, 62, "Output",ExpressionUUID->"1c75f7bf-61e8-4800-b4a5-cb3486e086a5"]
}, Open  ]],
Cell[288037, 6869, 444, 10, 37, "Text",ExpressionUUID->"d8bc94ef-f2df-4257-80df-8cb448e06b3c"],
Cell[288484, 6881, 149, 2, 20, "Text",ExpressionUUID->"52bddbfe-d991-4483-9e81-c40bccdbac72"],
Cell[CellGroupData[{
Cell[288658, 6887, 708, 17, 51, "Input",ExpressionUUID->"e08ca07a-1de1-4957-b2be-9c5aa20e0632",
 InitializationCell->True],
Cell[289369, 6906, 2430, 48, 198, "Output",ExpressionUUID->"ad6a9842-de2f-468c-afac-1607cb135e5c"]
}, Open  ]],
Cell[291814, 6957, 979, 28, 62, "Text",ExpressionUUID->"e434a226-1b04-4ec1-89cc-e209c6326fc4"],
Cell[CellGroupData[{
Cell[292818, 6989, 155, 2, 31, "Input",ExpressionUUID->"113c7974-4c2a-44a0-8818-d9a4b43f5bb7"],
Cell[292976, 6993, 426, 9, 31, "Output",ExpressionUUID->"8075834c-038c-425e-b6d8-1ff8dd0f9681"]
}, Open  ]],
Cell[CellGroupData[{
Cell[293439, 7007, 527, 13, 51, "Input",ExpressionUUID->"7a6dc0ec-08a9-4ab3-8dd1-f2ee8d552d9f"],
Cell[293969, 7022, 278, 6, 31, "Output",ExpressionUUID->"6bc02421-a0f4-409a-af05-311545abbc89"]
}, Open  ]],
Cell[294262, 7031, 845, 13, 81, "Text",ExpressionUUID->"12f75f7e-fecd-4a93-abf4-d317e5b0f6d8"],
Cell[CellGroupData[{
Cell[295132, 7048, 580, 15, 71, "Input",ExpressionUUID->"bdefbebd-2802-45d4-ab3d-7b2aa66c5a79"],
Cell[295715, 7065, 162, 3, 31, "Output",ExpressionUUID->"822d18fb-12b8-488e-a528-d900805875c5"]
}, Open  ]],
Cell[295892, 7071, 752, 22, 43, "Text",ExpressionUUID->"2cd8830b-0ceb-458c-a9a9-967ec4797ec2"],
Cell[296647, 7095, 323, 7, 36, "Text",ExpressionUUID->"82c0bfbe-1fa1-4058-9c5b-ab88b583f41c"],
Cell[296973, 7104, 741, 20, 90, "Input",ExpressionUUID->"832b91b6-0cde-4cbe-989f-118ee76b0f8e",
 InitializationCell->True],
Cell[297717, 7126, 415, 8, 36, "Text",ExpressionUUID->"4e8de1c9-1373-4156-a5be-bd45b5065240"],
Cell[CellGroupData[{
Cell[298157, 7138, 381, 10, 16, "Item1Numbered",ExpressionUUID->"e1c4ce36-664c-4dde-ac0d-1209987593f1"],
Cell[298541, 7150, 413, 12, 16, "Item1Numbered",ExpressionUUID->"2a179c87-92f8-4b4f-b509-56b3af8cf293"],
Cell[298957, 7164, 746, 20, 29, "Item1Numbered",ExpressionUUID->"0c4eda7d-9c84-44c7-a4f3-084aef3f16c6"],
Cell[299706, 7186, 398, 10, 16, "Item1Numbered",ExpressionUUID->"f51ac8a3-8fd7-4bf4-93a4-b3f3f3c5e33f"],
Cell[300107, 7198, 863, 22, 42, "Item1Numbered",ExpressionUUID->"4b9c40ab-ea7b-44a7-91d6-5c95534b08bf"]
}, Open  ]],
Cell[300985, 7223, 664, 20, 43, "Text",ExpressionUUID->"01239376-5c19-4d9d-8ccf-64cb09b0e04b"],
Cell[301652, 7245, 4728, 123, 431, "Input",ExpressionUUID->"7e99a805-99d3-457c-b35f-2de49fd6c396"],
Cell[306383, 7370, 339, 7, 20, "Text",ExpressionUUID->"ccfb0a35-6e92-4d56-8f5c-f46a66436046"],
Cell[CellGroupData[{
Cell[306747, 7381, 1844, 43, 108, "Input",ExpressionUUID->"1c193ff8-b12b-4e18-8e9c-12d83dc544c8",
 InitializationCell->True],
Cell[308594, 7426, 2726, 56, 213, "Output",ExpressionUUID->"cd9f1585-6843-46e4-a0e3-0f90d7f26eb6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311357, 7487, 152, 2, 31, "Input",ExpressionUUID->"633ca587-c081-49f2-8c38-2a08d0d3d76f"],
Cell[311512, 7491, 111, 1, 31, "Output",ExpressionUUID->"133647e3-3e1a-4d16-9ccb-001add8b101b"]
}, Open  ]],
Cell[311638, 7495, 1096, 26, 67, "Text",ExpressionUUID->"c3321a87-a745-4ecf-9de5-b1bb2fa3ba5b"],
Cell[CellGroupData[{
Cell[312759, 7525, 1386, 28, 108, "Input",ExpressionUUID->"8a56b41a-5452-4f66-b611-9b45a20c8105",
 InitializationCell->True],
Cell[314148, 7555, 4142, 84, 366, "Output",ExpressionUUID->"abb7b6d0-81e6-4739-9666-463c109ecd35"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318327, 7644, 152, 2, 31, "Input",ExpressionUUID->"8b79b2d3-f626-43c8-92fa-eaa207918d29"],
Cell[318482, 7648, 4843, 101, 366, "Output",ExpressionUUID->"3782a067-4a79-4ea0-b0f3-10dff45520e0"]
}, Open  ]],
Cell[323340, 7752, 680, 15, 52, "Text",ExpressionUUID->"3fb073bf-817d-44e1-9acd-39dceba67b1c"],
Cell[324023, 7769, 1112, 28, 75, "Text",ExpressionUUID->"a10c8912-7be5-46cc-ab27-64d825c1bf57"],
Cell[325138, 7799, 4801, 125, 431, "Input",ExpressionUUID->"962bfc11-e098-4549-a16b-b6dc27286eea",
 InitializationCell->True],
Cell[329942, 7926, 123, 0, 20, "Text",ExpressionUUID->"3d37175c-6a36-4d99-be62-63140158a5e2"],
Cell[CellGroupData[{
Cell[330090, 7930, 218, 4, 31, "Input",ExpressionUUID->"ed6df31c-9bde-473e-a808-c15c905d24c3"],
Cell[330311, 7936, 4919, 102, 366, "Output",ExpressionUUID->"221f5285-1afa-4b01-862d-b6f0b5bac761"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[335291, 8045, 131, 1, 32, "Subsection",ExpressionUUID->"ea23e0b8-564d-4c56-9d82-6481f39dc5da",
 CellTags->"Section2e"],
Cell[335425, 8048, 386, 7, 35, "Text",ExpressionUUID->"cb77e345-0a68-4984-83a7-6b7cb2553f1f"],
Cell[CellGroupData[{
Cell[335836, 8059, 730, 14, 51, "Input",ExpressionUUID->"26371fa4-e45a-4a41-8870-d38ead04b79c",
 InitializationCell->True],
Cell[336569, 8075, 658, 12, 31, "Output",ExpressionUUID->"75a0d48c-95e6-4ed8-989e-2626e99c297f"]
}, Open  ]],
Cell[337242, 8090, 669, 14, 52, "Text",ExpressionUUID->"4192d0d7-7f36-48f6-94e9-6eafd9b13ddf"],
Cell[CellGroupData[{
Cell[337936, 8108, 366, 8, 31, "Input",ExpressionUUID->"589ba845-5ffb-4805-9d8d-29eb66b0eaf0",
 InitializationCell->True],
Cell[338305, 8118, 294, 5, 31, "Output",ExpressionUUID->"64854585-9c54-404f-ad7e-30dea681408d"]
}, Open  ]],
Cell[338614, 8126, 700, 18, 37, "Text",ExpressionUUID->"316bf266-3f33-43d2-8bd9-06145dec687a"],
Cell[CellGroupData[{
Cell[339339, 8148, 569, 17, 31, "Input",ExpressionUUID->"25d21c98-65da-41da-b557-1023cd0234dc",
 InitializationCell->True],
Cell[339911, 8167, 604, 19, 31, "Output",ExpressionUUID->"76150600-37db-412c-a006-ae5806c7f9ae"]
}, Open  ]],
Cell[340530, 8189, 95, 0, 20, "Text",ExpressionUUID->"7d9c328b-7478-4635-92df-d7cbebefab9f"],
Cell[CellGroupData[{
Cell[340650, 8193, 516, 11, 51, "Input",ExpressionUUID->"8e306cb2-588d-48c5-afc5-84eaf2a83b42",
 InitializationCell->True],
Cell[341169, 8206, 3238, 62, 146, "Output",ExpressionUUID->"465b4121-9b47-4307-bea9-7633e3b6dec1"]
}, Open  ]],
Cell[344422, 8271, 479, 10, 36, "Text",ExpressionUUID->"e09b8db1-6188-4de2-87be-54d364ce1b26"],
Cell[CellGroupData[{
Cell[344926, 8285, 228, 4, 31, "Input",ExpressionUUID->"f8f0a348-55c1-4f5e-9c12-b9b385ef422d",
 InitializationCell->True],
Cell[345157, 8291, 409, 8, 31, "Output",ExpressionUUID->"8c442b01-2468-46fc-8018-f10ea7b02db3"]
}, Open  ]],
Cell[345581, 8302, 291, 6, 35, "Text",ExpressionUUID->"e9e55cb3-1e45-4011-bb11-b8917da180cd"],
Cell[345875, 8310, 629, 13, 52, "Text",ExpressionUUID->"d2cd514e-95c2-4a92-8e09-0940ca301932"],
Cell[CellGroupData[{
Cell[346529, 8327, 193, 3, 31, "Input",ExpressionUUID->"e6630359-f588-4e47-bf83-ed6a50503a58"],
Cell[346725, 8332, 3576, 70, 146, "Output",ExpressionUUID->"a54e86f4-3191-4f4d-b561-24126e251277"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[350350, 8408, 132, 1, 32, "Subsection",ExpressionUUID->"e237cf2e-31dc-49d5-b7ea-299bee614096",
 CellTags->"Section2f"],
Cell[350485, 8411, 658, 16, 37, "Text",ExpressionUUID->"f8b3b7e7-12e2-42e4-bd91-a667d63d9e6b"],
Cell[351146, 8429, 1594, 43, 50, "Text",ExpressionUUID->"b6bb0be4-7769-48fe-b87a-57e29bd22d26"],
Cell[CellGroupData[{
Cell[352765, 8476, 286, 6, 31, "Input",ExpressionUUID->"feef7aca-f227-4973-b5a7-e1e6822ed40d",
 InitializationCell->True],
Cell[353054, 8484, 3280, 71, 431, "Output",ExpressionUUID->"a5c9876c-1ad6-46aa-b319-b4e8a50028bc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[356371, 8560, 446, 11, 31, "Input",ExpressionUUID->"da90dae7-d192-4aa0-b40c-44ddbd02666c",
 InitializationCell->True],
Cell[356820, 8573, 1934, 43, 334, "Output",ExpressionUUID->"e21d4cb8-a8c7-4bea-b449-05a73206ee1e"]
}, Open  ]],
Cell[358769, 8619, 521, 9, 51, "Text",ExpressionUUID->"afdcd814-f87a-4d2b-aab1-72d21385974c"],
Cell[CellGroupData[{
Cell[359315, 8632, 1139, 27, 70, "Input",ExpressionUUID->"90a7357d-fcbe-4616-8a13-3976f7702ac7",
 InitializationCell->True],
Cell[360457, 8661, 6827, 144, 339, "Output",ExpressionUUID->"fb9207c2-49c0-4a56-8c22-aec5c8ddac9f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[367321, 8810, 388, 9, 31, "Input",ExpressionUUID->"689c71ee-94dd-48f3-abfd-70d6a1f67225",
 InitializationCell->True],
Cell[367712, 8821, 3199, 70, 426, "Output",ExpressionUUID->"8ec80807-7177-49d6-afac-0a0caa257a8d"]
}, Open  ]],
Cell[370926, 8894, 372, 8, 36, "Text",ExpressionUUID->"03666e72-f925-402b-99bb-9ce013825b5e"],
Cell[CellGroupData[{
Cell[371323, 8906, 178, 3, 31, "Input",ExpressionUUID->"a8613240-9611-467c-accf-af81b6f0e9fe"],
Cell[371504, 8911, 3739, 80, 431, "Output",ExpressionUUID->"89c90d2f-02c8-4629-95f5-2182e6ff5b00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[375280, 8996, 180, 3, 31, "Input",ExpressionUUID->"84a4c2b5-9f75-4d8e-a017-902c64a99610"],
Cell[375463, 9001, 7760, 164, 339, "Output",ExpressionUUID->"bb208353-dd5e-4597-b30b-67c5591e35a4"]
}, Open  ]],
Cell[383238, 9168, 364, 7, 36, "Text",ExpressionUUID->"d1d07186-8a21-4c70-91f6-2bc5785a085f"],
Cell[CellGroupData[{
Cell[383627, 9179, 388, 8, 51, "Input",ExpressionUUID->"37934446-a773-42a1-ab72-4996500ee45e"],
Cell[384018, 9189, 4171, 86, 370, "Output",ExpressionUUID->"49579771-c43d-4042-bfd5-e767c655553c"]
}, Open  ]],
Cell[CellGroupData[{
Cell[388226, 9280, 490, 13, 31, "Input",ExpressionUUID->"a67bcf12-2a8e-4a85-897c-0f00cc6ca563"],
Cell[388719, 9295, 6208, 134, 370, "Output",ExpressionUUID->"170c4c52-c73d-40d5-90f9-4fda26d9ef48"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[394976, 9435, 153, 2, 32, "Subsection",ExpressionUUID->"f15c3815-6b55-43f0-a8c0-a3e9d8e3122a",
 PageBreakWithin->False,
 CellTags->"Section2g"],
Cell[395132, 9439, 1043, 25, 67, "Text",ExpressionUUID->"09946c38-c68c-4b05-84e9-265f5bcfcc03",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[396178, 9466, 227, 5, 31, "Input",ExpressionUUID->"4d6e7b6c-1294-4c91-996b-1f1df85bf7dc",
 InitializationCell->True],
Cell[396408, 9473, 266, 7, 31, "Input",ExpressionUUID->"a03fa5f2-f401-46fa-93ec-5872b040ae23",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[396699, 9484, 259, 5, 31, "Input",ExpressionUUID->"07865a80-6808-4372-9c5d-17f9cd238285"],
Cell[396961, 9491, 4569, 101, 363, "Output",ExpressionUUID->"00bd4676-0b81-40c3-8c61-7c20001853e7"]
}, Open  ]],
Cell[401545, 9595, 460, 11, 36, "Text",ExpressionUUID->"f8111675-635b-46c7-a899-a3c92e904a59"],
Cell[402008, 9608, 231, 6, 31, "Input",ExpressionUUID->"6212519a-5903-40e7-92f4-976dd58d4fc4",
 InitializationCell->True],
Cell[402242, 9616, 498, 15, 31, "Input",ExpressionUUID->"0db71905-0d0d-4abd-94c9-803bca599a20",
 InitializationCell->True],
Cell[402743, 9633, 280, 8, 31, "Input",ExpressionUUID->"18087067-bb74-42b1-8977-cf7c33dd0dfb",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[403048, 9645, 197, 4, 31, "Input",ExpressionUUID->"0b3ecda1-e552-453d-8c1f-c272b18cc14b"],
Cell[403248, 9651, 3901, 84, 358, "Output",ExpressionUUID->"80b72944-0d21-49fd-a170-95e55ec0e7aa"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[407198, 9741, 151, 2, 32, "Subsection",ExpressionUUID->"3e726959-9a34-4b84-b7d9-03ef53fda0a7",
 PageBreakWithin->False,
 CellTags->"Section2h"],
Cell[407352, 9745, 394, 10, 36, "Text",ExpressionUUID->"ce1e26a0-1972-4db0-9e4b-8efb09cc7d1d",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[CellGroupData[{
Cell[407771, 9759, 300, 8, 31, "Input",ExpressionUUID->"c8aec17e-c84b-4d55-b1c5-f1fb428e5393"],
Cell[408074, 9769, 4082, 87, 431, "Output",ExpressionUUID->"55f935e4-de63-4d43-b39c-730c34b49b57"]
}, Open  ]],
Cell[412171, 9859, 450, 10, 36, "Text",ExpressionUUID->"bd9f9fac-77ac-42c2-ae72-c9d134c094ad"],
Cell[CellGroupData[{
Cell[412646, 9873, 485, 13, 31, "Input",ExpressionUUID->"f9f0060a-85da-4a7d-a2ed-93575131d4cd"],
Cell[413134, 9888, 2266, 43, 426, "Output",ExpressionUUID->"2d21e996-8b03-4d9c-a296-53de49926e4d"]
}, Open  ]],
Cell[415415, 9934, 927, 25, 40, "Text",ExpressionUUID->"ff021e29-70a8-446e-b911-f4993031a141"],
Cell[CellGroupData[{
Cell[416367, 9963, 664, 17, 51, "Input",ExpressionUUID->"9b480e1b-f8d0-4f91-a039-fbe33a797b77"],
Cell[417034, 9982, 4338, 89, 431, "Output",ExpressionUUID->"b71ede0e-2a61-40b9-b7b4-1004a3dd6161"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[421421, 10077, 117, 1, 32, "Subsection",ExpressionUUID->"7d548293-46b5-428a-ad06-4d80e1c772be",
 CellTags->"Section2i"],
Cell[421541, 10080, 1500, 42, 50, "Text",ExpressionUUID->"0ec1baf5-2a90-45d8-8904-410c9092067c"],
Cell[CellGroupData[{
Cell[423066, 10126, 1529, 43, 89, "Input",ExpressionUUID->"a5f4038a-59e3-471e-b8e7-9e7edbac7078",
 InitializationCell->True],
Cell[424598, 10171, 3161, 67, 154, "Output",ExpressionUUID->"d0c02628-a62b-4939-958f-f821ec2aae10"]
}, Open  ]],
Cell[427774, 10241, 285, 6, 21, "Text",ExpressionUUID->"719610d1-f686-45da-8c12-04703bfb32d8"],
Cell[428062, 10249, 318, 7, 21, "Text",ExpressionUUID->"19617917-bdbc-4353-9b5f-dbca0414bc5a"],
Cell[CellGroupData[{
Cell[428405, 10260, 218, 5, 31, "Input",ExpressionUUID->"e5dfc16e-fe98-4eb7-afa2-c842ab5d59a0"],
Cell[428626, 10267, 3282, 70, 166, "Output",ExpressionUUID->"7893cbf8-d4cd-4fdd-8e5b-ff4df1a13eaa"]
}, Open  ]],
Cell[431923, 10340, 794, 17, 52, "Text",ExpressionUUID->"ca745a24-d234-4958-ba22-fc9e8c615cb6"],
Cell[CellGroupData[{
Cell[432742, 10361, 239, 6, 31, "Input",ExpressionUUID->"60613b9e-97f0-4f7d-a7a2-8a260fc4e5af"],
Cell[432984, 10369, 3282, 70, 166, "Output",ExpressionUUID->"b7db639d-ecb0-4083-9648-ff3e1eb62785"]
}, Open  ]],
Cell[436281, 10442, 209, 3, 20, "Text",ExpressionUUID->"aaa81d69-0ecc-4959-bccf-836cc66af7a6"],
Cell[CellGroupData[{
Cell[436515, 10449, 682, 18, 51, "Input",ExpressionUUID->"8baf1ab6-499a-433d-8531-e4e9be10fb1d"],
Cell[437200, 10469, 4380, 92, 370, "Output",ExpressionUUID->"41e3e4cc-0058-4105-b6af-1d11753dd963"]
}, Open  ]],
Cell[CellGroupData[{
Cell[441617, 10566, 411, 11, 31, "Input",ExpressionUUID->"565355d4-e019-4386-a9e4-8053d5d25ff1"],
Cell[442031, 10579, 3836, 82, 356, "Output",ExpressionUUID->"a081c276-bf60-4928-8747-58589a95005f"]
}, Open  ]],
Cell[445882, 10664, 862, 16, 81, "Text",ExpressionUUID->"891ceba0-f844-46bd-a3f9-525774ef5d8b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446781, 10685, 133, 1, 32, "Subsection",ExpressionUUID->"2cf5a3b0-20b2-4f16-b51f-bbc0c08de9a5",
 CellTags->"Section2j"],
Cell[446917, 10688, 226, 3, 35, "Text",ExpressionUUID->"79ba4537-83f5-4569-aaed-c6e7fcba80f5"],
Cell[447146, 10693, 320, 5, 35, "Text",ExpressionUUID->"2a0b9033-0d47-466a-8ec2-2b548ddf4fd8"],
Cell[CellGroupData[{
Cell[447491, 10702, 1274, 31, 89, "Input",ExpressionUUID->"30b62c73-769b-4d49-81f6-641c75872aee",
 InitializationCell->True],
Cell[448768, 10735, 6007, 117, 335, "Output",ExpressionUUID->"ad491ac7-4a8e-44ef-8ef4-9ea33cdd8578"]
}, Open  ]],
Cell[454790, 10855, 1298, 29, 83, "Text",ExpressionUUID->"4f3121e5-fa07-4de3-b7ba-ce1773d52dfd"],
Cell[CellGroupData[{
Cell[456113, 10888, 534, 15, 51, "Input",ExpressionUUID->"33b6cc2e-f0ea-400b-af6d-15aaef1e1749",
 InitializationCell->True],
Cell[456650, 10905, 4900, 102, 426, "Output",ExpressionUUID->"ae5c3c94-021d-4ce6-af33-5b74a69906d2"]
}, Open  ]],
Cell[461565, 11010, 553, 11, 37, "Text",ExpressionUUID->"faf705e4-fb41-45ce-9713-d88db8137021"],
Cell[462121, 11023, 367, 7, 36, "Text",ExpressionUUID->"8fd1f664-d508-48d5-bca3-8c872580015d"],
Cell[CellGroupData[{
Cell[462513, 11034, 158, 3, 31, "Input",ExpressionUUID->"4250574a-62ad-4185-91a4-33d6412e4c0c"],
Cell[462674, 11039, 202, 5, 31, "Output",ExpressionUUID->"5dc07636-d37a-4452-90df-741f0daf241c"]
}, Open  ]],
Cell[462891, 11047, 351, 7, 36, "Text",ExpressionUUID->"9db377b1-d81e-4040-80ac-b67814ad460f"],
Cell[CellGroupData[{
Cell[463267, 11058, 441, 10, 51, "Input",ExpressionUUID->"f459f7af-1497-4b9e-aa8f-be938137807c",
 InitializationCell->True],
Cell[463711, 11070, 7662, 148, 330, "Output",ExpressionUUID->"a99acdf7-2535-4fee-bccf-ef54c0341880"]
}, Open  ]],
Cell[471388, 11221, 903, 20, 67, "Text",ExpressionUUID->"797a8dea-9284-4aee-935d-b02210f23df0"],
Cell[CellGroupData[{
Cell[472316, 11245, 558, 13, 70, "Input",ExpressionUUID->"b981b866-1e49-4371-848c-02efe7cf0380",
 InitializationCell->True],
Cell[472877, 11260, 8072, 153, 335, "Output",ExpressionUUID->"5abba517-d2b1-4ea3-b651-911d1e4e624c"]
}, Open  ]],
Cell[480964, 11416, 1857, 50, 69, "Text",ExpressionUUID->"934a6472-ff8e-4d97-b123-5de6bfc86662"],
Cell[CellGroupData[{
Cell[482846, 11470, 340, 8, 31, "Input",ExpressionUUID->"80426a08-1d22-4bff-bef9-35b11ceab158"],
Cell[483189, 11480, 4595, 96, 366, "Output",ExpressionUUID->"f5ac5f79-f95d-4fe3-b627-7155b07261ba"]
}, Open  ]],
Cell[CellGroupData[{
Cell[487821, 11581, 383, 9, 51, "Input",ExpressionUUID->"d6dad0b9-49fa-40a3-8ed1-c526f6b61f19"],
Cell[488207, 11592, 4683, 96, 374, "Output",ExpressionUUID->"e1049834-a189-45ca-a3f0-759d5022ef36"]
}, Open  ]],
Cell[492905, 11691, 227, 6, 20, "Text",ExpressionUUID->"a268771e-f795-4c40-b9d4-323d57ca01fc"],
Cell[CellGroupData[{
Cell[493157, 11701, 597, 14, 70, "Input",ExpressionUUID->"52930a25-f467-4b99-afd8-bb4bc94ceac4",
 InitializationCell->True],
Cell[493757, 11717, 5197, 107, 366, "Output",ExpressionUUID->"c2984c2a-cc47-4d85-9b58-f30c5c1dcb51"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[499015, 11831, 143, 1, 55, "Section",ExpressionUUID->"13a377f2-f887-483f-af22-7be649862fe0",
 CellTags->"Section3"],
Cell[499161, 11834, 317, 5, 35, "Text",ExpressionUUID->"f3d1bc53-8eae-47cb-a8ab-65edbc029a30"],
Cell[CellGroupData[{
Cell[499503, 11843, 119, 1, 32, "Subsection",ExpressionUUID->"bb97f9de-d74d-4260-bc5f-37c7990131ee",
 CellTags->"Section3a"],
Cell[499625, 11846, 1214, 34, 36, "Text",ExpressionUUID->"106a8cce-33e6-4ea3-b750-83fc8a0553f4"],
Cell[500842, 11882, 127, 0, 20, "Text",ExpressionUUID->"85c0c982-0756-4148-a26a-e7145a79b964"],
Cell[CellGroupData[{
Cell[500994, 11886, 623, 17, 51, "Input",ExpressionUUID->"66bd552e-7a55-4756-9563-2bf7b8e3903b",
 InitializationCell->True],
Cell[501620, 11905, 756, 22, 92, "Output",ExpressionUUID->"0b4948c0-8337-4986-867d-ca41aafea52b"]
}, Open  ]],
Cell[502391, 11930, 478, 11, 37, "Text",ExpressionUUID->"039676a4-45d8-42bb-b8c6-ee4b0a0af1da"],
Cell[502872, 11943, 256, 6, 21, "Text",ExpressionUUID->"df3eebde-344f-415d-ae96-4bd12885edc1"],
Cell[CellGroupData[{
Cell[503153, 11953, 160, 3, 31, "Input",ExpressionUUID->"8233d738-1050-4749-8148-b2e1e472134f"],
Cell[503316, 11958, 1374, 30, 374, "Output",ExpressionUUID->"77baca62-abdf-4adf-9c29-36ede66cd67c"]
}, Open  ]],
Cell[504705, 11991, 431, 8, 36, "Text",ExpressionUUID->"0ad54285-a0bf-4e8a-98db-d11cb0e00adc"],
Cell[CellGroupData[{
Cell[505161, 12003, 475, 12, 51, "Input",ExpressionUUID->"db4d2cb9-c97d-4e16-90ac-85214be4f6ae"],
Cell[505639, 12017, 2103, 44, 374, "Output",ExpressionUUID->"dfdc4e26-a174-4030-84b9-fce0ea930e26"]
}, Open  ]],
Cell[507757, 12064, 201, 3, 20, "Text",ExpressionUUID->"8618e784-2a99-41f7-b878-2723e02f9b33"],
Cell[507961, 12069, 1019, 25, 69, "Text",ExpressionUUID->"df397c3f-f3d1-47cc-a093-47045144ba6d"],
Cell[CellGroupData[{
Cell[509005, 12098, 252, 6, 31, "Input",ExpressionUUID->"d80d1742-a7f4-4cdf-a16c-88738ae20221",
 InitializationCell->True],
Cell[509260, 12106, 9121, 194, 61, "Output",ExpressionUUID->"4b1a988a-c119-4cc9-84cf-7c5d5f06141f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[518418, 12305, 155, 3, 31, "Input",ExpressionUUID->"88cb2818-5f9e-449f-9656-b681107a46ec"],
Cell[518576, 12310, 1237, 31, 142, "Output",ExpressionUUID->"df466704-1064-4997-a7f8-c807c3e7ac21"]
}, Open  ]],
Cell[CellGroupData[{
Cell[519850, 12346, 151, 3, 31, "Input",ExpressionUUID->"ea58eef8-7ff5-405f-bda9-942ebda0d2a2"],
Cell[520004, 12351, 934, 25, 51, "Output",ExpressionUUID->"d72e5992-2c62-4318-8a25-7dd8bd42562b"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[520987, 12382, 116, 1, 32, "Subsection",ExpressionUUID->"087546c0-0b7a-4730-b97e-d29b61b0d382",
 CellTags->"Section3b"],
Cell[521106, 12385, 192, 3, 20, "Text",ExpressionUUID->"a63cc783-91f2-454e-94db-d45ae007bf02"],
Cell[521301, 12390, 415, 8, 36, "Text",ExpressionUUID->"1dd4b689-177e-4116-9eb3-afbee681cbf9"],
Cell[CellGroupData[{
Cell[521741, 12402, 210, 5, 31, "Input",ExpressionUUID->"d369e03e-d0ca-45cf-90fd-836bdd3eed6e"],
Cell[521954, 12409, 166, 4, 31, "Output",ExpressionUUID->"772b280b-6863-4307-a90a-a683c794eb56"]
}, Open  ]],
Cell[522135, 12416, 622, 14, 37, "Text",ExpressionUUID->"57855267-76e8-446f-83d8-65e3615a7d41"],
Cell[522760, 12432, 497, 14, 51, "Input",ExpressionUUID->"5ea10e16-5dad-4a24-9cce-3e9f949b750f",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[523282, 12450, 217, 5, 31, "Input",ExpressionUUID->"e76f7788-7213-4252-a38b-5e0b4e12d532"],
Cell[523502, 12457, 24318, 469, 256, "Output",ExpressionUUID->"4cd3f1e7-80c0-4908-b5f7-98ff687d53fd"]
}, Open  ]],
Cell[547835, 12929, 901, 18, 85, "Text",ExpressionUUID->"60ff907a-fe80-4473-803d-85cf9114eaa4"],
Cell[548739, 12949, 376, 5, 50, "Text",ExpressionUUID->"df4996a2-2d4a-4c76-aa2e-7ebc50777a09"],
Cell[549118, 12956, 292, 8, 31, "Input",ExpressionUUID->"71941755-ec20-47d5-9d53-2c4b7e515386",
 InitializationCell->True],
Cell[549413, 12966, 147, 2, 20, "Text",ExpressionUUID->"413bbb3a-d8d6-4c6e-bcff-de01fddd58d9"],
Cell[549563, 12970, 288, 7, 31, "Input",ExpressionUUID->"79d78e9c-52cc-49bf-af96-99b12f676444",
 Evaluatable->False],
Cell[549854, 12979, 289, 6, 21, "Text",ExpressionUUID->"999e4745-9e7b-4cd9-bb9f-bd0fd4df74e3"],
Cell[550146, 12987, 1693, 44, 86, "Text",ExpressionUUID->"8d0d3b6f-3cfb-4cbe-99d5-3515b2dd6bde"],
Cell[CellGroupData[{
Cell[551864, 13035, 586, 18, 31, "Input",ExpressionUUID->"d8baa098-b2cb-40ff-b0dd-60478b6e39b0"],
Cell[552453, 13055, 846, 25, 92, "Output",ExpressionUUID->"0ed8831a-1727-4488-a6ca-6d6a561870b5"]
}, Open  ]],
Cell[553314, 13083, 604, 10, 51, "Text",ExpressionUUID->"4c648c11-c08f-4828-9314-16e458941e64"],
Cell[553921, 13095, 251, 5, 39, "Text",ExpressionUUID->"d14f62a4-ee29-4e56-906c-ea9394e73d22"],
Cell[CellGroupData[{
Cell[554197, 13104, 484, 13, 31, "Input",ExpressionUUID->"34a05393-f0bd-4c44-8d9f-26aa64899b96",
 InitializationCell->True],
Cell[554684, 13119, 450, 11, 33, "Output",ExpressionUUID->"bbbf2d0d-d710-4a73-a485-0a94a040ee90"]
}, Open  ]],
Cell[555149, 13133, 896, 20, 70, "Text",ExpressionUUID->"9de9d7f6-3141-426a-8b74-878d601d1044"],
Cell[CellGroupData[{
Cell[556070, 13157, 353, 9, 31, "Input",ExpressionUUID->"411b34fd-6057-4fe6-8ad5-e28b8528c789",
 InitializationCell->True],
Cell[556426, 13168, 203, 5, 31, "Output",ExpressionUUID->"13545100-04bd-4317-bc34-fcac7ae0e069"]
}, Open  ]],
Cell[556644, 13176, 818, 15, 67, "Text",ExpressionUUID->"91f7c7c9-6949-40fa-8adf-e3a6a5df088d"],
Cell[CellGroupData[{
Cell[557487, 13195, 372, 11, 31, "Input",ExpressionUUID->"4ce8d680-4f66-4bd7-8207-a240d4f0fb60"],
Cell[557862, 13208, 352, 10, 31, "Output",ExpressionUUID->"483f682b-60f5-4e6f-b180-6e0d0156ec9b"]
}, Open  ]],
Cell[558229, 13221, 188, 3, 20, "Text",ExpressionUUID->"c3940e75-1cd9-4d79-be3f-dc7bbde894d7"],
Cell[CellGroupData[{
Cell[558442, 13228, 386, 10, 31, "Input",ExpressionUUID->"d5eb3f3e-b3f2-4445-924a-c81a61d92327",
 InitializationCell->True],
Cell[558831, 13240, 357, 9, 33, "Output",ExpressionUUID->"69e06659-eedd-4997-8309-5a5a23a58bd9"]
}, Open  ]],
Cell[559203, 13252, 163, 3, 20, "Text",ExpressionUUID->"9480b0e5-ccc6-4b57-bdc5-889bbbb4d6de"],
Cell[CellGroupData[{
Cell[559391, 13259, 161, 3, 31, "Input",ExpressionUUID->"e22ad285-3196-4008-9999-577513d1ca2e"],
Cell[559555, 13264, 111, 2, 31, "Output",ExpressionUUID->"5c51ff87-69a4-4469-bacf-03cfbc3089ad"]
}, Open  ]],
Cell[559681, 13269, 494, 11, 36, "Text",ExpressionUUID->"631b3b50-51c8-49ae-a64b-a3eab037b747"],
Cell[CellGroupData[{
Cell[560200, 13284, 538, 16, 51, "Input",ExpressionUUID->"5ecd273f-a546-4b27-bf39-831878dc2851"],
Cell[560741, 13302, 705, 24, 31, "Output",ExpressionUUID->"a5c10bf2-069a-4ca5-a7b6-d9c82397896c"]
}, Open  ]],
Cell[561461, 13329, 275, 6, 21, "Text",ExpressionUUID->"6baff404-2a23-48ae-98ce-59a7907e1636"],
Cell[CellGroupData[{
Cell[561761, 13339, 690, 20, 70, "Input",ExpressionUUID->"5f6f3438-e26e-4256-9f4e-8f4318922911"],
Cell[562454, 13361, 869, 26, 92, "Output",ExpressionUUID->"d185ca26-03af-4660-adc8-f06e59d2af39"]
}, Open  ]],
Cell[563338, 13390, 540, 15, 40, "Text",ExpressionUUID->"0e19baf2-5173-4d06-a71c-e6fa2e0c5974"],
Cell[563881, 13407, 1955, 56, 165, "Input",ExpressionUUID->"bfd71859-adff-45db-8331-d00d901ec524",
 InitializationCell->True],
Cell[565839, 13465, 108, 0, 20, "Text",ExpressionUUID->"d4265b9a-35dc-4a88-b265-4779db428800"],
Cell[CellGroupData[{
Cell[565972, 13469, 926, 26, 70, "Input",ExpressionUUID->"1d20e2b4-0d8c-4bf5-b851-676e3e338545",
 InitializationCell->True],
Cell[566901, 13497, 2539, 54, 125, "Output",ExpressionUUID->"e2c015dc-f526-4a47-8da8-c585b1a3f8c6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[569489, 13557, 119, 1, 32, "Subsection",ExpressionUUID->"84121ca9-c025-445e-a2c2-ac551fbded43",
 CellTags->"Section3c"],
Cell[569611, 13560, 1200, 34, 35, "Text",ExpressionUUID->"17c3ab9f-e050-445d-a9e0-c3c9c7f5f07a"],
Cell[570814, 13596, 621, 13, 51, "Text",ExpressionUUID->"3779fabf-1c2d-4097-b346-86e8dff3af3b"],
Cell[CellGroupData[{
Cell[571460, 13613, 846, 23, 70, "Input",ExpressionUUID->"c77a919e-2ab0-4e2d-ba47-43489d4b000c",
 InitializationCell->True],
Cell[572309, 13638, 833, 23, 108, "Output",ExpressionUUID->"0d1ca385-6e54-4781-b459-0e986ca935e2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[573179, 13666, 922, 25, 70, "Input",ExpressionUUID->"5049bccb-e104-47ef-86b2-3394e305eaf8",
 InitializationCell->True],
Cell[574104, 13693, 1854, 37, 218, "Output",ExpressionUUID->"2e333ab4-8934-4bcc-95c7-83ad910c1927"]
}, Open  ]],
Cell[575973, 13733, 672, 15, 51, "Text",ExpressionUUID->"caff71ce-7c29-4c13-8750-d617c3c61109"],
Cell[576648, 13750, 752, 19, 37, "Text",ExpressionUUID->"c9e2c968-3e33-4a99-8faf-0e78826b4923"],
Cell[CellGroupData[{
Cell[577425, 13773, 203, 5, 31, "Input",ExpressionUUID->"613bd4a5-b273-4654-b808-438c1ebe693a"],
Cell[577631, 13780, 1007, 27, 108, "Output",ExpressionUUID->"ccb26a96-bca9-4460-83a1-605401bbd665"]
}, Open  ]],
Cell[578653, 13810, 925, 25, 36, "Text",ExpressionUUID->"bb63c743-9e11-4598-a012-dbd120fefa9e"],
Cell[CellGroupData[{
Cell[579603, 13839, 1067, 29, 70, "Input",ExpressionUUID->"38e4408b-91fb-4df9-ade9-4b01f4707ce3",
 InitializationCell->True],
Cell[580673, 13870, 1359, 28, 374, "Output",ExpressionUUID->"4bd5902a-9332-4682-9998-977916ddf07b"]
}, Open  ]],
Cell[CellGroupData[{
Cell[582069, 13903, 204, 5, 31, "Input",ExpressionUUID->"e2cfa573-7c83-4b70-b10a-bf5df16d409a"],
Cell[582276, 13910, 1046, 31, 92, "Output",ExpressionUUID->"fad1572f-6479-4037-b96f-1912c949657e"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[583371, 13947, 122, 1, 32, "Subsection",ExpressionUUID->"b384af5d-602f-4ad5-89a8-20097b4e5998",
 CellTags->"Section3d"],
Cell[583496, 13950, 346, 5, 35, "Text",ExpressionUUID->"5b161143-3654-4a9c-9dab-f367ab3f7560"],
Cell[583845, 13957, 633, 9, 80, "Text",ExpressionUUID->"da12f7e7-a740-465e-83d8-4ff3f582c4a8"],
Cell[584481, 13968, 1208, 28, 83, "Text",ExpressionUUID->"cabf7f87-d0f1-495f-8a15-f3861e3c3977"],
Cell[585692, 13998, 285, 4, 35, "Text",ExpressionUUID->"ba4ea62b-cc55-4d52-80e1-6948ccb58af2"],
Cell[585980, 14004, 3252, 79, 507, "Input",ExpressionUUID->"414eee9e-68f5-4d85-9b1a-b8c22d9db707",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[589257, 14087, 207, 4, 31, "Input",ExpressionUUID->"d7f6f518-e553-4ac3-ac6a-8f696f57e507"],
Cell[CellGroupData[{
Cell[589489, 14095, 170, 3, 24, "Print",ExpressionUUID->"60cf4436-4dfa-4195-aefc-366d5f055c39"],
Cell[589662, 14100, 167, 3, 24, "Print",ExpressionUUID->"20600cf5-1b98-4249-8528-48768cd69b96"],
Cell[589832, 14105, 175, 3, 24, "Print",ExpressionUUID->"963d1f1d-3203-4cd6-bbb1-8c6060440f39"],
Cell[590010, 14110, 170, 3, 24, "Print",ExpressionUUID->"21bf77dc-f36e-46de-8683-5f1e747dfd65"],
Cell[590183, 14115, 170, 3, 24, "Print",ExpressionUUID->"b1efc861-ef38-400c-8342-4c9f7c28c9b3"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[590402, 14124, 253, 6, 31, "Input",ExpressionUUID->"bab9f54a-ddda-40b3-acc7-5bb5c3d9f597"],
Cell[590658, 14132, 169, 3, 24, "Print",ExpressionUUID->"7c5c53e9-7074-4725-ae96-4e5e5c2254a6"]
}, Open  ]],
Cell[590842, 14138, 790, 19, 52, "Text",ExpressionUUID->"82b3eda0-dc98-44de-b6fa-934f3a01d6a7"],
Cell[CellGroupData[{
Cell[591657, 14161, 254, 6, 31, "Input",ExpressionUUID->"9580e130-8119-485c-87f0-a32bf14385b4"],
Cell[591914, 14169, 114, 2, 31, "Output",ExpressionUUID->"ddff1f31-ac73-46c5-98db-94363eeedeb9"]
}, Open  ]],
Cell[592043, 14174, 1441, 33, 87, "Text",ExpressionUUID->"0984b386-cd3c-4427-9e11-e2d7a178d326"],
Cell[593487, 14209, 167, 3, 20, "Text",ExpressionUUID->"97057be5-5d0f-4680-99e7-078389af0c4a"],
Cell[CellGroupData[{
Cell[593679, 14216, 1447, 40, 89, "Input",ExpressionUUID->"97076ae9-4b96-4db3-ad3f-6ce35dd2c446",
 InitializationCell->True],
Cell[595129, 14258, 2049, 41, 267, "Output",ExpressionUUID->"6ee52e94-cc29-4b91-9b84-c2e5bbbde2f6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[597215, 14304, 1449, 40, 89, "Input",ExpressionUUID->"ec2c24c3-7e72-4c51-9d5c-3465221cee65",
 InitializationCell->True],
Cell[598667, 14346, 2391, 53, 267, "Output",ExpressionUUID->"c77907d8-957a-48cc-bcfd-19f9a0b98cad"]
}, Open  ]],
Cell[601073, 14402, 265, 6, 21, "Text",ExpressionUUID->"9291b735-b1b2-4b2d-b3d8-56e740408616"],
Cell[CellGroupData[{
Cell[601363, 14412, 191, 4, 31, "Input",ExpressionUUID->"1d2f0551-5aac-4a2d-a54e-3a25bf782838"],
Cell[601557, 14418, 114, 2, 31, "Output",ExpressionUUID->"392ea034-e308-4524-93bf-6a2e5f3b1f24"]
}, Open  ]],
Cell[601686, 14423, 246, 5, 21, "Text",ExpressionUUID->"f7cb6bac-08a3-4537-a39f-476cd7ffcff3"],
Cell[CellGroupData[{
Cell[601957, 14432, 195, 4, 31, "Input",ExpressionUUID->"6ec6eacf-8944-4e05-8251-a054eecdf321"],
Cell[602155, 14438, 443, 12, 31, "Output",ExpressionUUID->"a0b5a10b-9edb-41fc-97a6-4a23dc24cbe9"]
}, Open  ]],
Cell[602613, 14453, 122, 0, 20, "Text",ExpressionUUID->"8953e4be-2fc4-408d-b2fe-2e8ee5af17d3"],
Cell[CellGroupData[{
Cell[602760, 14457, 207, 4, 31, "Input",ExpressionUUID->"f6c852b4-8c20-4379-89b7-a3dcada40d86"],
Cell[602970, 14463, 1443, 37, 51, "Output",ExpressionUUID->"2214e747-28ac-4521-af3b-c7ef7b2cb465"]
}, Open  ]],
Cell[604428, 14503, 905, 17, 67, "Text",ExpressionUUID->"24a659bf-9493-45e5-9fd1-b574743c04b7"],
Cell[CellGroupData[{
Cell[605358, 14524, 174, 4, 31, "Input",ExpressionUUID->"59db8d88-013a-4464-822b-677f52e0f72c"],
Cell[605535, 14530, 1905, 50, 31, "Output",ExpressionUUID->"7216b793-ce64-471d-9f2d-2e4ab377bef9"]
}, Open  ]],
Cell[607455, 14583, 528, 13, 40, "Text",ExpressionUUID->"5f7e7312-80bc-4f67-b77d-9133f8e434c8"],
Cell[CellGroupData[{
Cell[608008, 14600, 181, 4, 31, "Input",ExpressionUUID->"aa44d0ba-2e86-4320-b02a-e9d23a3a712a"],
Cell[608192, 14606, 3779, 83, 61, "Output",ExpressionUUID->"3ff83839-0c59-4eef-8ee4-20a2f25fa74a"]
}, Open  ]],
Cell[611986, 14692, 346, 7, 36, "Text",ExpressionUUID->"7a400198-a3fa-4674-b98f-1af1e3f85fcc"],
Cell[CellGroupData[{
Cell[612357, 14703, 308, 9, 31, "Input",ExpressionUUID->"5a85c005-372d-4933-a65b-fc02a87c3bf3"],
Cell[612668, 14714, 12009, 312, 31, "Output",ExpressionUUID->"0cce4c55-1764-4996-9963-b30d18897705"]
}, Open  ]],
Cell[624692, 15029, 387, 10, 21, "Text",ExpressionUUID->"b7627b10-9a5a-4069-83ab-c2ca22d2c71d"],
Cell[CellGroupData[{
Cell[625104, 15043, 1703, 47, 89, "Input",ExpressionUUID->"8f74ee77-b443-4380-af02-e6258820f67d"],
Cell[626810, 15092, 3473, 74, 274, "Output",ExpressionUUID->"7be131c6-1d6b-4fe2-a84e-2653e53ee24e"]
}, Open  ]],
Cell[630298, 15169, 277, 6, 20, "Text",ExpressionUUID->"350390e6-324d-4a7f-a01f-0539764250f5"],
Cell[CellGroupData[{
Cell[630600, 15179, 303, 7, 31, "Input",ExpressionUUID->"4105b7d4-14a2-4f4b-8531-dc1413007e33"],
Cell[630906, 15188, 411, 10, 31, "Output",ExpressionUUID->"be3c90c8-7caf-4836-9569-3b29254db1e8"]
}, Open  ]],
Cell[CellGroupData[{
Cell[631354, 15203, 1754, 48, 108, "Input",ExpressionUUID->"b7335a0a-9298-4c77-97d2-73f2bb086a51"],
Cell[633111, 15253, 3181, 67, 273, "Output",ExpressionUUID->"0fd3cfc7-9e29-4e9c-a882-9eaa1891aef1"]
}, Open  ]],
Cell[636307, 15323, 442, 11, 36, "Text",ExpressionUUID->"a2f7f1dd-2b38-4eca-b074-a4d46f9da29e"],
Cell[CellGroupData[{
Cell[636774, 15338, 220, 5, 31, "Input",ExpressionUUID->"4b153d66-e90e-4959-b995-4c5df522766d"],
Cell[636997, 15345, 7619, 155, 203, "Output",ExpressionUUID->"0456f9fa-0d34-4571-9dbc-e94c09ec9356"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[644677, 15507, 114, 1, 55, "Section",ExpressionUUID->"295b098c-70f8-44f6-94d5-da31bbedd722",
 CellTags->"Section4"],
Cell[644794, 15510, 167, 3, 20, "Text",ExpressionUUID->"bdd498ab-8796-4a7e-ba8b-3a74d2c58ef0"],
Cell[644964, 15515, 579, 13, 39, "Text",ExpressionUUID->"737136be-1b47-441e-b3a6-98b69ee4e116"],
Cell[CellGroupData[{
Cell[645568, 15532, 196, 5, 31, "Input",ExpressionUUID->"d926ec9b-92d8-48bc-8248-27ef63a7a871"],
Cell[645767, 15539, 1512, 33, 280, "Output",ExpressionUUID->"800c182d-1f11-487e-a9b7-4f54634bc5d1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[647316, 15577, 240, 6, 31, "Input",ExpressionUUID->"5ac03750-2d01-4492-8e2f-3ebe31d854be"],
Cell[647559, 15585, 114, 2, 31, "Output",ExpressionUUID->"d5fe9294-304b-47d9-8aae-bc99ebc10707"]
}, Open  ]],
Cell[CellGroupData[{
Cell[647710, 15592, 240, 6, 31, "Input",ExpressionUUID->"b84b7b89-f09c-4ff6-8368-ad2cd36ac3d9"],
Cell[647953, 15600, 1557, 31, 358, "Output",ExpressionUUID->"05216e24-ef26-4b85-a5a8-0dd8843b9702"]
}, Open  ]],
Cell[CellGroupData[{
Cell[649547, 15636, 285, 7, 31, "Input",ExpressionUUID->"a3478fe8-8798-4249-beba-84c3c3ed0f9f"],
Cell[649835, 15645, 115, 2, 31, "Output",ExpressionUUID->"02e5d97f-4561-479a-afc2-c018e9ea741e"]
}, Open  ]],
Cell[649965, 15650, 405, 10, 21, "Text",ExpressionUUID->"5009e94d-0c88-4fd3-b147-9898b7362822"],
Cell[CellGroupData[{
Cell[650395, 15664, 574, 15, 31, "Input",ExpressionUUID->"332e4ac2-b07d-450f-bf0a-e6d0fa1a56ac",
 InitializationCell->True],
Cell[650972, 15681, 1287, 26, 373, "Output",ExpressionUUID->"fcae593d-76e5-4623-9b29-8b5cd978dc27"]
}, Open  ]],
Cell[652274, 15710, 265, 6, 21, "Text",ExpressionUUID->"4be76e03-fe22-44a5-aa40-93445a59b795"],
Cell[CellGroupData[{
Cell[652564, 15720, 169, 3, 31, "Input",ExpressionUUID->"6e9eae8e-e3da-45d0-871e-0cf0218eee89"],
Cell[652736, 15725, 114, 2, 31, "Output",ExpressionUUID->"a6cf5b98-3582-4eb3-aad9-acd6ca45faa9"]
}, Open  ]],
Cell[652865, 15730, 418, 11, 21, "Text",ExpressionUUID->"3f5c83f6-57a9-452b-a8b2-0d1b2a5f034d"],
Cell[CellGroupData[{
Cell[653308, 15745, 656, 17, 51, "Input",ExpressionUUID->"750d0bca-fba3-4ad7-92ea-d961af64289c",
 InitializationCell->True],
Cell[653967, 15764, 1673, 31, 81, "Output",ExpressionUUID->"ab161af4-0c1e-4464-8312-3b26a1443b14"]
}, Open  ]],
Cell[CellGroupData[{
Cell[655677, 15800, 167, 3, 31, "Input",ExpressionUUID->"636786aa-def6-4fd9-8ebd-12d8b83a0a58"],
Cell[655847, 15805, 115, 2, 31, "Output",ExpressionUUID->"e6171fbd-a601-4526-9d1e-aa62f127118a"]
}, Open  ]],
Cell[655977, 15810, 372, 7, 36, "Text",ExpressionUUID->"17fe7850-2ec2-4985-9523-5aeedb434450"],
Cell[CellGroupData[{
Cell[656374, 15821, 173, 3, 31, "Input",ExpressionUUID->"92d8a710-3803-4049-ac08-ab23598a4f47"],
Cell[656550, 15826, 114, 2, 31, "Output",ExpressionUUID->"2106bfa5-bb69-47b6-aa8f-1d4f29f5ecf4"]
}, Open  ]],
Cell[656679, 15831, 411, 7, 36, "Text",ExpressionUUID->"7155f219-be2b-4b8e-ba5d-ae533e70eb41"],
Cell[657093, 15840, 300, 8, 23, "Text",ExpressionUUID->"76b19247-320d-463e-b1a1-ce556b74160c"],
Cell[CellGroupData[{
Cell[657418, 15852, 318, 8, 31, "Input",ExpressionUUID->"99ea4a50-847f-444c-9a2e-88cb3a209dcf"],
Cell[657739, 15862, 2808, 65, 363, "Output",ExpressionUUID->"55e50d5b-6377-40cd-938e-f29d89c4252f"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660584, 15932, 289, 7, 31, "Input",ExpressionUUID->"fb08f507-216c-4896-9b95-25b3c5cab535"],
Cell[660876, 15941, 264, 8, 31, "Output",ExpressionUUID->"8d02cf7b-2f52-4dd9-8ed6-7abcb317e088"]
}, Open  ]],
Cell[661155, 15952, 709, 20, 38, "Text",ExpressionUUID->"c46cc1f4-bb7a-40e5-b1f1-0ee97db44242"],
Cell[661867, 15974, 496, 10, 37, "Text",ExpressionUUID->"d170df3f-cd3d-4c29-aae2-001b953658c8"],
Cell[CellGroupData[{
Cell[662388, 15988, 171, 3, 31, "Input",ExpressionUUID->"cdbf4e12-6abc-4572-b855-2266e1fbf945"],
Cell[662562, 15993, 328, 9, 31, "Output",ExpressionUUID->"eb24eb34-5a9b-423f-b257-9759659ab1db"]
}, Open  ]],
Cell[CellGroupData[{
Cell[662927, 16007, 180, 4, 31, "Input",ExpressionUUID->"84099db6-5870-419b-8c3c-9cf43bdf86fa"],
Cell[663110, 16013, 259, 7, 31, "Output",ExpressionUUID->"afb79e27-98e7-4c13-9518-6d2f40454089"]
}, Open  ]],
Cell[CellGroupData[{
Cell[663406, 16025, 124, 1, 32, "Subsection",ExpressionUUID->"92db923f-3f31-43f5-b0c7-852336f274ae",
 CellTags->"Section4a"],
Cell[663533, 16028, 240, 4, 35, "Text",ExpressionUUID->"c03e6756-047d-455c-adb8-c4db36c8c158"],
Cell[663776, 16034, 1287, 32, 69, "Text",ExpressionUUID->"cb5ae444-6e97-4798-b9b2-37cdf28a9215"],
Cell[665066, 16068, 1765, 47, 203, "Input",ExpressionUUID->"4f92f03f-1352-4493-8074-853f8386c6d7",
 InitializationCell->True],
Cell[666834, 16117, 130, 0, 20, "Text",ExpressionUUID->"481e0be2-6e71-438f-9dde-add8147d571b"],
Cell[CellGroupData[{
Cell[666989, 16121, 238, 5, 31, "Input",ExpressionUUID->"0ce6255f-f8a5-4285-a8e1-50959d671e78"],
Cell[667230, 16128, 3208, 65, 84, "Output",ExpressionUUID->"3be8d9c2-159d-4cc4-8637-097c2d71b291"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[670487, 16199, 132, 1, 32, "Subsection",ExpressionUUID->"57f070f6-2b90-4bde-857a-9713e6787c03",
 CellTags->"Section4b"],
Cell[670622, 16202, 1206, 32, 50, "Text",ExpressionUUID->"82440408-505f-400c-8887-3409a2377c6e"],
Cell[671831, 16236, 394, 8, 40, "Text",ExpressionUUID->"2ea29578-a7f4-4123-9d48-7005a98e0683"],
Cell[CellGroupData[{
Cell[672250, 16248, 315, 8, 51, "Input",ExpressionUUID->"beea9479-6612-4ec2-b9f2-9ae8e10fda6b",
 InitializationCell->True],
Cell[672568, 16258, 948, 27, 108, "Output",ExpressionUUID->"8279410f-f428-4b5b-8a83-fd6e7fe4e390"]
}, Open  ]],
Cell[673531, 16288, 907, 26, 37, "Text",ExpressionUUID->"d3b11ced-ed61-4391-b73d-42c0c60a27ec"],
Cell[674441, 16316, 121, 0, 20, "Text",ExpressionUUID->"5eb94f53-d1b1-4db4-80dc-8b01f1077c44"],
Cell[CellGroupData[{
Cell[674587, 16320, 333, 9, 31, "Input",ExpressionUUID->"813b3d7f-f5e1-42eb-a2d1-f7302e5d07da"],
Cell[674923, 16331, 6118, 157, 197, "Output",ExpressionUUID->"7150565b-90ff-49ae-8e43-5a7d5e4e44cf"]
}, Open  ]],
Cell[681056, 16491, 461, 12, 39, "Text",ExpressionUUID->"2c5b6104-1c5d-4034-932e-3c7eafe00bb5"],
Cell[681520, 16505, 811, 18, 50, "Text",ExpressionUUID->"f7e8e71a-1d42-4a23-abdb-aa82bb376d4c"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[682380, 16529, 126, 1, 55, "Section",ExpressionUUID->"883be708-f882-4ed6-9e72-06db73a03569",
 CellTags->"Section5"],
Cell[682509, 16532, 674, 11, 65, "Text",ExpressionUUID->"006a7469-9117-451a-aec6-a72040fa14a2"],
Cell[CellGroupData[{
Cell[683208, 16547, 115, 1, 32, "Subsection",ExpressionUUID->"16c74548-e882-4f80-aaf0-225dca35522e",
 CellTags->"Section5a"],
Cell[683326, 16550, 760, 19, 51, "Text",ExpressionUUID->"ed36fb97-8000-434c-8550-809f1c8ccdd5"],
Cell[CellGroupData[{
Cell[684111, 16573, 190, 4, 31, "Input",ExpressionUUID->"31a7a6a5-d4ab-4974-81d3-a08040343a51"],
Cell[684304, 16579, 114, 2, 31, "Output",ExpressionUUID->"3b7e974a-2482-46a2-9191-8551e8b24f7a"]
}, Open  ]],
Cell[684433, 16584, 899, 19, 67, "Text",ExpressionUUID->"98632c14-6b9a-447e-a2ed-db65dc3946d8"],
Cell[CellGroupData[{
Cell[685357, 16607, 193, 4, 31, "Input",ExpressionUUID->"dd73c6a9-b18c-4ea0-a212-5c22efd31370"],
Cell[685553, 16613, 670, 15, 31, "Output",ExpressionUUID->"4a0acf2f-a49c-4067-a257-31eef7d77648"]
}, Open  ]],
Cell[686238, 16631, 586, 12, 51, "Text",ExpressionUUID->"b8cd6e88-56e3-484c-9243-26365da06b81"],
Cell[CellGroupData[{
Cell[686849, 16647, 217, 5, 31, "Input",ExpressionUUID->"cac7af16-2345-4133-8daa-3ee77412f625"],
Cell[687069, 16654, 1813, 40, 70, "Output",ExpressionUUID->"1feacedc-d7f5-4997-9872-b4b992170ab5"]
}, Open  ]],
Cell[688897, 16697, 403, 10, 36, "Text",ExpressionUUID->"3bf1bdce-ceaf-4658-b697-ff9fe1158226"],
Cell[CellGroupData[{
Cell[689325, 16711, 257, 7, 31, "Input",ExpressionUUID->"3ce37e20-c092-4f8b-8cf4-799ecb3ea9ca"],
Cell[689585, 16720, 113, 2, 31, "Output",ExpressionUUID->"ed9b6ab0-763b-4797-a2c0-832f589f9100"]
}, Open  ]],
Cell[689713, 16725, 990, 22, 67, "Text",ExpressionUUID->"ff89fe66-b77f-44aa-997e-0588d6f12917"],
Cell[690706, 16749, 1051, 23, 68, "Text",ExpressionUUID->"a382b5f7-28f8-47b5-a05a-ced8e9a37cf3"],
Cell[691760, 16774, 665, 17, 40, "Text",ExpressionUUID->"ea5d10d0-78e9-4161-96b6-5c2b96ad6e2e"],
Cell[CellGroupData[{
Cell[692450, 16795, 291, 7, 31, "Input",ExpressionUUID->"9cadf90d-b27f-40f9-a0a2-6684bb932d21",
 InitializationCell->True],
Cell[692744, 16804, 636, 14, 31, "Output",ExpressionUUID->"818be0ce-b3d6-446e-af80-e86d2067ab51"]
}, Open  ]],
Cell[693395, 16821, 435, 8, 36, "Text",ExpressionUUID->"0a9db13a-17ab-4350-8405-71d264e5b73a"],
Cell[693833, 16831, 790, 21, 58, "Text",ExpressionUUID->"b685e1bf-87a6-48b2-a731-cb101b8afc8b"],
Cell[CellGroupData[{
Cell[694648, 16856, 308, 8, 31, "Input",ExpressionUUID->"09e31f20-958c-47c6-8f6a-c39596b01cb7"],
Cell[694959, 16866, 3365, 71, 358, "Output",ExpressionUUID->"052b5b34-b9a0-4ae3-abec-4c55d6d18d28"]
}, Open  ]],
Cell[698339, 16940, 171, 3, 20, "Text",ExpressionUUID->"9685c40f-3cc4-4a5b-8f55-57b715a011cf"],
Cell[CellGroupData[{
Cell[698535, 16947, 198, 5, 31, "Input",ExpressionUUID->"9f2c5162-d701-402d-9640-b18e56333fbd"],
Cell[698736, 16954, 128, 3, 31, "Output",ExpressionUUID->"90f47ad7-c120-420a-b1d8-88612c697019"]
}, Open  ]],
Cell[CellGroupData[{
Cell[698901, 16962, 308, 8, 31, "Input",ExpressionUUID->"33e9a646-7148-4254-afe8-6955e9743a2b"],
Cell[699212, 16972, 2045, 38, 358, "Output",ExpressionUUID->"d1938e02-b6e6-4d53-94c2-27ecab80a879"]
}, Open  ]],
Cell[701272, 17013, 1122, 29, 56, "Text",ExpressionUUID->"eb863aef-0698-4bae-b4ac-c46f838bd590"],
Cell[CellGroupData[{
Cell[702419, 17046, 612, 15, 70, "Input",ExpressionUUID->"5a98ae5a-7162-461b-8161-97213356752a"],
Cell[703034, 17063, 2090, 43, 536, "Output",ExpressionUUID->"089a308f-4ad6-4f13-bdc0-027ccb427580"]
}, Open  ]],
Cell[705139, 17109, 557, 10, 58, "Text",ExpressionUUID->"5d9a9938-ee9c-4146-8cdd-a088536edef9"],
Cell[705699, 17121, 1041, 28, 108, "Input",ExpressionUUID->"615a8938-dc74-4988-ac6c-ab2570f08891",
 InitializationCell->True],
Cell[706743, 17151, 187, 3, 20, "Text",ExpressionUUID->"0edc27ce-1fea-4f87-876d-c7a34a1f6dd1"],
Cell[CellGroupData[{
Cell[706955, 17158, 540, 14, 70, "Input",ExpressionUUID->"1a52f0ac-f6e6-4700-82c9-bf9f297590d3"],
Cell[707498, 17174, 3354, 73, 546, "Output",ExpressionUUID->"cd250282-e810-48eb-89e5-3f40fc8d43b0"]
}, Open  ]],
Cell[710867, 17250, 151, 2, 20, "Text",ExpressionUUID->"e5f716c5-2872-4db7-aa66-c43dea4443d0"],
Cell[711021, 17254, 282, 4, 35, "Text",ExpressionUUID->"484b3c74-d186-402a-aa01-bb10525be0d0"],
Cell[CellGroupData[{
Cell[711328, 17262, 874, 19, 70, "Input",ExpressionUUID->"99d7a1aa-2614-4e06-9e16-75d464cef9b5"],
Cell[712205, 17283, 2865, 58, 206, "Output",ExpressionUUID->"bceee35f-a22a-4101-a433-1f30b846cced"]
}, Open  ]],
Cell[CellGroupData[{
Cell[715107, 17346, 159, 3, 31, "Input",ExpressionUUID->"7ec61bdc-908f-42be-9841-9af830d85113"],
Cell[715269, 17351, 115, 2, 31, "Output",ExpressionUUID->"5f5c0291-82ce-4a90-a26f-8336a779c219"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[715433, 17359, 112, 1, 32, "Subsection",ExpressionUUID->"f921781e-e24e-488e-b181-f9fd929304b2",
 CellTags->"Section5b"],
Cell[715548, 17362, 736, 10, 80, "Text",ExpressionUUID->"b133ff35-422e-4380-808b-34f7fc86bcb9"],
Cell[716287, 17374, 1412, 31, 101, "Text",ExpressionUUID->"edcf8933-a80b-4a72-bc08-1d7e883c9fe4"],
Cell[717702, 17407, 587, 17, 51, "Input",ExpressionUUID->"47c4d6df-432e-4e89-a464-1346627fcc84",
 InitializationCell->True],
Cell[718292, 17426, 301, 7, 20, "Text",ExpressionUUID->"f6400ff5-5d8a-47f2-9690-a3f1bf219d0d"],
Cell[CellGroupData[{
Cell[718618, 17437, 1921, 45, 108, "Input",ExpressionUUID->"d16a03a2-7945-41c7-8ac7-244112fc514d",
 InitializationCell->True],
Cell[720542, 17484, 2343, 45, 159, "Output",ExpressionUUID->"de23d146-b9f4-4e53-8abf-f88422efb421"]
}, Open  ]],
Cell[CellGroupData[{
Cell[722922, 17534, 158, 3, 31, "Input",ExpressionUUID->"6be58db2-b8b2-4ef1-af46-0793bc618536"],
Cell[723083, 17539, 115, 2, 31, "Output",ExpressionUUID->"c67cee85-113d-4336-b375-58483152610e"]
}, Open  ]],
Cell[CellGroupData[{
Cell[723235, 17546, 154, 3, 31, "Input",ExpressionUUID->"ce11be54-8a12-4546-a3c1-4ec07111f827"],
Cell[723392, 17551, 114, 2, 31, "Output",ExpressionUUID->"e984b709-2f9b-4a16-9e3c-ae2c9b694c66"]
}, Open  ]],
Cell[723521, 17556, 625, 9, 65, "Text",ExpressionUUID->"20593bdd-9475-4a65-9948-1f54c642fde2"],
Cell[724149, 17567, 8565, 208, 1020, "Input",ExpressionUUID->"b0459a0f-33d4-4b01-8f15-7eb26f76e8b4",
 InitializationCell->True],
Cell[732717, 17777, 671, 12, 73, "Text",ExpressionUUID->"ca37a71c-7611-42b8-b115-6734e47feef2"],
Cell[733391, 17791, 930, 18, 96, "Text",ExpressionUUID->"3151628e-31d4-45fb-bbee-5965d224e077"],
Cell[734324, 17811, 946, 18, 99, "Text",ExpressionUUID->"dd14f1ff-10bc-43d6-9cdb-efd7fa11c79d"],
Cell[735273, 17831, 807, 23, 59, "Text",ExpressionUUID->"12cafa15-6b96-474a-87a0-ef6e494b743f"],
Cell[736083, 17856, 1178, 27, 107, "Text",ExpressionUUID->"ac93333b-ca6e-4401-9e2e-5f5afad82d0a"],
Cell[737264, 17885, 982, 31, 62, "Text",ExpressionUUID->"60283a27-91ef-47e9-a5fd-7af2e29535a5"],
Cell[738249, 17918, 609, 16, 59, "Text",ExpressionUUID->"065b237e-51e8-4d2d-9287-a4a629eeaf28"],
Cell[738861, 17936, 1249, 33, 93, "Text",ExpressionUUID->"ed662172-c26b-4ae7-ada5-8476a8a5b26d"],
Cell[740113, 17971, 1760, 36, 158, "Text",ExpressionUUID->"34bc06c3-30b2-4f05-a00a-b9d7d6df16c3"],
Cell[741876, 18009, 124, 0, 20, "Text",ExpressionUUID->"43a997b6-3a00-48bc-a8a6-bc4d99098401"],
Cell[CellGroupData[{
Cell[742025, 18013, 1678, 41, 108, "Input",ExpressionUUID->"b0dc2f9c-0565-4886-97e8-37f582367267",
 InitializationCell->True],
Cell[743706, 18056, 2651, 51, 374, "Output",ExpressionUUID->"f223d859-6c94-4d74-8fda-61fc7baeec9d"]
}, Open  ]],
Cell[CellGroupData[{
Cell[746394, 18112, 227, 5, 31, "Input",ExpressionUUID->"97e29389-a4f9-410c-ae1f-a4d804c3ae75",
 InitializationCell->True],
Cell[746624, 18119, 778, 15, 31, "Output",ExpressionUUID->"c0f242f9-d9c0-4c12-9dc4-c907b11cb9be"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[747451, 18140, 118, 1, 32, "Subsection",ExpressionUUID->"41acec3b-72f3-45a8-854c-131b2698b7a4",
 CellTags->"Section5c"],
Cell[747572, 18143, 645, 13, 52, "Text",ExpressionUUID->"1bfa9832-e019-4b42-a23a-d552a8d779e9"],
Cell[CellGroupData[{
Cell[748242, 18160, 213, 5, 31, "Input",ExpressionUUID->"cf96aa4d-da29-42f3-8fab-baac583d6b1c",
 InitializationCell->True],
Cell[748458, 18167, 2391, 44, 288, "Output",ExpressionUUID->"b3cd140f-1e77-4181-8d9a-98d3f30ebfd7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[750886, 18216, 166, 3, 31, "Input",ExpressionUUID->"c16d3428-85ea-4973-bb5f-2233a79fd33d"],
Cell[751055, 18221, 114, 2, 31, "Output",ExpressionUUID->"27091af3-4586-4e6f-86dc-f0c6aed8dfda"]
}, Open  ]],
Cell[751184, 18226, 540, 12, 36, "Text",ExpressionUUID->"71e4931d-89cd-4661-8ef3-6b692816f573"],
Cell[CellGroupData[{
Cell[751749, 18242, 169, 3, 31, "Input",ExpressionUUID->"93daa322-dd6a-4761-9b5d-4b9580b33331"],
Cell[751921, 18247, 568, 13, 31, "Output",ExpressionUUID->"83b89b59-0c8d-439d-b159-12d4ea169070"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752526, 18265, 194, 4, 31, "Input",ExpressionUUID->"8a0beea2-2818-4038-b812-6b62109cfbc5"],
Cell[752723, 18271, 2866, 64, 127, "Output",ExpressionUUID->"d56e735b-0e9b-489c-b840-62c7fcb737bc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[755626, 18340, 276, 7, 31, "Input",ExpressionUUID->"15d6a140-8c43-4b21-a99a-fa686bf66ef3",
 InitializationCell->True],
Cell[755905, 18349, 536, 12, 31, "Output",ExpressionUUID->"7cb600ad-0784-449e-af5a-b682077ef3c8"]
}, Open  ]],
Cell[756456, 18364, 257, 6, 21, "Text",ExpressionUUID->"336255e2-e663-467f-8fed-269c00af8469"],
Cell[CellGroupData[{
Cell[756738, 18374, 206, 4, 31, "Input",ExpressionUUID->"1ebbf5fc-75b8-487a-999f-11b33d7e9fdb"],
Cell[756947, 18380, 4186, 87, 288, "Output",ExpressionUUID->"c0c703c2-e2dd-40df-9322-75d00e06378b"]
}, Open  ]],
Cell[761148, 18470, 199, 4, 24, "Text",ExpressionUUID->"05515278-3695-4b0f-a8b3-9cb1c2156d1f"],
Cell[CellGroupData[{
Cell[761372, 18478, 203, 4, 31, "Input",ExpressionUUID->"2f7d8df1-6e8b-4054-aa2a-dc2da60e7a6c"],
Cell[761578, 18484, 2694, 56, 448, "Output",ExpressionUUID->"4220d439-1d7d-450a-9382-0f27f48e42fe"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[764333, 18547, 124, 1, 55, "Section",ExpressionUUID->"ba88e25b-5334-48e1-91be-9c29178d2dee",
 CellTags->"Section6"],
Cell[764460, 18550, 337, 5, 35, "Text",ExpressionUUID->"12834ff3-6eaf-4f4c-b773-bf2a91666e99"],
Cell[764800, 18557, 750, 17, 52, "Text",ExpressionUUID->"44b94d3b-9d6a-42d4-bf97-4f5cc3b21038"],
Cell[765553, 18576, 161, 3, 20, "Text",ExpressionUUID->"6758678c-a7f2-4856-a9e3-12561728afd8"],
Cell[CellGroupData[{
Cell[765739, 18583, 2068, 54, 127, "Input",ExpressionUUID->"694369cd-3628-403a-a573-996ae2f87d50",
 InitializationCell->True],
Cell[767810, 18639, 3026, 58, 159, "Output",ExpressionUUID->"6bfa0ba6-69ca-46a2-9a0b-ba08938409d2"]
}, Open  ]],
Cell[770851, 18700, 2135, 55, 140, "Text",ExpressionUUID->"25d20cbc-4208-4452-be30-bf2ddd2cb5f4"],
Cell[772989, 18757, 615, 12, 52, "Text",ExpressionUUID->"373960ea-0967-4e9a-a09e-4e7ff78fa8a1"],
Cell[CellGroupData[{
Cell[773629, 18773, 2617, 70, 165, "Input",ExpressionUUID->"d0c5d656-cc5c-48db-89ca-f913d75a5353"],
Cell[776249, 18845, 2831, 51, 159, "Output",ExpressionUUID->"f5e26aef-3608-4aad-b440-22b4f58e5bfd"]
}, Open  ]],
Cell[779095, 18899, 319, 6, 21, "Text",ExpressionUUID->"d3d63b7b-8668-4027-a038-81caf6c9c8b3"],
Cell[779417, 18907, 1140, 24, 83, "Text",ExpressionUUID->"a6208d09-1348-4ffc-8a37-087397a6ed88"],
Cell[CellGroupData[{
Cell[780582, 18935, 1207, 33, 70, "Input",ExpressionUUID->"44f090e2-6145-44ed-b505-d452c4ae44c9",
 InitializationCell->True],
Cell[781792, 18970, 1070, 24, 126, "Output",ExpressionUUID->"754dd8b1-ca95-42a8-81f3-c333ab4aaf96"]
}, Open  ]],
Cell[CellGroupData[{
Cell[782899, 18999, 1275, 36, 89, "Input",ExpressionUUID->"77c5bb74-a620-4732-a673-002dea20527f"],
Cell[784177, 19037, 3147, 61, 159, "Output",ExpressionUUID->"5098c2d1-841b-44f4-a443-52d482df581c"]
}, Open  ]],
Cell[787339, 19101, 860, 19, 66, "Text",ExpressionUUID->"f1baf928-9272-4246-9f1b-ecaf67353c21"],
Cell[CellGroupData[{
Cell[788224, 19124, 209, 4, 31, "Input",ExpressionUUID->"5ec39d6b-f8fc-4035-ab68-7ad3acfcaf21"],
Cell[788436, 19130, 230, 5, 31, "Output",ExpressionUUID->"8999e3ff-65f9-43af-90c6-45044b9175da"]
}, Open  ]],
Cell[788681, 19138, 1145, 28, 68, "Text",ExpressionUUID->"b73e878f-d705-4408-b59d-08228b5ca7bd"],
Cell[CellGroupData[{
Cell[789851, 19170, 321, 8, 31, "Input",ExpressionUUID->"f247b74e-d576-4dae-ae62-e7b1451e4951"],
Cell[790175, 19180, 3725, 71, 159, "Output",ExpressionUUID->"412fc3c8-2d4a-4990-9816-2682009e23d4"]
}, Open  ]],
Cell[793915, 19254, 301, 6, 21, "Text",ExpressionUUID->"bbaafb7c-c115-4ce5-af65-01c9e373b44d"],
Cell[CellGroupData[{
Cell[794241, 19264, 206, 4, 31, "Input",ExpressionUUID->"210bd2b7-f78e-4bce-9ff1-41cc486ca865"],
Cell[794450, 19270, 113, 2, 31, "Output",ExpressionUUID->"3e4acd26-809f-4d46-bc7d-65871b55ebf8"]
}, Open  ]],
Cell[794578, 19275, 367, 7, 36, "Text",ExpressionUUID->"04e39816-5b40-4bcd-9e76-c226b25e8651"],
Cell[CellGroupData[{
Cell[794970, 19286, 188, 4, 31, "Input",ExpressionUUID->"f1163dbe-a2de-4806-9709-1a7422722872"],
Cell[795161, 19292, 209, 5, 31, "Output",ExpressionUUID->"171d7ae8-8b57-4341-b0f0-a81e79c115cc"]
}, Open  ]],
Cell[795385, 19300, 498, 9, 51, "Text",ExpressionUUID->"b3a70403-28fa-4eb2-b8cf-4f90f2105113"],
Cell[CellGroupData[{
Cell[795908, 19313, 199, 5, 31, "Input",ExpressionUUID->"9539013f-b09e-4ebe-9382-df1ddc22c3c5"],
Cell[796110, 19320, 942, 24, 126, "Output",ExpressionUUID->"e41e20b9-6790-491a-ad30-14f0c62fc4d2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[797101, 19350, 115, 1, 55, "Section",ExpressionUUID->"d5724981-81d0-4aeb-a1fd-f528100e9e21",
 CellTags->"Section7"],
Cell[797219, 19353, 702, 14, 66, "Text",ExpressionUUID->"10c48b9e-c6e6-4450-9343-7a98fb6274cc"],
Cell[797924, 19369, 578, 14, 36, "Text",ExpressionUUID->"409ddd99-40e1-4ed8-b977-0ccd7e51991e"],
Cell[CellGroupData[{
Cell[798527, 19387, 189, 4, 31, "Input",ExpressionUUID->"4428afac-d3f4-4e98-a6a2-1569b4a58c3d"],
Cell[798719, 19393, 114, 2, 31, "Output",ExpressionUUID->"b24fd399-05dd-4565-9bf2-838ca09cac60"]
}, Open  ]],
Cell[CellGroupData[{
Cell[798870, 19400, 188, 4, 31, "Input",ExpressionUUID->"25ab1454-9e9f-483a-be08-2469a5d05ec2"],
Cell[799061, 19406, 115, 2, 31, "Output",ExpressionUUID->"3b45621f-4314-436e-acdb-d48931c7c826"]
}, Open  ]],
Cell[CellGroupData[{
Cell[799213, 19413, 162, 2, 32, "Subsection",ExpressionUUID->"e2e7f02a-e5a9-43bd-8115-46e50f8fe350",
 CellTags->"Section7a"],
Cell[799378, 19417, 1039, 27, 50, "Text",ExpressionUUID->"07c7e560-211f-4343-be38-833d111633d5"],
Cell[CellGroupData[{
Cell[800442, 19448, 411, 10, 51, "Input",ExpressionUUID->"076c9628-2206-47a9-9e2b-8221bad39437",
 InitializationCell->True],
Cell[800856, 19460, 2739, 60, 359, "Output",ExpressionUUID->"0db12089-2614-471b-989b-21360bcf16ba"]
}, Open  ]],
Cell[803610, 19523, 942, 24, 55, "Text",ExpressionUUID->"05fd51c2-b17a-40ab-ac92-e417ffcd2dc5"],
Cell[CellGroupData[{
Cell[804577, 19551, 371, 9, 31, "Input",ExpressionUUID->"7afbd1f6-537d-447e-b355-59b1f2afd176",
 InitializationCell->True],
Cell[804951, 19562, 2878, 60, 422, "Output",ExpressionUUID->"3b7cd094-a7a1-4083-b46b-294ec3897d0a"]
}, Open  ]],
Cell[807844, 19625, 1032, 25, 68, "Text",ExpressionUUID->"a756ac14-dae4-4c4c-8853-af254efaaacd"],
Cell[CellGroupData[{
Cell[808901, 19654, 665, 17, 70, "Input",ExpressionUUID->"e84fd38d-03ea-408f-ae65-a539a16be5f5",
 InitializationCell->True],
Cell[809569, 19673, 3185, 67, 359, "Output",ExpressionUUID->"88e40ee2-879f-45d7-ae4c-743387e5cefc"]
}, Open  ]],
Cell[812769, 19743, 354, 7, 36, "Text",ExpressionUUID->"6ca3e411-f350-49eb-89b2-bb6bac5e1011"],
Cell[CellGroupData[{
Cell[813148, 19754, 725, 22, 70, "Input",ExpressionUUID->"c3f3d529-920a-4666-8fd6-9d1a5012ef5b",
 InitializationCell->True],
Cell[813876, 19778, 231, 6, 31, "Output",ExpressionUUID->"c9387b13-149c-492c-924b-2108d1cd0fd6"]
}, Open  ]],
Cell[CellGroupData[{
Cell[814144, 19789, 126, 2, 31, "Input",ExpressionUUID->"ea8c95e7-00a6-443d-a93f-44a522a68695"],
Cell[814273, 19793, 3312, 69, 359, "Output",ExpressionUUID->"c6e8f253-376d-4ca5-8e0d-77e6bc35d427"]
}, Open  ]],
Cell[817600, 19865, 876, 23, 40, "Text",ExpressionUUID->"31c38212-7a00-4b9f-8536-be942158aaae"],
Cell[CellGroupData[{
Cell[818501, 19892, 583, 15, 51, "Input",ExpressionUUID->"dcf6bb99-5ace-4fe7-aa4b-24865434cd55",
 InitializationCell->True],
Cell[819087, 19909, 3482, 72, 359, "Output",ExpressionUUID->"49abb31b-62dd-4942-9036-b706d68efcbd"]
}, Open  ]],
Cell[822584, 19984, 127, 0, 20, "Text",ExpressionUUID->"e6e69d3e-d91c-44d1-9378-11ae272a41d3"],
Cell[822714, 19986, 3523, 100, 375, "Input",ExpressionUUID->"ccd77f77-2b72-4c59-965f-87b4994759b2",
 InitializationCell->True],
Cell[826240, 20088, 1365, 37, 50, "Text",ExpressionUUID->"d765ac0d-895f-4287-99ef-d21eba4ff8c7"],
Cell[827608, 20127, 1937, 53, 222, "Input",ExpressionUUID->"fa505704-dc1c-4a32-be58-75eea4f2bce9",
 InitializationCell->True],
Cell[829548, 20182, 178, 2, 20, "Text",ExpressionUUID->"40313240-af38-484d-8460-5fe9d886b69d"],
Cell[CellGroupData[{
Cell[829751, 20188, 196, 4, 31, "Input",ExpressionUUID->"71b628ee-3997-4dab-87c3-0cc31d20114a"],
Cell[829950, 20194, 3116, 66, 359, "Output",ExpressionUUID->"b2ac4811-a0b0-47e8-8fb0-a368e33d8050"]
}, Open  ]],
Cell[833081, 20263, 1190, 28, 80, "Text",ExpressionUUID->"8a137350-f30a-45f9-a760-dd00605a85da"]
}, Open  ]],
Cell[CellGroupData[{
Cell[834308, 20296, 147, 1, 32, "Subsection",ExpressionUUID->"759c9208-1e0c-4d49-b19d-bf97a15ff664",
 CellTags->"Section7b"],
Cell[834458, 20299, 814, 19, 53, "Text",ExpressionUUID->"71ad8f7a-5fa2-4f56-9ef3-06bcdbe70609"],
Cell[CellGroupData[{
Cell[835297, 20322, 416, 10, 51, "Input",ExpressionUUID->"178efa70-3afa-4690-a51f-60488a2dc353",
 InitializationCell->True],
Cell[835716, 20334, 5238, 108, 358, "Output",ExpressionUUID->"890c03fd-5607-4831-9d30-4f0d20734d16"]
}, Open  ]],
Cell[840969, 20445, 196, 3, 20, "Text",ExpressionUUID->"04a437a9-47fe-477a-b318-a18d5b86456f"],
Cell[CellGroupData[{
Cell[841190, 20452, 237, 6, 31, "Input",ExpressionUUID->"804ac305-53b7-46e9-ba8d-540c0e0b5920",
 InitializationCell->True],
Cell[841430, 20460, 4582, 95, 431, "Output",ExpressionUUID->"1d9c672d-cb50-45cc-9536-88acdb1f5d48"]
}, Open  ]],
Cell[846027, 20558, 303, 6, 35, "Text",ExpressionUUID->"45b555ab-5acc-4477-8b67-0898fc43757c"],
Cell[CellGroupData[{
Cell[846355, 20568, 501, 13, 70, "Input",ExpressionUUID->"8379a1bc-5ec6-4863-ac9b-3050236f939e",
 InitializationCell->True],
Cell[846859, 20583, 3575, 76, 391, "Output",ExpressionUUID->"7c097e18-e454-4bd1-afd6-62f32616954d"]
}, Open  ]],
Cell[850449, 20662, 877, 25, 42, "Text",ExpressionUUID->"28fe7105-5903-4c03-a466-d215b2f5fb0a"],
Cell[CellGroupData[{
Cell[851351, 20691, 275, 7, 31, "Input",ExpressionUUID->"5f00f345-7c35-4de2-a6a4-19f51bbde32d"],
Cell[851629, 20700, 114, 2, 31, "Output",ExpressionUUID->"8de4f50c-cbc9-47f9-9801-84f131ad24a1"]
}, Open  ]],
Cell[851758, 20705, 354, 9, 23, "Text",ExpressionUUID->"63e66462-150b-44b8-8853-527c303a2ca1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[852161, 20720, 116, 1, 55, "Section",ExpressionUUID->"d784a58b-4e77-458f-aa87-baf1c76553df",
 CellTags->"Section8"],
Cell[852280, 20723, 279, 4, 35, "Text",ExpressionUUID->"2bb935e0-3645-446d-b19f-da5a7a01efb8"],
Cell[852562, 20729, 217, 3, 20, "Text",ExpressionUUID->"b483a67b-b29f-4325-9de7-3d870fef70df"],
Cell[852782, 20734, 610, 8, 65, "Text",ExpressionUUID->"8c9f33a1-8fc9-4449-8c7e-a235be4a975d"],
Cell[853395, 20744, 649, 9, 65, "Text",ExpressionUUID->"39244808-5f1a-459a-a4ea-46b76a98ed64"],
Cell[854047, 20755, 1021, 20, 83, "Text",ExpressionUUID->"5ff16e5e-4b91-4276-a97b-77d027b374a3"],
Cell[855071, 20777, 323, 7, 21, "Text",ExpressionUUID->"5a18d6c8-7668-42aa-aef1-9efd1f77f572"],
Cell[855397, 20786, 503, 14, 51, "Input",ExpressionUUID->"36086765-076c-4c0d-9183-ba0ce1605b34",
 InitializationCell->True],
Cell[855903, 20802, 242, 4, 35, "Text",ExpressionUUID->"f4c654e7-6e28-4e47-a096-92d87afe2e5a"],
Cell[CellGroupData[{
Cell[856170, 20810, 145, 3, 31, "Input",ExpressionUUID->"4ea52fb9-a3a1-4346-96b6-2da6b3d3bad0"],
Cell[856318, 20815, 12084, 269, 60, "Output",ExpressionUUID->"45e680ca-b6cf-4bf4-a520-fb0a9ea16201"]
}, Open  ]],
Cell[868417, 21087, 444, 8, 36, "Text",ExpressionUUID->"7c5ab2eb-f138-4166-8b9e-cbbbefaf5603"],
Cell[868864, 21097, 118, 0, 20, "Text",ExpressionUUID->"1bbbcaa3-25cf-415c-86d4-d5b69e8994b9"],
Cell[868985, 21099, 3364, 83, 374, "Input",ExpressionUUID->"3e20a829-05e9-49e3-b751-22b99f0bfcd4",
 InitializationCell->True],
Cell[872352, 21184, 737, 17, 58, "Text",ExpressionUUID->"c57d94b6-2d3e-4ea1-a6d7-3282378f933c"],
Cell[873092, 21203, 474, 9, 58, "Text",ExpressionUUID->"f01159e6-c3bf-4267-89d3-4b3d78727052"],
Cell[873569, 21214, 870, 24, 58, "Text",ExpressionUUID->"72ed1039-3d6b-48e2-93ad-07774f1fec72"],
Cell[874442, 21240, 123, 0, 20, "Text",ExpressionUUID->"e3c98c7e-500a-4d08-a595-b78e19d9cf48"],
Cell[CellGroupData[{
Cell[874590, 21244, 2482, 57, 127, "Input",ExpressionUUID->"eabed605-e01c-4ec5-a50e-dbdde6abff99",
 InitializationCell->True],
Cell[877075, 21303, 3630, 60, 268, "Output",ExpressionUUID->"8ce3fc75-f6b1-43ba-a0e6-a187fa48e281"]
}, Open  ]],
Cell[CellGroupData[{
Cell[880742, 21368, 158, 3, 31, "Input",ExpressionUUID->"8ab82144-85d6-4325-a322-c41d038cbe8d"],
Cell[880903, 21373, 4767, 80, 268, "Output",ExpressionUUID->"5cd1809d-d833-4c9c-bac7-173593d80cfd"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[885719, 21459, 170, 3, 55, "Section",ExpressionUUID->"cb288fcc-7132-4aa8-b178-1230335363a2",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[885914, 21466, 138, 2, 32, "Subsection",ExpressionUUID->"64f25afc-0aed-47e8-a139-c15c3085465c",
 CellTags->"SolutionsCE1"],
Cell[886055, 21470, 119, 0, 42, "Problem",ExpressionUUID->"f46125cf-6439-4b1b-9561-f66bd70461d7"],
Cell[886177, 21472, 602, 12, 50, "Text",ExpressionUUID->"98a5630f-64b8-4055-befd-8a13b0258f3d"],
Cell[886782, 21486, 1361, 39, 165, "Input",ExpressionUUID->"75e78af1-fa41-417b-a69e-2734f48c5fa5",
 InitializationCell->True],
Cell[888146, 21527, 1009, 28, 38, "Text",ExpressionUUID->"8912dec1-3a22-4f9d-b2eb-5ba9ede804ce"],
Cell[889158, 21557, 981, 22, 67, "Text",ExpressionUUID->"ace56ef7-d8eb-4c4e-8443-de94bfe85655"],
Cell[CellGroupData[{
Cell[890164, 21583, 386, 10, 31, "Input",ExpressionUUID->"8c66bca9-369f-444b-bfe1-5c28d872a483"],
Cell[890553, 21595, 14059, 286, 507, "Output",ExpressionUUID->"9612f1f9-3505-4a23-beb5-28e929e653e7"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[904661, 21887, 138, 2, 32, "Subsection",ExpressionUUID->"8a238ba0-33e6-438e-9880-b9f82caf5bc8",
 CellTags->"SolutionsCE2"],
Cell[904802, 21891, 150, 2, 42, "Problem",ExpressionUUID->"8ff022e5-18a9-4e3f-8e32-6e9430a74522"],
Cell[904955, 21895, 830, 18, 52, "Text",ExpressionUUID->"ca919fa7-5f65-49c0-84a4-62743f36372d"],
Cell[905788, 21915, 1507, 41, 184, "Input",ExpressionUUID->"a27180cd-6386-4dc3-9fa6-21e70885f062",
 InitializationCell->True],
Cell[907298, 21958, 154, 2, 20, "Text",ExpressionUUID->"d5e574df-bf46-4d3f-aee1-89e15056e563"],
Cell[907455, 21962, 223, 6, 31, "Input",ExpressionUUID->"cbbb1237-d81f-42e6-a769-7cef0d3511a2",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[907703, 21972, 148, 3, 31, "Input",ExpressionUUID->"7acf9c26-231f-4e20-b65d-563fe4d90333"],
Cell[907854, 21977, 112, 2, 31, "Output",ExpressionUUID->"dc7ff650-0a7b-45e5-9985-a05b8f83fb4a"]
}, Open  ]],
Cell[907981, 21982, 166, 3, 20, "Text",ExpressionUUID->"2d158777-c79c-4a55-953a-3957a96176c1"],
Cell[CellGroupData[{
Cell[908172, 21989, 354, 10, 31, "Input",ExpressionUUID->"067835ea-8a74-4ce2-8d92-a416926deff3"],
Cell[908529, 22001, 21612, 415, 549, "Output",ExpressionUUID->"ca20aa90-6e10-40f5-b9d0-a06e127eebb6"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[930190, 22422, 138, 2, 32, "Subsection",ExpressionUUID->"e88e5896-4992-4eee-b4c3-c3b982466197",
 CellTags->"SolutionsCE9"],
Cell[930331, 22426, 232, 3, 55, "Problem",ExpressionUUID->"4690db05-14ed-4f87-8d3c-3573baed1567"],
Cell[930566, 22431, 542, 11, 51, "Text",ExpressionUUID->"47697946-1bac-4c48-abb3-272270d45f38"],
Cell[CellGroupData[{
Cell[931133, 22446, 196, 5, 31, "Input",ExpressionUUID->"dbbae5db-d405-44ce-9a0b-99bc4d5e8eda"],
Cell[931332, 22453, 2814, 49, 243, "Output",ExpressionUUID->"337076b6-8672-428f-9456-d44f901a2a16"]
}, Open  ]],
Cell[934161, 22505, 915, 16, 82, "Text",ExpressionUUID->"783a5267-adbf-414f-8b25-9fc7fdda5349"],
Cell[CellGroupData[{
Cell[935101, 22525, 226, 5, 31, "Input",ExpressionUUID->"a9135198-bd48-4de8-a39a-8865c44c13ee"],
Cell[935330, 22532, 3234, 55, 318, "Output",ExpressionUUID->"2e8c5d74-9e60-4e34-b929-80be3057c19b"]
}, Open  ]],
Cell[938579, 22590, 321, 6, 21, "Text",ExpressionUUID->"f3e22802-0bdb-4cba-b326-d1118970080c"],
Cell[938903, 22598, 363, 7, 36, "Text",ExpressionUUID->"3aa2ad82-0ebc-4ff7-b127-d8762675671e"],
Cell[939269, 22607, 929, 21, 56, "Text",ExpressionUUID->"99d0f0bb-e9f7-4d32-a5cf-bc7dcc1bd0b1"],
Cell[940201, 22630, 926, 26, 89, "Input",ExpressionUUID->"ca97698f-90b3-4a62-86ee-841bdcc5fc32",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[941152, 22660, 153, 3, 31, "Input",ExpressionUUID->"f3bc9153-5223-45b8-ab2a-ee3c0221b33c"],
Cell[941308, 22665, 3123, 69, 386, "Output",ExpressionUUID->"f22e02d4-99d3-4471-9b84-fc74d226000f"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[944480, 22740, 140, 2, 32, "Subsection",ExpressionUUID->"f3eda7c2-7e9d-4229-b39f-fddd85708cd4",
 CellTags->"SolutionsCE13"],
Cell[944623, 22744, 540, 11, 55, "Problem",ExpressionUUID->"8eaa4f02-8fc8-457c-93b4-bf33cd166ecd"],
Cell[945166, 22757, 634, 14, 51, "Text",ExpressionUUID->"7c93f37c-d834-46a1-9c17-5283109519c6"],
Cell[945803, 22773, 1230, 34, 165, "Input",ExpressionUUID->"06acd114-54be-40b8-835c-c69bac74592f",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[947058, 22811, 289, 8, 31, "Input",ExpressionUUID->"d94443d0-1d45-4d72-abb5-da63b3c926fb"],
Cell[947350, 22821, 456, 13, 47, "Output",ExpressionUUID->"8ce3c9c5-5686-49ec-9e94-5941c3db78ee"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[947867, 22841, 107, 1, 55, "Section",ExpressionUUID->"44306461-a2f0-47bc-88cd-2a26cd3eaa77",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[947999, 22846, 231, 5, 20, "Item1Exercise",ExpressionUUID->"ba4f6430-4eda-4d3f-9e83-76adda5cf51c"],
Cell[948233, 22853, 201, 3, 20, "Item1Exercise",ExpressionUUID->"58c8bc2d-77fe-473b-b4a2-644c295ce261"],
Cell[948437, 22858, 183, 3, 20, "Item1Exercise",ExpressionUUID->"604a5d99-4463-41b3-bfcd-b3da9ba0d164"],
Cell[948623, 22863, 303, 4, 33, "Item1Exercise",ExpressionUUID->"8975beb5-ce52-41ca-95fd-f2d21be33e1b"],
Cell[948929, 22869, 185, 3, 20, "Item1Exercise",ExpressionUUID->"943cc907-946e-4d6c-8ce0-36e56ef9c27d"],
Cell[949117, 22874, 512, 13, 33, "Item1Exercise",ExpressionUUID->"11230cba-f1d8-4c1a-a62b-eb1bfff04c8b"],
Cell[949632, 22889, 1182, 30, 46, "Item1Exercise",ExpressionUUID->"6090ebc2-b451-4aee-a5a6-178437eedefd"],
Cell[950817, 22921, 1176, 30, 46, "Item1Exercise",ExpressionUUID->"04714530-c022-4425-b79a-b533d7d6a7d9"],
Cell[951996, 22953, 255, 4, 33, "Item1Exercise",ExpressionUUID->"15c4ee4d-be01-4a8f-9b1a-3149a0ae7e89"],
Cell[952254, 22959, 263, 4, 33, "Item1Exercise",ExpressionUUID->"eb58b103-6f19-4f89-861c-1e8a81786e7b"],
Cell[952520, 22965, 261, 4, 33, "Item1Exercise",ExpressionUUID->"aec3641c-1743-4b95-9822-35f3d5f4fcd9"],
Cell[952784, 22971, 282, 4, 33, "Item1Exercise",ExpressionUUID->"045ab48d-89a2-460d-95ea-3a6072055599"],
Cell[953069, 22977, 612, 15, 33, "Item1Exercise",ExpressionUUID->"3cc0e99a-3ad0-4656-8f29-3c5b4ac1d0f0"],
Cell[953684, 22994, 594, 13, 46, "Item1Exercise",ExpressionUUID->"7b2552cf-5c77-4c0c-92ba-8091b94e7488"],
Cell[954281, 23009, 451, 6, 46, "Item1Exercise",ExpressionUUID->"b3878cd3-da7f-45c5-b51f-9d892938562b"],
Cell[954735, 23017, 573, 9, 59, "Item1Exercise",ExpressionUUID->"923f43c7-6651-43bf-a133-1a5616a37b2f"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

